#!/usr/bin/env node
var _nodeOs = require("node:os");
var _nodePath = require("node:path");
var _nodeFs = require("node:fs");
var _function = require("../library/env/function.js");
var _format = require("../library/common/format.js");
var _function2 = require("../library/common/function.js");
var _string = require("../library/common/string.js");
var _time = require("../library/common/time.js");
var _check = require("../library/common/check.js");
var _Tree = require("../library/common/data/Tree.js");
var _Stream = require("../library/node/data/Stream.js");
var _Z64String = require("../library/node/data/Z64String.js");
var _Path = require("../library/node/fs/Path.js");
var _File = require("../library/node/fs/File.js");
var _Directory = require("../library/node/fs/Directory.js");
var _Modify = require("../library/node/fs/Modify.js");
var _function3 = require("../library/node/server/function.js");
var _Server = require("../library/node/server/Server.js");
var _Proxy = require("../library/node/server/Proxy.js");
var _DefaultOpen = require("../library/node/system/DefaultOpen.js");
var _ResolveCommand = require("../library/node/system/ResolveCommand.js");
var _run = require("../library/node/run.js");
var _Process = require("../library/node/system/Process.js");
var _Status = require("../library/node/system/Status.js");
var _archive = require("../library/node/module/Archive/archive.js");
var _docker = require("../library/node/module/Software/docker.js");
var _npm = require("../library/node/module/Software/npm.js");
var _PingRace = require("../library/node/module/PingRace.js");
var _Auth = require("../library/node/module/Auth.js");
var _testConnection = require("./server/testConnection.js");
var _serveStatic = require("./server/serveStatic.js");
var _websocketGroup = require("./server/websocketGroup.js");
var _httpRequestProxy = require("./server/httpRequestProxy.js");
var _function4 = require("./function.js");
var _option = require("./option.js");
var _package = require("../package.json");
(async () => {
  const optionData = await (0, _option.parseOption)();
  if (optionData.getToggle("version")) return (0, _function4.logAuto)({
    packageName: _package.name,
    packageVersion: _package.version,
    nodeVersion: process.version,
    platform: process.platform,
    arch: process.arch,
    endianness: (0, _function.getEndianness)(),
    cpuCount: ((0, _nodeOs.cpus)() || [ "TERMUX FIX" ]).length
  });
  if (optionData.getToggle("help")) return (0, _function4.logAuto)((0, _option.formatUsage)());
  const modeName = _option.MODE_NAME_LIST.find((name => optionData.tryGet(name)));
  if (!modeName) throw new Error("no mode specified");
  await (async (optionData, modeName) => {
    const sharedPack = (0, _function4.sharedOption)(optionData, modeName);
    const {getFirst, tryGetFirst, getToggle} = optionData;
    const {argumentList, log, inputFile, outputFile, outputValueAuto, outputStream} = sharedPack;
    const isOutputJSON = getToggle("json");
    const root = tryGetFirst("root") || process.cwd();
    const logTaskResult = (task, path) => task(path).then((() => log(`[${modeName}] done: ${path}`)), (error => log(`[${modeName}] error: ${path}\n${error.stack || error}`)));
    const quickServerExot = async (protocol = "http:") => {
      const {hostname, port} = (0, _function3.parseHostString)(tryGetFirst("host") || "", "0.0.0.0");
      return (0, _Server.createServerExot)({
        protocol,
        hostname,
        port: port || await (0, _function3.autoTestServerPort)([ 80, 8080, 8888, 8800, 8e3 ], hostname)
      });
    };
    const startServer = async (configureFunc, option) => {
      const serverExot = await quickServerExot();
      const routePrefix = tryGetFirst("route-prefix") || "";
      (0, _testConnection.commonServerDown)(serverExot, log);
      return (0, _testConnection.commonServerUp)({
        serverExot,
        log,
        routePrefix,
        ...configureFunc({
          serverExot,
          log,
          routePrefix,
          ...option
        })
      });
    };
    switch (modeName) {
     case "wait":
      {
        const waitTime = argumentList[0] || 2e3;
        return (0, _time.setTimeoutAsync)(waitTime);
      }
     case "echo":
      return log(argumentList);
     case "cat":
      if (argumentList.length) for (const path of argumentList) await (0, _Stream.quickRunletFromStream)((0, _nodeFs.createReadStream)(path), process.stdout); else process.stdin.isTTY || await (0, _Stream.quickRunletFromStream)(process.stdin, process.stdout);
      return;
     case "write":
     case "append":
      {
        if (process.stdin.isTTY) throw new Error("unsupported TTY stdin");
        const flags = "write" === modeName ? "w" : "a";
        return (0, _Stream.quickRunletFromStream)(process.stdin, (0, _nodeFs.createWriteStream)(argumentList[0], {
          flags
        }));
      }
     case "merge":
      {
        const [mergedFile, ...fileList] = argumentList;
        for (const path of fileList) await (0, _Stream.quickRunletFromStream)((0, _nodeFs.createReadStream)(path), (0, _nodeFs.createWriteStream)(mergedFile, {
          flags: "a"
        }));
        return;
      }
     case "text-file":
      {
        const [openMode = "write"] = argumentList;
        return (openMode.startsWith("w") ? _File.writeText : _File.appendText)(outputFile, `${getFirst("note")}\n`);
      }
     case "text-replace":
     case "text-replace-all":
      {
        const [fromString, toString] = argumentList;
        return (0, _File.editTextSync)((string => string["text-replace" === modeName ? "replace" : "replaceAll"](fromString, toString)), inputFile);
      }
     case "open":
      {
        const [uri = ".", isDetached = !1] = argumentList;
        return (isDetached ? _run.runDetached : _run.runSync)([ ...(0, _DefaultOpen.getDefaultOpenCommandList)(), uri.includes("://") ? uri : (0, _nodePath.normalize)(uri) ]);
      }
     case "which":
      {
        const commandNameOrPath = argumentList[0];
        const resultCommand = (0, _ResolveCommand.resolveCommand)(commandNameOrPath, root);
        if (!resultCommand) throw new Error(`failed to resolve command: ${commandNameOrPath}`);
        return log(resultCommand);
      }
     case "run":
      {
        const commandNameOrPath = argumentList[0];
        return (0, _run.run)([ (0, _ResolveCommand.resolveCommand)(commandNameOrPath, root), ...argumentList.slice(1) ]).promise;
      }
     case "detach":
      {
        const commandNameOrPath = argumentList[0];
        return (0, _run.runDetached)([ (0, _ResolveCommand.resolveCommand)(commandNameOrPath, root), ...argumentList.slice(1) ], {
          stdoutFile: outputFile || void 0
        });
      }
     case "status":
      return log(isOutputJSON ? (0, _Status.getSystemStatus)() : (0, _Status.describeSystemStatus)());
     case "create-directory":
      for (const path of argumentList) await logTaskResult(_Directory.createDirectory, path);
      return;
     case "modify-copy":
      return (0, _Modify.modifyCopy)(argumentList[0], argumentList[1]);
     case "modify-rename":
      return (0, _Modify.modifyRename)(argumentList[0], argumentList[1]);
     case "modify-delete":
      for (const path of argumentList) await logTaskResult(_Modify.modifyDelete, path);
      return;
     case "process-status":
      {
        const [outputMode = "pid--"] = argumentList;
        return log(await (isOutputJSON ? _Process.getAllProcessStatusAsync : _Process.describeAllProcessStatusAsync)(outputMode));
      }
     case "process-signal":
      {
        inputFile && argumentList.unshift(await (0, _File.readText)(inputFile));
        const [pidString, signal = "SIGTERM"] = argumentList;
        return process.kill(Number(pidString), signal);
      }
     case "json-format":
      {
        const [unfoldLevel = 2] = argumentList;
        return (0, _File.writeText)(outputFile || inputFile, (0, _format.prettyStringifyJSON)(await (0, _File.readJSON)(inputFile), unfoldLevel));
      }
     case "encode":
     case "decode":
      {
        const [codecType = "b64"] = argumentList;
        const [encode, decode] = {
          b64: [ _Z64String.packB64, _Z64String.unpackB64 ],
          gz64: [ _Z64String.packGz64, _Z64String.unpackGz64 ],
          br64: [ _Z64String.packBr64, _Z64String.unpackBr64 ]
        }[codecType];
        return outputValueAuto(("encode" === modeName ? encode : decode)(getFirst("note")));
      }
     case "file-list":
     case "file-list-all":
     case "file-tree":
      {
        const prettyStringifyFileTree = async rootPath => {
          const {dirInfoListMap} = await (0, _Directory.getDirInfoTree)(rootPath);
          const resultList = [];
          (0, _Tree.prettyStringifyTreeNode)((([[path], level]) => {
            var _dirInfoListMap$get;
            return null == (_dirInfoListMap$get = dirInfoListMap.get(path)) ? void 0 : _dirInfoListMap$get.map((({name, path: subPath}, subIndex, {length}) => [ [ subPath, name ], level + 1, subIndex !== length - 1 ]));
          }), [ [ rootPath, "NAME" ], -1, !1 ], ((prefix, [, name]) => resultList.push(`${prefix}${name}`)));
          return resultList.join("\n");
        };
        const rootPath = argumentList[0] || process.cwd();
        return outputValueAuto("file-list" === modeName ? (await (0, _Directory.getDirInfoList)(rootPath)).map((({type, name}) => type === _Path.PATH_TYPE.Directory ? `${name}/` : name)) : "file-list-all" === modeName ? await (0, _Directory.getFileList)(rootPath) : "file-tree" === modeName ? await prettyStringifyFileTree(rootPath) : "");
      }
     case "compress":
      return (0, _archive.compressAutoAsync)(inputFile, outputFile);
     case "extract":
      return (0, _archive.extractAutoAsync)(inputFile, outputFile);
     case "docker":
      return (0, _docker.runDocker)(argumentList).promise;
     case "docker-compose":
      return (0, _docker.runCompose)(argumentList).promise;
     case "auth-file-describe":
      return outputValueAuto(await (0, _Auth.describeAuthFile)(inputFile));
     case "auth-check-code-generate":
      return outputValueAuto(await (0, _Auth.generateAuthCheckCode)(inputFile, argumentList[0]));
     case "auth-check-code-verify":
      await (0, _Auth.verifyAuthCheckCode)(inputFile, argumentList[0], argumentList[1] && Number(argumentList[1]));
      return outputValueAuto("pass verify");
     case "auth-gen-tag":
      await (0, _Auth.generateAuthFile)(outputFile, {
        tag: argumentList[0],
        size: tryGetFirst("auth-gen-size"),
        tokenSize: tryGetFirst("auth-gen-token-size"),
        timeGap: tryGetFirst("auth-gen-time-gap"),
        info: tryGetFirst("auth-gen-info")
      });
      return log(await (0, _Auth.describeAuthFile)(outputFile));
     case "ping-race":
     case "ping-stat":
      return outputValueAuto(await ("ping-race" === modeName ? _PingRace.pingRaceUrlList : _PingRace.pingStatUrlList)(argumentList, {
        timeout: tryGetFirst("timeout") || 5e3
      }));
     case "server-serve-static":
     case "server-serve-static-simple":
     case "server-serve-static-api":
      {
        const [expireTime = 5e3] = argumentList;
        return startServer(_serveStatic.configure, {
          isSimpleServe: modeName.endsWith("-simple"),
          isSimpleApi: modeName.endsWith("-api"),
          expireTime: Number(expireTime),
          staticRoot: root
        });
      }
     case "server-websocket-group":
      return startServer(_websocketGroup.configure);
     case "server-test-connection":
     case "server-test-connection-simple":
     case "server-test-connection-simple-payload":
      return startServer(_testConnection.configure, {
        isSimpleTest: modeName.includes("-simple"),
        isSimplePayload: modeName.endsWith("-payload")
      });
     case "server-tcp-proxy":
      {
        let targetOptionList;
        let getTargetOption;
        if ((0, _check.isBasicFunction)(argumentList[0])) {
          targetOptionList = [ {
            hostname: "custom-hostname",
            port: "custom-port"
          } ];
          getTargetOption = argumentList[0];
        } else {
          targetOptionList = argumentList.map((host => (0, _function3.parseHostString)(host, "127.0.0.1")));
          let targetOptionIndex = 0;
          getTargetOption = socket => {
            targetOptionIndex = (targetOptionIndex + 1) % targetOptionList.length;
            const targetOption = targetOptionList[targetOptionIndex];
            log(`[CONNECT] ${socket.remoteAddress}:${socket.remotePort} => ${targetOption.hostname}:${targetOption.port}`);
            return targetOption;
          };
        }
        const serverExot = await quickServerExot("tcp:");
        serverExot.server.on("connection", (0, _Proxy.createTCPProxyListener)({
          getTargetOption
        }));
        await serverExot.up();
        (0, _testConnection.commonServerDown)(serverExot, log);
        return log((0, _string.indentList)("[TCPProxy]", [ `pid: ${process.pid}`, `at: ${serverExot.option.hostname}:${serverExot.option.port}`, ...targetOptionList.map((option => `proxy to: ${option.hostname}:${option.port}`)) ]));
      }
     case "server-http-request-proxy":
      {
        const [toOriginString, isSetXForwardString] = argumentList;
        const {protocol, origin} = new URL(toOriginString);
        if (!/^https?:$/.test(protocol)) throw new Error(`invalid protocol: ${protocol} from: ${toOriginString}`);
        return startServer(_httpRequestProxy.configure, {
          targetOrigin: origin,
          isSetXForward: Boolean(isSetXForwardString),
          timeout: tryGetFirst("timeout") || 42e3
        });
      }
     case "fetch":
      {
        let [initialUrl, method = "GET", jumpMax = 4] = argumentList;
        jumpMax = Number(jumpMax) || 0;
        const timeout = tryGetFirst("timeout") || 0;
        log(`[fetch] jumpMax: ${jumpMax}, timeout: ${timeout || "none"}`);
        const body = inputFile ? await (0, _File.readBuffer)(inputFile) : null;
        let isDone = !1;
        const response = await (0, _npm.fetchWithJumpProxy)(initialUrl, {
          method,
          timeout,
          jumpMax,
          body,
          headers: {
            accept: "*/*",
            "user-agent": `${_package.name}/${_package.version}`
          },
          onProgressUpload: (0, _function2.throttleT)(((now, total) => isDone || log(`[fetch-upload] ${(0, _format.percent)(now / total)} (${(0, _format.binary)(now)}B / ${(0, _format.binary)(total)}B)`)), 1e3),
          onProgressDownload: (0, _function2.throttleT)(((now, total) => isDone || log(`[fetch-download] ${(0, _format.percent)(now / total)} (${(0, _format.binary)(now)}B / ${(0, _format.binary)(total)}B)`)), 1e3),
          preFetch: (url, jumpCount, cookieList) => log(`[fetch] <${method}>${url}, jump: ${jumpCount}/${jumpMax}, timeout: ${timeout ? (0, _format.time)(timeout) : "none"}, cookie: ${cookieList.length}`)
        });
        if (!response.ok) throw new Error(`bad status: ${response.status}`);
        const contentLength = Number(response.headers["content-length"]);
        log(`[fetch] status: ${response.status}, header: ${(0, _format.prettyStringifyJSON)(response.headers)}`);
        log(`[fetch] fetch response content${contentLength ? ` (${(0, _format.binary)(contentLength)}B)` : ""}...`);
        await outputStream(response.stream());
        isDone = !0;
        return log("\n[fetch] done");
      }
     default:
      return (0, _function4.sharedMode)(sharedPack);
    }
  })(optionData, modeName).catch((error => {
    console.warn(`[Error] in mode: ${modeName}: ${error.stack || error}`);
    process.exit(2);
  }));
})().catch((error => {
  console.warn((0, _option.formatUsage)(error.stack || error, "simple"));
  process.exit(1);
}));