var _nodePath = require("node:path");
var _compare = require("../../library/common/compare.js");
var _format = require("../../library/common/format.js");
var _string = require("../../library/common/string.js");
var _MIME = require("../../library/common/module/MIME.js");
var _HTML = require("../../library/common/module/HTML.js");
var _Path = require("../../library/node/fs/Path.js");
var _Directory = require("../../library/node/fs/Directory.js");
var _Common = require("../../library/node/server/Responder/Common.js");
var _Send = require("../../library/node/server/Responder/Send.js");
var _Router = require("../../library/node/server/Responder/Router.js");
var _ServeStatic = require("../../library/node/server/Responder/ServeStatic.js");
exports.configure = ({routePrefix, isSimpleServe, isSimpleApi, expireTime, staticRoot}) => {
  const fromStaticRoot = (0, _Path.createPathPrefixLock)(staticRoot);
  const getParamFilePath = store => fromStaticRoot(decodeURIComponent((0, _Router.getRouteParamAny)(store) || ""));
  const responderServeStatic = (0, _ServeStatic.createResponderServeStatic)({
    expireTime
  });
  return {
    routeConfigList: isSimpleApi ? [ [ [ "/", "/*" ], Object.keys(_Router.METHOD_MAP), store => responderServeStatic(store, fromStaticRoot(`${store.request.url.replace(/\//g, "#")}#[${store.request.method}]`)) ] ] : [ [ isSimpleServe ? "/*" : "/file/*", [ "GET", "HEAD" ], store => responderServeStatic(store, getParamFilePath(store)) ], !isSimpleServe && [ "/list/*", "GET", store => responderFilePathList(store, getParamFilePath(store), routePrefix, staticRoot) ], !isSimpleServe && [ [ "/", "/*", "/file", "/list" ], "GET", store => (0, 
    _Common.responderEndWithRedirect)(store, {
      redirectUrl: `${routePrefix}/list/`
    }) ] ],
    isAddFavicon: !isSimpleServe,
    title: "ServerServeStatic|" + (isSimpleServe ? "no-list" : "with-list"),
    extraInfo: {
      staticRoot,
      expireTime: (0, _format.time)(expireTime)
    }
  };
};
const responderFilePathList = async (store, rootPath, routePrefix, staticRoot) => {
  const relativeRoot = (0, _nodePath.relative)(staticRoot, rootPath);
  const titleHTML = `/${(0, _string.escapeHTML)((0, _Path.toPosixPath)(relativeRoot))}`;
  const directoryInfoList = [];
  const fileInfoList = [];
  for (const dirInfo of await (0, _Directory.getDirInfoList)(rootPath)) {
    dirInfo.stat = await (0, _Path.getPathStat)(dirInfo.path);
    dirInfo.stat.isDirectory() ? directoryInfoList.push(dirInfo) : fileInfoList.push(dirInfo);
  }
  return (0, _Send.responderSendBufferCompress)(store, {
    type: _MIME.BASIC_EXTENSION_MAP.html,
    buffer: Buffer.from((0, _HTML.COMMON_LAYOUT)([ `<title>${titleHTML}</title>`, (0, _HTML.COMMON_STYLE)(), mainStyle ], [ `<b class="auto-height">${titleHTML}</b>`, relativeRoot && renderItem(routePrefix, "/list", [ (0, _nodePath.dirname)(relativeRoot) ], "🔙", ".."), ...directoryInfoList.sort(compareInfo).map((({name, stat: {mtimeMs}}) => renderItem(routePrefix, "/list", [ relativeRoot, name ], "📁", `${name}/`, 0, mtimeMs))), ...fileInfoList.sort(compareInfo).map((({name, stat: {size, mtimeMs}}) => renderItem(routePrefix, "/file", [ relativeRoot, name ], "📄", name, size, mtimeMs, !0))) ]))
  });
};
const mainStyle = "<style>\nbody { white-space: pre; }\na, b { display: flex; align-items: center; }\na { text-decoration: none; border-top: 1px solid var(--c-fill-n); }\na:hover { background: var(--c-fill-s); }\np.name { overflow: hidden; text-overflow: ellipsis; }\np.size, p.date { padding-left: 1em; text-align: right; opacity: 0.5 }\np.size { flex: 1; }\n@media only screen and (max-width: 768px) { p.date { display: none; } }\n</style>";
const compareInfo = ({name: a}, {name: b}) => (0, _compare.compareStringWithNumber)(a, b);
const renderItem = (routePrefix, hrefPrefix, hrefFragList, tag, name, size, mtimeMs, isDownload, HREF = `href="${routePrefix}${hrefPrefix}/${(0, _string.lazyEncodeURI)((0, _Path.toPosixPath)((0, _nodePath.join)(...hrefFragList)))}"`, NAME = (0, 
_string.escapeHTML)(name), NAME_EXT = (/\.(\w{1,5})$/.exec(NAME) || [])[1] || "") => `<a class="auto-height" ${HREF} ${isDownload ? 'target="_blank"' : ""}>\n<object class="name">${isDownload ? `<a ${HREF} download="${NAME}">${tag}</a>` : tag}</object>\n<p class="name">|${NAME.slice(0, -NAME_EXT.length || void 0)}</p>\n<p>${NAME_EXT}</p>\n<p class="size">${size ? `${(0, 
_format.binary)(size)}B` : ""}</p>\n<p class="date">${mtimeMs ? (0, _string.escapeHTML)(new Date(mtimeMs).toISOString()) : ""}</p>\n</a>`;