var _time = require("../../library/common/time.js");
var _Array = require("../../library/common/immutable/Array.js");
var _MIME = require("../../library/common/module/MIME.js");
var _Stream = require("../../library/node/data/Stream.js");
var _Server = require("../../library/node/server/Server.js");
var _Common = require("../../library/node/server/Responder/Common.js");
var _Send = require("../../library/node/server/Responder/Send.js");
var _Router = require("../../library/node/server/Responder/Router.js");
var _ExitListener = require("../../library/node/system/ExitListener.js");
exports.commonServerUp = async ({serverExot: {up, server, option}, log, routePrefix, routeConfigList, isAddFavicon, title, extraInfo = {}}) => {
  const responderLogEnd = (0, _Common.createResponderLogEnd)({
    log
  });
  server.on("request", (0, _Server.createRequestListener)({
    responderList: [ (0, _Common.createResponderLog)({
      log
    }), (0, _Router.createResponderRouter)({
      routeMap: (0, _Router.createRouteMap)([ ...routeConfigList, !routePrefix && isAddFavicon && [ [ "/favicon", "/favicon.ico" ], "GET", (0, _Send.createResponderFavicon)() ] ].filter(Boolean), routePrefix),
      baseUrl: option.baseUrl
    }) ],
    responderEnd: store => {
      (0, _Common.responderEnd)(store);
      responderLogEnd(store);
    }
  }));
  await up();
  routePrefix && Object.assign(extraInfo, {
    routePrefix
  });
  log((0, _Server.describeServerOption)(option, title, extraInfo));
};
exports.commonServerDown = (serverExot, log) => (0, _ExitListener.addExitListenerLossyOnce)((({eventType, error}) => {
  log && log(`[exit|${eventType}] ${error}`);
  return serverExot.down();
}));
const BASIC_METHOD_LIST = [ "GET", "POST", "PUT", "DELETE" ];
exports.configure = ({log, routePrefix, isSimpleTest, isSimplePayload}) => {
  const bufferData = (0, _Send.prepareBufferData)(Buffer.from("TEST CONTENT"));
  const routeConfigList = isSimpleTest ? [ [ [ "/*", "/", "" ], Object.keys(_Router.METHOD_MAP), async store => {
    const {url, method, httpVersion, rawHeaders, socket: {remoteAddress, remotePort}} = store.request;
    const describeObject = {
      from: `${remoteAddress}:${remotePort}`,
      request: {
        url,
        method,
        httpVersion,
        headers: (0, _Array.arraySplitChunk)(rawHeaders, 2).map((fragList => fragList.join(": "))),
        payload: isSimplePayload ? (await (0, _Stream.readableStreamToBufferAsync)(store.request)).toString("base64") : void 0
      }
    };
    log(`[test-describe]\n${JSON.stringify(describeObject, null, 2)}`);
    return (0, _Send.responderSendJSON)(store, {
      object: describeObject
    });
  } ] ] : [ [ [ "/test-describe", "/test-describe/*" ], Object.keys(_Router.METHOD_MAP), async store => {
    const {url, method, httpVersion, rawHeaders, socket: {remoteAddress, remotePort}} = store.request;
    const describeObject = {
      from: `${remoteAddress}:${remotePort}`,
      request: {
        url,
        method,
        httpVersion,
        headers: (0, _Array.arraySplitChunk)(rawHeaders, 2).map((fragList => fragList.join(": ")))
      }
    };
    log(`[test-describe]\n${JSON.stringify(describeObject, null, 2)}`);
    return (0, _Send.responderSendJSON)(store, {
      object: describeObject
    });
  } ], [ [ "/test-echo-post", "/test-echo-post/:mime" ], "POST", async store => {
    const {url: {searchParams}} = store.getState();
    return (searchParams.has("no-compress") ? _Send.responderSendBuffer : _Send.responderSendBufferCompress)(store, {
      buffer: await (0, _Stream.readableStreamToBufferAsync)(store.request),
      type: _MIME.BASIC_EXTENSION_MAP[(0, _Router.getRouteParam)(store, "mime")]
    });
  } ], [ "/test-buffer", BASIC_METHOD_LIST, store => (0, _Send.responderSendBufferCompress)(store, bufferData) ], [ "/test-json", BASIC_METHOD_LIST, store => (0, _Send.responderSendJSON)(store, {
    object: {
      testKey: "TEST VALUE"
    }
  }) ], [ "/test-destroy", BASIC_METHOD_LIST, async store => store.response.destroy() ], [ [ "/test-timeout", "/test-timeout/:wait" ], BASIC_METHOD_LIST, async store => {
    const wait = parseInt((0, _Router.getRouteParam)(store, "wait")) || 200;
    await (0, _time.setTimeoutAsync)(wait);
    return (0, _Send.responderSendJSON)(store, {
      object: {
        wait
      }
    });
  } ], [ [ "/test-status-code", "/test-status-code/:status-code" ], BASIC_METHOD_LIST, async store => {
    const statusCode = parseInt((0, _Router.getRouteParam)(store, "status-code")) || 200;
    return (0, _Common.responderEndWithStatusCode)(store, {
      statusCode
    });
  } ], [ [ "/test-retry", "/test-retry/:count" ], BASIC_METHOD_LIST, (() => {
    const retryMap = {};
    return async store => {
      const count = parseInt((0, _Router.getRouteParam)(store, "count")) || 4;
      const currentCount = (retryMap[count] || 0) + 1;
      retryMap[count] = currentCount % count;
      currentCount === count ? await (0, _Send.responderSendJSON)(store, {
        object: {
          count
        }
      }) : store.response.destroy();
    };
  })() ], [ [ "/", "" ], "GET", (0, _Router.createResponderRouteListHTML)({
    getRouteMap: () => (0, _Router.createRouteMap)(routeConfigList, routePrefix)
  }) ] ];
  return {
    routeConfigList,
    isAddFavicon: !0,
    title: "TestConnection",
    extraInfo: {
      isSimpleTest,
      isSimplePayload
    }
  };
};