var _MIME = require("../../library/common/module/MIME.js");
var _HTML = require("../../library/common/module/HTML.js");
var _BufferPacket = require("../../library/node/data/BufferPacket.js");
var _Common = require("../../library/node/server/Responder/Common.js");
var _Send = require("../../library/node/server/Responder/Send.js");
var _Router = require("../../library/node/server/Responder/Router.js");
var _function = require("../../library/node/server/WS/function.js");
var _Server = require("../../library/node/server/WS/Server.js");
var _function2 = require("../../library/node/server/function.js");
const groupInfoMap = {};
const groupIdSetMap = {};
const fixUserIdClash = (groupPath, id) => {
  const groupIdSet = groupIdSetMap[groupPath];
  if (!groupIdSet || !groupIdSet.has(id)) return id;
  const prefix = id.replace(/-*\d+$/, "");
  let suffix = 1;
  for (;groupIdSet.has(`${prefix}-${suffix}`); ) suffix++;
  return `${prefix}-${suffix}`;
};
const PROTOCOL_TYPE_SET = new Set([ "group-binary-packet" ]);
const responderWebSocketGroupUpgrade = async store => {
  const {url: {searchParams}, info: {protocolList}} = store.getState();
  const groupPath = decodeURIComponent((0, _Router.getRouteParamAny)(store) || "");
  const id = searchParams.get("id");
  const protocol = getProtocol(protocolList, PROTOCOL_TYPE_SET);
  if (groupPath && id && protocol) {
    store.upgradeWS(protocol);
    store.setState({
      groupPath,
      id: fixUserIdClash(groupPath, id)
    });
    await (async store => {
      const {ws} = store;
      const {groupPath, id} = store.getState();
      const sendGroupInfo = groupInfo => {
        if (0 === groupInfo.size) return;
        const buffer = (0, _BufferPacket.packBufferPacket)(JSON.stringify({
          type: "#INFO_GROUP",
          payload: Array.from(groupIdSetMap[groupPath])
        }));
        groupInfo.forEach((v => v.ws.sendBinary(buffer)));
      };
      const sendGroupBuffer = (groupInfo, type, payload, payloadBuffer) => {
        if (groupInfo.size <= 1) return;
        const buffer = (0, _BufferPacket.packBufferPacket)(JSON.stringify({
          type,
          payload: {
            ...payload,
            id
          }
        }), payloadBuffer);
        groupInfo.forEach((v => {
          v.id !== id && v.ws.sendBinary(buffer);
        }));
      };
      const sendTargetBuffer = (groupInfo, type, payload, targetId, payloadBuffer) => {
        const targetInfo = groupInfo.get(targetId);
        targetInfo && targetInfo.ws.sendBinary((0, _BufferPacket.packBufferPacket)(JSON.stringify({
          type,
          targetId,
          payload: {
            ...payload,
            id
          }
        }), payloadBuffer));
      };
      const groupInfo = ((groupPath, id, ws) => {
        void 0 === groupIdSetMap[groupPath] && (groupIdSetMap[groupPath] = new Set);
        const groupIdSet = groupIdSetMap[groupPath];
        groupIdSet.add(id);
        void 0 === groupInfoMap[groupPath] && (groupInfoMap[groupPath] = new Map);
        const groupInfo = groupInfoMap[groupPath];
        groupInfo.set(id, {
          id,
          ws,
          groupIdSet
        });
        return groupInfo;
      })(groupPath, id, ws);
      await ws.sendBinary((0, _BufferPacket.packBufferPacket)(JSON.stringify({
        type: "#INFO_USER",
        payload: {
          groupPath,
          id
        }
      })));
      sendGroupInfo(groupInfo);
      try {
        for await (const {opcode, buffer} of ws) {
          if (opcode !== _function.OPCODE_TYPE.BINARY) throw new Error("expect BINARY");
          const [headerString, payloadBuffer] = (0, _BufferPacket.parseBufferPacket)(buffer);
          const {type, payload, targetId} = JSON.parse(headerString);
          "#CLOSE" === type && ws.close(1e3, "CLOSE received");
          "#BUFFER_GROUP" === type && sendGroupBuffer(groupInfo, type, payload, payloadBuffer);
          "#BUFFER_SINGLE" === type && sendTargetBuffer(groupInfo, type, payload, targetId, payloadBuffer);
        }
      } catch (error) {
        error && ws.close(1e3, String(error));
      }
      ((groupPath, id) => {
        const groupIdSet = groupIdSetMap[groupPath];
        groupIdSet && groupIdSet.delete(id);
        groupIdSet && 0 === groupIdSet.size && delete groupIdSetMap[groupPath];
        const groupInfo = groupInfoMap[groupPath];
        groupInfo && groupInfo.delete(id);
        groupInfo && 0 === groupInfo.size && delete groupInfoMap[groupPath];
      })(groupPath, id);
      sendGroupInfo(groupInfo);
    })(store);
  }
};
const getProtocol = (protocolList, protocolTypeSet) => protocolList.find(protocolTypeSet.has, protocolTypeSet);
exports.configure = ({serverExot, log, routePrefix}) => {
  const bufferData = (0, _Send.prepareBufferData)(Buffer.from((0, _HTML.COMMON_LAYOUT)([ (0, _HTML.COMMON_STYLE)(), mainStyle ], [ mainHTML, (0, _HTML.COMMON_SCRIPT)({
    TYPE_CLOSE: "#CLOSE",
    TYPE_INFO_GROUP: "#INFO_GROUP",
    TYPE_INFO_USER: "#INFO_USER",
    TYPE_BUFFER_GROUP: "#BUFFER_GROUP",
    TYPE_BUFFER_SINGLE: "#BUFFER_SINGLE",
    DATA_LENGTH_LIMIT: 268435456,
    ROUTE_PREFIX: routePrefix,
    onload: mainScriptInit
  }), (0, _function2.DR_BROWSER_SCRIPT_TAG)() ])), _MIME.BASIC_EXTENSION_MAP.html);
  const routeConfigList = [ [ [ "/", "" ], "GET", store => (0, _Send.responderSendBufferCompress)(store, bufferData) ], [ "/*", "GET", store => (0, _Common.responderEndWithRedirect)(store, {
    redirectUrl: `${routePrefix}/`
  }) ] ];
  (0, _Server.enableWSServer)(serverExot.server, {
    onUpgradeRequest: (0, _Server.createUpgradeRequestListener)({
      responderList: [ (0, _Common.createResponderLog)({
        log
      }), (0, _Router.createResponderRouter)({
        routeMap: (0, _Router.createRouteMap)([ [ "/websocket-group/*", "GET", responderWebSocketGroupUpgrade ] ], routePrefix),
        serverExot
      }) ]
    }),
    dataLengthLimit: 268435456
  });
  return {
    routeConfigList,
    isAddFavicon: !0,
    title: "WebSocketGroup"
  };
};
const mainStyle = "<style>\nlabel { display: flex; align-items: center; }\np { position: relative; padding: 14px 4px 2px; border-top: 1px solid var(--c-fill-n); word-break: break-all; }\np:hover { background: var(--c-fill-s); }\npre { overflow: auto; padding: 0 2px; max-height: 10em; border-left: 1px solid var(--c-fill-n); color: #888; }\ninput { flex: 1; }\n.flex-column { display: flex; flex-flow: column; }\n.non-flex { flex-shrink: 0; }\n.time-tag, .id-tag { position: absolute; top: 0; font-size: 12px; line-height: 12px; }\n.time-tag { right: 0; color: var(--c-fill-d); }\n.id-tag { left: 0; }\n.color-self { color: var(--c-dr); }\n.color-system { color: var(--c-fill-d); }\n</style>";
const mainHTML = '\n<div id="setup" class="flex-column">\n  <input id="group-path" class="auto-height" type="text" placeholder="group-path" autofocus />\n  <input id="id" class="auto-height" type="text" placeholder="id" />\n</div>\n<button id="button-toggle"></button>\n<div id="log" style="overflow-y: auto; flex: 1; min-height: 0;"></div>\n<button id="button-log-clear" class="non-flex">Clear Log [Ctrl+l]</button>\n<div id="main" class="flex-column non-flex">\n  <label for="payload-text">Message Text: </label>\n  <textarea id="payload-text" style="min-height: 6em;"></textarea>\n  <label>Message File: <input id="payload-file" class="auto-height" type="file" /></label>\n  <button id="button-send"></button>\n</div>\n';
const mainScriptInit = () => {
  const {document, alert, getSelection, location, Blob, WebSocket, qS, cE, TYPE_CLOSE, TYPE_INFO_GROUP, TYPE_INFO_USER, TYPE_BUFFER_GROUP, TYPE_BUFFER_SINGLE, DATA_LENGTH_LIMIT, ROUTE_PREFIX, Dr: {Common: {Time: {setTimeoutAsync}, Function: {lossyAsync}, Error: {withFallbackResult}, Math: {getRandomInt, getRandomId62S}, Format: {binary}}, Browser: {Data: {BlobPacket: {packBlobPacket, parseBlobPacket}}, DOM: {applyReceiveFileListListener, createDownloadWithBlob}, Input: {KeyCommand: {createKeyCommandHub}}}}} = window;
  const idTag = (id, className = "") => cE("span", {
    innerText: `[${id}]`,
    className: `id-tag ${className}`
  });
  const timeTag = () => cE("span", {
    innerText: (new Date).toLocaleString(),
    className: "time-tag"
  });
  const appendLog = (...elementList) => {
    const log = qS("#log");
    const item = cE("p", {
      id: getRandomId62S()
    }, elementList);
    log.appendChild(item);
    log.scrollTop = log.scrollHeight;
    return item;
  };
  const addLog = ({id, text, className}) => appendLog(cE("pre", {
    innerText: text,
    ondblclick: event => getSelection().selectAllChildren(event.currentTarget.parentNode.querySelector("pre"))
  }), idTag(id, className), timeTag());
  const addLogWithFile = ({isSend, id, fileName, fileSize, fileId, className}) => appendLog(cE("pre", {
    innerText: isSend ? "sharing: " : "share: "
  }), cE(isSend ? "b" : "button", {
    innerText: `📄 ${fileName} (${binary(fileSize)}B)`,
    onclick: isSend ? null : () => requestFile(id, fileId)
  }), idTag(id, className), timeTag());
  const addLogSystem = text => appendLog(cE("pre", {
    innerText: text,
    className: "color-system"
  }), idTag("System", "color-system"), timeTag());
  const clearLog = () => qS("#log", "");
  const STATE = {
    fileWeakMap: new WeakMap,
    retryCount: 0
  };
  const onCloseWebSocket = () => {
    addLogSystem(`Left group: ${STATE.groupPath}`);
    qS("#setup").style.display = "";
    qS("#main").style.display = "none";
    qS("#group-path").focus();
    qS("#button-toggle", "Enter Group [Ctrl+d]");
    document.title = "WebSocket Group";
    STATE.websocket = null;
    STATE.groupPath = null;
    STATE.id = null;
    STATE.groupInfo = [];
  };
  const onErrorRetry = lossyAsync((async error => {
    onCloseWebSocket();
    await setTimeoutAsync(200 * STATE.retryCount);
    STATE.retryCount++;
    STATE.retryCount <= 10 && addLogSystem(`(${STATE.retryCount}) connection dropped, try re-connect... [${error.message || error.type || error}]`);
    STATE.retryCount <= 10 && toggleWebSocket();
  })).trigger;
  onCloseWebSocket();
  clearLog();
  const requestFile = (id, fileId) => {
    STATE.websocket.send(packBlobPacket(JSON.stringify({
      type: TYPE_BUFFER_SINGLE,
      targetId: id,
      payload: {
        fileId,
        intent: "request"
      }
    })));
  };
  const toggleWebSocket = () => {
    if (STATE.websocket) {
      STATE.websocket.send(packBlobPacket(JSON.stringify({
        type: TYPE_CLOSE
      })));
      return;
    }
    const groupPath = qS("#group-path").value.trim() || "public";
    const id = qS("#id").value.trim() || getRandomUserId();
    const websocket = new WebSocket(((groupPath, id) => {
      const {protocol, host} = location;
      return `${"https:" === protocol ? "wss:" : "ws:"}//${host}${ROUTE_PREFIX}/websocket-group/${encodeURIComponent(groupPath)}?id=${encodeURIComponent(id)}`;
    })(groupPath, id), "group-binary-packet");
    websocket.binaryType = "blob";
    const onOpenInfo = id => {
      qS("#group-path").value = groupPath;
      qS("#id").value = id;
      (({websocket, groupPath, id}) => {
        addLogSystem(`Join group: ${groupPath}, as: ${id}`);
        qS("#setup").style.display = "none";
        qS("#main").style.display = "";
        qS("#payload-text").focus();
        qS("#button-toggle", `Exit Group: ${groupPath} [Ctrl+d]`);
        ((groupPath, id) => {
          location.hash = `#${new URLSearchParams({
            GP: groupPath,
            ID: id
          })}`;
        })(groupPath, id);
        document.title = `[${groupPath}/${id}]`;
        STATE.websocket = websocket;
        STATE.groupPath = groupPath;
        STATE.id = id;
        STATE.groupInfo = [];
        STATE.retryCount = 0;
      })({
        websocket,
        groupPath,
        id
      });
    };
    websocket.addEventListener("error", onErrorRetry);
    websocket.addEventListener("close", (({code}) => 1e3 === code ? onCloseWebSocket() : onErrorRetry(new Error(`server close with code: ${code}`))));
    websocket.addEventListener("message", (({data}) => onMessage(data, onOpenInfo)));
  };
  const getRandomUserId = () => {
    const tagList = Object.keys(window);
    return `User-${tagList[getRandomInt(tagList.length - 1)]}`;
  };
  const onMessage = async (data, onOpenInfo) => {
    const [headerString, payloadBlob] = await parseBlobPacket(data);
    const {type, payload} = JSON.parse(headerString);
    if (type === TYPE_INFO_USER) onOpenInfo(payload.id); else if (type === TYPE_INFO_GROUP) {
      addLogSystem(`Current ${payload.length} user: ${payload.join(", ")}`);
      qS("#button-send").disabled = payload.length <= 1;
      qS("#button-send", `Send to ${payload.length - 1} User [Ctrl+Enter]`);
      STATE.groupInfo = payload;
    } else if (type === TYPE_BUFFER_GROUP) {
      const {id, text, fileName, fileSize, fileId} = payload;
      text && addLog({
        id,
        text
      });
      fileName && addLogWithFile({
        id,
        fileName,
        fileSize,
        fileId
      });
    } else if (type === TYPE_BUFFER_SINGLE) {
      const {id, intent} = payload;
      if ("request" === intent) {
        const {fileId} = payload;
        const file = STATE.fileWeakMap.get(qS(`#${fileId}`));
        const fileName = file && file.name;
        const fileType = file && file.type;
        STATE.websocket.send(packBlobPacket(JSON.stringify({
          type: TYPE_BUFFER_SINGLE,
          targetId: id,
          payload: {
            intent: "response",
            ok: Boolean(file),
            fileName,
            fileType
          }
        }), file));
        addLogSystem(file ? `Send file: ${fileName} to ${id}` : `Miss file request from ${id}`);
      } else if ("response" === intent) {
        const {ok, fileName, fileType} = payload;
        ok && createDownloadWithBlob(fileName, new Blob([ payloadBlob ], {
          type: fileType
        }));
        addLogSystem(ok ? `Get file: ${fileName} from ${id}` : `Miss file response from ${id}`);
      }
    }
  };
  const sendPayload = () => {
    if (!STATE.websocket || STATE.groupInfo.length <= 1 || !qS("#payload-text")) return;
    const text = qS("#payload-text").value.trim();
    const file = qS("#payload-file").files[0];
    qS("#payload-text").value = "";
    qS("#payload-file").value = "";
    if (!text && !file) return;
    const fileName = file && file.name;
    const fileSize = file && file.size;
    if (fileSize > DATA_LENGTH_LIMIT) return alert(`fill size too big! max: ${binary(DATA_LENGTH_LIMIT)}B, get ${binary(fileSize)}B`);
    text && addLog({
      id: STATE.id,
      text,
      className: "color-self"
    });
    const fileTag = fileName && addLogWithFile({
      isSend: !0,
      id: STATE.id,
      fileName,
      fileSize,
      className: "color-self"
    });
    STATE.websocket.send(packBlobPacket(JSON.stringify({
      type: TYPE_BUFFER_GROUP,
      payload: {
        text,
        fileName,
        fileSize,
        fileId: fileTag && fileTag.id
      }
    })));
    fileName && STATE.fileWeakMap.set(fileTag, file);
  };
  qS("#button-log-clear").onclick = clearLog;
  qS("#button-toggle").onclick = toggleWebSocket;
  qS("#button-send").onclick = sendPayload;
  const {groupPath, id} = (() => {
    const {GP: groupPath = "", ID: id = ""} = withFallbackResult({}, (() => Object.fromEntries(new URLSearchParams(location.hash.slice(1)).entries())));
    return {
      groupPath,
      id
    };
  })();
  qS("#group-path").value = groupPath;
  qS("#id").value = id;
  const {start, addKeyCommand} = createKeyCommandHub();
  addKeyCommand({
    checkMap: {
      ctrlKey: !0,
      key: "d"
    },
    callback: toggleWebSocket
  });
  addKeyCommand({
    checkMap: {
      ctrlKey: !0,
      key: "l"
    },
    callback: clearLog
  });
  addKeyCommand({
    checkMap: {
      ctrlKey: !0,
      key: "Enter"
    },
    callback: sendPayload
  });
  addKeyCommand({
    target: qS("#group-path"),
    checkMap: {
      key: "Enter"
    },
    callback: toggleWebSocket
  });
  addKeyCommand({
    target: qS("#id"),
    checkMap: {
      key: "Enter"
    },
    callback: toggleWebSocket
  });
  start();
  applyReceiveFileListListener(document, (fileList => {
    const payloadFile = qS("#payload-file");
    payloadFile && (payloadFile.files = fileList);
  }));
};