var _ArrayBuffer = require("./ArrayBuffer.js");
var _TextEnDecoder = require("../module/TextEnDecoder.js");
const MAX_PACKET_HEADER_SIZE = exports.MAX_PACKET_HEADER_SIZE = 4294967295;
const HEADER_BYTE_SIZE = exports.HEADER_BYTE_SIZE = 4;
const EMPTY_ARRAY_BUFFER = new ArrayBuffer(0);
const packArrayBufferHeader = headerArrayBuffer => {
  const headerSize = headerArrayBuffer.byteLength;
  if (headerSize > MAX_PACKET_HEADER_SIZE) throw new Error(`headerArrayBuffer exceeds max size ${MAX_PACKET_HEADER_SIZE}, get: ${headerSize}`);
  const headerSizeDataView = new DataView(new ArrayBuffer(HEADER_BYTE_SIZE));
  headerSizeDataView.setUint32(0, headerSize, !1);
  return [ headerSizeDataView.buffer, headerArrayBuffer ];
};
exports.packArrayBufferHeader = packArrayBufferHeader;
const parseArrayBufferHeader = arrayBufferPair => {
  const headerSize = new DataView(arrayBufferPair, 0, HEADER_BYTE_SIZE).getUint32(0, !1);
  return [ arrayBufferPair.slice(HEADER_BYTE_SIZE, HEADER_BYTE_SIZE + headerSize), HEADER_BYTE_SIZE + headerSize ];
};
exports.parseArrayBufferHeader = parseArrayBufferHeader;
exports.packArrayBufferPacket2 = (headerString, payloadArrayBuffer = EMPTY_ARRAY_BUFFER) => (0, _ArrayBuffer.concatArrayBuffer)([ ...packArrayBufferHeader((0, _TextEnDecoder.encodeUTF8)(headerString).buffer), payloadArrayBuffer ]);
exports.parseArrayBufferPacket2 = arrayBufferPacket => {
  const [headerArrayBuffer, payloadOffset] = parseArrayBufferHeader(arrayBufferPacket);
  return [ (0, _TextEnDecoder.decodeUTF8)(headerArrayBuffer), arrayBufferPacket.slice(payloadOffset) ];
};
exports.packArrayBufferPacket = (headerU16String, payloadArrayBuffer = EMPTY_ARRAY_BUFFER) => (0, _ArrayBuffer.concatArrayBuffer)([ ...packArrayBufferHeader((0, _ArrayBuffer.fromU16String)(headerU16String)), payloadArrayBuffer ]);
exports.parseArrayBufferPacket = arrayBufferPacket => {
  const [headerArrayBuffer, payloadOffset] = parseArrayBufferHeader(arrayBufferPacket);
  return [ (0, _ArrayBuffer.toU16String)(headerArrayBuffer), arrayBufferPacket.slice(payloadOffset) ];
};
exports.packChainArrayBufferPacket = (arrayBufferList = []) => {
  const headerDataView = new DataView(new ArrayBuffer(4 * arrayBufferList.length));
  arrayBufferList.forEach((({byteLength}, index) => headerDataView.setUint32(4 * index, byteLength, !1)));
  return (0, _ArrayBuffer.concatArrayBuffer)([ ...packArrayBufferHeader(headerDataView.buffer), ...arrayBufferList ]);
};
exports.parseChainArrayBufferPacket = chainArrayBufferPacket => {
  const [headerArrayBuffer, payloadOffset] = parseArrayBufferHeader(chainArrayBufferPacket);
  const headerDataView = new DataView(headerArrayBuffer);
  const byteLengthList = [];
  for (let index = 0, indexMax = headerDataView.byteLength / 4; index < indexMax; index++) byteLengthList.push(headerDataView.getUint32(4 * index, !1));
  return (0, _ArrayBuffer.deconcatArrayBuffer)(chainArrayBufferPacket, byteLengthList, payloadOffset);
};
exports.packArrayBufferListPacket = arrayBufferList => {
  const packetList = [];
  for (const arrayBuffer of arrayBufferList) {
    const headerSize = arrayBuffer.byteLength;
    if (headerSize > MAX_PACKET_HEADER_SIZE) throw new Error(`arrayBuffer exceeds max size ${MAX_PACKET_HEADER_SIZE}, get: ${headerSize}`);
    const headerSizeDataView = new DataView(new ArrayBuffer(HEADER_BYTE_SIZE));
    headerSizeDataView.setUint32(0, headerSize, !1);
    packetList.push(headerSizeDataView.buffer, arrayBuffer);
  }
  return (0, _ArrayBuffer.concatArrayBuffer)(packetList);
};
exports.parseArrayBufferListPacket = arrayBufferPacket => {
  const arrayBufferList = [];
  let index = 0;
  for (;index < arrayBufferPacket.byteLength; ) {
    const headerSize = new DataView(arrayBufferPacket.slice(index, index + HEADER_BYTE_SIZE)).getUint32(0, !1);
    arrayBufferList.push(arrayBufferPacket.slice(index + HEADER_BYTE_SIZE, index + HEADER_BYTE_SIZE + headerSize));
    index += HEADER_BYTE_SIZE + headerSize;
  }
  return arrayBufferList;
};