const B64_CHAR = [];
const B64_CODE = [];
{
  const B64_CHAR_STRING = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
  for (let index = 0, indexMax = B64_CHAR_STRING.length; index < indexMax; ++index) {
    B64_CHAR[index] = B64_CHAR_STRING.charAt(index);
    B64_CODE[B64_CHAR_STRING.charCodeAt(index)] = index;
  }
  B64_CODE[45] = 62;
  B64_CODE[95] = 63;
}
const encodeU24Chunk = (u8List, index, indexMax) => {
  const stringList = [];
  for (;index < indexMax; index += 3) {
    const u24 = (u8List[index] << 16) + (u8List[index + 1] << 8) + u8List[index + 2];
    stringList.push(B64_CHAR[u24 >> 18 & 63] + B64_CHAR[u24 >> 12 & 63] + B64_CHAR[u24 >> 6 & 63] + B64_CHAR[63 & u24]);
  }
  return stringList.join("");
};
exports.encode = (arrayBuffer = new ArrayBuffer(0)) => {
  const u8List = new Uint8Array(arrayBuffer);
  const u8Count = u8List.length;
  const u8TailCount = u8Count % 3;
  const stringList = [];
  for (let index = 0, indexMax = u8Count - u8TailCount; index < indexMax; index += 12288) stringList.push(encodeU24Chunk(u8List, index, Math.min(indexMax, index + 12288)));
  if (1 === u8TailCount) {
    const u8 = u8List[u8Count - 1];
    stringList.push(B64_CHAR[u8 >> 2 & 63] + B64_CHAR[u8 << 4 & 48] + "==");
  } else if (2 === u8TailCount) {
    const u16 = (u8List[u8Count - 2] << 8) + u8List[u8Count - 1];
    stringList.push(B64_CHAR[u16 >> 10 & 63] + B64_CHAR[u16 >> 4 & 63] + B64_CHAR[u16 << 2 & 60] + "=");
  }
  return stringList.join("");
};
exports.decode = (b64String = "") => {
  const padIndex = b64String.indexOf("=");
  const u6Count = -1 === padIndex ? b64String.length : padIndex;
  if (u6Count % 4 == 1) throw new Error("invalid Base64 string u6Count");
  const u6TailCount = u6Count % 4;
  const u8Count = 3 * Math.floor(u6Count / 4) + (u6TailCount && (2 === u6TailCount ? 1 : 2));
  const u8List = new Uint8Array(u8Count);
  for (let index = 0, indexMax = u6Count - u6TailCount, u8Index = 0; index < indexMax; index += 4, u8Index += 3) {
    const u24 = B64_CODE[b64String.charCodeAt(index)] << 18 | B64_CODE[b64String.charCodeAt(index + 1)] << 12 | B64_CODE[b64String.charCodeAt(index + 2)] << 6 | B64_CODE[b64String.charCodeAt(index + 3)];
    u8List[u8Index] = u24 >> 16 & 255;
    u8List[u8Index + 1] = u24 >> 8 & 255;
    u8List[u8Index + 2] = 255 & u24;
  }
  if (2 === u6TailCount) {
    const u8 = B64_CODE[b64String.charCodeAt(u6Count - 2)] << 2 | B64_CODE[b64String.charCodeAt(u6Count - 1)] >> 4;
    u8List[u8Count - 1] = 255 & u8;
  } else if (3 === u6TailCount) {
    const u16 = B64_CODE[b64String.charCodeAt(u6Count - 3)] << 10 | B64_CODE[b64String.charCodeAt(u6Count - 2)] << 4 | B64_CODE[b64String.charCodeAt(u6Count - 1)] >> 2;
    u8List[u8Count - 2] = u16 >> 8 & 255;
    u8List[u8Count - 1] = 255 & u16;
  }
  return u8List.buffer;
};