var _Event = require("../module/Event.js");
var _LinkedList = require("./LinkedList.js");
const createCache = (key, value, size, expireAt) => ({
  ...(0, _LinkedList.createNode)(value),
  key,
  size,
  expireAt
});
exports.createCache = createCache;
exports.createCacheMap = ({valueSizeSumMax, valueSizeSingleMax = Math.max(.05 * valueSizeSumMax, 1), expireAfter = 6e4, eventHub = (0, _Event.createHub)()}) => {
  const hasEventHub = Boolean(eventHub);
  const {clear: clearEventHub, subscribe, unsubscribe, send} = hasEventHub ? eventHub : {};
  const map = new Map;
  const linkedList = (0, _LinkedList.createDoublyLinkedList)();
  let valueSizeSum = 0;
  const cacheAdd = cache => {
    map.set(cache.key, cache);
    linkedList.unshift(cache);
    valueSizeSum += cache.size;
    hasEventHub && send({
      type: "add",
      key: cache.key,
      payload: cache.value
    });
  };
  const cacheDelete = cache => {
    map.delete(cache.key);
    linkedList.remove(cache);
    valueSizeSum -= cache.size;
    hasEventHub && send({
      type: "delete",
      key: cache.key,
      payload: cache.value
    });
  };
  return {
    hasEventHub,
    clearEventHub,
    subscribe,
    unsubscribe,
    clear: () => map.forEach(cacheDelete),
    getSize: linkedList.getLength,
    getValueSizeSum: () => valueSizeSum,
    set: (key, value, size = 1, expireAt = Date.now() + expireAfter) => {
      const prevCache = map.get(key);
      prevCache && cacheDelete(prevCache);
      if (!(size > valueSizeSingleMax)) {
        for (;size + valueSizeSum > valueSizeSumMax; ) cacheDelete(linkedList.getTail().prev);
        cacheAdd(createCache(key, value, size, expireAt));
      }
    },
    get: (key, time = Date.now()) => {
      const cache = map.get(key);
      if (cache) {
        if (cache.expireAt <= time) return cacheDelete(cache);
        linkedList.moveToFirst(cache);
        return cache.value;
      }
    },
    touch: (key, expireAt = Date.now() + expireAfter) => {
      const cache = map.get(key);
      if (cache) {
        cache.expireAt = expireAt;
        linkedList.moveToFirst(cache);
        return cache.value;
      }
    },
    delete: key => {
      const cache = map.get(key);
      cache && cacheDelete(cache);
      return cache && cache.value;
    },
    saveCacheList: () => {
      const cacheList = [];
      linkedList.forEachReverse((node => {
        const {key, value, size, expireAt} = node;
        cacheList.push({
          key,
          value,
          size,
          expireAt
        });
      }));
      return cacheList;
    },
    loadCacheList: (cacheList, time = Date.now()) => cacheList.forEach((({key, value, size, expireAt}) => {
      if (expireAt <= time) return;
      const cache = map.get(key);
      if (!(cache && cache.expireAt >= expireAt)) {
        cache && cacheDelete(cache);
        cacheAdd(createCache(key, value, size, expireAt));
      }
    }))
  };
};