exports.createCacheMap2 = ({sizeMax, expireAfter = 6e4, map = new Map}) => ({
  clear: () => map.clear(),
  getSize: () => map.size,
  set: (key, value, expireAt = Date.now() + expireAfter) => {
    map.delete(key);
    for (;map.size >= sizeMax; ) map.delete(map.keys().next().value);
    map.set(key, {
      value,
      expireAt
    });
  },
  get: (key, time = Date.now()) => {
    const cache = map.get(key);
    if (cache) {
      map.delete(key);
      if (!(cache.expireAt <= time)) {
        map.set(key, cache);
        return cache.value;
      }
    }
  },
  touch: (key, expireAt = Date.now() + expireAfter) => {
    const cache = map.get(key);
    if (cache) {
      cache.expireAt = expireAt;
      map.delete(key);
      map.set(key, cache);
      return cache.value;
    }
  },
  delete: key => {
    const cache = map.get(key);
    if (cache) {
      map.delete(cache);
      return cache.value;
    }
  }
});