const SEARCH_END = exports.SEARCH_END = Symbol("TREE:SEARCH-END");
const SEARCH_SKIP = exports.SEARCH_SKIP = Symbol("TREE:SEARCH-SKIP");
const createTree2DepthFirstSearch = getSubNodeListFunc => {
  const unshiftStack = (stack, node, extra) => {
    const nodeList = getSubNodeListFunc(node, extra);
    Array.isArray(nodeList) && stack.unshift(...nodeList);
  };
  return (initialNode, func, extra) => {
    const stack = [];
    unshiftStack(stack, initialNode, extra);
    let node;
    for (;node = stack.shift(); ) {
      const output = func(node);
      if (output === SEARCH_END) return node;
      output !== SEARCH_SKIP && unshiftStack(stack, node, extra);
    }
  };
};
exports.createTree2DepthFirstSearch = createTree2DepthFirstSearch;
exports.createTree2DepthFirstSearchAsync = getSubNodeListFunc => {
  const unshiftStackAsync = async (stack, node, extra) => {
    const nodeList = await getSubNodeListFunc(node, extra);
    Array.isArray(nodeList) && stack.unshift(...nodeList);
  };
  return async (initialNode, func, extra) => {
    const stack = [];
    await unshiftStackAsync(stack, initialNode, extra);
    let node;
    for (;node = stack.shift(); ) {
      const output = await func(node);
      if (output === SEARCH_END) return node;
      output !== SEARCH_SKIP && await unshiftStackAsync(stack, node, extra);
    }
  };
};
exports.createTree2BreadthFirstSearch = getSubNodeListFunc => {
  const pushStack = (stack, node, extra) => {
    const nodeList = getSubNodeListFunc(node, extra);
    Array.isArray(nodeList) && stack.push(...nodeList);
  };
  return (initialNode, func, extra) => {
    const stack = [];
    pushStack(stack, initialNode, extra);
    let node;
    for (;node = stack.shift(); ) {
      const output = func(node);
      if (output === SEARCH_END) return node;
      output !== SEARCH_SKIP && pushStack(stack, node, extra);
    }
  };
};
exports.createTree2BreadthFirstSearchAsync = getSubNodeListFunc => {
  const pushStackAsync = async (stack, node, extra) => {
    const nodeList = await getSubNodeListFunc(node, extra);
    Array.isArray(nodeList) && stack.push(...nodeList);
  };
  return async (initialNode, func, extra) => {
    const stack = [];
    await pushStackAsync(stack, initialNode, extra);
    let node;
    for (;node = stack.shift(); ) {
      const output = await func(node);
      if (output === SEARCH_END) return node;
      output !== SEARCH_SKIP && await pushStackAsync(stack, node, extra);
    }
  };
};
exports.createTree2BottomUpSearch = getSubNodeListFunc => {
  const pushPairStack = (pairStack, node, extra) => {
    const nodeList = getSubNodeListFunc(node, extra);
    return Boolean(Array.isArray(nodeList) && pairStack.push([ node, nodeList ]));
  };
  return (initialNode, func, extra) => {
    const pairStack = [];
    pushPairStack(pairStack, initialNode, extra);
    for (;0 !== pairStack.length; ) {
      const [node, subNodeList] = pairStack[pairStack.length - 1];
      if (0 === subNodeList.length) {
        pairStack.pop();
        if (0 !== pairStack.length && func(node) === SEARCH_END) return node;
      } else {
        const subNode = subNodeList.shift();
        if (!pushPairStack(pairStack, subNode, extra) && func(subNode) === SEARCH_END) return subNode;
      }
    }
  };
};
exports.createTree2BottomUpSearchAsync = getSubNodeListFunc => {
  const pushPairStackAsync = async (pairStack, node, extra) => {
    const nodeList = await getSubNodeListFunc(node, extra);
    return Boolean(Array.isArray(nodeList) && pairStack.push([ node, nodeList ]));
  };
  return async (initialNode, func, extra) => {
    const pairStack = [];
    await pushPairStackAsync(pairStack, initialNode, extra);
    for (;0 !== pairStack.length; ) {
      const [node, subNodeList] = pairStack[pairStack.length - 1];
      if (0 === subNodeList.length) {
        pairStack.pop();
        if (0 !== pairStack.length && await func(node) === SEARCH_END) return node;
      } else {
        const subNode = subNodeList.shift();
        if (!await pushPairStackAsync(pairStack, subNode, extra) && await func(subNode) === SEARCH_END) return subNode;
      }
    }
  };
};
exports.prettyStringifyTree2Node = (getSubNodeLevelHasMoreListFunc, initialNode, func, extra) => {
  const studList = [];
  createTree2DepthFirstSearch(getSubNodeLevelHasMoreListFunc)(initialNode, (([node, level, hasMore]) => {
    studList.length = level;
    0 !== level && (studList[level - 1] = hasMore ? "├─ " : "└─ ");
    func(studList.join(""), node);
    0 !== level && (studList[level - 1] = hasMore ? "│  " : "   ");
  }), extra);
};