exports.remessageError = (error, nextMessage) => {
  error.message && error.stack && (error.stack = error.stack.replace(error.message, nextMessage));
  error.message = nextMessage;
  return error;
};
exports.rethrowError = error => {
  console.warn(error);
  throw error;
};
exports.withFallbackResult = (fallbackResult, func, ...args) => {
  try {
    return func(...args);
  } catch (error) {
    return fallbackResult;
  }
};
exports.withFallbackResultAsync = async (fallbackResult, func, ...args) => {
  try {
    return await func(...args);
  } catch (error) {
    return fallbackResult;
  }
};
exports.catchSync = (func, ...args) => {
  let result, resultError;
  try {
    result = func(...args);
  } catch (error) {
    resultError = error || new Error;
  }
  return {
    result,
    error: resultError
  };
};
exports.catchAsync = async (func, ...args) => {
  let result, resultError;
  try {
    result = await func(...args);
  } catch (error) {
    resultError = error || new Error;
  }
  return {
    result,
    error: resultError
  };
};
exports.catchPromise = promise => promise.then(packResult, packError);
const packResult = result => ({
  result,
  error: void 0
});
const packError = error => ({
  result: void 0,
  error: error || new Error
});