var _base = require("../../math/base.js");
var _Vector = require("./Vector.js");
const PI_HALF = .5 * Math.PI;
exports.fromPoint = (pointA, pointB, data) => ({
  ...data,
  center: {
    x: .5 * (pointA.x + pointB.x),
    y: .5 * (pointA.y + pointB.y)
  },
  size: {
    x: Math.abs(pointA.x - pointB.x),
    y: Math.abs(pointA.y - pointB.y)
  },
  rotate: 0
});
exports.fromLine = ({begin, end}, width, data) => ({
  ...data,
  center: {
    x: .5 * (begin.x + end.x),
    y: .5 * (begin.y + end.y)
  },
  size: {
    x: width,
    y: (0, _Vector.getDist)(end, begin)
  },
  rotate: (0, _Vector.getRotate)(end, begin) - PI_HALF
});
exports.fromBoundingRect = ({left, right, top, bottom}, data) => ({
  ...data,
  center: {
    x: .5 * (left + right),
    y: .5 * (top + bottom)
  },
  size: {
    x: right - left,
    y: bottom - top
  },
  rotate: 0
});
exports.getBoundingSize = ({size: {x, y}, rotate}) => {
  const cos = Math.cos(rotate);
  const sin = Math.sin(rotate);
  return {
    x: Math.abs(x * cos) + Math.abs(y * sin),
    y: Math.abs(x * sin) + Math.abs(y * cos)
  };
};
const getBoundingWidth = ({size: {x, y}, rotate}) => Math.abs(x * Math.cos(rotate)) + Math.abs(y * Math.sin(rotate));
exports.getBoundingWidth = getBoundingWidth;
const getBoundingHeight = ({size: {x, y}, rotate}) => Math.abs(x * Math.sin(rotate)) + Math.abs(y * Math.cos(rotate));
exports.getBoundingHeight = getBoundingHeight;
exports.getBoundingLeft = widget => widget.center.x - .5 * getBoundingWidth(widget);
exports.getBoundingRight = widget => widget.center.x + .5 * getBoundingWidth(widget);
exports.getBoundingTop = widget => widget.center.y - .5 * getBoundingHeight(widget);
exports.getBoundingBottom = widget => widget.center.y + .5 * getBoundingHeight(widget);
exports.round = widget => ({
  ...widget,
  center: (0, _Vector.round)(widget.center),
  size: (0, _Vector.round)(widget.size),
  rotate: (0, _base.roundFloat)(widget.rotate)
});
exports.localPoint = ({center, rotate}, {x, y}) => {
  const offsetX = x - center.x;
  const offsetY = y - center.y;
  const cos = Math.cos(-rotate);
  const sin = Math.sin(-rotate);
  return {
    x: offsetX * cos - offsetY * sin,
    y: offsetX * sin + offsetY * cos
  };
};
exports.localBoundingRect = ({center, rotate}, targetWidget) => {
  const offsetX = targetWidget.center.x - center.x;
  const offsetY = targetWidget.center.y - center.y;
  const offsetRotate = targetWidget.rotate - rotate;
  const cos = Math.cos(offsetRotate);
  const sin = Math.sin(offsetRotate);
  const x = offsetX * cos - offsetY * sin;
  const y = offsetX * sin + offsetY * cos;
  const halfBoundingWidth = .5 * (targetWidget.size.x * cos - targetWidget.size.y * sin);
  const halfBoundingHeight = .5 * (targetWidget.size.x * sin + targetWidget.size.y * cos);
  return {
    left: x - halfBoundingWidth,
    right: x + halfBoundingWidth,
    top: y - halfBoundingHeight,
    bottom: y + halfBoundingHeight
  };
};
exports.isContainBoundingRect = ({center, size}, boundingRect) => center.x - .5 * size.x <= boundingRect.left && center.x + .5 * size.x >= boundingRect.right && center.y - .5 * size.y <= boundingRect.top && center.y + .5 * size.y >= boundingRect.bottom;
exports.isInterceptBoundingRect = ({center, size}, boundingRect) => !(center.x + .5 * size.x < boundingRect.left || center.x - .5 * size.x > boundingRect.right || center.y + .5 * size.y < boundingRect.top || center.y - .5 * size.y > boundingRect.bottom);