var _verify = require("../verify.js");
var _Object = require("./Object.js");
var _Array = require("./Array.js");
const createStateStore = state => {
  (0, _verify.basicObject)(state, "initialState should be basic Object");
  let listenerList = [];
  return {
    subscribe: listener => {
      listenerList = (0, _Array.arrayMatchPush)(listenerList, listener);
    },
    unsubscribe: listener => {
      listenerList = (0, _Array.arrayMatchDelete)(listenerList, listener);
    },
    getState: () => state,
    setState: nextState => {
      if ((nextState = (0, _Object.objectMerge)(state, nextState)) === state) return state;
      const prevState = state;
      state = nextState;
      listenerList.forEach((listener => listener(state, prevState)));
      return state;
    }
  };
};
exports.createStateStore = createStateStore;
exports.createStateStoreLite = state => ({
  getState: () => state,
  setState: nextState => state = {
    ...state,
    ...nextState
  }
});
exports.createStateStoreEnhanced = ({initialState, enhancer, reducer}) => {
  (0, _verify.basicFunction)(enhancer, "enhancer function required");
  (0, _verify.basicFunction)(reducer, "reducer function required");
  const {subscribe, unsubscribe, getState: getStoreState, setState: setStoreState} = createStateStore(initialState);
  let dispatchingState = null;
  let isDispatching = !1;
  let isReducing = !1;
  const getState = () => isDispatching ? dispatchingState : getStoreState();
  const dispatch = action => isDispatching ? (action => {
    enhancer(enhancerStore, action);
    dispatchingState = reducer(dispatchingState, action);
  })(action) : (action => {
    dispatchingState = getStoreState();
    isDispatching = !0;
    enhancer(enhancerStore, action);
    isDispatching = !1;
    isReducing = !0;
    const nextState = reducer(dispatchingState, action);
    isReducing = !1;
    dispatchingState = null;
    setStoreState(nextState);
  })(action);
  const enhancerStore = {
    getState,
    dispatch
  };
  return {
    subscribe,
    unsubscribe,
    getState,
    dispatch
  };
};
exports.toReduxStore = store => {
  const {subscribe: subscribeStore, unsubscribe} = store;
  (0, _verify.basicFunction)(subscribeStore, "store.subscribe required");
  (0, _verify.basicFunction)(unsubscribe, "store.unsubscribe required");
  return {
    ...store,
    subscribe: listener => {
      subscribeStore(listener);
      return () => unsubscribe(listener);
    }
  };
};
exports.reducerFromMap = reducerMap => {
  const keyList = Object.keys(reducerMap);
  const keyListLength = keyList.length;
  return (state, action) => {
    const nextState = {};
    let isChanged = !1;
    for (let index = 0; index < keyListLength; index++) {
      const key = keyList[index];
      const keyState = state[key];
      const nextKeyState = reducerMap[key](keyState, action);
      nextState[key] = nextKeyState;
      isChanged = isChanged || nextKeyState !== keyState;
    }
    return isChanged ? nextState : state;
  };
};
exports.createEntryEnhancer = entryMap => (enhancerStore, action) => {
  const entryFunction = entryMap[action.type];
  return entryFunction && entryFunction(enhancerStore, action);
};
exports.createStoreStateSyncReducer = (actionType, {getState, setState}) => (state, {type, payload}) => {
  type === actionType && setState(payload);
  return getState();
};