var _ArrayBuffer = require("../data/ArrayBuffer.js");
var _ArrayBufferPacket = require("../data/ArrayBufferPacket.js");
var _Base = require("../data/Base64.js");
const packArrayBufferChunk = async ({chunkArrayBuffer, key, chunkIndex, chunkTotal}, isSkipVerifyHash = !1) => (0, _ArrayBufferPacket.packChainArrayBufferPacket)([ (0, _ArrayBuffer.fromU16String)(JSON.stringify({
  key,
  chunkIndex,
  chunkTotal,
  chunkByteLength: chunkArrayBuffer.byteLength
})), isSkipVerifyHash ? new ArrayBuffer(0) : await (0, _ArrayBuffer.calcSHA256ArrayBuffer)(chunkArrayBuffer), chunkArrayBuffer ]);
exports.packArrayBufferChunk = packArrayBufferChunk;
exports.parseArrayBufferChunk = async (arrayBufferPacket, allowSkipHashVerify = !0) => {
  const [headerArrayBuffer, chunkHashArrayBuffer, chunkArrayBuffer] = (0, _ArrayBufferPacket.parseChainArrayBufferPacket)(arrayBufferPacket);
  const {key, chunkIndex, chunkTotal, chunkByteLength} = JSON.parse((0, _ArrayBuffer.toU16String)(headerArrayBuffer));
  if (chunkByteLength !== chunkArrayBuffer.byteLength) throw new Error(`chunk length mismatch, get: ${chunkArrayBuffer.byteLength}, expect ${chunkByteLength}`);
  if (chunkHashArrayBuffer.byteLength) {
    const verifyChunkHashBuffer = await (0, _ArrayBuffer.calcSHA256ArrayBuffer)(chunkArrayBuffer);
    if (!(0, _ArrayBuffer.isEqualArrayBuffer)(chunkHashArrayBuffer, verifyChunkHashBuffer)) throw new Error(`chunk ${chunkIndex} of ${key} hash mismatch, get: ${(0, _Base.encode)(verifyChunkHashBuffer)}, expect ${(0, _Base.encode)(chunkHashArrayBuffer)}`);
  } else if (!allowSkipHashVerify) throw new Error(`missing chunk ${chunkIndex} of ${key} hash to verify`);
  return {
    chunkArrayBuffer,
    key,
    chunkIndex,
    chunkTotal
  };
};
exports.uploadArrayBufferByChunk = async ({arrayBuffer, size = arrayBuffer && arrayBuffer.byteLength, getChunk = arrayBuffer && ((index, chunkSize) => arrayBuffer.slice(index, index + chunkSize)), key, chunkSizeMax = 1048576, isSkipVerifyHash = !1, uploadChunk, onProgress}) => {
  let chunkIndex = 0;
  const chunkTotal = Math.ceil(size / chunkSizeMax) || 1;
  for (;chunkIndex < chunkTotal; ) {
    onProgress && await onProgress(chunkIndex * chunkSizeMax, size);
    const chunkSize = chunkIndex < chunkTotal - 1 ? chunkSizeMax : size % chunkSizeMax;
    const chunkInfo = {
      chunkArrayBuffer: await getChunk(chunkIndex * chunkSizeMax, chunkSize),
      key,
      chunkIndex,
      chunkTotal
    };
    await uploadChunk(await packArrayBufferChunk(chunkInfo, isSkipVerifyHash), chunkInfo);
    chunkIndex++;
  }
  onProgress && onProgress(size, size);
};