exports.createHub = () => {
  const set = new Set;
  return {
    clear: () => set.clear(),
    subscribe: listener => {
      set.add(listener);
    },
    unsubscribe: listener => {
      set.delete(listener);
    },
    send: data => set.forEach((listener => listener(data)))
  };
};
const createEventBase = () => {
  const setMap = new Map;
  return {
    setMap,
    clear: () => setMap.clear(),
    on: (type, listener) => {
      const listenerSet = setMap.get(type);
      listenerSet ? listenerSet.add(listener) : setMap.set(type, new Set([ listener ]));
    },
    off: (type, listener) => {
      const listenerSet = setMap.get(type);
      listenerSet && listenerSet.delete(listener);
      listenerSet && 0 === listenerSet.size && setMap.delete(type);
    }
  };
};
exports.createEventTarget = () => {
  const {setMap, clear, on, off} = createEventBase();
  return {
    clear,
    dispatchEvent: event => {
      const listenerSet = setMap.get(event.type);
      listenerSet && listenerSet.forEach((listener => listener(event)));
    },
    addEventListener: on,
    removeEventListener: off
  };
};
exports.createEventEmitter = () => {
  const {setMap, clear, on, off} = createEventBase();
  return {
    clear,
    emit: (eventName, ...args) => {
      const listenerSet = setMap.get(eventName);
      listenerSet && listenerSet.forEach((listener => listener(...args)));
    },
    on,
    off,
    addListener: on,
    removeListener: off,
    removeAllListeners: (...eventNameList) => {
      eventNameList.length ? eventNameList.forEach((eventName => setMap.delete(eventName))) : setMap.clear();
    }
  };
};