var _check = require("../check.js");
var _random = require("../math/random.js");
exports.createExotError = (exotId, message) => {
  const exotError = new Error(message);
  exotError.exotId = exotId;
  return exotError;
};
exports.createDummyExot = ({idPrefix = "DUMMY-EXOT-", id = (0, _random.getRandomId62S)(idPrefix), onUp, onDown, ...extra} = {}) => {
  let isActive = !1;
  return {
    ...extra,
    id,
    up: async onExotError => {
      if (isActive) throw new Error("already up");
      if (!onExotError) throw new Error("expect onExotError to receive ExotError notice");
      onUp && await onUp(onExotError);
      isActive = !0;
    },
    down: async () => {
      if (isActive) {
        isActive = !1;
        onDown && await onDown();
      }
    },
    isUp: () => isActive
  };
};
exports.createExotGroup = ({idPrefix = "EXOT-GROUP-", id = (0, _random.getRandomId62S)(idPrefix), getOnExotError, onExotError, exotList = [], exotMap = toExotMap(...exotList), isBatch = !1} = {}) => {
  const upEach = async (onExotError = onExotErrorGroup) => {
    for (const exot of exotMap.values()) exot.isUp() || await exot.up(onExotError);
  };
  const upBatch = async (onExotError = onExotErrorGroup) => {
    await Promise.all(mapExotMapValue(exotMap, (exot => exot.isUp() || exot.up(onExotError))));
  };
  const downEach = async () => {
    for (const exot of Array.from(exotMap.values()).reverse()) await exot.down();
  };
  const downBatch = async () => {
    await Promise.all(Array.from(exotMap.values()).reverse().map((exot => exot.down())));
  };
  const set = exot => {
    if (!isExot(exot)) throw new Error(`invalid exot: ${exot}`);
    if (exotMap.has(exot.id)) throw new Error(`duplicate exot id: ${exot.id}`);
    exotMap.set(exot.id, exot);
  };
  const get = exotId => exotMap.get(exotId);
  const deleteKeyword = exotId => {
    const exot = get(exotId);
    void 0 !== exot && exotMap.delete(exotId);
    return exot;
  };
  const down = isBatch ? downBatch : downEach;
  const exotGroup = {
    id,
    up: isBatch ? upBatch : upEach,
    down,
    isUp: () => findExotMapValue(exotMap, (exot => exot.isUp())),
    upEach,
    upBatch,
    downEach,
    downBatch,
    getSize: () => exotMap.size,
    set,
    get,
    delete: deleteKeyword,
    load: async (exot, onExotError = onExotErrorGroup) => {
      set(exot);
      exot.isUp() || await exot.up(onExotError);
    },
    drop: async exotId => {
      const exot = deleteKeyword(exotId);
      exot && await exot.down();
      return exot;
    },
    exotMap
  };
  const onExotErrorGroup = onExotError || getOnExotError && getOnExotError(exotGroup) || down;
  return exotGroup;
};
const isExot = value => (0, _check.isObjectAlike)(value) && (0, _check.isString)(value.id) && (0, _check.isBasicFunction)(value.up) && (0, _check.isBasicFunction)(value.down) && (0, _check.isBasicFunction)(value.isUp);
exports.isExot = isExot;
const toExotMap = (...exotList) => {
  const exotMap = new Map;
  for (const exot of exotList) exotMap.set(exot.id, exot);
  return exotMap;
};
exports.toExotMap = toExotMap;
const mapExotMapValue = (exotMap, func) => {
  const resultList = [];
  for (const exot of exotMap.values()) resultList.push(func(exot));
  return resultList;
};
exports.mapExotMapValue = mapExotMapValue;
const findExotMapValue = (exotMap, func) => {
  for (const exot of exotMap.values()) {
    const result = func(exot);
    if (result) return result;
  }
};
exports.findExotMapValue = findExotMapValue;