var _compare = require("../compare.js");
var _Tree = require("../data/Tree.js");
const REGEXP_NAME = /[\w-]+/;
const createKeyTree = ({NAME_KEY = "key", NAME_SUB_LIST = "subList"}) => {
  if (!REGEXP_NAME.test(NAME_KEY)) throw new Error(`expect ${NAME_KEY} to match: ${REGEXP_NAME}`);
  if (!REGEXP_NAME.test(NAME_SUB_LIST)) throw new Error(`expect ${NAME_SUB_LIST} to match: ${REGEXP_NAME}`);
  const REGEXP_MAX_COMMA = new RegExp(`"},{"${NAME_KEY}":"`, "g");
  const REGEXP_MAX_BRACKET_LEFT = new RegExp(`(?:","${NAME_SUB_LIST}":|^)\\[{"${NAME_KEY}":"`, "g");
  const REGEXP_MAX_BRACKET_RIGHT = new RegExp(`"((?:}])+)(?:},{"${NAME_KEY}":"|$)`, "g");
  const REGEXP_MIN_COMMA = /,/g;
  const REGEXP_MIN_BRACKET_LEFT = /\[/g;
  const REGEXP_MIN_BRACKET_RIGHT = /(]+)/g;
  return {
    stringify: keyTreeJSON => JSON.stringify(keyTreeJSON).replace(REGEXP_MAX_BRACKET_LEFT, "[").replace(REGEXP_MAX_BRACKET_RIGHT, ((match, $1) => "]".repeat($1.length / 2))).replace(REGEXP_MAX_COMMA, ","),
    parse: keyTreeString => JSON.parse(keyTreeString.replace(REGEXP_MIN_COMMA, `"},{"${NAME_KEY}":"`).replace(REGEXP_MIN_BRACKET_RIGHT, ((match, $1, offset, string) => `"${"}]".repeat($1.length)}${offset + match.length !== string.length ? `},{"${NAME_KEY}":"` : ""}`)).replace(REGEXP_MIN_BRACKET_LEFT, ((match, offset) => `${0 !== offset ? `","${NAME_SUB_LIST}":` : ""}[{"${NAME_KEY}":"`)))
  };
};
exports.createKeyTree = createKeyTree;
exports.createKeyTreeEnhanced = ({NAME_KEY = "key", NAME_SUB_LIST = "subList"}) => {
  const keyTreeJSONDepthFirstSearch = (0, _Tree.createTreeDepthFirstSearch)((([node, index, upperKey]) => node[NAME_SUB_LIST] && node[NAME_SUB_LIST].length && node[NAME_SUB_LIST].map(((subNode, index) => [ subNode, index, node[NAME_KEY] ]))));
  return {
    ...createKeyTree({
      NAME_KEY,
      NAME_SUB_LIST
    }),
    createBuilder: rootKey => {
      const listMap = new Map;
      const buildSub = key => {
        const object = {
          [NAME_KEY]: key
        };
        const subList = listMap.get(key);
        listMap.delete(key);
        void 0 !== subList && 0 !== subList.length && (object[NAME_SUB_LIST] = subList.sort(sortFunc).map((([subKey]) => buildSub(subKey))));
        return object;
      };
      const sortFunc = ([keyA, positionA], [keyB, positionB]) => positionA - positionB || (0, _compare.compareString)(keyA, keyB);
      return {
        add: (key, position, upperKey) => {
          let list = listMap.get(upperKey);
          if (void 0 === list) {
            list = [];
            listMap.set(upperKey, list);
          }
          list.push([ key, position ]);
        },
        build: () => {
          const rootSubList = [];
          let initKey = rootKey;
          for (;initKey; ) {
            const {[NAME_SUB_LIST]: subList} = buildSub(initKey);
            void 0 !== subList && rootSubList.push(...subList);
            initKey = listMap.keys().next().value;
          }
          return 0 === rootSubList.length ? {
            [NAME_KEY]: rootKey
          } : {
            [NAME_KEY]: rootKey,
            [NAME_SUB_LIST]: rootSubList
          };
        }
      };
    },
    walkKeyTreeJSON: (keyTreeJSON, func = (([node, index, upperKey]) => {})) => keyTreeJSONDepthFirstSearch([ keyTreeJSON, 0, void 0 ], func)
  };
};