var _check = require("../check.js");
var _function = require("../data/function.js");
var _Object = require("../mutable/Object.js");
const PACKAGE_KEY_SORT_REQUIRED = [ "bundledDependencies", "peerDependencies", "dependencies", "devDependencies", "optionalDependencies" ];
const PACKAGE_KEY_ORDER = [ "private", "name", "version", "description", "author", "contributors", "maintainers", "license", "keywords", "repository", "homepage", "bugs", "type", "main", "bin", "browser", "man", "files", "directories", "scripts", "config", "publishConfig", "os", "cpu", "engines", "engineStrict", ...PACKAGE_KEY_SORT_REQUIRED ];
const getPackageKeyOrder = key => {
  const index = PACKAGE_KEY_ORDER.indexOf(key);
  return -1 === index ? PACKAGE_KEY_ORDER.length : index;
};
exports.sortPackageJSON = packageJSON => {
  packageJSON = (0, _function.dupJSON)(packageJSON);
  PACKAGE_KEY_SORT_REQUIRED.forEach((key => {
    packageJSON[key] && (0, _Object.objectSortKey)(packageJSON[key]);
  }));
  (0, _Object.objectSortKey)(packageJSON, ((a, b) => getPackageKeyOrder(a) - getPackageKeyOrder(b)));
  return packageJSON;
};
exports.packPackageJSON = packageJSON => `${JSON.stringify(packageJSON, null, 2)}\n`;
exports.toPackageInfo = ({packageJSON, packageJSONPath = "", packageRootPath = ""}) => ({
  sourcePackageJSON: (0, _function.dupJSON)(packageJSON),
  packageJSON,
  packageJSONPath,
  packageRootPath
});
exports.collectDependency = (packageInfo, {dependencyMap = {}, dependencyInfoMap = {}, duplicateInfoList = []} = {}) => {
  const {dependencies = {}, devDependencies = {}, peerDependencies = {}, optionalDependencies = {}} = packageInfo.packageJSON;
  for (const dependencyObject of [ dependencies, devDependencies, peerDependencies, optionalDependencies ]) for (const [name, versionSpec] of Object.entries(dependencyObject)) if (dependencyInfoMap[name]) duplicateInfoList.push({
    name,
    versionSpec,
    packageInfo,
    existPackageInfo: dependencyInfoMap[name]
  }); else {
    dependencyMap[name] = versionSpec;
    dependencyInfoMap[name] = {
      name,
      versionSpec,
      packageInfo
    };
  }
  return {
    dependencyMap,
    dependencyInfoMap,
    duplicateInfoList
  };
};
exports.getFirstBinPath = packageJSON => (0, _check.isString)(packageJSON.bin) ? packageJSON.bin : (0, _check.isBasicObject)(packageJSON.bin) ? Object.values(packageJSON.bin)[0] : void 0;
exports.parsePackageNameAndVersion = nameAndVersion => {
  const nameAndVersionList = nameAndVersion.split("@");
  if (nameAndVersionList.length < 2) return [];
  const version = nameAndVersionList.pop();
  const name = nameAndVersionList.join("@");
  return name && version ? [ name, version ] : [];
};
exports.toPackageTgzName = (name, version) => `${name}-${version}.tgz`.replace(/^@/, "").replace(/\//, "-");