const nextObject = (object, key) => void 0 === object[key] ? object[key] = {} : object[key];
const parseRouteToMap = (routeNode, route) => {
  const paramNameList = [];
  return {
    routeNode: routeNode = route.split("/").reduce(((routeNode, frag) => {
      if ("*" === frag) {
        paramNameList.push("/*");
        return nextObject(routeNode, "/*");
      }
      if (":" === frag[0]) {
        const paramName = frag.slice(1);
        if (!paramName || paramNameList.includes(paramName)) throw new Error(`invalid frag [${frag}] for route: ${route}`);
        paramNameList.push(paramName);
        return nextObject(routeNode, "/:PARAM");
      }
      return nextObject(routeNode, frag);
    }), routeNode),
    paramNameList
  };
};
exports.parseRouteToMap = parseRouteToMap;
const findRouteFromMap = (routeNode, route) => {
  const paramValueList = [];
  const routeFragList = route.split("/");
  for (let index = 0, indexMax = routeFragList.length; index < indexMax; index++) {
    const frag = routeFragList[index];
    if (routeNode[frag]) routeNode = routeNode[frag]; else {
      if (!routeNode["/:PARAM"]) {
        if (routeNode["/*"]) {
          paramValueList.push(routeFragList.slice(index).join("/"));
          routeNode = routeNode["/*"];
          break;
        }
        return;
      }
      paramValueList.push(frag);
      routeNode = routeNode["/:PARAM"];
    }
  }
  return {
    routeNode,
    paramValueList
  };
};
exports.findRouteFromMap = findRouteFromMap;
const appendRouteMap = (routeMap = {}, routeInput = "/", data, routePrefix = "") => {
  if (Array.isArray(routeInput)) return routeInput.reduce(((o, v) => appendRouteMap(routeMap, v, data, routePrefix)), routeMap);
  const route = routePrefix ? `${routePrefix}${routeInput}` : routeInput;
  const {routeNode, paramNameList} = parseRouteToMap(routeMap, route);
  if (routeNode["/DATA"]) throw new Error(`duplicate route: ${route}`);
  routeNode["/DATA"] = {
    route,
    paramNameList,
    data
  };
  return routeMap;
};
exports.appendRouteMap = appendRouteMap;
exports.createRouteMap = (configList, routePrefix) => configList.reduce(((o, [route, data]) => appendRouteMap(o, route, data, routePrefix)), {});
exports.parseRouteUrl = (routeMap, url) => {
  const routeData = findRouteFromMap(routeMap, url);
  if (void 0 === routeData) return;
  const {routeNode, paramValueList} = routeData;
  if (void 0 === routeNode["/DATA"]) return;
  const {route, paramNameList, data} = routeNode["/DATA"];
  return {
    route,
    paramMap: paramNameList.reduce(((o, paramName, index) => {
      o[paramName] = paramValueList[index];
      return o;
    }), {}),
    data
  };
};
exports.getRouteParamAny = ({paramMap}) => paramMap["/*"];
exports.getRouteParam = ({paramMap}, paramName) => paramMap[paramName];