var _Iter = require("../data/Iter.js");
var _function = require("../function.js");
var _Runlet = require("./Runlet.js");
exports.createArrayInputChip = ({array = [], state = {
  index: 0,
  indexMax: array.length,
  array
}, key = "chip:input:array", ...extra}) => ({
  ...extra,
  key,
  prevPoolKey: _Runlet.KEY_POOL_IO,
  prevPendKey: _Runlet.KEY_PEND_INPUT,
  state,
  process: async (pack, state, error) => {
    if (!error) {
      if (state.index === state.indexMax) pack[1] = _Runlet.END; else {
        pack[0] = state.array[state.index];
        pack[1] = void 0;
        state.index++;
      }
      return {
        pack,
        state
      };
    }
  },
  describe: () => `[${state.index}/${state.indexMax}]`
});
exports.createArrayOutputChip = ({array = [], state = {
  array
}, IOP = (0, _function.createInsideOutPromise)(), key = "chip:output:array", ...extra}) => ({
  ...extra,
  key,
  nextPoolKey: _Runlet.KEY_POOL_IO,
  nextPendKey: _Runlet.KEY_PEND_OUTPUT,
  state,
  process: async (pack, state, error) => {
    if (error) return IOP.reject(error);
    if (pack[1] === _Runlet.END) IOP.resolve(state.array); else {
      state.array.push(pack[0]);
      pack[0] = void 0;
      pack[1] = _Runlet.SKIP;
    }
    return {
      pack,
      state
    };
  },
  describe: () => `[${array.length}]`,
  promise: IOP.promise
});
exports.createAsyncIterInputChip = ({iterable, iterator, next = (0, _Iter.unwrap)({
  iterable,
  iterator
}), key = "chip:input:async-iter", ...extra}) => ({
  ...extra,
  key,
  prevPoolKey: _Runlet.KEY_POOL_IO,
  prevPendKey: _Runlet.KEY_PEND_INPUT,
  process: async (pack, state, error) => {
    if (error) return;
    const {value, done} = await next();
    const isEND = !0 === done;
    pack[0] = isEND ? void 0 : value;
    pack[1] = isEND ? _Runlet.END : void 0;
    return {
      pack,
      state
    };
  }
});
exports.createAsyncIterOutputChip = ({LSAI = (0, _Iter.createLockStepAsyncIter)(), key = "chip:output:async-iter", ...extra}) => ({
  ...LSAI,
  ...extra,
  key,
  nextPoolKey: _Runlet.KEY_POOL_IO,
  nextPendKey: _Runlet.KEY_PEND_OUTPUT,
  process: async (pack, state, error) => {
    if (error) return LSAI.throw(error);
    const [value, hint] = pack;
    const isEND = hint === _Runlet.END;
    await LSAI.send(value, isEND);
    return {
      pack: isEND ? pack : (0, _Runlet.createPack)(void 0, _Runlet.SKIP),
      state
    };
  }
});
exports.createENDRegulatorChip = ({inputChipCount = 1, outputChipCount = 1, key = "chip:end-regulator", ...extra}) => ({
  ...extra,
  key,
  state: {
    inputEND: inputChipCount,
    outputEND: outputChipCount
  },
  process: async (pack, state, error) => {
    if (!error) {
      if (pack[1] === _Runlet.END) {
        state.inputEND--;
        return {
          pack: state.inputEND > 0 ? (0, _Runlet.createPack)(void 0, _Runlet.SKIP) : (0, _Runlet.createPack)(state.outputEND >= 2 ? state.outputEND : void 0, _Runlet.END),
          state
        };
      }
      return {
        pack,
        state
      };
    }
  }
});