var _time = require("../time.js");
var _random = require("../math/random.js");
var _function = require("../data/function.js");
var _ArrayBufferPacket = require("../data/ArrayBufferPacket.js");
const calcCode = (size, tokenSize, dataView, seed = 0) => {
  const seedBinaryCharList = (seed = Math.floor(seed)).toString(2).padStart(tokenSize, "0").split("");
  const valueMax = Math.pow(16, tokenSize);
  let dataViewIndex = seed % size;
  let value = dataView.getUint8(dataViewIndex);
  for (let seedIndex = seedBinaryCharList.length - 1; seedIndex >= 0; seedIndex--) {
    const stepMode = "1" === seedBinaryCharList[seedIndex];
    dataViewIndex = (dataViewIndex + (stepMode ? 1 : dataView.getUint8((dataViewIndex + 1) % size))) % size;
    value = ((stepMode ? value : 16 * value) + dataView.getUint8(dataViewIndex)) % valueMax;
  }
  return (value % Math.pow(2, 4 * tokenSize)).toString(16).padStart(tokenSize, "0");
};
const verifyOption = ({tag = (0, _time.getTimestamp)().toString(36), size = 65568, tokenSize = 8, timeGap = 30, info = null}) => {
  if (!/^\w*$/.test(tag)) throw new Error(`invalid tag: ${tag}`);
  if (!Number.isInteger(size) || size <= 1024 || size % 32) throw new Error(`invalid size: ${size}`);
  if (!Number.isInteger(tokenSize) || tokenSize > 13 || tokenSize < 2) throw new Error(`invalid tokenSize: ${tokenSize}`);
  if (!Number.isInteger(timeGap) || timeGap < 1) throw new Error(`invalid timeGap: ${timeGap}`);
  return {
    tag,
    size,
    tokenSize,
    timeGap,
    info
  };
};
exports.verifyOption = verifyOption;
exports.verifyCheckCode = (timedLookupData, checkCode, timestamp = (0, _time.getTimestamp)()) => {
  if ("string" != typeof checkCode || checkCode.length < timedLookupData.tokenSize) throw new Error(`invalid checkCode: ${checkCode}`);
  verifyParsedCheckCode(timedLookupData, parseCheckCode(checkCode), timestamp);
};
const verifyParsedCheckCode = ({tag, size, tokenSize, timeGap, dataView}, [verifyTag, verifyTimestamp, verifyCode], timestamp = (0, _time.getTimestamp)()) => {
  if (verifyTag !== tag) throw new Error(`tag mismatch: ${verifyTag}, expect: ${tag}`);
  if (Math.abs(timestamp - verifyTimestamp) > timeGap) throw new Error(`timestamp mismatch: ${verifyTimestamp}, expect: ${timestamp}±${timeGap}`);
  const code = calcCode(size, tokenSize, dataView, verifyTimestamp / timeGap);
  if (code !== verifyCode) throw new Error(`code mismatch: ${verifyCode}, expect: ${code}`);
};
exports.verifyParsedCheckCode = verifyParsedCheckCode;
exports.generateLookupData = option => {
  option = verifyOption(option);
  return {
    ...option,
    dataView: new DataView((0, _random.getRandomArrayBuffer)(option.size))
  };
};
exports.generateCheckCode = ({tag, size, tokenSize, timeGap, dataView}, timestamp = (0, _time.getTimestamp)()) => {
  const code = calcCode(size, tokenSize, dataView, timestamp / timeGap);
  return packCheckCode(tag, timestamp, code);
};
const packCheckCode = (tagString, timestamp, codeString) => (0, _function.swapObfuscateString)([ tagString, timestamp.toString(36), codeString ].join("-"));
exports.packCheckCode = packCheckCode;
const parseCheckCode = checkCodeString => {
  const resultList = (0, _function.swapObfuscateString)(checkCodeString).split("-");
  resultList[1] = Number.parseInt(resultList[1], 36);
  return resultList;
};
exports.parseCheckCode = parseCheckCode;
exports.packDataArrayBuffer = ({tag, size, tokenSize, timeGap, info, dataView}) => (0, _ArrayBufferPacket.packArrayBufferPacket)(JSON.stringify([ tag, size, tokenSize, timeGap, info ]), dataView.buffer);
exports.parseDataArrayBuffer = dataArrayBuffer => {
  const [headerString, payloadArrayBuffer] = (0, _ArrayBufferPacket.parseArrayBufferPacket)(dataArrayBuffer);
  const [tag, size, tokenSize, timeGap, info = null] = JSON.parse(headerString);
  return {
    tag,
    size,
    tokenSize,
    timeGap,
    info,
    dataView: new DataView(payloadArrayBuffer)
  };
};
exports.packDataArrayBuffer2 = ({tag, size, tokenSize, timeGap, info, dataView}) => (0, _ArrayBufferPacket.packArrayBufferPacket2)(JSON.stringify([ tag, size, tokenSize, timeGap, info ]), dataView.buffer);
exports.parseDataArrayBuffer2 = dataArrayBuffer => {
  const [headerString, payloadArrayBuffer] = (0, _ArrayBufferPacket.parseArrayBufferPacket2)(dataArrayBuffer);
  const [tag, size, tokenSize, timeGap, info = null] = JSON.parse(headerString);
  return {
    tag,
    size,
    tokenSize,
    timeGap,
    info,
    dataView: new DataView(payloadArrayBuffer)
  };
};