var _time = require("../time.js");
var _Object = require("../immutable/Object.js");
const createUpdater = () => {
  let funcList;
  let funcMap;
  let funcMapValueList;
  let prevTime;
  const clear = () => {
    funcList = [];
    funcMap = {};
    funcMapValueList = [];
    prevTime = Date.now();
  };
  clear();
  return {
    clear,
    pushFunc: func => {
      funcList.push(func);
    },
    setFunc: (key, func) => {
      funcMap = (0, _Object.objectSet)(funcMap, key, func);
      funcMapValueList = Object.values(funcMap);
    },
    deleteFunc: key => {
      funcMap = (0, _Object.objectDelete)(funcMap, key);
      funcMapValueList = Object.values(funcMap);
    },
    update: () => {
      const time = Date.now();
      const deltaTime = time - prevTime;
      const doUpdate = func => func(deltaTime * _time.CLOCK_TO_SECOND);
      prevTime = time;
      funcList.length && (funcList = funcList.filter(doUpdate));
      funcMapValueList.forEach(doUpdate);
    }
  };
};
exports.createUpdater = createUpdater;
exports.createUpdateLoop = ({delay, queueTask = _time.requestFrameUpdate, cancelTask = _time.cancelFrameUpdate} = {}) => {
  const {clear, pushFunc, setFunc, deleteFunc, update} = createUpdater();
  const {start, stop} = (0, _time.createTimer)({
    func: update,
    queueTask,
    cancelTask,
    delay
  });
  return {
    start,
    stop,
    clear,
    pushFunc,
    setFunc,
    deleteFunc
  };
};