const REGEXP_INDENT_LINE = /\n/g;
const indentLine = (string, indentString = "  ", indentStringStart = indentString) => `${indentStringStart}${string.replace(REGEXP_INDENT_LINE, `\n${indentString}`)}`;
exports.indentLine = indentLine;
exports.indentLineList = (stringList, indentString = "  ", indentStringStart = indentString) => `${indentStringStart}${stringList.join(`\n${indentString}`)}`;
exports.indentList = (title, itemList = [], indentStringStart = "  - ", indentString = " ".repeat(indentStringStart.length)) => [ title, ...itemList.map((item => indentLine(item, indentString, indentStringStart))) ].join("\n");
exports.filterJoin = (array, delimiter = "\n") => array.filter(Boolean).join(delimiter);
exports.autoEllipsis = (string = "", limit = 64, head = 32, tail = 16) => string.length > limit ? `${string.slice(0, head)}...${tail > 0 ? string.slice(-tail) : ""} (+${string.length - head - tail})` : string;
const REGEXP_SPLIT_CAMEL_CASE_TEST = /[^A-Z]/;
const REGEXP_SPLIT_CAMEL_CASE = /([^A-Z]|[A-Z]{2,})([A-Z])/g;
exports.splitCamelCase = string => REGEXP_SPLIT_CAMEL_CASE_TEST.test(string) ? string.replace(REGEXP_SPLIT_CAMEL_CASE, "$1 $2").split(" ") : [ string ];
exports.splitSnakeCase = string => string.split("_");
exports.splitKebabCase = string => string.split("-");
exports.joinCamelCase = (stringList, fromIndex = 1) => stringList.reduce(((o, string, index) => o + (index < fromIndex ? string : capFirst(string))), "");
exports.joinSnakeCase = stringList => stringList.join("_").toUpperCase();
exports.joinKebabCase = stringList => stringList.join("-").toLowerCase();
const capFirst = string => string.charAt(0).toUpperCase() + string.slice(1);
exports.createMarkReplacer = (valueMap = {}) => {
  const REGEXP_KEY = /[\w-]+/;
  const markMap = {};
  for (const [key, value] of Object.entries(valueMap)) {
    if (!REGEXP_KEY.test(key)) throw new Error(`invalid key: ${key}`);
    if (void 0 !== value) {
      if ("string" != typeof value) throw new Error(`invalid value: ${value} of key: ${key}`);
      markMap[`{${key}}`] = value;
    }
  }
  const REGEXP_MARK = /{[\w-]+}/g;
  const replacerFunc = mark => markMap[mark] || mark;
  return string => string.replace(REGEXP_MARK, replacerFunc);
};
const REGEXP_ESCAPE_REGEXP = /[.*+?^${}()|[\]\\]/g;
const escapeRegExp = string => string.replace(REGEXP_ESCAPE_REGEXP, "\\$&");
exports.escapeRegExp = escapeRegExp;
exports.replaceAll = (string, fromString, to) => string.replace(new RegExp(escapeRegExp(fromString), "g"), to);
const ESCAPE_HTML_MAP = {
  '"': "&#34;",
  "&": "&#38;",
  "'": "&#39;",
  "<": "&#60;",
  ">": "&#62;"
};
const REGEXP_ESCAPE_HTML = /["&'<>]/g;
const replaceEscapeHTML = string => ESCAPE_HTML_MAP[string] || string;
exports.escapeHTML = string => string && string.replace(REGEXP_ESCAPE_HTML, replaceEscapeHTML);
const UNESCAPE_HTML_MAP = {
  "&#34;": '"',
  "&quot;": '"',
  "&#38;": "&",
  "&amp;": "&",
  "&#39;": "'",
  "&apos;": "'",
  "&#60;": "<",
  "&lt;": "<",
  "&#62;": ">",
  "&gt;": ">"
};
const REGEXP_UNESCAPE_HTML = /&(?:#34|quot|#38|amp|#39|apos|#60|lt|#62|gt);/g;
const replaceUnescapeHTML = string => UNESCAPE_HTML_MAP[string] || string;
exports.unescapeHTML = string => string && string.replace(REGEXP_UNESCAPE_HTML, replaceUnescapeHTML);
const REGEXP_INVALID_CHAR_XML = /[^\x09\x0A\x0D\x20-\xFF\x85\xA0-\uD7FF\uE000-\uFDCF\uFDE0-\uFFFD]/gm;
exports.removeInvalidCharXML = string => string.replace(REGEXP_INVALID_CHAR_XML, "");
exports.lazyEncodeURI = string => encodeURI(string).replace(/[?#]/g, (string => "?" === string ? "%3F" : "%23"));
exports.forEachRegExpExec = (regexpGlobal, string, func) => {
  let result;
  for (;result = regexpGlobal.exec(string); ) if (func(result)) return;
};
exports.forEachLine = (string, func) => {
  const length = string.length;
  const maxIndex = length + 1;
  let fromIndex = 0;
  do {
    let toIndex = string.indexOf("\n", fromIndex);
    -1 === toIndex && (toIndex = length);
    if (func(string.slice(fromIndex, toIndex))) return;
    fromIndex = toIndex + 1;
  } while (fromIndex !== maxIndex);
};