var _check = require("./check.js");
var _format = require("./format.js");
const __throw = (title, message, detail) => {
  throw new Error(`[verify|${title}]${message ? ` ${message};` : ""} ${detail || ""}`);
};
const __toVerify = (title, checkFunc) => (value, message) => {
  checkFunc(value) || __throw(title, message, `get: ${(0, _format.describe)(value)}`);
};
exports.string = __toVerify("String", _check.isString);
exports.boolean = __toVerify("Boolean", _check.isBoolean);
exports.truthy = __toVerify("Truthy", _check.isTruthy);
exports.number = __toVerify("Number", _check.isNumber);
exports.integer = __toVerify("Integer", _check.isInteger);
exports.regexp = __toVerify("RegExp", _check.isRegExp);
exports.arrayBuffer = __toVerify("ArrayBuffer", _check.isArrayBuffer);
exports.objectAlike = __toVerify("ObjectAlike", _check.isObjectAlike);
exports.basicObject = __toVerify("BasicObject", _check.isBasicObject);
exports.objectKey = (value, key, message) => {
  (0, _check.isObjectKey)(value, key) || __throw("ObjectKey", message, `expect to have key: ${key}`);
};
exports.objectContain = (value, target, message) => {
  (0, _check.isObjectContain)(value, target) || __throw("ObjectContain", message, `expect to contain: ${(0, _format.describe)(target)}`);
};
exports.basicArray = __toVerify("Array", _check.isBasicArray);
exports.arrayLength = (value, length, message) => {
  (0, _check.isArrayLength)(value, length) || __throw("ArrayLength", message, `expect length: ${length}, get: ${(0, _check.isBasicArray)(value) ? value.length : (0, _format.describe)(value)}`);
};
exports.basicFunction = __toVerify("Function", _check.isBasicFunction);
exports.promiseAlike = __toVerify("PromiseAlike", _check.isPromiseAlike);
exports.oneOf = (value, validList, message) => {
  (0, _check.isOneOf)(value, validList) || __throw("OneOf", message, `expect one of: [${validList}], get: ${(0, _format.describe)(value)}`);
};
exports.doThrow = (func, message) => {
  (0, _check.isFunctionThrow)(func) || __throw("DoThrow", message);
};
exports.doNotThrow = (func, message) => {
  (0, _check.isFunctionThrow)(func) && __throw("DoNotThrow", message);
};
exports.doThrowAsync = async (func, message) => {
  await (0, _check.isFunctionThrowAsync)(func) || __throw("DoThrowAsync", message);
};
exports.doNotThrowAsync = async (func, message) => {
  await (0, _check.isFunctionThrowAsync)(func) && __throw("DoNotThrowAsync", message);
};
const __detailEqual = (actual, expect) => `\nactual: ${(0, _format.describe)(actual)}\nexpect: ${(0, _format.describe)(expect)}`;
exports.strictEqual = (actual, expect, message) => {
  (0, _check.isStrictEqual)(actual, expect) || __throw("StrictEqual", message, __detailEqual(actual, expect));
};
exports.notStrictEqual = (actual, expect, message) => {
  (0, _check.isStrictEqual)(actual, expect) && __throw("NotStrictEqual", message, __detailEqual(actual, expect));
};
exports.stringifyEqual = (actual, expect, message = "should stringify equal") => {
  (0, _check.isStringifyEqual)(actual, expect) || __throw("StringifyEqual", message, __detailEqual(actual, expect));
};
exports.notStringifyEqual = (actual, expect, message = "should not stringify equal") => {
  (0, _check.isStringifyEqual)(actual, expect) && __throw("NotStringifyEqual", message, __detailEqual(actual, expect));
};
exports.includes = (actual, expect, message) => {
  actual && actual.includes && actual.includes(expect) || __throw("Includes", message, `expect ${(0, _format.describe)(actual)} to include ${expect}`);
};
exports.notIncludes = (actual, expect, message) => {
  actual && actual.includes && !actual.includes(expect) || __throw("NotIncludes", message, `expect ${(0, _format.describe)(actual)} to not include ${expect}`);
};