var _nodeCrypto = require("node:crypto");
const randomFillAsync = (0, require("node:util").promisify)(_nodeCrypto.randomFill);
exports.getRandomBufferAsync = size => randomFillAsync(Buffer.allocUnsafe(size));
exports.calcHash = (bufferOrString, algorithm = "sha1", encoding = "base64") => (0, _nodeCrypto.createHash)(algorithm).update(bufferOrString).digest(encoding);
exports.createBufferRefragPool = () => {
  const pool = [];
  let poolSumLength = 0;
  return {
    pushFrag: bufferFrag => {
      pool.push(bufferFrag);
      poolSumLength += bufferFrag.length;
    },
    hasLength: length => poolSumLength >= length,
    tryGetRefragBuffer: length => {
      if (!(poolSumLength < length)) {
        poolSumLength -= length;
        if (length === pool[0].length) return pool.shift();
        if (length < pool[0].length) {
          const buffer = pool[0].subarray(0, length);
          pool[0] = pool[0].subarray(length);
          return buffer;
        }
        {
          const buffer = Buffer.allocUnsafe(length);
          let offset = 0;
          for (;length > 0; ) {
            const frag = pool[0];
            const fragLength = frag.length;
            if (length >= fragLength) {
              frag.copy(buffer, offset);
              pool.shift();
              offset += fragLength;
              length -= fragLength;
            } else {
              frag.copy(buffer, offset, 0, length);
              pool[0] = frag.subarray(length);
              length = 0;
            }
          }
          return buffer;
        }
      }
    }
  };
};