var _nodePath = require("node:path");
var _nodeFs = require("node:fs");
var _nodeCrypto = require("node:crypto");
var _compare = require("../../common/compare.js");
var _AsyncLane = require("../../common/module/AsyncLane.js");
var _Stream = require("../data/Stream.js");
var _Path = require("./Path.js");
var _Directory = require("./Directory.js");
const getChecksumInfoOfFile = async (absolutePath, fromPath) => [ (0, _nodePath.relative)(fromPath, absolutePath), (await (0, _Stream.readableStreamToBufferAsync)((0, _Stream.setupStreamPipe)((0, _nodeFs.createReadStream)(absolutePath), (0, 
_nodeCrypto.createHash)("sha1")))).toString("base64") ];
exports.getChecksumInfoOfFile = getChecksumInfoOfFile;
const getChecksumInfoListOfPath = async (path = ".", fromPath = process.cwd(), checksumInfoList = []) => {
  path = (0, _nodePath.resolve)(fromPath, path);
  const collector = async filePath => {
    checksumInfoList.push(await getChecksumInfoOfFile(filePath, fromPath));
  };
  const pathType = (0, _Path.getPathTypeFromStat)(await (0, _Path.getPathStat)(path));
  switch (pathType) {
   case _Path.PATH_TYPE.File:
    await collector(path);
    break;
   case _Path.PATH_TYPE.Directory:
    {
      const {getTailPromise, pushAuto} = (0, _AsyncLane.extendAutoSelectLane)((0, _AsyncLane.createAsyncLane)({
        laneSize: 4
      }));
      await (0, _Directory.walkDirInfoTree)(await (0, _Directory.getDirInfoTree)(path), (async dirInfo => {
        dirInfo.type === _Path.PATH_TYPE.File && pushAuto((() => collector(dirInfo.path)));
      }));
      await getTailPromise();
      break;
    }
   default:
    throw new Error(`invalid pathType: ${pathType} for ${path}`);
  }
  return checksumInfoList;
};
exports.getChecksumInfoListOfPath = getChecksumInfoListOfPath;
const getChecksumInfoListOfPathList = async (pathList = [], fromPath = process.cwd(), checksumInfoList = []) => {
  for (const path of pathList) await getChecksumInfoListOfPath(path, fromPath, checksumInfoList);
  return checksumInfoList;
};
exports.getChecksumInfoListOfPathList = getChecksumInfoListOfPathList;
const describeChecksumInfoList = checksumInfoList => checksumInfoList.sort((([a], [b]) => (0, _compare.compareString)(a, b))).map((([relativePath, hashBase64]) => `${hashBase64} ${relativePath}`)).join("\n");
exports.describeChecksumInfoList = describeChecksumInfoList;
exports.describeChecksumOfPathList = async ({pathList = [], fromPath = process.cwd()}) => describeChecksumInfoList(await getChecksumInfoListOfPathList(pathList, fromPath));