var _nodePath = require("node:path");
var _nodeFs = require("node:fs");
var _nodeOs = require("node:os");
var _error = require("../../common/error.js");
var _random = require("../../common/math/random.js");
var _Tree = require("../../common/data/Tree.js");
var _Path = require("./Path.js");
const getPathTypeFromDirent = dirent => dirent.isSymbolicLink() ? _Path.PATH_TYPE.Symlink : dirent.isDirectory() ? _Path.PATH_TYPE.Directory : dirent.isFile() ? _Path.PATH_TYPE.File : _Path.PATH_TYPE.Other;
exports.getPathTypeFromDirent = getPathTypeFromDirent;
const getDirInfoList = async path => {
  const dirInfoList = [];
  for (const dirent of await _nodeFs.promises.readdir(path, {
    withFileTypes: !0
  })) {
    const type = getPathTypeFromDirent(dirent);
    const {name} = dirent;
    dirInfoList.push({
      type,
      name,
      path: (0, _nodePath.join)(path, name)
    });
  }
  return dirInfoList;
};
exports.getDirInfoList = getDirInfoList;
const getDirInfoListSync = path => {
  const dirInfoList = [];
  for (const dirent of (0, _nodeFs.readdirSync)(path, {
    withFileTypes: !0
  })) {
    const type = getPathTypeFromDirent(dirent);
    const {name} = dirent;
    dirInfoList.push({
      type,
      name,
      path: (0, _nodePath.join)(path, name)
    });
  }
  return dirInfoList;
};
exports.getDirInfoListSync = getDirInfoListSync;
const getDirInfoTree = async path => {
  path = (0, _Path.dropTrailingSep)(path);
  const dirInfoListMap = new Map;
  const queue = [ path ];
  for (;queue.length; ) {
    const upperPath = queue.shift();
    const dirInfoList = await getDirInfoList(upperPath);
    dirInfoListMap.set(upperPath, dirInfoList);
    dirInfoList.forEach((({type, path}) => type === _Path.PATH_TYPE.Directory && queue.push(path)));
  }
  return {
    root: path,
    dirInfoListMap
  };
};
exports.getDirInfoTree = getDirInfoTree;
const getDirInfoTreeSync = path => {
  path = (0, _Path.dropTrailingSep)(path);
  const dirInfoListMap = new Map;
  const queue = [ path ];
  for (;queue.length; ) {
    const upperPath = queue.shift();
    const dirInfoList = getDirInfoListSync(upperPath);
    dirInfoListMap.set(upperPath, dirInfoList);
    dirInfoList.forEach((({type, path}) => type === _Path.PATH_TYPE.Directory && queue.push(path)));
  }
  return {
    root: path,
    dirInfoListMap
  };
};
exports.getDirInfoTreeSync = getDirInfoTreeSync;
const getSubNodeListFunc = (dirInfo, dirInfoListMap) => dirInfoListMap.get(dirInfo.path);
const dirInfoTreeBreadthFirstSearchAsync = (0, _Tree.createTreeBreadthFirstSearchAsync)(getSubNodeListFunc);
const dirInfoTreeBreadthFirstSearchSync = (0, _Tree.createTreeBreadthFirstSearch)(getSubNodeListFunc);
const dirInfoTreeBottomUpSearchAsync = (0, _Tree.createTreeBottomUpSearchAsync)(getSubNodeListFunc);
const dirInfoTreeBottomUpSearchSync = (0, _Tree.createTreeBottomUpSearch)(getSubNodeListFunc);
const walkDirInfoTree = async ({root, dirInfoListMap}, callback) => dirInfoTreeBreadthFirstSearchAsync({
  path: root
}, callback, dirInfoListMap);
exports.walkDirInfoTree = walkDirInfoTree;
const walkDirInfoTreeSync = ({root, dirInfoListMap}, callback) => dirInfoTreeBreadthFirstSearchSync({
  path: root
}, callback, dirInfoListMap);
exports.walkDirInfoTreeSync = walkDirInfoTreeSync;
const walkDirInfoTreeBottomUp = async ({root, dirInfoListMap}, callback) => dirInfoTreeBottomUpSearchAsync({
  path: root
}, callback, dirInfoListMap);
exports.walkDirInfoTreeBottomUp = walkDirInfoTreeBottomUp;
const walkDirInfoTreeBottomUpSync = ({root, dirInfoListMap}, callback) => dirInfoTreeBottomUpSearchSync({
  path: root
}, callback, dirInfoListMap);
exports.walkDirInfoTreeBottomUpSync = walkDirInfoTreeBottomUpSync;
const copyDirInfoTree = async (dirInfoTree, pathTo) => {
  await createDirectory(pathTo);
  const pathToMap = new Map;
  pathToMap.set(dirInfoTree.root, pathTo);
  return walkDirInfoTree(dirInfoTree, (async ({name, path}) => {
    const upperPath = (0, _nodePath.dirname)(path);
    const pathTo = (0, _nodePath.join)(pathToMap.get(upperPath), name);
    pathToMap.set(path, pathTo);
    await (0, _Path.copyPath)(path, pathTo);
  }));
};
exports.copyDirInfoTree = copyDirInfoTree;
const copyDirInfoTreeSync = (dirInfoTree, pathTo) => {
  createDirectorySync(pathTo);
  const pathToMap = new Map;
  pathToMap.set(dirInfoTree.root, pathTo);
  return walkDirInfoTreeSync(dirInfoTree, (({name, path}) => {
    const upperPath = (0, _nodePath.dirname)(path);
    const pathTo = (0, _nodePath.join)(pathToMap.get(upperPath), name);
    pathToMap.set(path, pathTo);
    (0, _Path.copyPathSync)(path, pathTo);
  }));
};
exports.copyDirInfoTreeSync = copyDirInfoTreeSync;
exports.renameDirInfoTree = async ({root, dirInfoListMap}, pathTo) => {
  await createDirectory(pathTo);
  for (const {name, path} of dirInfoListMap.get(root)) await (0, _Path.renamePath)(path, (0, _nodePath.join)(pathTo, name));
};
exports.renameDirInfoTreeSync = ({root, dirInfoListMap}, pathTo) => {
  createDirectorySync(pathTo);
  for (const {name, path} of dirInfoListMap.get(root)) (0, _Path.renamePathSync)(path, (0, _nodePath.join)(pathTo, name));
};
const deleteDirInfoTree = async dirInfoListMap => walkDirInfoTreeBottomUp(dirInfoListMap, (({path}) => (0, _Path.deletePath)(path)));
exports.deleteDirInfoTree = deleteDirInfoTree;
const deleteDirInfoTreeSync = dirInfoListMap => walkDirInfoTreeBottomUpSync(dirInfoListMap, (({path}) => (0, _Path.deletePathSync)(path)));
exports.deleteDirInfoTreeSync = deleteDirInfoTreeSync;
const createDirectory = async (path, pathStat) => {
  if (void 0 !== pathStat) {
    if (pathStat.isDirectory()) return;
    if (pathStat !== _Path.STAT_ERROR) throw new Error(`path already taken by non-directory: ${path}`);
  }
  await _nodeFs.promises.mkdir(path, {
    recursive: !0
  });
};
exports.createDirectory = createDirectory;
const createDirectorySync = (path, pathStat) => {
  if (void 0 !== pathStat) {
    if (pathStat.isDirectory()) return;
    if (pathStat !== _Path.STAT_ERROR) throw new Error(`path already taken by non-directory: ${path}`);
  }
  (0, _nodeFs.mkdirSync)(path, {
    recursive: !0
  });
};
exports.createDirectorySync = createDirectorySync;
exports.copyDirectory = async (pathFrom, pathTo, pathStat) => copyDirInfoTree(await getDirInfoTree(pathFrom), pathTo);
exports.copyDirectorySync = (pathFrom, pathTo, pathStat) => copyDirInfoTreeSync(getDirInfoTreeSync(pathFrom), pathTo);
const deleteDirectory = async (path, pathStat) => {
  await deleteDirInfoTree(await getDirInfoTree(path));
  await (0, _Path.deletePath)(path, pathStat);
};
exports.deleteDirectory = deleteDirectory;
const deleteDirectorySync = (path, pathStat) => {
  deleteDirInfoTreeSync(getDirInfoTreeSync(path));
  (0, _Path.deletePathSync)(path, pathStat);
};
exports.deleteDirectorySync = deleteDirectorySync;
const resetDirectory = async (path, pathStat) => {
  void 0 === pathStat && (pathStat = await (0, _Path.getPathStat)(path));
  if (pathStat.isDirectory()) await deleteDirInfoTree(await getDirInfoTree(path)); else {
    pathStat !== _Path.STAT_ERROR && await (0, _Path.deletePath)(path, pathStat);
    await createDirectory(path);
  }
};
exports.resetDirectory = resetDirectory;
const resetDirectorySync = (path, pathStat) => {
  void 0 === pathStat && (pathStat = (0, _Path.getPathStatSync)(path));
  if (pathStat.isDirectory()) deleteDirInfoTreeSync(getDirInfoTreeSync(path)); else {
    pathStat !== _Path.STAT_ERROR && (0, _Path.deletePathSync)(path, pathStat);
    createDirectorySync(path);
  }
};
exports.resetDirectorySync = resetDirectorySync;
exports.withTempDirectory = async (asyncFunc, pathTemp = (0, _nodePath.join)((0, _nodeOs.tmpdir)(), "dr-js", (0, _random.getRandomId)("temp-"))) => {
  await resetDirectory(pathTemp);
  const {result, error} = await (0, _error.catchAsync)(asyncFunc, pathTemp);
  await deleteDirectory(pathTemp);
  if (error) throw error;
  return result;
};
exports.withTempDirectorySync = (syncFunc, pathTemp = (0, _nodePath.join)((0, _nodeOs.tmpdir)(), "dr-js", (0, _random.getRandomId)("temp-"))) => {
  resetDirectorySync(pathTemp);
  const {result, error} = (0, _error.catchSync)(syncFunc, pathTemp);
  deleteDirectorySync(pathTemp);
  if (error) throw error;
  return result;
};
exports.getFileList = async (path, fileCollector = (async (fileList, {path}) => {
  fileList.push(path);
})) => {
  const fileList = [];
  const pathStat = await (0, _Path.getPathStat)(path);
  const pathType = (0, _Path.getPathTypeFromStat)(pathStat);
  switch (pathType) {
   case _Path.PATH_TYPE.File:
    await fileCollector(fileList, {
      type: pathType,
      name: (0, _nodePath.basename)(path),
      path
    });
    break;
   case _Path.PATH_TYPE.Directory:
    await walkDirInfoTree(await getDirInfoTree(path), (dirInfo => dirInfo.type === _Path.PATH_TYPE.File && fileCollector(fileList, dirInfo)));
    break;
   default:
    throw new Error(`invalid pathType: ${pathType} for ${path}`);
  }
  return fileList;
};
exports.getFileListSync = (path, fileCollector = ((fileList, {path}) => {
  fileList.push(path);
})) => {
  const fileList = [];
  const pathStat = (0, _Path.getPathStatSync)(path);
  const pathType = (0, _Path.getPathTypeFromStat)(pathStat);
  switch (pathType) {
   case _Path.PATH_TYPE.File:
    fileCollector(fileList, {
      type: pathType,
      name: (0, _nodePath.basename)(path),
      path
    });
    break;
   case _Path.PATH_TYPE.Directory:
    walkDirInfoTreeSync(getDirInfoTreeSync(path), (dirInfo => dirInfo.type === _Path.PATH_TYPE.File && fileCollector(fileList, dirInfo)));
    break;
   default:
    throw new Error(`invalid pathType: ${pathType} for ${path}`);
  }
  return fileList;
};