var _nodeFs = require("node:fs");
var _tryRequire = require("../../env/tryRequire.js");
var _ArrayBuffer = require("../../common/data/ArrayBuffer.js");
var _function = require("../../common/data/function.js");
const readBuffer = async path => _nodeFs.promises.readFile(path);
exports.readBuffer = readBuffer;
const readBufferSync = path => (0, _nodeFs.readFileSync)(path);
exports.readBufferSync = readBufferSync;
const writeBuffer = async (path, bufferAlike) => _nodeFs.promises.writeFile(path, bufferAlike);
exports.writeBuffer = writeBuffer;
const writeBufferSync = (path, bufferAlike) => (0, _nodeFs.writeFileSync)(path, bufferAlike);
exports.writeBufferSync = writeBufferSync;
const appendBuffer = async (path, bufferAlike) => _nodeFs.promises.appendFile(path, bufferAlike);
exports.appendBuffer = appendBuffer;
const appendBufferSync = (path, bufferAlike) => (0, _nodeFs.appendFileSync)(path, bufferAlike);
exports.appendBufferSync = appendBufferSync;
exports.editBuffer = async (editFunc = (async buffer => buffer), pathFrom, pathTo = pathFrom) => writeBuffer(pathTo, await editFunc(await readBuffer(pathFrom)));
exports.editBufferSync = (editFunc = (buffer => buffer), pathFrom, pathTo = pathFrom) => writeBufferSync(pathTo, editFunc(readBufferSync(pathFrom)));
const readArrayBuffer = async path => (0, _ArrayBuffer.fromNodejsBuffer)(await readBuffer(path));
exports.readArrayBuffer = readArrayBuffer;
const readArrayBufferSync = path => (0, _ArrayBuffer.fromNodejsBuffer)(readBufferSync(path));
exports.readArrayBufferSync = readArrayBufferSync;
const writeArrayBuffer = async (path, arrayBuffer) => writeBuffer(path, Buffer.from(arrayBuffer));
exports.writeArrayBuffer = writeArrayBuffer;
const writeArrayBufferSync = (path, arrayBuffer) => writeBufferSync(path, Buffer.from(arrayBuffer));
exports.writeArrayBufferSync = writeArrayBufferSync;
exports.appendArrayBuffer = async (path, arrayBuffer) => appendBuffer(path, Buffer.from(arrayBuffer));
exports.appendArrayBufferSync = (path, arrayBuffer) => appendBufferSync(path, Buffer.from(arrayBuffer));
exports.editArrayBuffer = async (editFunc = (async arrayBuffer => arrayBuffer), pathFrom, pathTo = pathFrom) => writeArrayBuffer(pathTo, await editFunc(await readArrayBuffer(pathFrom)));
exports.editArrayBufferSync = (editFunc = (arrayBuffer => arrayBuffer), pathFrom, pathTo = pathFrom) => writeArrayBufferSync(pathTo, editFunc(readArrayBufferSync(pathFrom)));
const readText = async path => String(await readBuffer(path));
exports.readText = readText;
const readTextSync = path => String(readBufferSync(path));
exports.readTextSync = readTextSync;
const writeText = exports.writeText = writeBuffer;
const writeTextSync = exports.writeTextSync = writeBufferSync;
exports.appendText = appendBuffer;
exports.appendTextSync = appendBufferSync;
exports.editText = async (editFunc = (async string => string), pathFrom, pathTo = pathFrom) => writeText(pathTo, await editFunc(await readText(pathFrom)));
exports.editTextSync = (editFunc = (string => string), pathFrom, pathTo = pathFrom) => writeTextSync(pathTo, editFunc(readTextSync(pathFrom)));
const readJSON = async path => JSON.parse(await readText(path));
exports.readJSON = readJSON;
const readJSONSync = path => JSON.parse(readTextSync(path));
exports.readJSONSync = readJSONSync;
exports.readJSONAlike = async path => readJSONAlikeSync(path);
const readJSONAlikeSync = path => {
  delete require.cache[(0, _tryRequire.tryRequireResolve)(path)];
  const jsonAlike = (0, _tryRequire.tryRequire)(path);
  if (void 0 === jsonAlike) throw new Error(`fail to load file as JSON: ${path}`);
  return (0, _function.dupJSON)(jsonAlike);
};
exports.readJSONAlikeSync = readJSONAlikeSync;
const writeJSON = async (path, value) => writeBuffer(path, JSON.stringify(value));
exports.writeJSON = writeJSON;
const writeJSONSync = (path, value) => writeBufferSync(path, JSON.stringify(value));
exports.writeJSONSync = writeJSONSync;
const writeJSONPretty = async (path, value) => writeBuffer(path, JSON.stringify(value, null, 2));
exports.writeJSONPretty = writeJSONPretty;
const writeJSONPrettySync = (path, value) => writeBufferSync(path, JSON.stringify(value, null, 2));
exports.writeJSONPrettySync = writeJSONPrettySync;
exports.editJSON = async (editFunc = (async value => value), pathFrom, pathTo = pathFrom) => writeJSON(pathTo, await editFunc(await readJSON(pathFrom)));
exports.editJSONSync = (editFunc = (value => value), pathFrom, pathTo = pathFrom) => writeJSONSync(pathTo, editFunc(readJSONSync(pathFrom)));
exports.editJSONPretty = async (editFunc = (async value => value), pathFrom, pathTo = pathFrom) => writeJSONPretty(pathTo, await editFunc(await readJSON(pathFrom)));
exports.editJSONPrettySync = (editFunc = (value => value), pathFrom, pathTo = pathFrom) => writeJSONPrettySync(pathTo, editFunc(readJSONSync(pathFrom)));
exports.copyFile = async (pathFrom, pathTo) => _nodeFs.promises.copyFile(pathFrom, pathTo);
exports.copyFileSync = (pathFrom, pathTo) => (0, _nodeFs.copyFileSync)(pathFrom, pathTo);
exports.renameFile = async (pathFrom, pathTo) => _nodeFs.promises.rename(pathFrom, pathTo);
exports.renameFileSync = (pathFrom, pathTo) => (0, _nodeFs.renameSync)(pathFrom, pathTo);
const deleteFile = async path => _nodeFs.promises.unlink(path);
exports.deleteFile = deleteFile;
const deleteFileSync = path => (0, _nodeFs.unlinkSync)(path);
exports.deleteFileSync = deleteFileSync;
exports.deleteFileForce = async path => {
  try {
    await deleteFile(path);
  } catch (error) {}
};
exports.deleteFileForceSync = path => {
  try {
    deleteFileSync(path);
  } catch (error) {}
};