var _nodeFs = require("node:fs");
var _nodePath = require("node:path");
var _nodeOs = require("node:os");
var _error = require("../../common/error.js");
const STAT_ERROR = exports.STAT_ERROR = {
  isSymbolicLink: () => !1,
  isDirectory: () => !1,
  isFile: () => !1
};
const PATH_TYPE = exports.PATH_TYPE = {
  File: "File",
  Directory: "Directory",
  Symlink: "Symlink",
  Other: "Other",
  Error: "Error"
};
exports.getPathTypeFromStat = stat => stat.isSymbolicLink() ? PATH_TYPE.Symlink : stat.isDirectory() ? PATH_TYPE.Directory : stat.isFile() ? PATH_TYPE.File : stat === STAT_ERROR ? PATH_TYPE.Error : PATH_TYPE.Other;
const getPathLstat = async path => (0, _error.withFallbackResultAsync)(STAT_ERROR, _nodeFs.promises.lstat, path);
exports.getPathLstat = getPathLstat;
const getPathLstatSync = path => (0, _error.withFallbackResult)(STAT_ERROR, _nodeFs.lstatSync, path);
exports.getPathLstatSync = getPathLstatSync;
exports.getPathStat = async path => (0, _error.withFallbackResultAsync)(STAT_ERROR, _nodeFs.promises.stat, path);
exports.getPathStatSync = path => (0, _error.withFallbackResult)(STAT_ERROR, _nodeFs.statSync, path);
exports.copyPath = async (pathFrom, pathTo, pathStat) => {
  void 0 === pathStat && (pathStat = await getPathLstat(pathFrom));
  return pathStat.isSymbolicLink() ? _nodeFs.promises.symlink(await _nodeFs.promises.readlink(pathFrom), pathTo) : pathStat.isDirectory() ? (await getPathLstat(pathTo)).isDirectory() ? void 0 : _nodeFs.promises.mkdir(pathTo) : _nodeFs.promises.copyFile(pathFrom, pathTo);
};
exports.copyPathSync = (pathFrom, pathTo, pathStat) => {
  void 0 === pathStat && (pathStat = getPathLstatSync(pathFrom));
  return pathStat.isSymbolicLink() ? (0, _nodeFs.symlinkSync)((0, _nodeFs.readlinkSync)(pathFrom), pathTo) : pathStat.isDirectory() ? getPathLstatSync(pathTo).isDirectory() ? void 0 : (0, _nodeFs.mkdirSync)(pathTo) : (0, _nodeFs.copyFileSync)(pathFrom, pathTo);
};
exports.renamePath = async (pathFrom, pathTo) => _nodeFs.promises.rename(pathFrom, pathTo);
exports.renamePathSync = (pathFrom, pathTo) => (0, _nodeFs.renameSync)(pathFrom, pathTo);
const deletePath = async (path, pathStat) => {
  void 0 === pathStat && (pathStat = await getPathLstat(path));
  pathStat.isDirectory() ? await _nodeFs.promises.rmdir(path) : await _nodeFs.promises.unlink(path);
};
exports.deletePath = deletePath;
const deletePathSync = (path, pathStat) => {
  void 0 === pathStat && (pathStat = getPathLstatSync(path));
  pathStat.isDirectory() ? (0, _nodeFs.rmdirSync)(path) : (0, _nodeFs.unlinkSync)(path);
};
exports.deletePathSync = deletePathSync;
exports.deletePathForce = async (path, pathStat) => (0, _error.withFallbackResultAsync)(void 0, deletePath, path, pathStat);
exports.deletePathForceSync = (path, pathStat) => (0, _error.withFallbackResult)(void 0, deletePathSync, path, pathStat);
const existPath = async path => ErrorNotExist !== await (0, _error.withFallbackResultAsync)(ErrorNotExist, _nodeFs.promises.access, path);
exports.existPath = existPath;
const existPathSync = path => ErrorNotExist !== (0, _error.withFallbackResult)(ErrorNotExist, _nodeFs.accessSync, path);
exports.existPathSync = existPathSync;
const ErrorNotExist = new Error;
exports.nearestExistPath = async path => {
  for (;path && !await existPath(path); ) path = (0, _nodePath.dirname)(path);
  return path;
};
exports.nearestExistPathSync = path => {
  for (;path && !existPathSync(path); ) path = (0, _nodePath.dirname)(path);
  return path;
};
exports.toPosixPath = path => path.replace(REGEXP_PATH_SEP_WIN32, "/");
const REGEXP_PATH_SEP_WIN32 = /\\/g;
exports.addTrailingSep = path => (0, _nodePath.join)(path, _nodePath.sep);
exports.dropTrailingSep = path => (0, _nodePath.join)(path, ".");
const expandHome = path => "~" === path ? (0, _nodeOs.homedir)() : REGEXP_HOME_PREFIX.test(path) ? `${(0, _nodeOs.homedir)()}${path.slice(1)}` : path;
exports.expandHome = expandHome;
const REGEXP_HOME_PREFIX = /^~[/\\]/;
exports.resolveHome = (...argList) => (0, _nodePath.resolve)(...argList.map(expandHome));
exports.createPathPrefixLock = rootPath => {
  rootPath = (0, _nodePath.resolve)(rootPath);
  return relativePath => {
    const absolutePath = (0, _nodePath.resolve)(rootPath, relativePath);
    if (!absolutePath.startsWith(rootPath)) throw new Error(`invalid relativePath: ${relativePath}`);
    return absolutePath;
  };
};