var _nodePath = require("node:path");
var _nodeOs = require("node:os");
var _time = require("../common/time.js");
var _format = require("../common/format.js");
var _base = require("../common/math/base.js");
var _npm = require("./module/Software/npm.js");
var _TerminalTTY = require("./module/TerminalTTY.js");
var _ResolveCommand = require("./system/ResolveCommand.js");
var _run = require("./run.js");
const loadEnvKey = key => {
  try {
    return JSON.parse(process.env[key]);
  } catch (error) {
    return null;
  }
};
exports.loadEnvKey = loadEnvKey;
const saveEnvKey = (key, value) => {
  try {
    process.env[key] = JSON.stringify(value);
  } catch (error) {}
};
exports.saveEnvKey = saveEnvKey;
exports.syncEnvKey = (key, defaultValue) => {
  const value = loadEnvKey(key) || defaultValue;
  saveEnvKey(key, value);
  return value;
};
const argvFlag = (...checkFlagList) => process.argv.find((flag => checkFlagList.includes(flag)));
exports.argvFlag = argvFlag;
const ENV_KEY_LOGGER = exports.ENV_KEY_LOGGER = "__DRJS_KIT_LOGGER__";
const ENV_KEY_VERBOSE = exports.ENV_KEY_VERBOSE = "__DRJS_KIT_VERBOSE__";
const getKitLogger = ({title = "kit", isNoEnvKey = !1, isVerbose = Boolean(loadEnvKey(ENV_KEY_VERBOSE) || argvFlag("verbose") || process.env.KIT_VERBOSE), isQuiet = Boolean(argvFlag("quiet") || process.env.KIT_QUIET), padWidth = (0, _base.clamp)(process.env.KIT_LOGGER_WIDTH || process.stdout.isTTY && process.stdout.columns || 80, 32, 240), TerminalColor = (0, 
_TerminalTTY.createColor)(), colorPadLogFunc = TerminalColor.fg.yellow, colorStepLogFunc = TerminalColor.fg.yellow, colorLogFunc = TerminalColor.fg.darkGray, logFunc = console.log} = {}) => {
  const startTime = (0, _time.clock)();
  !isNoEnvKey && saveEnvKey(ENV_KEY_VERBOSE, Boolean(isVerbose));
  const envKit = loadEnvKey(ENV_KEY_LOGGER) || {
    titleList: [],
    startTime,
    pid: null
  };
  !isNoEnvKey && envKit.pid !== process.pid && saveEnvKey(ENV_KEY_LOGGER, {
    ...envKit,
    titleList: [ title, ...envKit.titleList ],
    pid: process.pid
  });
  title = [ title, ...envKit.titleList.map((v => v.slice(0, (0, _base.clamp)(Math.floor(padWidth / 20), 3, 9)))) ].join("|");
  let prevTime = (0, _time.clock)();
  const stepLog = (...args) => logFunc(`- ${colorStepLogFunc(`(+${(() => {
    const currentTime = (0, _time.clock)();
    const stepTime = currentTime - prevTime;
    prevTime = currentTime;
    return (0, _format.time)(stepTime);
  })()})`)} ${args.join(" ")}`);
  const log = (...args) => logFunc(colorLogFunc(`- ${args.join(" ")}`));
  const devLog = isVerbose ? log : () => {};
  return !isVerbose && isQuiet ? {
    padLog: stepLog,
    stepLog: devLog,
    log: devLog,
    devLog: () => {},
    isVerbose
  } : {
    padLog: (...args) => {
      const start = `## ${args.join(" ")} `;
      const end = ` [${title}|${(() => {
        const currentTime = (0, _time.clock)();
        prevTime = currentTime;
        return envKit.startTime === startTime ? (0, _format.time)(currentTime - startTime) : `${(0, _format.time)(currentTime - startTime)}/${(0, _format.time)(currentTime - envKit.startTime)}`;
      })()}]`;
      logFunc(`\n${start.padEnd(padWidth - end.length, "-")}${colorPadLogFunc(end)}`);
    },
    stepLog,
    log,
    devLog,
    isVerbose
  };
};
exports.getKitLogger = getKitLogger;
const getKitPathCombo = ({PATH_ROOT = (0, _npm.findUpPackageRoot)(process.cwd()) || process.cwd(), PATH_OUTPUT = "output-gitignore/", PATH_TEMP = ".temp-gitignore/", PATH_HOME = (0, _nodeOs.homedir)(), PATH_OSTEMP = (0, _nodeOs.tmpdir)()} = {}) => {
  PATH_OUTPUT = (0, _nodePath.resolve)(PATH_ROOT, PATH_OUTPUT);
  PATH_TEMP = (0, _nodePath.resolve)(PATH_ROOT, PATH_TEMP);
  PATH_HOME = (0, _nodePath.resolve)(PATH_ROOT, PATH_HOME);
  PATH_OSTEMP = (0, _nodePath.resolve)(PATH_ROOT, PATH_OSTEMP);
  return {
    PATH_ROOT,
    fromRoot: (...args) => (0, _nodePath.resolve)(PATH_ROOT, ...args),
    PATH_OUTPUT,
    fromOutput: (...args) => (0, _nodePath.resolve)(PATH_OUTPUT, ...args),
    PATH_TEMP,
    fromTemp: (...args) => (0, _nodePath.resolve)(PATH_TEMP, ...args),
    PATH_HOME,
    fromHome: (...args) => (0, _nodePath.resolve)(PATH_HOME, ...args),
    PATH_OSTEMP,
    fromOsTemp: (...args) => (0, _nodePath.resolve)(PATH_OSTEMP, ...args)
  };
};
exports.getKitPathCombo = getKitPathCombo;
const getKitRun = ({PATH_ROOT, log = console.log, isQuiet = Boolean(argvFlag("quiet") || process.env.KIT_QUIET), isDryRun = Boolean(process.env.KIT_DRY_RUN)} = {}) => {
  const toArgList = argListOrString => Array.isArray(argListOrString) ? [ ...argListOrString ] : argListOrString.split(" ").filter(Boolean);
  const RUN = (argListOrString, {isDetached = !1, ...option} = {}) => {
    const argList = toArgList(argListOrString);
    argList[0] = (0, _ResolveCommand.resolveCommand)(argList[0], PATH_ROOT);
    log(`[${isDryRun ? "DRY-RUN" : "RUN"}${isDetached ? "|DETACHED" : ""}] "${argList.join(" ")}"`);
    if (!isDryRun) return (isDetached ? _run.runDetached : _run.runSync)(argList, {
      cwd: PATH_ROOT,
      quiet: isQuiet,
      ...option
    });
  };
  return {
    RUN,
    RUN_SUDO_NPM: (argListOrString, option) => RUN([ ...(0, _npm.getSudoArgs)(), ...toArgList(argListOrString) ], option)
  };
};
exports.getKitRun = getKitRun;
const getKit = option => {
  const kitLogger = getKitLogger(option);
  const kitPathCombo = getKitPathCombo(option);
  const kitRun = getKitRun({
    ...option,
    ...kitLogger,
    ...kitPathCombo
  });
  return {
    ...kitLogger,
    ...kitPathCombo,
    ...kitRun
  };
};
exports.getKit = getKit;
exports.runKit = (asyncFunc, {kit, kitLogger, title = process.argv.slice(2).join("+") || void 0, ...kitOption} = {}) => {
  const kitOrKitLogger = kit || kitLogger || getKit({
    title,
    ...kitOption
  });
  const startTime = (0, _time.clock)();
  new Promise((resolve => resolve(asyncFunc(kitOrKitLogger)))).then((() => {
    kitOrKitLogger.padLog(`done in ${(0, _format.time)((0, _time.clock)() - startTime)}`);
  }), (error => {
    console.warn(error);
    kitOrKitLogger.padLog(`error after ${(0, _format.time)((0, _time.clock)() - startTime)}`);
    process.exit(-1);
  }));
};