var _nodePath = require("node:path");
var _nodeFs = require("node:fs");
var _error = require("../../../common/error.js");
var _Object = require("../../../common/immutable/Object.js");
var _Path = require("../../fs/Path.js");
var _Directory = require("../../fs/Directory.js");
var _Modify = require("../../fs/Modify.js");
const PATH_VISIBLE = "path.visible";
const PATH_STAT = "path.stat";
const PATH_COPY = "path.copy";
const PATH_RENAME = "path.rename";
const PATH_DELETE = "path.delete";
const PATH_DIRECTORY_CREATE = "path.dir.create";
const PATH_DIRECTORY_CONTENT = "path.dir.content";
const PATH_DIRECTORY_ALL_FILE_LIST = "path.dir.all-file-list";
exports.ACTION_TYPE = {
  PATH_VISIBLE: "path.visible",
  PATH_STAT: "path.stat",
  PATH_COPY: "path.copy",
  PATH_RENAME: "path.rename",
  PATH_DELETE: "path.delete",
  PATH_DIRECTORY_CREATE: "path.dir.create",
  PATH_DIRECTORY_CONTENT: "path.dir.content",
  PATH_DIRECTORY_ALL_FILE_LIST: "path.dir.all-file-list"
};
const ACTION_CORE_MAP = exports.ACTION_CORE_MAP = {
  [PATH_VISIBLE]: async absolutePath => (0, _Path.existPath)(absolutePath).then((isVisible => ({
    isVisible
  }))),
  [PATH_STAT]: async absolutePath => _nodeFs.promises.stat(absolutePath).then((({mode, size, mtimeMs}) => ({
    mode,
    size,
    mtimeMs
  }))),
  [PATH_COPY]: async (absolutePath, absolutePathTo) => {
    await (0, _Modify.modifyCopy)(absolutePath, absolutePathTo);
  },
  [PATH_RENAME]: async (absolutePath, absolutePathTo) => {
    await (0, _Modify.modifyRename)(absolutePath, absolutePathTo);
  },
  [PATH_DELETE]: async absolutePath => {
    await (0, _Modify.modifyDelete)(absolutePath);
  },
  [PATH_DIRECTORY_CREATE]: async absolutePath => {
    await (0, _Directory.createDirectory)(absolutePath);
  },
  [PATH_DIRECTORY_CONTENT]: async absolutePath => {
    const {result: dirInfoList = [], error} = await (0, _error.catchAsync)(_Directory.getDirInfoList, absolutePath);
    const directoryList = [];
    const fileList = [];
    for (const {type, name, path} of dirInfoList) {
      const stat = type === _Path.PATH_TYPE.Directory || await (0, _Path.getPathStat)(path);
      !0 === stat || stat.isDirectory() ? directoryList.push(name) : fileList.push([ name, stat.size, Math.round(stat.mtimeMs) ]);
    }
    return {
      directoryList,
      fileList
    };
  },
  [PATH_DIRECTORY_ALL_FILE_LIST]: async absolutePath => {
    const fileList = [];
    const {error} = await (0, _error.catchAsync)((async () => (0, _Directory.walkDirInfoTree)(await (0, _Directory.getDirInfoTree)(absolutePath), (async ({type, path}) => {
      const stat = type === _Path.PATH_TYPE.Directory || await (0, _Path.getPathStat)(path);
      !0 !== stat && !stat.isDirectory() && fileList.push([ (0, _Path.toPosixPath)((0, _nodePath.relative)(absolutePath, path)), stat.size, Math.round(stat.mtimeMs) ]);
    }))));
    return {
      fileList
    };
  }
};
exports.setupActionMap = ({actionCoreMap = ACTION_CORE_MAP, rootPath, getPath = (0, _Path.createPathPrefixLock)(rootPath), loggerExot}) => {
  const calcPath = (key, extraPath) => getPath(_nodePath.posix.normalize(_nodePath.posix.join(key, extraPath)));
  return (0, _Object.objectMap)(actionCoreMap, ((actionFunc, actionType) => async (store, {key, keyTo, batchList = [ "" ]}) => {
    loggerExot.add(`[ActionBox|${actionType}] ${key || ""}${keyTo ? `, ${keyTo}` : ""} (${batchList.join("|")})`);
    const resultList = [];
    const errorList = [];
    for (const extraPath of batchList) {
      const absolutePath = calcPath(key, extraPath);
      const absolutePathTo = keyTo && calcPath(keyTo, extraPath);
      const {result, error} = await (0, _error.catchAsync)(actionFunc, absolutePath, absolutePathTo);
      const response = {
        actionType,
        key,
        keyTo,
        ...result
      };
      error ? errorList.push({
        ...response,
        error: String(error)
      }) : resultList.push(response);
    }
    return {
      resultList,
      errorList
    };
  }));
};