var _time = require("../../../common/time.js");
var _string = require("../../../common/string.js");
var _error = require("../../../common/error.js");
var _Object = require("../../../common/immutable/Object.js");
var _hostStatus = require("../Software/hostStatus.js");
const STATUS_TIMESTAMP = "status.timestamp";
const STATUS_TIME_ISO = "status.time-iso";
const STATUS_SERVER_COMMON = "status.server-common";
exports.ACTION_TYPE = {
  STATUS_TIMESTAMP: "status.timestamp",
  STATUS_TIME_ISO: "status.time-iso",
  STATUS_SERVER_COMMON: "status.server-common"
};
const ACTION_CORE_MAP = exports.ACTION_CORE_MAP = {
  [STATUS_TIMESTAMP]: async () => ({
    status: (0, _time.getTimestamp)()
  }),
  [STATUS_TIME_ISO]: async () => ({
    status: (new Date).toISOString()
  }),
  [STATUS_SERVER_COMMON]: async ({rootPath, statusCommandList}) => ({
    status: (await (0, _hostStatus.getCommonHostStatus)(rootPath, statusCommandList)).map((([title, output]) => output && `${`[${title}] `.padEnd(80, "=")}\n${(0, _string.indentLine)(output, "  ")}`)).filter(Boolean).join("\n")
  })
};
exports.setupActionMap = ({actionCoreMap = ACTION_CORE_MAP, statusCommandList = _hostStatus.COMMON_HOST_STATUS_COMMAND_LIST, rootPath, loggerExot}) => {
  const option = {
    rootPath,
    statusCommandList
  };
  return (0, _Object.objectMap)(actionCoreMap, ((actionFunc, actionType) => async (store, actionPayload) => {
    loggerExot.add(`[ActionBox|${actionType}]`);
    const {result, error} = await (0, _error.catchAsync)(actionFunc, option);
    return error ? {
      actionType,
      error: String(error)
    } : {
      actionType,
      ...result
    };
  }));
};