var _nodePath = require("node:path");
var _nodeFs = require("node:fs");
var _Directory = require("../../fs/Directory.js");
var _run = require("../../run.js");
var _z = require("./7z.js");
var _npmTar = require("./npmTar.js");
var _fsp = require("./fsp.js");
var _function = require("./function.js");
exports.REGEXP_AUTO = /\.(?:tar|tgz|tar\.gz|tbr|tar\.br|t7z|tar\.7z|txz|tar\.xz|7z|zip|fsp|fsp\.gz|fsp\.br)$/;
exports.check = () => Boolean((0, _z.check)() && (0, _npmTar.check)());
exports.verify = () => (0, _z.verify)() && (0, _npmTar.verify)() && !0;
const compress7zAsync = async (sourceDirectory, outputFile) => (0, _run.run)((0, _z.compressArgs)(sourceDirectory, outputFile)).promise;
exports.compress7zAsync = compress7zAsync;
const extract7zAsync = async (sourceFile, outputPath) => (0, _run.run)((0, _z.extractArgs)(sourceFile, outputPath)).promise;
exports.extract7zAsync = extract7zAsync;
const compressT7zAsync = async (sourceDirectory, outputFile, pathTemp) => (0, _Directory.withTempDirectory)((async pathTemp => {
  await (0, _npmTar.compressAsync)(sourceDirectory, (0, _nodePath.resolve)(pathTemp, "archive.tar"));
  await compress7zAsync(pathTemp, outputFile);
}), pathTemp);
exports.compressT7zAsync = compressT7zAsync;
const extractT7zAsync = async (sourceFile, outputPath, pathTemp) => (0, _Directory.withTempDirectory)((async pathTemp => {
  await extract7zAsync(sourceFile, pathTemp);
  const nameList = await _nodeFs.promises.readdir(pathTemp);
  const tarName = nameList.find((name => name.endsWith(".tar")));
  if (!tarName) throw new Error(`expect tar in archive, get: ${nameList.join(", ")}`);
  await (0, _Directory.createDirectory)(outputPath);
  await (0, _npmTar.extractAsync)((0, _nodePath.resolve)(pathTemp, tarName), outputPath);
}), pathTemp);
exports.extractT7zAsync = extractT7zAsync;
const compressAutoAsync = async (sourceDirectory, outputFile, pathTemp) => _function.REGEXP_T7Z.test(outputFile) || _function.REGEXP_TXZ.test(outputFile) ? compressT7zAsync(sourceDirectory, outputFile, pathTemp) : (0, _Directory.createDirectory)((0, 
_nodePath.dirname)(outputFile)).then((() => (_npmTar.REGEXP_NPM_TAR.test(outputFile) ? _npmTar.compressAsync : _fsp.REGEXP_FSP.test(outputFile) ? _fsp.compressAsync : compress7zAsync)(sourceDirectory, outputFile)));
exports.compressAutoAsync = compressAutoAsync;
const extractAutoAsync = async (sourceFile, outputPath, pathTemp) => _function.REGEXP_T7Z.test(sourceFile) || _function.REGEXP_TXZ.test(sourceFile) ? extractT7zAsync(sourceFile, outputPath, pathTemp) : (0, _Directory.createDirectory)(outputPath).then((() => (_npmTar.REGEXP_NPM_TAR.test(sourceFile) ? _npmTar.extractAsync : _fsp.REGEXP_FSP.test(sourceFile) ? _fsp.extractAsync : extract7zAsync)(sourceFile, outputPath)));
exports.extractAutoAsync = extractAutoAsync;
exports.repackAsync = async (fileFrom, fileTo, pathTemp) => (0, _Directory.withTempDirectory)((async pathTemp => {
  await extractAutoAsync(fileFrom, pathTemp);
  await compressAutoAsync(pathTemp, fileTo);
}), pathTemp);
exports.repackTarAsync = async (fileFrom, fileTo, pathTemp) => (0, _Directory.withTempDirectory)((async pathTemp => {
  _function.REGEXP_TGZ.test(fileFrom) || _function.REGEXP_TBR.test(fileFrom) ? await (0, _function.extractGzBrFileAsync)(fileFrom, (0, _nodePath.resolve)(pathTemp, "file.tar")) : await extract7zAsync(fileFrom, pathTemp);
  if (_function.REGEXP_TGZ.test(fileTo) || _function.REGEXP_TBR.test(fileTo)) {
    const nameList = await _nodeFs.promises.readdir(pathTemp);
    const tarName = nameList.find((name => name.endsWith(".tar")));
    if (!tarName) throw new Error(`expect tar in archive, get: ${nameList.join(", ")}`);
    await (0, _function.compressGzBrFileAsync)((0, _nodePath.resolve)(pathTemp, tarName), fileTo);
  } else await compress7zAsync(pathTemp, fileTo);
}), pathTemp);