var _nodeFs = require("node:fs");
var _nodeZlib = require("node:zlib");
var _Stream = require("../../data/Stream.js");
exports.REGEXP_TGZ = /\.t(?:ar\.)?gz$/;
exports.REGEXP_TBR = /\.t(?:ar\.)?br$/;
exports.REGEXP_T7Z = /\.t(?:ar\.)?7z$/;
exports.REGEXP_TXZ = /\.t(?:ar\.)?xz$/;
const isBufferGzip = buffer => buffer && buffer.length > 3 && 31 === buffer[0] && 139 === buffer[1] && 8 === buffer[2];
exports.isBufferGzip = isBufferGzip;
exports.isFileGzip = async file => {
  const buffer = Buffer.allocUnsafe(3);
  await _nodeFs.promises.read(file, buffer, 0, 3, 0);
  return isBufferGzip(buffer);
};
const createGzipMax = () => (0, _nodeZlib.createGzip)({
  level: 9,
  windowBits: 15,
  memLevel: 9,
  chunkSize: 131072
});
exports.createGzipMax = createGzipMax;
const createBrotliCompressMax = () => (0, _nodeZlib.createBrotliCompress)({
  params: {
    [_nodeZlib.constants.BROTLI_PARAM_QUALITY]: 11
  },
  chunkSize: 131072
});
exports.createBrotliCompressMax = createBrotliCompressMax;
const REGEXP_GZ = exports.REGEXP_GZ = /\.t?gz$/;
exports.REGEXP_BR = /\.t?br$/;
exports.REGEXP_GZBR = /\.t?(?:gz|br)$/;
exports.compressGzBrFileAsync = async (fileFrom, fileTo, isGzip = REGEXP_GZ.test(fileTo)) => (0, _Stream.quickRunletFromStream)((0, _nodeFs.createReadStream)(fileFrom), isGzip ? createGzipMax() : createBrotliCompressMax(), (0, _nodeFs.createWriteStream)(fileTo));
exports.extractGzBrFileAsync = async (fileFrom, fileTo, isGzip = REGEXP_GZ.test(fileFrom)) => (0, _Stream.quickRunletFromStream)((0, _nodeFs.createReadStream)(fileFrom), isGzip ? (0, _nodeZlib.createGunzip)() : (0, _nodeZlib.createBrotliDecompress)(), (0, 
_nodeFs.createWriteStream)(fileTo));