var _nodePath = require("node:path");
var _nodeFs = require("node:fs");
var _error = require("../../common/error.js");
var _function = require("../../common/function.js");
var _function2 = require("../../common/data/function.js");
var _StateStore = require("../../common/immutable/StateStore.js");
var _Stream = require("../data/Stream.js");
var _File = require("../fs/File.js");
var _Directory = require("../fs/Directory.js");
var _Logger = require("./Logger.js");
var _Path = require("../fs/Path.js");
const INITIAL_STATE = {
  id: 0
};
const INITIAL_FACT_INFO = exports.INITIAL_FACT_INFO = {
  factId: 0,
  factState: INITIAL_STATE,
  factCacheFile: "",
  factLogFile: ""
};
exports.createFactDatabaseExot = ({id = "exot:fact-database", initialFactInfo, initialState = INITIAL_STATE, applyFact = (state, fact) => ({
  ...state,
  ...fact
}), encodeFact = JSON.stringify, decodeFact = JSON.parse, pathFactDirectory, nameFactLogFile = "factLog", nameFactCacheFile = "factCache", ...extraOption}) => {
  let isActive;
  let factId;
  let factLogger;
  let lossySaveFactCache;
  const {getState, setState, subscribe, unsubscribe} = (0, _StateStore.createStateStore)(INITIAL_STATE);
  return {
    id,
    up: async (onExotError = extraOption.onError) => {
      void 0 === initialFactInfo && (initialFactInfo = await tryLoadFactInfo({
        ...INITIAL_FACT_INFO,
        factState: initialState
      }, {
        applyFact,
        decodeFact,
        pathFactDirectory,
        nameFactLogFile,
        nameFactCacheFile
      }));
      setState(initialFactInfo.factState);
      factId = initialFactInfo.factId || 0;
      factLogger = (0, _Logger.createLoggerExot)({
        ...extraOption,
        pathLogDirectory: pathFactDirectory,
        getLogFileName: () => `${nameFactLogFile}.${factId + 1}.log`
      });
      await factLogger.up(onExotError);
      let prevFactCacheFile = initialFactInfo.factCacheFile || "";
      lossySaveFactCache = (0, _function.lossyAsync)((async () => {
        const factCacheFile = (0, _nodePath.join)(pathFactDirectory, `${nameFactCacheFile}.${factId}.json`);
        if (prevFactCacheFile !== factCacheFile) {
          await (0, _File.writeJSON)(factCacheFile, {
            factId,
            factState: getState()
          });
          prevFactCacheFile && await (0, _Path.deletePathForce)(prevFactCacheFile);
          prevFactCacheFile = factCacheFile;
        }
      }), onExotError);
      isActive = !0;
    },
    down: () => {
      if (!isActive) return;
      isActive = !1;
      factLogger.down();
      lossySaveFactCache.trigger();
      const runningPromise = lossySaveFactCache.getRunningPromise();
      factId = void 0;
      factLogger = void 0;
      lossySaveFactCache = void 0;
      return runningPromise;
    },
    isUp: () => isActive,
    getState,
    subscribe,
    unsubscribe,
    getSaveFactCachePromise: () => lossySaveFactCache && lossySaveFactCache.getRunningPromise(),
    add: fact => {
      if (isActive) {
        if (factId >= Number.MAX_SAFE_INTEGER) throw new Error(`factId is too big: ${factId}`);
        fact.id = factId + 1;
        factId++;
        factLogger.add(encodeFact(fact));
        setState(applyFact(getState(), fact));
      }
    },
    save: () => {
      isActive && factLogger.save();
    },
    split: () => {
      if (isActive) {
        factLogger.split();
        lossySaveFactCache.trigger();
      }
    }
  };
};
const tryLoadFactInfo = async (factInfo, {applyFact, decodeFact, pathFactDirectory, nameFactLogFile, nameFactCacheFile}) => {
  const factLogFileList = [];
  const factCacheFileList = [];
  const {error} = await (0, _error.catchAsync)((async () => (await (0, _Directory.getDirInfoList)(pathFactDirectory)).forEach((({name, path}) => {
    name.startsWith(nameFactLogFile) && REGEXP_LOG_FILE.test(name) && factLogFileList.push({
      fileId: parseInt(REGEXP_LOG_FILE.exec(name)[1]),
      name,
      path
    });
    name.startsWith(nameFactCacheFile) && REGEXP_CACHE_FILE.test(name) && factCacheFileList.push({
      fileId: parseInt(REGEXP_CACHE_FILE.exec(name)[1]),
      name,
      path
    });
  }))));
  factInfo = await tryLoadFactInfoFromCache(factInfo, {
    factCacheFileList
  });
  return await tryLoadFactInfoFromLog(factInfo, {
    factLogFileList,
    decodeFact,
    applyFact
  });
};
exports.tryLoadFactInfo = tryLoadFactInfo;
const REGEXP_LOG_FILE = /\.(\d+)\.log$/;
const REGEXP_CACHE_FILE = /\.(\d+)\.json$/;
const tryLoadFactInfoFromCache = async (factInfo, {factCacheFileList}) => {
  factCacheFileList.sort(((a, b) => b.fileId - a.fileId));
  for (const {name, path} of factCacheFileList) {
    const {factId, factState} = (0, _function2.tryParseJSONObject)(await (0, _File.readText)(path));
    if (factId) return {
      ...factInfo,
      factId,
      factState,
      factCacheFile: path
    };
  }
  return factInfo;
};
const tryLoadFactInfoFromLog = async (factInfo, {factLogFileList, decodeFact, applyFact}) => {
  let {factId, factState, factLogFile = ""} = factInfo;
  const maxFactLogId = factLogFileList.reduce(((o, {fileId}) => fileId <= factId + 1 ? Math.max(o, fileId) : o), 0);
  factLogFileList = factLogFileList.filter((({fileId}) => fileId >= maxFactLogId)).sort(((a, b) => a.fileId - b.fileId));
  for (const {name, path} of factLogFileList) await (0, _Stream.readlineOfStreamAsync)((0, _nodeFs.createReadStream)(path), (logText => {
    const fact = logText && decodeFact(logText);
    if (fact && !(fact.id <= factId)) {
      if (fact.id !== factId + 1) throw new Error(`invalid factId: ${fact.id}, should be: ${factId + 1}. file: ${name}`);
      factState = applyFact(factState, fact);
      factId = fact.id;
      factLogFile = path;
    }
  }));
  return {
    ...factInfo,
    factId,
    factState,
    factLogFile
  };
};
exports.tryDeleteExtraCache = async ({pathFactDirectory, nameFactCacheFile = "factCache", keepFactId = 1 / 0, keepFileCount = 2}) => {
  const factCacheFileList = (await (0, _Directory.getDirInfoList)(pathFactDirectory)).map((({name, path}) => {
    const fileId = name.startsWith(nameFactCacheFile) && REGEXP_CACHE_FILE.test(name) && parseInt(REGEXP_CACHE_FILE.exec(name)[1]);
    return Number.isInteger(fileId) && {
      fileId,
      name,
      path
    };
  })).filter(Boolean);
  if (!factCacheFileList.length) return;
  factCacheFileList.sort(((a, b) => b.fileId - a.fileId));
  let skippedFile = 0;
  for (const {fileId, name, path} of factCacheFileList) fileId >= keepFactId || (skippedFile < keepFileCount ? skippedFile += 1 : await (0, _Path.deletePathForce)(path));
};