var _nodePath = require("node:path");
var _nodeFs = require("node:fs");
var _error = require("../../common/error.js");
var _random = require("../../common/math/random.js");
var _CacheMap = require("../../common/data/CacheMap.js");
var _ArrayBuffer = require("../../common/data/ArrayBuffer.js");
var _AsyncFuncQueue = require("../../common/module/AsyncFuncQueue.js");
var _ChunkUpload = require("../../common/module/ChunkUpload.js");
var _Stream = require("../data/Stream.js");
var _File = require("../fs/File.js");
var _Path = require("../fs/Path.js");
var _Directory = require("../fs/Directory.js");
var _Modify = require("../fs/Modify.js");
exports.createOnFileChunkUpload = async ({rootPath, mergePath, onError = _error.rethrowError, allowSkipHashVerify = !0, expireTime = 6e5, chunkCacheMap = (0, _CacheMap.createCacheMap)({
  valueSizeSumMax: 64,
  expireAfter: expireTime
})}) => {
  await (0, _Directory.createDirectory)(rootPath);
  await (0, _Directory.createDirectory)(mergePath);
  const getPath = (0, _Path.createPathPrefixLock)(rootPath);
  const {push} = (0, _AsyncFuncQueue.createAsyncFuncQueue)(onError);
  chunkCacheMap.subscribe((({type, key, payload}) => {
    if ("delete" !== type) return;
    const {tempPath} = payload;
    push((() => (0, _Modify.modifyDeleteForce)(tempPath)));
  }));
  return async ({bufferPacket, cacheKeyPrefix = "", onUploadStart, onUploadChunk, onUploadEnd}) => {
    const {chunkArrayBuffer, key, chunkIndex, chunkTotal} = await (0, _ChunkUpload.parseArrayBufferChunk)((0, _ArrayBuffer.fromNodejsBuffer)(bufferPacket), allowSkipHashVerify);
    const cacheKey = `${cacheKeyPrefix}-${key}-${chunkTotal}`;
    let chunkData = chunkCacheMap.get(cacheKey);
    if (void 0 === chunkData) {
      const filePath = getPath(key);
      const tempPath = (0, _nodePath.resolve)(mergePath, (0, _random.getRandomId)(cacheKey).replace(/[^\w-.]/g, "_"));
      chunkData = {
        tempPath,
        filePath,
        key,
        chunkTotal,
        chunkList: []
      };
      onUploadStart && await onUploadStart(chunkData);
      await (0, _Directory.createDirectory)(tempPath);
    }
    const chunkPath = (0, _nodePath.resolve)(chunkData.tempPath, `chunk-${chunkIndex}-${chunkTotal}`);
    await (0, _File.writeBuffer)(chunkPath, Buffer.from(chunkArrayBuffer));
    chunkData.chunkList[chunkIndex] = {
      chunkIndex,
      chunkPath
    };
    onUploadChunk && await onUploadChunk(chunkData, chunkIndex);
    const chunkCacheCount = Object.keys(chunkData.chunkList).length;
    if (chunkCacheCount === chunkTotal) {
      await (0, _Directory.createDirectory)((0, _nodePath.dirname)(chunkData.filePath));
      await (0, _File.writeBuffer)(chunkData.filePath, "");
      for (const {chunkPath} of chunkData.chunkList) await (0, _Stream.quickRunletFromStream)((0, _nodeFs.createReadStream)(chunkPath), (0, _nodeFs.createWriteStream)(chunkData.filePath, {
        flags: "a"
      }));
      await (0, _Modify.modifyDelete)(chunkData.tempPath);
      chunkCacheMap.delete(cacheKey);
      onUploadEnd && await onUploadEnd(chunkData);
    } else chunkCacheCount > 1 ? chunkCacheMap.touch(cacheKey) : chunkCacheMap.set(cacheKey, chunkData);
  };
};
exports.uploadFileByChunk = async ({fileBuffer, filePath, fileSize, key, chunkSizeMax, uploadChunk, onProgress}) => {
  void 0 === fileSize && (fileSize = fileBuffer ? fileBuffer.length : (await _nodeFs.promises.stat(filePath)).size);
  const fileHandle = fileBuffer ? void 0 : await _nodeFs.promises.open(filePath);
  const getChunk = fileBuffer ? (index, chunkSize) => (0, _ArrayBuffer.fromNodejsBuffer)(fileBuffer.slice(index, index + chunkSize)) : async (index, chunkSize) => {
    const buffer = Buffer.allocUnsafe(chunkSize);
    await fileHandle.read(buffer, 0, chunkSize, index);
    return (0, _ArrayBuffer.fromNodejsBuffer)(buffer);
  };
  await (0, _ChunkUpload.uploadArrayBufferByChunk)({
    size: fileSize,
    getChunk,
    key,
    chunkSizeMax,
    uploadChunk,
    onProgress
  });
  fileHandle && await fileHandle.close();
};