var _nodePath = require("node:path");
var _Directory = require("../fs/Directory.js");
var _SafeWrite = require("./SafeWrite.js");
const createSimpleLoggerExot = ({id = "exot:logger-simple", queueLengthThreshold = 1024, ...safeWriteStreamOption}) => {
  const logQueue = [];
  let safeWriteStream;
  const isUp = () => void 0 !== safeWriteStream;
  const save = () => {
    if (!1 !== isUp() && 0 !== logQueue.length) {
      logQueue.push("");
      safeWriteStream.write(logQueue.join("\n"));
      logQueue.length = 0;
    }
  };
  return {
    id,
    up: onExotError => {
      if (!0 === isUp()) throw new Error("already up");
      safeWriteStream = (0, _SafeWrite.createSafeWriteStream)({
        onError: onExotError,
        ...safeWriteStreamOption
      });
    },
    down: () => {
      if (!1 !== isUp()) {
        save();
        safeWriteStream.end();
        safeWriteStream = void 0;
      }
    },
    isUp,
    add: logString => {
      logQueue.push(logString);
      logQueue.length > queueLengthThreshold && save();
    },
    save
  };
};
exports.createSimpleLoggerExot = createSimpleLoggerExot;
exports.createLoggerExot = ({id = "exot:logger", pathLogDirectory, getLogFileName = () => `${(new Date).toISOString().replace(/\W/g, "-")}.log`, saveInterval = 3e4, splitInterval = 864e5, ...simpleLoggerOption}) => {
  const getLoggerExot = () => createSimpleLoggerExot({
    ...simpleLoggerOption,
    pathOutputFile: (0, _nodePath.resolve)(pathLogDirectory, getLogFileName())
  });
  let loggerExot = getLoggerExot();
  let saveToken;
  let splitToken;
  const upSync = onExotError => {
    loggerExot.up(onExotError);
    saveToken = saveInterval ? setInterval(save, saveInterval) : void 0;
    splitToken = splitInterval ? setTimeout(split, splitInterval) : void 0;
  };
  const down = () => {
    if (!1 !== isUp()) {
      loggerExot.down();
      void 0 !== saveToken && clearInterval(saveToken);
      void 0 !== splitToken && clearTimeout(splitToken);
      loggerExot = getLoggerExot();
      saveToken = void 0;
      splitToken = void 0;
    }
  };
  const isUp = () => loggerExot.isUp();
  const save = () => {
    loggerExot.save();
  };
  const split = () => {
    if (!1 !== isUp()) {
      down();
      upSync();
    }
  };
  return {
    id,
    up: async onExotError => {
      if (!0 === isUp()) throw new Error("already up");
      await (0, _Directory.createDirectory)(pathLogDirectory);
      upSync(onExotError);
    },
    down,
    isUp,
    add: (...args) => {
      loggerExot.add(args.join(" "));
    },
    save,
    split
  };
};