var _nodePath = require("node:path");
var _tryRequire = require("../../../env/tryRequire.js");
var _string = require("../../../common/string.js");
var _verify = require("../../../common/verify.js");
var _function = require("../../../common/data/function.js");
var _Object = require("../../../common/immutable/Object.js");
var _Array = require("../../../common/immutable/Array.js");
var _Z64String = require("../../data/Z64String.js");
var _parser = require("./parser.js");
const throwError = message => {
  throw new Error(message);
};
const filterFormatValue = value => void 0 !== value && "" !== value;
const getPreset = (argumentCount, argumentListVerify, argumentListNormalize, description, optional) => (0, _Object.objectFilter)({
  argumentCount,
  argumentListNormalize,
  argumentListVerify,
  description,
  optional
}, filterFormatValue);
const Preset = exports.Preset = {
  Optional: {
    optional: !0
  },
  Path: {
    isPath: !0
  },
  Any: getPreset("0-", void 0, void 0, "any", !0),
  Toggle: getPreset("0-1", void 0, (argumentList => "false/no/n/0".split("/").includes(String(argumentList[0]).toLowerCase()) ? [ !1 ] : [ !0 ]), 'set to ANY value to enable, except "false/no/n/0"', !0)
};
Object.assign(Preset, ...[ [ "String", _verify.string, argumentList => argumentList.map(String) ], [ "Number", _verify.number, argumentList => argumentList.map(Number) ], [ "Boolean", _verify.boolean, argumentList => argumentList.map(Boolean) ], [ "Integer", _verify.integer, argumentList => argumentList.map(parseInt) ], [ "RegExp", _verify.regexp, void 0 ], [ "Object", _verify.basicObject, void 0 ], [ "Function", _verify.basicFunction, void 0 ] ].map((([typeName, verifyFunc, normalizeFunc]) => ({
  [`Single${typeName}`]: getPreset(1, (argumentList => {
    (0, _verify.arrayLength)(argumentList, 1);
    verifyFunc(argumentList[0], `expect ${typeName}`);
  }), normalizeFunc),
  [`All${typeName}`]: getPreset("1-", (argumentList => argumentList.forEach(((v, i) => verifyFunc(v, `expect ${typeName} at #${i}`)))), normalizeFunc)
}))));
const parseCompact = (compactFormat, extraOptionOrExtendFormatList) => {
  const [compactTag, ...descriptionList] = compactFormat.split("|");
  const [nameTag, presetTag = "", argumentCount] = compactTag.split(/\s*[\s/]\s*/);
  const nameTagList = nameTag.split(",");
  const presetList = presetTag.split(",");
  return Object.assign({}, ...presetList.map((presetName => {
    presetName && !Preset[presetName] && throwError(`invalid presetName: ${presetName}`);
    return Preset[presetName];
  })).filter(Boolean), (0, _Object.objectFilter)({
    name: nameTagList[0],
    shortName: nameTagList.find((nameTag => 1 === nameTag.length)),
    aliasNameList: nameTagList.length > 1 ? nameTagList.slice(1) : void 0,
    argumentCount,
    description: descriptionList.join("|")
  }, filterFormatValue), Array.isArray(extraOptionOrExtendFormatList) ? {
    extendFormatList: extraOptionOrExtendFormatList
  } : extraOptionOrExtendFormatList || {});
};
Object.assign(Preset, {
  SinglePath: parseCompact("/SingleString,Path"),
  AllPath: parseCompact("/AllString,Path"),
  Config: parseCompact([ 'config,c/SingleString,Optional|from JS/JSON: set to "path/to/config.js|json"', 'from ENV: set to "env" to enable, default not check env', 'from ENV JSON: set to "json-env:ENV_NAME" to read the ENV string as JSON, or "jz64/jb64-env"', 'from CLI JSON: set to "json-cli:JSON_STRING" to read the appended string as JSON, or "jz64/jb64-cli"' ].join("\n")),
  pickOneOf: (selectList, extraDescription = "") => {
    selectList.length <= 2 && throwError(`expect more to pick: ${selectList}`);
    return getPreset(1, (argumentList => {
      (0, _verify.arrayLength)(argumentList, 1);
      (0, _verify.oneOf)(argumentList[0], selectList);
    }), void 0, `${extraDescription}one of:\n${(0, _string.indentLineList)((0, _Array.arraySplitChunk)(selectList, 4).map((v => v.join(" "))))}`);
  },
  parseCompact,
  parseCompactList: (...args) => args.map((compactFormat => Array.isArray(compactFormat) ? parseCompact(...compactFormat) : parseCompact(compactFormat)))
});
Object.entries(Preset).forEach((([key, value]) => {
  const compactName = (0, _string.splitCamelCase)(key).map((string => string.charAt(0))).join("");
  Preset[compactName] = value;
}));
exports.getOptionalFormatFlag = (...formatNameList) => optionMap => !formatNameList.some((formatName => Boolean(optionMap[formatName])));
exports.getOptionalFormatValue = (formatName, ...valueList) => optionMap => {
  const format = optionMap[formatName];
  return format && !valueList.includes(format.argumentList[0]);
};
const parseOptionMap = async ({parseCLI, parseENV, parseCONFIG, processOptionMap, optionCLI = process.argv.slice(2), optionENV = process.env}) => {
  const optionMapCLI = optionMapResolvePathMutate(parseCLI(optionCLI));
  const [baseString, appendString] = splitConfigString(optionMapCLI.config && optionMapCLI.config.argumentList[0]);
  return processOptionMap({
    ...baseString ? "env" === baseString ? optionMapResolvePathMutate(parseENV(optionENV)) : /^j(son|z64|b64)-env$/.test(baseString) ? optionMapResolvePathMutate(parseCONFIG(tryParseObjectStringAlike(optionENV[appendString], baseString) || throwError(`failed to load config: ${baseString}`))) : /^j(son|z64|b64)-cli$/.test(baseString) ? optionMapResolvePathMutate(parseCONFIG(tryParseObjectStringAlike(appendString, baseString) || throwError(`failed to load config: ${baseString}`))) : optionMapResolvePathMutate(parseCONFIG((0, 
    _tryRequire.tryRequire)((0, _nodePath.resolve)(baseString)) || throwError(`failed to load config: ${baseString}`)), (0, _nodePath.dirname)((0, _nodePath.resolve)(baseString))) : null,
    ...optionMapCLI
  });
};
exports.parseOptionMap = parseOptionMap;
const splitConfigString = configString => {
  const [baseString, ...appendList] = configString ? /^j(son|z64|b64)-env:/.test(configString) || /^j(son|z64|b64)-cli:/.test(configString) ? configString.split(":") : [ configString ] : [];
  return [ baseString, appendList.join(":") ];
};
const optionMapResolvePathMutate = (optionMap, pathRoot = process.cwd()) => {
  Object.values(optionMap).forEach((({format: {isPath}, argumentList}) => isPath && argumentList.forEach(((v, i) => argumentList[i] = (0, _nodePath.resolve)(pathRoot, v)))));
  return optionMap;
};
const tryParseObjectStringAlike = (string, baseString) => {
  try {
    let jsonString = string;
    "6" === baseString.charAt(2) && (jsonString = ("z" === baseString.charAt(1) ? _Z64String.unpackGz64 : _Z64String.unpackBr64)(string));
    return (0, _function.tryParseJSONObject)(jsonString, null);
  } catch (error) {}
};
const createOptionGetter = optionMap => {
  const tryGet = name => optionMap[name] && optionMap[name].argumentList;
  const tryGetFirst = name => optionMap[name] && optionMap[name].argumentList[0];
  const get = name => tryGet(name) || throwError(`expect option ${name}`);
  const getFirst = name => get(name)[0];
  return {
    optionMap,
    tryGet,
    tryGetFirst,
    get,
    getFirst,
    getToggle: name => Boolean(tryGetFirst(name)),
    pwd: name => {
      const pathValue = optionMap[name] && (optionMap[name].format.isPath ? getFirst(name) : "CONFIG" === optionMap[name].source && !/^env|j(son|z64|b64)-(env|cli)$/.test(splitConfigString(getFirst("config"))[0]) && getFirst("config"));
      return pathValue ? (0, _nodePath.dirname)(pathValue) : "";
    }
  };
};
exports.createOptionGetter = createOptionGetter;
exports.prepareOption = optionConfig => {
  const {parseCLI, parseENV, parseCONFIG, processOptionMap, formatUsage} = (0, _parser.createOptionParser)(optionConfig);
  return {
    parseOption: async ({optionCLI, optionENV} = {}) => createOptionGetter(await parseOptionMap({
      parseCLI,
      parseENV,
      parseCONFIG,
      processOptionMap,
      optionCLI,
      optionENV
    })),
    formatUsage
  };
};