var _nodeFs = require("node:fs");
var _error = require("../../common/error.js");
exports.createSafeWriteStream = ({pathOutputFile, flag = "w", mode = 438, fileDescriptor = (0, _nodeFs.openSync)(pathOutputFile, flag, mode), onError = _error.rethrowError}) => {
  let writingQueue = [];
  let pendingQueue = [];
  const onWriteDone = error => error ? onError(error) : writingQueue.length = 0;
  const doWrite = string => void 0 !== fileDescriptor && (0, _nodeFs.write)(fileDescriptor, string, onWriteDone);
  return {
    write: string => {
      if (string) {
        pendingQueue.push(string);
        if (0 === writingQueue.length) {
          [writingQueue, pendingQueue] = [ pendingQueue, [] ];
          process.nextTick(doWrite, writingQueue.join(""));
        }
      }
    },
    end: () => {
      const remainQueue = [ ...writingQueue, ...pendingQueue ];
      0 !== remainQueue.length && (0, _nodeFs.writeSync)(fileDescriptor, remainQueue.join(""));
      (0, _nodeFs.closeSync)(fileDescriptor);
      fileDescriptor = void 0;
      pendingQueue.length = writingQueue.length = 0;
    }
  };
};