var _error = require("../../../common/error.js");
var _run = require("../../run.js");
var _function = require("../function.js");
const {getArgs, setArgs, check, verify} = (0, _function.createArgListPack)((() => (0, _function.probeSync)([ "docker", "version" ], "Server:") ? [ "docker" ] : (0, _function.probeSync)([ "sudo", "docker", "version" ], "Server:") ? [ "sudo", "docker" ] : void 0), 'expect "docker" in PATH, with dockerd up');
exports.verify = verify;
exports.check = check;
exports.setArgs = setArgs;
exports.getArgs = getArgs;
exports.runDocker = (argList = [], option) => (0, _run.run)([ ...verify(), ...argList ], option);
exports.runDockerSync = (argList = [], option) => (0, _run.runSync)([ ...verify(), ...argList ], option);
const runDockerStdout = (argList = [], option) => (0, _run.runStdout)([ ...verify(), ...argList ], option);
exports.runDockerStdout = runDockerStdout;
exports.runDockerStdoutSync = (argList = [], option) => (0, _run.runStdoutSync)([ ...verify(), ...argList ], option);
exports.parseDockerImage = (image = "") => {
  if (image.toLowerCase().includes("@sha256:")) throw new Error(`expect tag without digest: ${image}`);
  const stubList = image.split(":");
  if (stubList.length <= 1) throw new Error(`expect explicit tag: ${image}`);
  const tag = stubList.pop();
  if (!/^\w[\w-.]+$/.test(tag)) throw new Error(`invalid tag: ${image}`);
  return [ stubList.join(":"), tag ];
};
const checkLocalImage = async (imageRepo, imageTag) => {
  const stdoutString = String(await runDockerStdout([ "image", "ls", `${imageRepo}:${imageTag}` ]));
  return stdoutString.includes(imageRepo) && stdoutString.includes(imageTag);
};
exports.checkLocalImage = checkLocalImage;
const pullImage = async (imageRepo, imageTag) => runDockerStdout([ "pull", `${imageRepo}:${imageTag}` ]);
exports.pullImage = pullImage;
exports.checkPullImage = async (imageRepo, imageTag) => {
  if (await checkLocalImage(imageRepo, imageTag)) return !0;
  await (0, _error.catchAsync)(pullImage, imageRepo, imageTag);
  return checkLocalImage(imageRepo, imageTag);
};
exports.getContainerLsList = async (isListAll = !1) => String(await runDockerStdout([ "container", "ls", "--format", "{{.ID}}|{{.Image}}|{{.Names}}", isListAll && "--all" ].filter(Boolean))).trim().split("\n").filter(Boolean).map((string => {
  const [id, image, names] = string.split("|");
  return {
    id,
    image,
    names
  };
}));
exports.patchContainerLsListStartedAt = async (containerLsList = []) => {
  const idList = containerLsList.map((({id}) => id));
  String(await runDockerStdout([ "container", "inspect", "--format", "{{.Id}}|{{.State.StartedAt}}", ...idList ])).trim().split("\n").filter(Boolean).forEach((string => {
    const [id, startedAtString] = string.split("|");
    const item = containerLsList[idList.findIndex((v => id.startsWith(v)))];
    item && (item.startedAt = new Date(startedAtString));
  }));
};
exports.matchContainerLsList = (containerLsList = [], processList = []) => {
  containerLsList.forEach((item => {
    item.pid = (processList.find((({command}) => command.includes(item.id))) || {}).pid;
  }));
};
const {getArgs: getArgsCompose, setArgs: setArgsCompose, check: checkCompose, verify: verifyCompose} = (0, _function.createArgListPack)((() => {
  if (!check()) return;
  const argsListP = [ ...getArgs(), "compose" ];
  if ((0, _function.probeSync)([ ...argsListP, "version" ], "ompose version")) return argsListP;
  const argsList = [ ...getArgs().slice(0, -1), "docker-compose" ];
  return (0, _function.probeSync)([ ...argsList, "version" ], "ompose version") ? argsList : void 0;
}), 'expect both "docker" and "docker compose|docker-compose" in PATH, with dockerd up');
exports.verifyCompose = verifyCompose;
exports.checkCompose = checkCompose;
exports.setArgsCompose = setArgsCompose;
exports.getArgsCompose = getArgsCompose;
exports.runCompose = (argList = [], option) => (0, _run.run)([ ...verifyCompose(), ...argList ], option);
exports.runComposeSync = (argList = [], option) => (0, _run.runSync)([ ...verifyCompose(), ...argList ], option);
exports.runComposeStdout = (argList = [], option) => (0, _run.runStdout)([ ...verifyCompose(), ...argList ], option);
exports.runComposeStdoutSync = (argList = [], option) => (0, _run.runStdoutSync)([ ...verifyCompose(), ...argList ], option);