var _format = require("../../../common/format.js");
var _check = require("../../../common/check.js");
var _run = require("../../run.js");
var _Status = require("../../system/Status.js");
const IS_WIN32 = "win32" === process.platform;
const IS_DARWIN = "darwin" === process.platform;
const COMMON_HOST_STATUS_COMMAND_LIST = exports.COMMON_HOST_STATUS_COMMAND_LIST = [ [ "Disk", async rootPath => {
  if (IS_WIN32) {
    const freeByteString = (await runQuick("cmd.exe /C dir", rootPath)).match(/ ([\d,]+) bytes free/)[1].replace(/\D/g, "");
    return `${(0, _format.binary)(Number(freeByteString))}B free storage`;
  }
  {
    const diskStatus = await runQuick("df -h .", rootPath);
    return diskStatus.includes("/dev/") ? [ diskStatus, "Usage", await runQuick("du -hd1", rootPath) ].join("\n") : diskStatus;
  }
} ], !IS_WIN32 && [ "Network", "vnstat -s" ], [ "System", IS_DARWIN ? "top -l1 -n0" : async rootPath => (await runQuick("top -bn1", rootPath)).split("\n").slice(0, 5).join("\n"), () => (0, _Status.describeSystemStatus)() ], [ "Time", () => (new Date).toISOString() ] ].filter(Boolean);
const runQuick = async (command, rootPath) => String(await (0, _run.runStdout)(command.split(" "), {
  cwd: rootPath
}));
const runStatusCommand = async (statusCommand, rootPath) => {
  let output = "";
  (0, _check.isString)(statusCommand) ? output = await runQuick(statusCommand, rootPath) : (0, _check.isBasicFunction)(statusCommand) && (output = await statusCommand(rootPath));
  return output;
};
exports.getCommonHostStatus = async (rootPath, statusCommandList = COMMON_HOST_STATUS_COMMAND_LIST) => {
  const resultList = [];
  for (const [title, ...tryList] of statusCommandList) {
    let output = "";
    for (const statusCommand of tryList) {
      output = await runStatusCommand(statusCommand, rootPath).catch((() => ""));
      if (output) break;
    }
    resultList.push([ title, output ]);
  }
  return resultList;
};