var _nodePath = require("node:path");
var _nodeFs = require("node:fs");
var _tryRequire = require("../../../env/tryRequire.js");
var _SemVer = require("../../../common/module/SemVer.js");
var _ResolveCommand = require("../../system/ResolveCommand.js");
var _net = require("../../net.js");
var _run = require("../../run.js");
var _function = require("../function.js");
exports.findUpPackageRoot = (path = __dirname) => {
  path = (0, _nodePath.resolve)(path);
  let prevPath;
  for (;path !== prevPath; ) {
    if ((0, _nodeFs.existsSync)((0, _nodePath.resolve)(path, "package.json"))) return path;
    prevPath = path;
    path = (0, _nodePath.resolve)(path, "..");
  }
};
exports.findUpNpmrc = (path = __dirname) => {
  path = (0, _nodePath.resolve)(path);
  let prevPath;
  for (;path !== prevPath; ) {
    const npmrcPath = (0, _nodePath.resolve)(path, ".npmrc");
    if ((0, _nodeFs.existsSync)(npmrcPath)) return npmrcPath;
    prevPath = path;
    path = (0, _nodePath.resolve)(path, "..");
  }
};
let cachePathNpmExecutable;
const getPathNpmExecutable = () => {
  if (void 0 === cachePathNpmExecutable) {
    const command = (0, _ResolveCommand.resolveCommandName)("npm");
    cachePathNpmExecutable = command && (0, _nodeFs.realpathSync)(command);
  }
  return cachePathNpmExecutable;
};
exports.getPathNpmExecutable = getPathNpmExecutable;
const getSudoArgs = () => "win32" === process.platform ? [ getPathNpmExecutable() ] : [ "sudo", getPathNpmExecutable() ];
exports.getSudoArgs = getSudoArgs;
exports.runNpm = (argList = [], option) => (0, _run.run)([ getPathNpmExecutable(), ...argList ], option);
exports.runNpmSync = (argList = [], option) => (0, _run.runSync)([ getPathNpmExecutable(), ...argList ], option);
exports.runNpmStdout = (argList = [], option) => (0, _run.runStdout)([ getPathNpmExecutable(), ...argList ], option);
exports.runNpmStdoutSync = (argList = [], option) => (0, _run.runStdoutSync)([ getPathNpmExecutable(), ...argList ], option);
exports.runSudoNpm = (argList = [], option) => (0, _run.run)([ ...getSudoArgs(), ...argList ], option);
exports.runSudoNpmSync = (argList = [], option) => (0, _run.runSync)([ ...getSudoArgs(), ...argList ], option);
exports.runSudoNpmStdout = (argList = [], option) => (0, _run.runStdout)([ ...getSudoArgs(), ...argList ], option);
exports.runSudoNpmStdoutSync = (argList = [], option) => (0, _run.runStdoutSync)([ ...getSudoArgs(), ...argList ], option);
let cachePathNpmGlobalRoot;
const getPathNpmGlobalRoot = () => {
  void 0 === cachePathNpmGlobalRoot && (cachePathNpmGlobalRoot = getPathNpmExecutable() && (0, _function.spawnString)([ getPathNpmExecutable(), "root", "-g" ]).trim());
  return cachePathNpmGlobalRoot;
};
exports.getPathNpmGlobalRoot = getPathNpmGlobalRoot;
exports.fromGlobalNodeModules = (...args) => (0, _nodePath.resolve)(getPathNpmGlobalRoot(), ...args);
let cachePathNpm;
const getPathNpm = () => {
  if (void 0 === cachePathNpm) {
    {
      const npmHelpText = (0, _function.spawnString)([ getPathNpmExecutable() ]).trim();
      const npmHelpLastLine = npmHelpText && npmHelpText.split("\n").pop();
      const npmFullPath = npmHelpLastLine && npmHelpLastLine.split(" ").pop();
      npmFullPath && (cachePathNpm = npmFullPath);
    }
    const tryPath = (pathFrom, relativeNpm) => {
      try {
        (0, _nodeFs.statSync)((0, _nodePath.resolve)(pathFrom, relativeNpm, "./node_modules/semver/")).isDirectory() && (cachePathNpm = (0, _nodePath.resolve)(pathFrom, relativeNpm));
      } catch (error) {}
    };
    !cachePathNpm && getPathNpmExecutable().endsWith("npm-cli.js") && tryPath(getPathNpmExecutable(), "../../");
    !cachePathNpm && tryPath(getPathNpmGlobalRoot(), "./npm/");
    !cachePathNpm && getPathNpmExecutable().endsWith("npm.cmd") && tryPath(getPathNpmExecutable(), "../node_modules/npm/");
    cachePathNpm || (cachePathNpm = "");
  }
  return cachePathNpm;
};
exports.getPathNpm = getPathNpm;
const fromNpmNodeModules = (...args) => getPathNpm() && (0, _nodePath.resolve)(getPathNpm(), "./node_modules/", ...args);
exports.fromNpmNodeModules = fromNpmNodeModules;
exports.hasRepoVersion = async (packageName, packageVersion, cwd = process.cwd()) => {
  try {
    return JSON.parse(String(await (0, _run.runStdout)([ getPathNpmExecutable(), "view", `${packageName}@${packageVersion}`, "versions", "--json" ], {
      cwd
    }))).includes(packageVersion);
  } catch (error) {
    return !1;
  }
};
const fetchLikeRequestWithProxy = (url, option = {}) => {
  const npmVer = (0, _tryRequire.tryRequire)(fromNpmNodeModules("../package.json")).version;
  (0, _tryRequire.tryRequire)("node:https").request.__agent_base_https_request_patched__ = !0;
  return (0, _net.fetchLikeRequest)(url, {
    ...option,
    agent: ((0, _SemVer.compareSemVer)(npmVer, "10.0.0") < 0 ? (0, _tryRequire.tryRequire)(fromNpmNodeModules((0, _SemVer.compareSemVer)(npmVer, "7.16.0") < 0 ? "make-fetch-happen/agent" : "make-fetch-happen/lib/agent")) : (0, _tryRequire.tryRequire)(fromNpmNodeModules("@npmcli/agent/lib")).getAgent)(url, {
      dns: {
        ttl: 3e5,
        lookup: (0, _tryRequire.tryRequire)("node:dns").lookup
      },
      ...option
    }),
    secureEndpoint: "https:" === new URL(url).protocol
  });
};
exports.fetchLikeRequestWithProxy = fetchLikeRequestWithProxy;
exports.fetchWithJumpProxy = (initialUrl, option) => (0, _net.fetchWithJump)(initialUrl, {
  fetch: fetchLikeRequestWithProxy,
  ...option
});