var _nodeReadline = require("node:readline");
var _function = require("../../common/function.js");
exports.createColor = ({isEnableColor = process.stdout.isTTY && process.stdout.hasColors() || process.env.CI} = {}) => {
  const toAES = value => `[${value}m`;
  const createWrapper = isEnableColor ? (setAES, clearAES) => text => `${setAES}${text}${clearAES}` : () => text => text;
  const wrapperFg = {};
  const wrapperBg = {};
  "\nblack|30|40\nred|31|41\ngreen|32|42\nyellow|33|43\nblue|34|44\nmagenta|35|45\ncyan|36|46\nlightGray|37|47\n\ndefault|39|49\n\ndarkGray|90|100\nlightRed|91|101\nlightGreen|92|102\nlightYellow|93|103\nlightBlue|94|104\nlightMagenta|95|105\nlightCyan|96|106\nwhite|97|107\n".split("\n").filter(Boolean).forEach((controlSequenceText => {
    const [name, colorFg, colorBg] = controlSequenceText.split("|");
    wrapperFg[name] = createWrapper(toAES(colorFg), toAES(39));
    wrapperBg[name] = createWrapper(toAES(colorBg), toAES(49));
  }));
  return {
    fg: wrapperFg,
    bg: wrapperBg
  };
};
exports.createStatusBar = ({stream = process.stdout, throttleWait = 200, trimOverflow = !0} = {}) => {
  if (!stream.isTTY) return {
    update: () => {},
    done: () => {}
  };
  let isDone = !1;
  let nextText = "";
  let prevText = "";
  const tickFunc = () => {
    if (!isDone && nextText !== prevText) {
      stream.clearLine(0);
      stream.cursorTo(0);
      stream.write(trimOverflow ? nextText.slice(0, stream.columns) : nextText);
      prevText = nextText;
    }
  };
  const tick = throttleWait ? (0, _function.throttleT)(tickFunc, throttleWait) : tickFunc;
  return {
    update: text => {
      nextText = text;
      tick();
    },
    done: () => {
      isDone = !0;
      stream.clearLine(0);
      stream.cursorTo(0);
    }
  };
};
exports.promptAsync = async ({message = "", defaultAnswer}) => {
  const readlineInterface = (0, _nodeReadline.createInterface)({
    input: process.stdin,
    output: process.stdout
  });
  const answer = await new Promise((resolve => {
    readlineInterface.on("close", (() => resolve(defaultAnswer)));
    readlineInterface.question(message, (answer => resolve(answer)));
  }));
  readlineInterface.close();
  return answer;
};