var _nodeFs = require("node:fs");
var _nodeChild_process = require("node:child_process");
var _error = require("../common/error.js");
var _Stream = require("./data/Stream.js");
const patchWin32CmdBat = (command, argList) => /.\.(cmd|ext)$/.test(command) ? [ "cmd.exe", [ "/C", command, ...argList ] ] : [ command, argList ];
const getOption = (option, quiet) => ({
  stdio: quiet ? [ "ignore", "pipe", "pipe" ] : "inherit",
  ...option
});
const toRunError = (error, exitData) => Object.assign(error || new Error(`exit with code: ${exitData.code}, signal: ${exitData.signal}`), exitData);
const describeRunOutcome = data => data.describeOutcome || (data.stdoutPromise ? Promise.all([ data.stdoutPromise, data.stderrPromise ]).then((([stdout, stderr]) => describeRunOutcomeSync(Object.assign(data, {
  stdout,
  stderr
})))) : describeRunOutcomeSync(data));
exports.describeRunOutcome = describeRunOutcome;
const describeRunOutcomeSync = data => {
  if (void 0 === data.describeOutcome) {
    const {message, stack, code, signal, command, argList = [], stdout, stderr} = data;
    data.describeOutcome = [ stack || message, void 0 !== signal && `[code] ${code} [signal] ${signal}`, `[args] ${quote(command)} ${argList.map(quote).join(" ")}`, stdout && stdout.length && `[stdout] ${stdout}`, stderr && stderr.length && `[stderr] ${stderr}` ].filter(Boolean).join("\n");
  }
  return data.describeOutcome;
};
exports.describeRunOutcomeSync = describeRunOutcomeSync;
const quote = v => JSON.stringify(v);
const run = ([command, ...argList], {quiet = !1, describeError = quiet, ...option} = {}) => {
  "win32" === process.platform && ([command, argList] = patchWin32CmdBat(command, argList));
  const subProcess = (0, _nodeChild_process.spawn)(command, argList, getOption(option, quiet));
  const stdoutPromise = quiet ? (0, _Stream.readableStreamToBufferAsync)(subProcess.stdout) : void 0;
  const stderrPromise = quiet ? (0, _Stream.readableStreamToBufferAsync)(subProcess.stderr) : void 0;
  const getExitData = () => ({
    code: subProcess.exitCode,
    signal: subProcess.signalCode,
    command,
    argList,
    stdoutPromise,
    stderrPromise
  });
  let promise = new Promise(((resolve, reject) => {
    subProcess.on("error", (error => reject(toRunError(error, getExitData()))));
    subProcess.on("exit", ((code, signal) => 0 === code ? resolve(getExitData()) : reject(toRunError(null, getExitData()))));
  }));
  describeError && (promise = promise.catch((async error => {
    throw (0, _error.remessageError)(error, await describeRunOutcome(error));
  })));
  return {
    subProcess,
    promise,
    stdoutPromise,
    stderrPromise
  };
};
exports.run = run;
const runSync = ([command, ...argList], {quiet = !1, describeError = quiet, ...option} = {}) => {
  "win32" === process.platform && ([command, argList] = patchWin32CmdBat(command, argList));
  const {error, status: code, signal, stdout, stderr} = (0, _nodeChild_process.spawnSync)(command, argList, getOption(option, quiet));
  const exitData = {
    code,
    signal,
    command,
    argList,
    stdout,
    stderr
  };
  if (0 !== code) {
    const runError = toRunError(error, exitData);
    throw describeError ? (0, _error.remessageError)(runError, describeRunOutcomeSync(runError)) : runError;
  }
  return exitData;
};
exports.runSync = runSync;
exports.runDetached = ([command, ...argList], {stdoutFile = "", stderrFile = stdoutFile, stdoutFd = stdoutFile && (0, _nodeFs.openSync)(stdoutFile, "a"), stderrFd = stderrFile && stderrFile !== stdoutFile ? (0, _nodeFs.openSync)(stderrFile, "a") : stdoutFd, ...option} = {}) => {
  "win32" === process.platform && ([command, argList] = patchWin32CmdBat(command, argList));
  const subProcess = (0, _nodeChild_process.spawn)(command, argList, {
    detached: !0,
    stdio: stdoutFd ? [ "ignore", stdoutFd, stderrFd ] : "ignore",
    ...option
  });
  subProcess.on("error", (error => {}));
  subProcess.unref();
  stdoutFd && (0, _nodeFs.closeSync)(stdoutFd);
  stderrFd && stderrFd !== stdoutFd && (0, _nodeFs.closeSync)(stderrFd);
  return {
    subProcess
  };
};
exports.runStdout = async (argList = [], option) => {
  const {promise, stdoutPromise} = run(argList, {
    quiet: !0,
    ...option
  });
  await promise;
  return stdoutPromise;
};
exports.runStdoutSync = (argList = [], option) => runSync(argList, {
  quiet: !0,
  ...option
}).stdout;