var _nodeTls = require("node:tls");
var _Object = require("../../../../common/immutable/Object.js");
var _Exot = require("../../../../common/module/Exot.js");
var _File = require("../../../fs/File.js");
var _Log = require("../../../module/Log.js");
var _Pid = require("../../../module/Pid.js");
var _ExitListener = require("../../../system/ExitListener.js");
var _Common = require("../../Responder/Common.js");
var _Router = require("../../Responder/Router.js");
var _Send = require("../../Responder/Send.js");
var _Server = require("../../Server.js");
var _Server2 = require("../../WS/Server.js");
exports.configureServerExot = ({protocol = "http:", hostname = "127.0.0.1", port, TLSSNIConfig, TLSDHParam, ...extraOption}) => (0, _Server.createServerExot)({
  protocol,
  hostname,
  port,
  ..."https:" === protocol && loadTLS(TLSSNIConfig, TLSDHParam),
  ...extraOption
});
const loadTLS = (TLSSNIConfig, TLSDHParam) => {
  TLSSNIConfig.key && (TLSSNIConfig = {
    default: TLSSNIConfig
  });
  TLSSNIConfig.default || (TLSSNIConfig.default = Object.values(TLSSNIConfig)[0]);
  if (!TLSSNIConfig.default) throw new Error("no default TLS config");
  const dhparam = TLSDHParam && autoLoadBuffer(TLSDHParam);
  const optionMap = (0, _Object.objectMap)(TLSSNIConfig, (({key, cert, ca}) => ({
    key: autoLoadBuffer(key),
    cert: autoLoadBuffer(cert),
    ca: ca && autoLoadBuffer(ca),
    dhparam
  })));
  const secureContextMap = (0, _Object.objectMap)(optionMap, (option => (0, _nodeTls.createSecureContext)(option)));
  return {
    ...optionMap.default,
    SNICallback: Object.keys(optionMap).length >= 2 ? (hostname, callback) => callback(null, secureContextMap[hostname] || secureContextMap.default) : void 0
  };
};
const autoLoadBuffer = bufferOrPath => Buffer.isBuffer(bufferOrPath) ? bufferOrPath : (0, _File.readBufferSync)(bufferOrPath);
const setupServer = ({protocol = "http:", hostname = "127.0.0.1", port, TLSSNIConfig, TLSDHParam, ...extraOption}) => {
  (0, _Pid.configurePid)(extraOption);
  const {loggerExot} = (0, _Log.configureLog)(extraOption);
  return {
    loggerExot,
    serverExot: (0, _Server.createServerExot)({
      protocol,
      hostname,
      port,
      ..."https:" === protocol && loadTLS(TLSSNIConfig, TLSDHParam),
      ...extraOption
    })
  };
};
exports.setupServer = setupServer;
exports.configureFeature = (option, featureList = []) => setupFeature(featureList, option);
const setupFeature = (featureList = [], {serverExot, loggerExot, isRawServer = !1, isFavicon = !0, isDebugRoute = !1, rootRouteResponder, preResponderList = [], preRouteList = [], responderLogEnd = (0, _Common.createResponderLogEnd)({
  loggerExot
})}) => {
  serverExot.featureMap = new Map;
  serverExot.wsSet = void 0;
  preRouteList = preRouteList.filter(Boolean);
  let featureUrl;
  const featureRouteList = [];
  const featureWSRouteList = [];
  featureList.forEach((feature => {
    if (feature) {
      if (!feature.name) throw new Error(`expect feature.name of ${feature}`);
      if (serverExot.featureMap.has(feature.name)) throw new Error(`duplicate feature.name: ${feature.name}`);
      serverExot.featureMap.set(feature.name, feature);
      featureUrl || (featureUrl = feature.URL_HTML);
      feature.routeList && featureRouteList.push(...feature.routeList.filter(Boolean));
      feature.wsRouteList && featureWSRouteList.push(...feature.wsRouteList.filter(Boolean));
    }
  }));
  if (isRawServer) {
    if (preRouteList.length || featureRouteList.length || featureWSRouteList.length) throw new Error("route setting conflict with isRawServer");
    return;
  }
  const routeMap = (0, _Router.createRouteMap)([ ...preRouteList, ...featureRouteList, isFavicon && [ [ "/favicon", "/favicon.ico" ], "GET", (0, _Send.createResponderFavicon)() ], [ [ "/", "" ], "GET", rootRouteResponder || (isDebugRoute ? (0, 
  _Router.createResponderRouteListHTML)({
    getRouteMap: () => routeMap
  }) : featureUrl ? store => (0, _Common.responderEndWithRedirect)(store, {
    redirectUrl: featureUrl
  }) : store => (0, _Common.responderEndWithStatusCode)(store, {
    statusCode: 400
  })) ] ].filter(Boolean));
  serverExot.server.on("request", (0, _Server.createRequestListener)({
    responderList: [ ...preResponderList, (0, _Common.createResponderLog)({
      loggerExot
    }), (0, _Router.createResponderRouter)({
      routeMap,
      serverExot
    }) ].filter(Boolean),
    responderEnd: store => {
      (0, _Common.responderEnd)(store);
      return responderLogEnd(store);
    }
  }));
  if (0 !== featureWSRouteList.length) {
    const routeMap = (0, _Router.createRouteMap)(featureWSRouteList);
    serverExot.wsSet = (0, _Server2.enableWSServer)(serverExot.server, {
      onUpgradeRequest: (0, _Server2.createUpgradeRequestListener)({
        responderList: [ (0, _Common.createResponderLog)({
          loggerExot
        }), (0, _Router.createResponderRouter)({
          routeMap,
          serverExot
        }) ]
      })
    });
  }
  return {
    RouteList: (0, _Router.describeRouteMap)(routeMap),
    RouteListWS: 0 === featureWSRouteList.length ? [] : (0, _Router.describeRouteMap)((0, _Router.createRouteMap)(featureWSRouteList))
  };
};
exports.setupFeature = setupFeature;
const setupServerExotGroup = (serverExot, loggerExot, serverExotGroup = (0, _Exot.createExotGroup)({
  id: "exot:group-server",
  getOnExotError: serverExotGroup => error => {
    console.log("[exot-group-error]", error);
    return serverExotGroup.down();
  }
})) => {
  loggerExot && serverExotGroup.set(loggerExot);
  if (serverExot.featureMap) for (const {exotList} of serverExot.featureMap.values()) if (exotList && exotList.length) for (const exot of exotList) serverExotGroup.set(exot);
  serverExotGroup.set(serverExot);
  return serverExotGroup;
};
exports.setupServerExotGroup = setupServerExotGroup;
const startServerExotGroup = async ({loggerExot, serverExot, serverExotGroup = loggerExot ? setupServerExotGroup(serverExot, loggerExot) : serverExot, isMuteLog = !loggerExot || !1}) => {
  (0, _ExitListener.addExitListenerLossyOnce)((eventPack => {
    isMuteLog || loggerExot.add(`[SERVER] down... ${JSON.stringify(eventPack)}${eventPack.error ? ` ${eventPack.error.stack || eventPack.error}` : ""}`);
    return serverExotGroup.down().then((() => isMuteLog || loggerExot.add("[SERVER] down")));
  }));
  isMuteLog || loggerExot.add("[SERVER] up...");
  await serverExotGroup.up();
  isMuteLog || loggerExot.add("[SERVER] up");
};
exports.startServerExotGroup = startServerExotGroup;
const runServerExotGroup = async pack => {
  await startServerExotGroup(pack);
  return pack;
};
exports.runServerExotGroup = runServerExotGroup;
exports.runServer = async (configureServer, serverOption, featureOption, serverInfoTitle = (featureOption.packageName ? `${featureOption.packageName}@${featureOption.packageVersion}` : void 0)) => {
  const {loggerExot, serverExot} = setupServer(serverOption);
  const serverInfo = (0, _Server.describeServerOption)(serverExot.option, serverInfoTitle, featureOption);
  await configureServer({
    serverExot,
    loggerExot,
    serverInfo,
    ...featureOption
  });
  loggerExot.add(serverInfo);
  return runServerExotGroup({
    serverOption,
    featureOption,
    loggerExot,
    serverExot,
    serverExotGroup: setupServerExotGroup(serverExot, loggerExot)
  });
};