var _nodePath = require("node:path");
var _check = require("../../../../common/check.js");
var _string = require("../../../../common/string.js");
var _preset = require("../../../module/Option/preset.js");
var _function = require("../../function.js");
const {parseCompact, parseCompactList} = _preset.Preset;
const ServerHostFormat = exports.ServerHostFormat = 'host,H/SS,O|set "hostname:port"';
exports.getServerExotFormatConfig = (extraList = []) => parseCompact(ServerHostFormat, [ parseCompact(`TLS-SNI-config/SO,O|TLS SNI config map, set to enable https:\n${(0, _string.indentLineList)([ 'multi config: { [hostname]: { key: pathOrBuffer, cert: pathOrBuffer, ca: pathOrBuffer } }, default to special hostname "default", or the first config', "single config: { key: pathOrBuffer, cert: pathOrBuffer, ca: pathOrBuffer }", "key: Private keys in PEM format", "cert: Cert chains in PEM format", "ca: Optionally override the trusted CA certificates" ])}`, parseCompactList('TLS-dhparam/O/1|pathOrBuffer; Diffie-Hellman Key Exchange, generate with: "openssl dhparam -dsaparam -outform PEM -out output/path/dh4096.pem 4096"')), ...extraList ]);
exports.getServerExotOption = ({tryGet, tryGetFirst, pwd}, defaultHostname = "127.0.0.1") => {
  const {hostname, port} = (0, _function.parseHostString)(tryGetFirst("host") || "", defaultHostname);
  const pwdTLSSNIConfig = pwd("TLS-SNI-config");
  return {
    protocol: pwdTLSSNIConfig ? "https:" : "http:",
    hostname,
    port,
    ...pwdTLSSNIConfig && objectMapDeep({
      TLSSNIConfig: tryGetFirst("TLS-SNI-config"),
      TLSDHParam: tryGetFirst("TLS-dhparam")
    }, (value => (0, _check.isString)(value) ? (0, _nodePath.resolve)(pwdTLSSNIConfig, value) : value))
  };
};
const objectMapDeep = (object, mapFunc) => {
  const result = {};
  for (const [key, value] of Object.entries(object)) result[key] = (0, _check.isBasicObject)(value) && !Buffer.isBuffer(value) ? objectMapDeep(value, mapFunc) : mapFunc(value, key);
  return result;
};
exports.LogFormatConfig = parseCompact("log-path/SP,O", parseCompactList("log-file-prefix/SS,O"));
exports.getLogOption = ({tryGetFirst}) => ({
  pathLogDirectory: tryGetFirst("log-path"),
  logFilePrefix: tryGetFirst("log-file-prefix")
});
exports.PidFormatConfig = parseCompact("pid-file/SP,O", parseCompactList("pid-ignore-exist/T"));
exports.getPidOption = ({tryGetFirst, getToggle}) => ({
  filePid: tryGetFirst("pid-file"),
  shouldIgnoreExistPid: getToggle("pid-ignore-exist")
});