exports.pathContentStyle = "<style>\nh2, h6 { margin: 0.5em 4px; }\n.select, .directory, .file { display: flex; flex-flow: row nowrap; align-items: stretch; }\n.directory:hover, .file:hover { background: var(--c-fill-s); }\n.name { overflow:hidden; flex: 1; white-space:nowrap; text-align: left; text-overflow: ellipsis; background: transparent; }\n.select .name, .file .name { pointer-events: none; color: #888; }\n.edit { pointer-events: auto; min-width: 1.5em; min-height: auto; line-height: normal; }\n</style>";
exports.initPathContent = (URL_ACTION_JSON_ABBR, URL_FILE_SERVE_ABBR, IS_READ_ONLY = !0, IS_EXTRA_TAR = !1, IS_EXTRA_AUTO = !1, ACTION_TYPE, authFetch, withConfirmModal, withPromptModal) => {
  const {open, qS, cE, aCL, Dr: {Common: {Format, String: {lazyEncodeURI}, Compare: {compareStringWithNumber}}}} = window;
  const SORT_FUNC = {
    NAME: ([nameA], [nameB]) => compareStringWithNumber(nameA, nameB),
    TIME: ([, , mtimeMsA = 0], [, , mtimeMsB = 0]) => mtimeMsB - mtimeMsA,
    SIZE: ([, sizeA], [, sizeB]) => sizeB - sizeA
  };
  const SORT_TYPE_LIST = Object.keys(SORT_FUNC);
  const pathPush = (relativePath, name) => "." === relativePath ? name : `${relativePath}/${name}`;
  const pathName = relativePath => "." === relativePath ? "[ROOT]" : `${relativePath}/`;
  const initialPathContentState = {
    pathSortType: SORT_TYPE_LIST[0],
    pathContent: {
      relativePath: ".",
      directoryList: [],
      fileList: []
    }
  };
  const authFetchActionJSON = async (actionType, actionPayload = {}) => (await authFetch(`${URL_ACTION_JSON_ABBR}/${encodeURIComponent(actionType)}`, {
    method: "POST",
    body: JSON.stringify(actionPayload)
  })).json();
  const doLoadPath = async (pathContentStore, relativePath = pathContentStore.getState().pathContent.relativePath) => {
    const {resultList: [{key: nextRelativePath, directoryList, fileList}]} = await authFetchActionJSON(ACTION_TYPE.PATH_DIRECTORY_CONTENT, {
      key: relativePath || "."
    });
    pathContentStore.setState({
      pathContent: {
        relativePath: nextRelativePath || ".",
        directoryList,
        fileList
      }
    });
  };
  return {
    initialPathContentState,
    authFetchActionJSON,
    cyclePathSortType: (pathContentStore, pathSortType = pathContentStore.getState().pathSortType) => {
      const nextSortIndex = (SORT_TYPE_LIST.indexOf(pathSortType) + 1) % SORT_TYPE_LIST.length;
      pathContentStore.setState({
        pathSortType: SORT_TYPE_LIST[nextSortIndex]
      });
    },
    getLoadPathAsync: pathContentStore => async relativePath => doLoadPath(pathContentStore, relativePath),
    getPathActionAsync: pathContentStore => async (batchList, actionType, key, keyTo) => {
      if (key && null !== keyTo && (actionType !== ACTION_TYPE.PATH_COPY && actionType !== ACTION_TYPE.PATH_RENAME || keyTo !== key)) {
        await authFetchActionJSON(actionType, {
          key,
          keyTo,
          batchList
        });
        await doLoadPath(pathContentStore);
      }
    },
    getPreviewFile: (pathContentStore, authUrl) => async (relativePath, fileName) => open(authUrl(`${URL_FILE_SERVE_ABBR}/${lazyEncodeURI(pathPush(relativePath, fileName))}`)),
    getDownloadFile: (pathContentStore, authDownload) => async (relativePath, fileName) => authDownload(`${URL_FILE_SERVE_ABBR}/${lazyEncodeURI(pathPush(relativePath, fileName))}`, fileName),
    renderPathContent: (pathContentStore, parentElement, loadPath, pathAction, previewFile, downloadFile) => {
      const {pathSortType, pathContent: {relativePath, directoryList, fileList}} = pathContentStore.getState();
      const selectToggleMap = {};
      const selectNameSet = new Set;
      const doSelectRemaining = () => {
        Object.entries(selectToggleMap).forEach((([name, toggle]) => !selectNameSet.has(name) && toggle()));
        updateSelectStatus();
      };
      const TEXT_BATCH = text => `🗃️${text}`;
      const selectPathPop = cE("button", {
        className: "edit",
        innerText: "🔙..",
        onclick: () => loadPath((relativePath => "." === relativePath ? "." : relativePath.split("/").slice(0, -1).join("/"))(relativePath))
      });
      const selectEditSelectNone = cE("button", {
        className: "edit",
        innerText: TEXT_BATCH("☐"),
        onclick: doSelectRemaining
      });
      const selectEditSelectSome = cE("button", {
        className: "edit",
        innerText: TEXT_BATCH("☒"),
        onclick: doSelectRemaining
      });
      const selectEditSelectAll = cE("button", {
        className: "edit",
        innerText: TEXT_BATCH("☑"),
        onclick: () => {
          selectNameSet.forEach((name => selectToggleMap[name]()));
          updateSelectStatus();
        }
      });
      const selectEditCopy = cE("button", {
        className: "edit",
        innerText: TEXT_BATCH("📋"),
        onclick: async () => pathAction([ ...selectNameSet ], ACTION_TYPE.PATH_COPY, relativePath, await withPromptModal(`Batch Copy ${selectNameSet.size} Path To`, relativePath))
      });
      const selectEditRename = cE("button", {
        className: "edit",
        innerText: TEXT_BATCH("✂️"),
        onclick: async () => pathAction([ ...selectNameSet ], ACTION_TYPE.PATH_RENAME, relativePath, await withPromptModal(`Batch Rename ${selectNameSet.size} Path To`, relativePath))
      });
      const selectEditDelete = cE("button", {
        className: "edit",
        innerText: TEXT_BATCH("🗑️"),
        onclick: async () => await withConfirmModal(`Batch Delete ${selectNameSet.size} Path In: ${pathName(relativePath)}?`) && pathAction([ ...selectNameSet ], ACTION_TYPE.PATH_DELETE, relativePath)
      });
      const updateSelectStatus = () => aCL(qS(".select", ""), [ "." !== relativePath && selectPathPop, selectNameSet.size ? selectNameSet.size < directoryList.length + fileList.length ? selectEditSelectSome : selectEditSelectAll : selectEditSelectNone, selectNameSet.size && selectEditCopy, selectNameSet.size && selectEditRename, selectNameSet.size && selectEditDelete, cE("span", {
        className: "name button",
        innerText: `${selectNameSet.size} selected`
      }) ]);
      const renderSelectButton = name => {
        selectToggleMap[name] = () => {
          const prevIsSelect = selectNameSet.has(name);
          selectNameSet[prevIsSelect ? "delete" : "add"](name);
          const isSelect = !prevIsSelect;
          element.className = isSelect ? "edit select" : "edit";
          element.innerText = isSelect ? "☑" : "☐";
        };
        const element = cE("button", {
          className: "edit",
          innerText: "☐",
          onclick: () => {
            selectToggleMap[name]();
            updateSelectStatus();
          }
        });
        return element;
      };
      const editBlocker = (IS_READ_ONLY || window.innerWidth < 480) && [];
      const wideMBlocker = window.innerWidth < 640 && [];
      const renderCommonEditList = relativePath => [ cE("button", {
        className: "edit",
        innerText: "📋",
        onclick: async () => pathAction([ "" ], ACTION_TYPE.PATH_COPY, relativePath, await withPromptModal("Copy To", relativePath))
      }), cE("button", {
        className: "edit",
        innerText: "✂️",
        onclick: async () => pathAction([ "" ], ACTION_TYPE.PATH_RENAME, relativePath, await withPromptModal("Rename To", relativePath))
      }), cE("button", {
        className: "edit",
        innerText: "🗑️",
        onclick: async () => await withConfirmModal(`Delete path: ${relativePath}?`) && pathAction([ "" ], ACTION_TYPE.PATH_DELETE, relativePath)
      }) ];
      const createCompressButton = (relativePath, actionType, defaultExtension, extraMessage = "") => cE("button", {
        className: "edit",
        innerText: "📥",
        onclick: async () => pathAction([ "" ], actionType, relativePath, await withPromptModal(`Compress To ${extraMessage}`, `${relativePath}.${defaultExtension}`))
      });
      const renderExtraCompressEdit = relativePath => IS_EXTRA_AUTO ? createCompressButton(relativePath, ACTION_TYPE.PATH_COMPRESS_AUTO, "7z", "(support 7z|xz|tar|fsp)") : IS_EXTRA_TAR ? createCompressButton(relativePath, ACTION_TYPE.PATH_COMPRESS_TAR, "tgz") : null;
      const renderExtraExtractEdit = relativePath => {
        let actionType;
        IS_EXTRA_AUTO && REGEXP_EXTRACT_AUTO.test(relativePath) ? actionType = ACTION_TYPE.PATH_EXTRACT_AUTO : IS_EXTRA_TAR && REGEXP_EXTRACT_TAR.test(relativePath) && (actionType = ACTION_TYPE.PATH_EXTRACT_TAR);
        return actionType && cE("button", {
          className: "edit",
          innerText: "📤",
          onclick: async () => pathAction([ "" ], actionType, relativePath, await withPromptModal("Extract To", `${relativePath}.content/`))
        });
      };
      const REGEXP_EXTRACT_AUTO = /\.(?:tar|tgz|tar\.gz|t7z|tar\.7z|txz|tar\.xz|7z|zip|fsp|fsp\.gz)$/;
      const REGEXP_EXTRACT_TAR = /\.(?:tar|tgz|tar\.gz)$/;
      parentElement.innerHTML = "";
      aCL(parentElement, [ cE("h2", {
        innerText: pathName(relativePath)
      }), cE("h6", {
        innerText: `${directoryList.length} directory, ${fileList.length} file (${Format.binary(fileList.reduce(((o, [, size]) => o + size), 0))}B)`
      }), cE("div", {
        className: "select"
      }), ...directoryList.sort(((nameA, nameB) => SORT_FUNC[pathSortType]([ nameA ], [ nameB ]))).map((name => cE("div", {
        className: "directory"
      }, [ !IS_READ_ONLY && renderSelectButton(name), cE("button", {
        className: "name",
        innerText: `📁|${name}/`,
        onclick: () => loadPath(pathPush(relativePath, name))
      }), ...editBlocker || wideMBlocker || [ renderExtraCompressEdit(pathPush(relativePath, name)) ], ...editBlocker || renderCommonEditList(pathPush(relativePath, name)) ]))), ...fileList.sort(SORT_FUNC[pathSortType]).map((([name, size, mtimeMs]) => cE("div", {
        className: "file"
      }, [ !IS_READ_ONLY && renderSelectButton(name), cE("span", {
        className: "name button",
        innerText: `📄|${name} - ${new Date(mtimeMs).toLocaleString()}`
      }), cE("button", {
        className: "edit",
        innerText: `${Format.binary(size)}B|💾`,
        onclick: () => downloadFile(relativePath, name)
      }), ...editBlocker || wideMBlocker || [ renderExtraExtractEdit(pathPush(relativePath, name)) ], cE("button", {
        className: "edit",
        innerText: "🔍",
        onclick: () => previewFile(relativePath, name)
      }), ...editBlocker || renderCommonEditList(pathPush(relativePath, name)) ]))) ]);
      !IS_READ_ONLY && updateSelectStatus();
    }
  };
};