var _nodePath = require("node:path");
var _nodeFs = require("node:fs");
var _function = require("../../../../common/function.js");
var _format = require("../../../../common/format.js");
var _string = require("../../../../common/string.js");
var _Stream = require("../../../data/Stream.js");
var _Directory = require("../../../fs/Directory.js");
var _FileChunkUpload = require("../../../module/FileChunkUpload.js");
exports.fileUpload = async ({fileInputPath: filePath, fileBuffer, key, urlFileUpload, timeout = 3e4, maxRetry = 4, wait = 1e3, authFetch, log, onProgress = (uploadedSize, totalSize) => log && log(`[Upload] upload: ${(0, _format.percent)(uploadedSize / totalSize)}`)}) => {
  const fileSize = fileBuffer ? fileBuffer.length : (await _nodeFs.promises.stat(filePath)).size;
  log && log(`[Upload] key: ${key}, size: ${(0, _format.binary)(fileSize)}B`);
  await (0, _FileChunkUpload.uploadFileByChunk)({
    fileBuffer,
    filePath,
    fileSize,
    key,
    onProgress,
    uploadChunk: async (arrayBufferPacket, {key, chunkIndex, chunkTotal}) => (0, _function.withRetryAsync)((async () => authFetch(urlFileUpload, {
      method: "POST",
      body: arrayBufferPacket,
      timeout
    }).catch((error => {
      const message = `[ERROR][Upload] upload chunk ${chunkIndex}/${chunkTotal} of ${key}, packet size: ${arrayBufferPacket.byteLength}`;
      log && log(message, error);
      throw new Error(message);
    }))), maxRetry, wait)
  });
  log && log(`[Upload] done: ${key}`);
};
exports.fileDownload = async ({fileOutputPath, key, urlFileDownload, timeout = 3e4, authFetch, log}) => {
  log && log(`[Download] key: ${key}`);
  fileOutputPath && await (0, _Directory.createDirectory)((0, _nodePath.dirname)(fileOutputPath));
  const response = await authFetch(`${urlFileDownload}/${(0, _string.lazyEncodeURI)(key)}`, {
    method: "GET",
    timeout
  });
  if (!fileOutputPath) {
    const fileBuffer = await response.buffer();
    log && log(`[Download] get: ${(0, _format.binary)(fileBuffer.length)}B`);
    return fileBuffer;
  }
  await (0, _Stream.quickRunletFromStream)(response.stream(), (0, _nodeFs.createWriteStream)(fileOutputPath));
  log && log(`[Download] done: ${fileOutputPath}`);
};