var _time = require("../../../common/time.js");
exports.responderError = (store, error) => {
  store.setState({
    error
  });
};
exports.responderEnd = store => {
  if (store.response.writableEnded) return;
  const {error} = store.getState();
  !store.response.headersSent && store.response.writeHead(error ? error.status || 400 : 500);
  store.response.end();
};
exports.responderEndWithStatusCode = (store, {statusCode = 500, headerMap, data}) => {
  if (!store.response.finished) {
    !store.response.headersSent && store.response.writeHead(statusCode, headerMap);
    store.response.end(data);
  }
};
exports.responderEndWithRedirect = (store, {statusCode = 302, redirectUrl}) => {
  if (!store.response.finished) {
    !store.response.headersSent && store.response.writeHead(statusCode, {
      location: redirectUrl
    });
    store.response.end();
  }
};
exports.responderSetHeaderCacheControlImmutable = store => {
  store.response.setHeader("cache-control", "max-age=315360000, public, immutable");
};
const DEFAULT_DESCRIBE_REQUEST = ({url, method, headers: {host}, socket: {remoteAddress, remotePort}}) => `[${method}] ${host || ""}${url} (${remoteAddress}:${remotePort})`;
exports.createResponderLog = ({loggerExot, log = loggerExot && loggerExot.add, describeRequest = DEFAULT_DESCRIBE_REQUEST}) => store => {
  const userAgentString = store.request.headers["user-agent"] || "none";
  log(describeRequest(store.request), "[UA]", userAgentString);
};
exports.createResponderLogEnd = ({loggerExot, log = loggerExot && loggerExot.add, describeRequest = DEFAULT_DESCRIBE_REQUEST}) => store => {
  const {statusCode} = store.response;
  const {time, error} = store.getState();
  const timeString = `${Math.round((0, _time.clock)() - time)}ms`;
  log(describeRequest(store.request), error ? `[ERROR|${statusCode}|${timeString}] ${error.stack || error}` : `[END|${statusCode}|${timeString}]`);
};
exports.createResponderSetHeaderHSTS = ({protocol}) => "https:" !== protocol ? () => {} : store => {
  store.response.setHeader("strict-transport-security", "max-age=31536000; includeSubDomains; preload");
};
exports.createResponderHostMapper = ({hostResponderMap, responderDefault}) => store => {
  const responder = hostResponderMap[(store.request.headers.host || "").toLowerCase()] || responderDefault;
  return responder && responder(store);
};