var _nodeZlib = require("node:zlib");
var _nodeUtil = require("node:util");
var _MIME = require("../../../common/module/MIME.js");
var _Stream = require("../../data/Stream.js");
var _EntityTag = require("../../module/EntityTag.js");
const gzipAsync = (0, _nodeUtil.promisify)(_nodeZlib.gzip);
const checkPair = (store, entityTag, isEnableGzip = !1) => {
  entityTag && store.response.setHeader("etag", entityTag);
  const headers = store.request.headers;
  const shouldSend = !entityTag || !(headers["if-none-match"] || "").includes(entityTag);
  return [ shouldSend, isEnableGzip && shouldSend && (headers["accept-encoding"] || "").includes("gzip") ];
};
const responseHead = (store, type, length, [shouldSend, shouldGzip], range) => {
  let statusCode = 200;
  const headers = {
    "content-type": type
  };
  length >= 0 && (headers["content-length"] = length);
  if (range) {
    const [start, end] = range;
    statusCode = 206;
    headers["content-length"] = end - start + 1;
    headers["content-range"] = `bytes ${start}-${end}/${length}`;
    shouldSend = !0;
  } else shouldSend ? shouldGzip && (headers["content-encoding"] = "gzip") : statusCode = 304;
  store.response.writeHead(statusCode, headers);
  return shouldSend;
};
exports.responderSendBuffer = (store, {buffer, entityTag, type = _MIME.DEFAULT_MIME, length = buffer.length}) => responseHead(store, type, length, checkPair(store, entityTag)) && (0, _Stream.writeBufferToStreamAsync)(store.response, buffer);
exports.responderSendBufferRange = (store, {buffer, entityTag, type = _MIME.DEFAULT_MIME, length = buffer.length}, [start, end]) => {
  end = Math.min(end, length - 1);
  return responseHead(store, type, length, checkPair(store, entityTag), [ start, end ]) && (0, _Stream.writeBufferToStreamAsync)(store.response, buffer.slice(start, end + 1));
};
const responderSendBufferCompress = async (store, {buffer, bufferGzip, entityTag, type = _MIME.DEFAULT_MIME, length = buffer.length}) => {
  const [shouldSend, shouldGzip] = checkPair(store, entityTag, !0);
  shouldGzip && (length = (buffer = bufferGzip || await gzipAsync(buffer)).length);
  return responseHead(store, type, length, [ shouldSend, shouldGzip ]) && (0, _Stream.writeBufferToStreamAsync)(store.response, buffer);
};
exports.responderSendBufferCompress = responderSendBufferCompress;
exports.responderSendStream = (store, {stream, entityTag, type = _MIME.DEFAULT_MIME, length}) => responseHead(store, type, length, checkPair(store, entityTag)) && (0, _Stream.quickRunletFromStream)(stream, store.response);
exports.responderSendStreamRange = (store, {streamRange, entityTag, type = _MIME.DEFAULT_MIME, length = "*"}, [start, end]) => {
  length >= 0 && (end = Math.min(end, length - 1));
  return responseHead(store, type, length, checkPair(store, entityTag), [ start, end ]) && (0, _Stream.quickRunletFromStream)(streamRange, store.response);
};
exports.responderSendStreamCompress = async (store, {stream, streamGzip, entityTag, type = _MIME.DEFAULT_MIME, length}) => {
  const [shouldSend, shouldGzip] = checkPair(store, entityTag, !0);
  return responseHead(store, type, shouldGzip ? void 0 : length, [ shouldSend, shouldGzip ]) && _Stream.quickRunletFromStream.apply(null, shouldGzip ? streamGzip ? [ streamGzip, store.response ] : [ stream, (0, _nodeZlib.createGzip)(), store.response ] : [ stream, store.response ]);
};
exports.responderSendJSON = (store, {object, entityTag}) => responderSendBufferCompress(store, {
  buffer: Buffer.from(JSON.stringify(object)),
  type: _MIME.BASIC_EXTENSION_MAP.json,
  entityTag
});
const prepareBufferData = (buffer, type) => ({
  buffer,
  type,
  bufferGzip: (0, _nodeZlib.gzipSync)(buffer),
  entityTag: (0, _EntityTag.getEntityTagByContentHash)(buffer)
});
exports.prepareBufferData = prepareBufferData;
exports.createResponderFavicon = (buffer = Buffer.from("iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAADUlEQVR42mNMXvf/PwAGnQMR4CJUOAAAAABJRU5ErkJggg==", "base64"), type = _MIME.BASIC_EXTENSION_MAP.png) => store => responderSendBufferCompress(store, prepareBufferData(buffer, type));