var _nodeFs = require("node:fs");
var _CacheMap = require("../../../common/data/CacheMap.js");
var _MIME = require("../../../common/module/MIME.js");
var _File = require("../../fs/File.js");
var _Path = require("../../fs/Path.js");
var _EntityTag = require("../../module/EntityTag.js");
var _Common = require("./Common.js");
var _Send = require("./Send.js");
const createDefaultCacheMap = () => (0, _CacheMap.createCacheMap)({
  valueSizeSumMax: 33554432
});
exports.createDefaultCacheMap = createDefaultCacheMap;
exports.createResponderServeStatic = ({sizeSingleMax = 524288, expireTime = 6e4, isEnableGzip = !1, isEnableRange = !0, serveCacheMap = createDefaultCacheMap()}) => {
  const serveCache = async (store, filePath, filePathGz, rangePair) => {
    const filePathServe = filePathGz || filePath;
    const bufferData = serveCacheMap.get(filePathServe);
    if (!bufferData) return !1;
    filePathGz && store.response.setHeader("content-encoding", "gzip");
    isEnableRange && store.response.setHeader("accept-ranges", "bytes");
    rangePair ? await (0, _Send.responderSendBufferRange)(store, bufferData, rangePair) : await (0, _Send.responderSendBuffer)(store, bufferData);
    return !0;
  };
  const serve = async (store, filePath, filePathGz, rangePair) => {
    const filePathServe = filePathGz || filePath;
    const stat = await (0, _Path.getPathStat)(filePathServe);
    if (!stat.isFile()) return !1;
    const entityTag = (0, _EntityTag.getWeakEntityTagByStat)(stat);
    const type = (0, _MIME.getMIMETypeFromFileName)(filePath);
    const length = stat.size;
    filePathGz && store.response.setHeader("content-encoding", "gzip");
    isEnableRange && store.response.setHeader("accept-ranges", "bytes");
    if (rangePair) {
      rangePair[1] = Math.min(rangePair[1], length - 1);
      await (0, _Send.responderSendStreamRange)(store, {
        streamRange: (0, _nodeFs.createReadStream)(filePathServe, {
          start: rangePair[0],
          end: rangePair[1]
        }),
        entityTag,
        type,
        length
      }, rangePair);
    } else if (length > sizeSingleMax) await (0, _Send.responderSendStream)(store, {
      stream: (0, _nodeFs.createReadStream)(filePathServe),
      entityTag,
      type,
      length
    }); else {
      const bufferData = {
        buffer: await (0, _File.readBuffer)(filePathServe),
        entityTag,
        type,
        length
      };
      serveCacheMap.set(filePathServe, bufferData, length, Date.now() + expireTime);
      await (0, _Send.responderSendBuffer)(store, bufferData);
    }
    return !0;
  };
  return async (store, filePath) => {
    if (await (async (store, filePath) => {
      if ("HEAD" !== store.request.method) return !1;
      let bufferData = serveCacheMap.get(filePath);
      if (!bufferData) {
        const stat = await (0, _Path.getPathStat)(filePath);
        if (!stat.isFile()) throw new Error(`miss file: ${filePath}`);
        bufferData = {
          entityTag: (0, _EntityTag.getWeakEntityTagByStat)(stat),
          type: (0, _MIME.getMIMETypeFromFileName)(filePath),
          length: stat.size
        };
      }
      const headerMap = {
        etag: bufferData.entityTag,
        "content-type": bufferData.type,
        "content-length": bufferData.length,
        "accept-ranges": isEnableRange ? "bytes" : "none"
      };
      await (0, _Common.responderEndWithStatusCode)(store, {
        statusCode: 200,
        headerMap
      });
      return !0;
    })(store, filePath)) return;
    const filePathGz = isEnableGzip && REGEXP_ENCODING_GZIP.test(store.request.headers["accept-encoding"]) && filePath + ".gz";
    const rangePair = isEnableRange && parseRangeHeader(store.request.headers.range);
    if (!(filePathGz && await serveCache(store, filePath, filePathGz) || await serveCache(store, filePath, void 0, rangePair) || filePathGz && await serve(store, filePath, filePathGz) || await serve(store, filePath, void 0, rangePair))) throw new Error(`miss file: ${filePath}`);
  };
};
const REGEXP_ENCODING_GZIP = /gzip/i;
const parseRangeHeader = rangeString => {
  const result = REGEXP_HEADER_RANGE.exec(rangeString);
  if (!result) return;
  const [, startString, endString] = result;
  const start = parseInt(startString);
  const end = endString ? parseInt(endString) : 1 / 0;
  return start < end ? [ start, end ] : void 0;
};
const REGEXP_HEADER_RANGE = /bytes=(\d+)-(\d+)?$/i;