var _nodeOs = require("node:os");
var _nodeCrypto = require("node:crypto");
var _nodeNet = require("node:net");
var _nodeTls = require("node:tls");
var _nodeHttp = require("node:http");
var _nodeHttps = require("node:https");
var _time = require("../../common/time.js");
var _check = require("../../common/check.js");
var _format = require("../../common/format.js");
var _CacheMap = require("../../common/data/CacheMap2.js");
var _StateStore = require("../../common/immutable/StateStore.js");
var _Object = require("../../common/immutable/Object.js");
var _Common = require("./Responder/Common.js");
exports.createServerExot = ({id = "server", protocol, skipSessionPatch = !1, forceCloseTimeout = 2e3, ...option}) => {
  if (![ "tcp:", "tls:", "http:", "https:" ].includes(protocol)) throw new Error(`invalid protocol: ${protocol}`);
  const isSecure = [ "tls:", "https:" ].includes(protocol);
  (option = {
    isSecure,
    protocol,
    port: isSecure ? 443 : 80,
    hostname: "127.0.0.1",
    ...isSecure && !skipSessionPatch && {
      sessionTimeout: 14400
    },
    ...option
  }).baseUrl = `${protocol}//${option.hostname}:${option.port}`;
  const serverSec = isSecure ? "tls:" === protocol ? (0, _nodeTls.createServer)(option) : (0, _nodeHttps.createServer)(option) : void 0;
  const server = serverSec || ("tcp:" === protocol ? (0, _nodeNet.createServer)() : (0, _nodeHttp.createServer)());
  serverSec && !skipSessionPatch && (server => {
    const sessionCacheMap = (0, _CacheMap.createCacheMap2)({
      sizeMax: 4096,
      expireAfter: 6e5
    });
    server.on("newSession", ((sessionId, sessionData, next) => {
      sessionCacheMap.set(sessionId.toString("base64"), sessionData);
      next();
    }));
    server.on("resumeSession", ((sessionId, next) => {
      next(null, sessionCacheMap.get(sessionId.toString("base64")) || null);
    }));
  })(serverSec);
  const socketSet = new Set;
  server.on("connection", (socket => {
    socketSet.add(socket);
    socket.once("close", (() => {
      socketSet.delete(socket);
    }));
  }));
  let sessionTicketRotateToken;
  return {
    id,
    up: async () => !server.listening && new Promise(((resolve, reject) => {
      server.on("error", reject);
      server.listen(option.port, option.hostname.replace(/[[\]]/g, ""), (() => {
        server.off("error", reject);
        if (serverSec && !skipSessionPatch) {
          const resetSessionTicketKey = () => serverSec.setTicketKeys((0, _nodeCrypto.randomBytes)(48));
          sessionTicketRotateToken && clearInterval(sessionTicketRotateToken);
          sessionTicketRotateToken = setInterval(resetSessionTicketKey, 144e5);
          resetSessionTicketKey();
        }
        resolve();
      }));
    })),
    down: async () => server.listening && new Promise((resolve => {
      sessionTicketRotateToken && clearInterval(sessionTicketRotateToken);
      const forceCloseToken = (0, _check.isNumber)(forceCloseTimeout) && forceCloseTimeout !== 1 / 0 && setTimeout((() => {
        for (const socket of socketSet) socket.destroy();
      }), forceCloseTimeout);
      server.close((() => {
        forceCloseToken && clearTimeout(forceCloseToken);
        resolve();
      }));
    })),
    isUp: () => server.listening,
    server,
    option,
    socketSet
  };
};
const DEFAULT_RESPONDER_ERROR = _Common.responderError;
const DEFAULT_RESPONDER_END = _Common.responderEnd;
exports.createRequestListener = ({responderList = [], responderError = DEFAULT_RESPONDER_ERROR, responderEnd = DEFAULT_RESPONDER_END}) => async (request, response) => {
  const store = (0, _StateStore.createStateStoreLite)({
    time: (0, _time.clock)(),
    status: 500,
    error: null
  });
  store.socket = request.socket;
  store.request = request;
  store.response = response;
  try {
    for (const responder of responderList) await responder(store);
  } catch (error) {
    await responderError(store, error);
  }
  await responderEnd(store);
};
exports.describeServerOption = ({baseUrl, protocol, hostname, port}, title = "server", extraOption) => `[${title}]\n${(0, _format.prettyStringifyConfigObject)({
  pid: process.pid,
  baseUrl,
  ...hostname && "0.0.0.0" !== hostname && "[::]" !== hostname ? {} : (0, _Object.objectFromEntries)(Object.entries((0, _nodeOs.networkInterfaces)()).reduce(((o, [interfaceName, interfaceList]) => {
    interfaceList.forEach((({address, family, isIPv4 = "IPv4" === family}) => (hostname.startsWith("[") || isIPv4) && o.push([ isIPv4 ? address : `[${address}]`, interfaceName ])));
    return o;
  }), []).map((([address, interfaceName]) => [ `localUrl[${interfaceName}]`, `${protocol}//${address}:${port}` ]))),
  ...(0, _check.isBasicArray)(extraOption) && extraOption.length ? {
    extraOption
  } : (0, _check.isBasicObject)(extraOption) ? extraOption : {}
}, "  ", "  ")}`;