var _function = require("../../../common/function.js");
var _time = require("../../../common/time.js");
var _Iter = require("../../../common/data/Iter.js");
var _AsyncFuncQueue = require("../../../common/module/AsyncFuncQueue.js");
var _Runlet = require("../../../common/module/Runlet.js");
var _RunletChip = require("../../../common/module/RunletChip.js");
var _Stream = require("../../data/Stream.js");
var _frameDecode = require("./frameDecode.js");
var _frameEncode = require("./frameEncode.js");
const DEFAULT_BUFFER = Buffer.from("Dr");
exports.createWSBase = ({socket, info = {}, dataLengthLimit = 8388608, isMask = !1, shouldPing = !1}) => {
  let readyState = 0;
  const closeClear = async error => {
    if (3 !== readyState) {
      readyState = 3;
      Object.assign(info, {
        error
      });
      closeTimeoutToken && clearTimeout(closeTimeoutToken);
      pingCheckTimeoutToken && clearTimeout(pingCheckTimeoutToken);
      if (runletRead.getIsValid()) {
        LSAIRead.abort();
        runletRead.detach();
      }
      if (runletWrite.getIsValid()) {
        await pushLSAI(void 0, !0);
        await promise;
        runletWrite.detach();
      }
      socket.destroy();
    }
  };
  let isCloseReceived = !1;
  let closeTimeoutToken = null;
  const close = async (code = 1e3, reason = "") => {
    if (3 === readyState || 2 === readyState) return closeClear();
    if (1 !== readyState) throw new Error(`invalid readyState on close: ${readyState}`);
    readyState = 2;
    closeTimeoutToken = (0, _time.setWeakTimeout)(closeClear, 5e3);
    return pushLSAI((0, _frameEncode.createCloseFramePack)(code, reason)).then(isCloseReceived ? closeClear : () => promise, closeClear);
  };
  let pingBuffer = null;
  let pongBuffer = null;
  let pingCheckTimeoutToken = null;
  const ping = async (buffer = DEFAULT_BUFFER) => {
    if (1 === readyState && !pingBuffer) {
      pingCheckTimeoutToken && clearTimeout(pingCheckTimeoutToken);
      pingCheckTimeoutToken = (0, _time.setWeakTimeout)(pingCheck, 6e4);
      pingBuffer = buffer;
      pongBuffer = null;
      return pushLSAI((0, _frameEncode.encodePingFramePack)(buffer));
    }
  };
  const pingCheck = async () => pongBuffer && 0 === Buffer.compare(pingBuffer, pongBuffer) ? shouldPing && ping() : close(1006, "ping check failed");
  const onError = error => runletWrite.getIsValid() ? close(1006, `error: ${error.message}`) : closeClear(error);
  const LSAIRead = (0, _Iter.createLockStepAsyncIter)();
  const LSAIWrite = (0, _Iter.createLockStepAsyncIter)();
  const {push} = (0, _AsyncFuncQueue.createAsyncFuncQueue)();
  const pushLSAI = (framePack, done) => push((() => LSAIWrite.send(framePack, done)));
  const IOP = (0, _function.createInsideOutPromise)();
  const promise = IOP.promise.catch(onError);
  const runletRead = (0, _Runlet.createRunlet)((0, _Runlet.quickConfigPend)((0, _Runlet.toPoolMap)([ _Runlet.PoolIO, (0, _Runlet.createCountPool)({
    sizeLimit: 16
  }) ]), (0, _Runlet.toChipMap)((0, _Runlet.toLinearChipList)([ (0, _Stream.createReadableStreamInputChip)({
    stream: socket
  }), (0, _frameDecode.createFrameDecodeChip)({
    respondClose: async (code, reason) => {
      Object.assign(info, {
        code,
        reason
      });
      isCloseReceived = !0;
      return close(code, reason);
    },
    respondPing: async buffer => {
      if (1 === readyState) return pushLSAI((0, _frameEncode.encodePongFramePack)(buffer));
    },
    respondPong: buffer => {
      pongBuffer = buffer;
    },
    dataLengthLimit
  }), (0, _RunletChip.createAsyncIterOutputChip)({
    LSAI: LSAIRead
  }) ])), {
    onError
  }));
  const runletWrite = (0, _Runlet.createRunlet)((0, _Runlet.quickConfigPend)((0, _Runlet.toPoolMap)([ _Runlet.PoolIO, (0, _Runlet.createCountPool)({
    sizeLimit: 16
  }) ]), (0, _Runlet.toChipMap)((0, _Runlet.toLinearChipList)([ (0, _RunletChip.createAsyncIterInputChip)({
    next: LSAIWrite.next
  }), (0, _frameEncode.createFrameEncodeChip)({
    isMask,
    dataLengthLimit
  }), (0, _Stream.createWritableStreamOutputChip)({
    stream: socket,
    IOP
  }) ])), {
    onError
  }));
  readyState = 1;
  runletRead.attach();
  runletWrite.attach();
  runletRead.trigger();
  runletWrite.trigger();
  shouldPing && ping();
  return {
    ...(0, _Iter.wrapAsync)(LSAIRead.next),
    promise,
    CONNECTING: 0,
    OPEN: 1,
    CLOSING: 2,
    CLOSED: 3,
    socket,
    info,
    dataLengthLimit,
    isMask,
    shouldPing,
    getIsOpen: () => 1 === readyState,
    close,
    closeClear,
    sendText: async string => {
      if (1 !== readyState) throw new Error(`not open yet: readyState = ${readyState}`);
      return pushLSAI((0, _frameEncode.encodeTextFramePack)(string));
    },
    sendBinary: async buffer => {
      if (1 !== readyState) throw new Error(`not open yet: readyState = ${readyState}`);
      return pushLSAI((0, _frameEncode.encodeBinaryFramePack)(buffer));
    },
    ping
  };
};