var _function = require("../../../common/function.js");
var _net = require("../../net.js");
var _Base = require("./Base.js");
var _function2 = require("./function.js");
const VALID_PROTOCOL_SET = new Set([ "wss:", "ws:", "https:", "http:" ]);
const SECURE_PROTOCOL_SET = new Set([ "wss:", "https:" ]);
exports.createWSClient = (url, {protocolList, requestKey = (0, _function2.getRequestKey)(), onUpgradeResponse = (response, socket, headBuffer, info) => {}, dataLengthLimit, isMask = !0, shouldPing = !1, ...option}) => {
  url = url instanceof URL ? url : new URL(url);
  if (!VALID_PROTOCOL_SET.has(url.protocol)) throw new Error(`invalid url protocol: ${url.protocol}`);
  if (!url.host) throw new Error(`invalid url host: ${url.host}`);
  const isSecure = SECURE_PROTOCOL_SET.has(url.protocol);
  url.protocol = isSecure ? "https:" : "http:";
  option.headers = {
    ...option.headers,
    upgrade: "websocket",
    connection: "upgrade",
    "sec-websocket-version": _function2.WEBSOCKET_VERSION,
    "sec-websocket-key": requestKey,
    "sec-websocket-protocol": (0, _function2.packProtocolList)(protocolList)
  };
  const IOP = (0, _function.createInsideOutPromise)();
  const onError = error => {
    request.destroy();
    IOP.reject(error);
  };
  const {request, promise} = (0, _net.requestHttp)(url, option);
  promise.then((response => {
    onError(new Error("unexpected response"));
  }), onError);
  request.once("upgrade", (async (response, socket, headBuffer) => {
    try {
      const protocol = response.headers["sec-websocket-protocol"];
      if (!protocolList.includes(protocol)) throw new Error(`unexpected protocol ${protocol}`);
      if ((0, _function2.getRespondKey)(requestKey) !== response.headers["sec-websocket-accept"]) throw new Error("wrong sec-websocket-accept");
      const info = {
        isSecure,
        protocol,
        protocolList,
        headers: response.headers,
        url
      };
      await onUpgradeResponse(response, socket, headBuffer, info);
      IOP.resolve((0, _Base.createWSBase)({
        socket,
        info,
        dataLengthLimit,
        isMask,
        shouldPing
      }));
    } catch (error) {
      onError(error);
    }
  }));
  return IOP.promise;
};