var _time = require("../../../common/time.js");
var _StateStore = require("../../../common/immutable/StateStore.js");
var _Base = require("./Base.js");
var _function = require("./function.js");
exports.enableWSServer = (server, {onUpgradeRequest = (request, socket, headBuffer, info) => {}, onError = error => {
  console.error("[WSServer|onError]", error);
}, dataLengthLimit, isMask = !1, shouldPing = !0, wsSet = new Set}) => {
  server.on("upgrade", (async (request, socket, headBuffer) => {
    try {
      const requestKey = request.headers["sec-websocket-key"];
      const version = parseInt(request.headers["sec-websocket-version"]);
      const protocolList = (0, _function.parseProtocolString)(request.headers["sec-websocket-protocol"]);
      if (!requestKey || version !== _function.WEBSOCKET_VERSION || "GET" !== request.method || "websocket" !== request.headers.upgrade.toLowerCase()) throw new Error("invalid upgrade request");
      const info = {
        isSecure: request.socket.encrypted,
        protocol: void 0,
        protocolList,
        headers: request.headers
      };
      let ws;
      info.getWS = protocol => {
        if (void 0 !== ws) return ws;
        if (!protocol || !protocolList.includes(protocol)) throw new Error(`no/wrong selected protocol: "${protocol}"`);
        socket.write(joinHeader("HTTP/1.1 101 Switching Protocols", "upgrade: websocket", "connection: upgrade", `sec-websocket-accept: ${(0, _function.getRespondKey)(requestKey)}`, `sec-websocket-protocol: ${protocol}`));
        info.protocol = protocol;
        ws = (0, _Base.createWSBase)({
          socket,
          info,
          dataLengthLimit,
          isMask,
          shouldPing
        });
        wsSet.add(ws);
        ws.promise.then((() => {
          wsSet.delete(ws);
        }));
        return ws;
      };
      await onUpgradeRequest(request, socket, headBuffer, info);
    } catch (error) {
      socket.writableEnded || socket.end(joinHeader("HTTP/1.1 400 Bad Request", "connection: close"));
      onError(error);
    }
  }));
  return wsSet;
};
const joinHeader = (...args) => [ ...args, "\r\n" ].filter(Boolean).join("\r\n");
exports.createUpgradeRequestListener = ({responderList = [], responderError = DEFAULT_RESPONDER_ERROR, responderEnd = DEFAULT_RESPONDER_END}) => async (request, socket, headBuffer, info) => {
  const store = (0, _StateStore.createStateStoreLite)({
    time: (0, _time.clock)(),
    error: null,
    socket,
    headBuffer,
    info
  });
  store.request = request;
  store.response = {
    finished: !0,
    headersSent: !0,
    writableEnded: !0,
    statusCode: 101
  };
  store.ws = void 0;
  store.upgradeWS = protocol => {
    store.ws = info.getWS(protocol);
  };
  try {
    for (const responder of responderList) await responder(store);
  } catch (error) {
    await responderError(store, error);
  }
  await responderEnd(store);
};
const DEFAULT_RESPONDER_ERROR = (store, error) => {
  store.setState({
    error
  });
};
const DEFAULT_RESPONDER_END = store => store.ws && store.ws.closeClear(store.getState().error);