var _nodeCrypto = require("node:crypto");
var _Runlet = require("../../../common/module/Runlet.js");
var _function = require("./function.js");
const DEFAULT_MASK_QUADLET_BUFFER = Buffer.alloc(4);
exports.createFrameEncodeChip = ({isMask, dataLengthLimit = 16777216, key = "chip:frame-encode", ...extra}) => ({
  ...extra,
  key,
  state: {
    isMask,
    dataLengthLimit
  },
  sync: !0,
  process: (pack, state, error) => {
    if (error) return;
    if (pack[1] === _Runlet.END) return {
      pack,
      state
    };
    const {opcode, buffer} = pack[0];
    if (buffer.length > state.dataLengthLimit) throw new Error(`dataLength ${buffer.length} exceeds limit: ${dataLengthLimit}`);
    pack[0] = ((opcode, dataBuffer, isMask) => {
      const dataLength = dataBuffer.length;
      const [quadbitFIN, quadbitOpcodeMask] = _function.FRAME_CONFIG.COMPLETE;
      const initialOctet = quadbitFIN << 4 | opcode & quadbitOpcodeMask;
      let maskLengthOctet, extendLengthOctetCount;
      if (dataLength <= 125) {
        maskLengthOctet = dataLength;
        extendLengthOctetCount = 0;
      } else if (dataLength <= 65535) {
        maskLengthOctet = 126;
        extendLengthOctetCount = 2;
      } else {
        if (!(dataLength <= _function.BUFFER_MAX_LENGTH)) throw new Error(`dataLength too big: ${dataLength}`);
        maskLengthOctet = 127;
        extendLengthOctetCount = 8;
      }
      const maskOctetCount = isMask ? 4 : 0;
      const maskQuadletBuffer = isMask && dataLength ? (0, _nodeCrypto.randomBytes)(4) : DEFAULT_MASK_QUADLET_BUFFER;
      isMask && (maskLengthOctet |= 128);
      const headerBuffer = Buffer.allocUnsafe(2 + extendLengthOctetCount + maskOctetCount);
      headerBuffer.writeUInt16BE(initialOctet << 8 | maskLengthOctet, 0);
      2 === extendLengthOctetCount && headerBuffer.writeUInt16BE(dataLength, 2);
      8 === extendLengthOctetCount && headerBuffer.writeUInt32BE(0, 2);
      8 === extendLengthOctetCount && headerBuffer.writeUInt32BE(dataLength, 6);
      isMask && maskQuadletBuffer.copy(headerBuffer, 2 + extendLengthOctetCount);
      const frameBuffer = Buffer.concat([ headerBuffer, dataBuffer ]);
      isMask && dataLength && (0, _function.applyMaskQuadletBufferInPlace)(frameBuffer, maskQuadletBuffer, headerBuffer.length);
      return frameBuffer;
    })(opcode, buffer, state.isMask);
    return {
      pack,
      state
    };
  }
});
exports.encodeTextFramePack = string => ({
  opcode: _function.OPCODE_TYPE.TEXT,
  buffer: Buffer.from(string)
});
exports.encodeBinaryFramePack = buffer => ({
  opcode: _function.OPCODE_TYPE.BINARY,
  buffer
});
exports.createCloseFramePack = (code = 1e3, reason = "") => {
  const stringLength = Buffer.byteLength(reason);
  const buffer = Buffer.allocUnsafe(2 + stringLength);
  buffer.writeUInt16BE(code, 0);
  buffer.write(reason, 2, stringLength);
  return {
    opcode: _function.OPCODE_TYPE.CLOSE,
    buffer
  };
};
exports.encodePingFramePack = buffer => ({
  opcode: _function.OPCODE_TYPE.PING,
  buffer
});
exports.encodePongFramePack = buffer => ({
  opcode: _function.OPCODE_TYPE.PONG,
  buffer
});