var _nodeNet = require("node:net");
var _tryRequire = require("../../env/tryRequire.js");
var _error = require("../../common/error.js");
var _Stream = require("../data/Stream.js");
var _File = require("../fs/File.js");
const DR_BROWSER_FILE_PATH = () => [ "./Dr.browser.js", "../Dr.browser.js", "@dr-js/core/library/Dr.browser.js" ].reduce(((o, path) => o || (0, _tryRequire.tryRequireResolve)(path)), null);
exports.DR_BROWSER_FILE_PATH = DR_BROWSER_FILE_PATH;
let cache = "";
exports.DR_BROWSER_SCRIPT_TAG = () => {
  "" === cache && (cache = `<script>${(0, _File.readTextSync)(DR_BROWSER_FILE_PATH())}<\/script>`);
  return cache;
};
const getUnusedPort = (expectPort = 0, hostname = "0.0.0.0") => new Promise(((resolve, reject) => {
  const server = (0, _nodeNet.createServer)();
  server.on("error", reject);
  server.listen({
    host: hostname,
    port: expectPort,
    exclusive: !0
  }, (error => {
    if (error) return reject(error);
    const {port} = server.address();
    server.close((() => resolve(port)));
  }));
}));
exports.getUnusedPort = getUnusedPort;
exports.autoTestServerPort = async (expectPortList, hostname) => {
  for (const expectPort of expectPortList) {
    const {result, error} = await (0, _error.catchAsync)(getUnusedPort, expectPort, hostname);
    if (result) return result;
  }
  return getUnusedPort(0, hostname);
};
exports.parseHostString = (host, defaultHostname) => {
  const hostnameList = host.split(":");
  const port = Number(hostnameList.pop()) || void 0;
  return {
    hostname: hostnameList.join(":") || defaultHostname || void 0,
    port
  };
};
const parseCookieString = cookieString => cookieString.split(";").reduce(((o, v) => {
  const [key, ...valueList] = v.split("=");
  const value = valueList.join("=").trim();
  "" !== value && (o[key.trim()] = value);
  return o;
}), {});
exports.parseCookieString = parseCookieString;
exports.isRequestAborted = store => Boolean(store.request.destroyed || store.request.aborted);
exports.getRequestParam = (store, key) => {
  const {headers} = store.request;
  return headers[key] || store.getState().url.searchParams.get(key) || headers.cookie && decodeURIComponent(parseCookieString(headers.cookie)[key] || "") || store.info && getWSProtocolListParam(store.info.protocolList, key);
};
const getRequestBuffer = store => (0, _Stream.readableStreamToBufferAsync)(store.request);
exports.getRequestBuffer = getRequestBuffer;
exports.getRequestJSON = store => getRequestBuffer(store).then((buffer => 0 === buffer.length ? void 0 : JSON.parse(String(buffer))));
const getWSProtocolListParam = (protocolList = [], key) => {
  const header = `${encodeURIComponent(key)}=`;
  const protocol = protocolList.find((protocol => protocol.startsWith(header)));
  return protocol && decodeURIComponent(protocol.slice(header.length));
};
exports.getWSProtocolListParam = getWSProtocolListParam;
exports.packWSProtocolListParam = (key, value) => `${encodeURIComponent(key)}=${encodeURIComponent(value)}`;
exports.isPrivateAddress = address => "127.0.0.1" === address || "0.0.0.0" === address || "[::1]" === address || "[::]" === address || address.startsWith("192.168.") || address.startsWith("127.") || address.startsWith("10.") || address.startsWith("[fc") || address.startsWith("[fd") || address.startsWith("[fe") || "localhost" === address;