var _error = require("../../common/error.js");
var _function = require("../../common/function.js");
const EXIT_LISTENER_LIST = [ [ "exit", code => exitListenerSync({
  eventType: "exit",
  code
}) ], [ "uncaughtException", error => exitListenerAsync({
  eventType: "uncaughtException",
  error
}) ], [ "unhandledRejection", (error, promise) => exitListenerAsync({
  eventType: "unhandledRejection",
  error,
  promise
}) ], ...[ "SIGINT", "SIGHUP", "SIGQUIT", "SIGTERM" ].map((eventName => [ eventName, () => exitListenerAsync({
  eventType: "signal",
  signalEventType: eventName
}) ])) ];
const listenerSyncSet = new Set;
const listenerAsyncSet = new Set;
const exitListenerSync = eventPack => {
  isBind && unbindExitListener();
  for (const listenerSync of Array.from(listenerSyncSet)) (0, _error.catchSync)(listenerSync, eventPack);
  process.exitCode = eventPack.code || eventPack.error && 1 || 0;
};
const exitListenerAsync = async eventPack => {
  isBind && unbindExitListener();
  for (const listenerAsync of Array.from(listenerAsyncSet)) await (0, _error.catchAsync)(listenerAsync, eventPack);
  exitListenerSync(eventPack);
};
let isBind = !1;
const bindExitListener = () => {
  isBind = !0;
  EXIT_LISTENER_LIST.forEach((([eventName, listenerFunc]) => process.on(eventName, listenerFunc)));
};
const unbindExitListener = () => {
  isBind = !1;
  EXIT_LISTENER_LIST.forEach((([eventName, listenerFunc]) => process.off(eventName, listenerFunc)));
};
exports.clearExitListener = () => {
  isBind && unbindExitListener();
  listenerSyncSet.clear();
  listenerAsyncSet.clear();
};
const addExitListenerSync = (...listenerSyncList) => {
  listenerSyncList.forEach((listenerSync => listenerSyncSet.add(listenerSync)));
  !isBind && bindExitListener();
};
exports.addExitListenerSync = addExitListenerSync;
const addExitListenerAsync = (...listenerSyncList) => {
  listenerSyncList.forEach((listenerAsync => listenerAsyncSet.add(listenerAsync)));
  !isBind && bindExitListener();
};
exports.addExitListenerAsync = addExitListenerAsync;
exports.addExitListenerLossyOnce = listener => {
  const onceListener = (0, _function.once)(listener);
  addExitListenerAsync(onceListener);
  addExitListenerSync(onceListener);
  return onceListener;
};
exports.deleteExitListenerSync = (...listenerSyncList) => {
  listenerSyncList.forEach((listenerSync => listenerSyncSet.delete(listenerSync)));
};
exports.deleteExitListenerAsync = (...listenerSyncList) => {
  listenerSyncList.forEach((listenerAsync => listenerAsyncSet.delete(listenerAsync)));
};
exports.guardPromiseEarlyExit = async (callback, promise) => {
  const earlyExitCheck = code => 0 === code && !0 === isTestRunning && callback();
  let isTestRunning = !0;
  process.on("exit", earlyExitCheck);
  const {result, error} = await (0, _error.catchPromise)(promise);
  isTestRunning = !1;
  process.off("exit", earlyExitCheck);
  if (error) throw error;
  return result;
};