var _time = require("../../common/time.js");
var _string = require("../../common/string.js");
var _format = require("../../common/format.js");
var _Tree = require("../../common/data/Tree.js");
var _run = require("../run.js");
const GET_PROCESS_LIST_ASYNC_MAP = {
  INIT: !1,
  linux: null,
  win32: null,
  darwin: null,
  android: null
};
const getProcessListAsync = () => {
  !1 === GET_PROCESS_LIST_ASYNC_MAP.INIT && (() => {
    const createGetProcessListAsync = (commandString, lineSeparator, itemSwapFunc) => {
      const argList = commandString.split(" ");
      return async () => ((outputString, lineSeparator, itemSwapFunc = (v => v), processList = []) => {
        const [titleLine, ...rowList] = outputString.split(lineSeparator).filter(Boolean);
        const colStartIndexList = (titleString => {
          let flipCharType = " " === titleString.charAt(0);
          let flipCount = 0;
          const colStartIndexList = [ 0 ];
          for (let index = 0, indexMax = titleString.length; index < indexMax; index++) if (flipCharType !== (" " === titleString.charAt(index))) {
            flipCharType = !flipCharType;
            flipCount++;
            flipCount % 2 == 0 && colStartIndexList.push(index);
          }
          return colStartIndexList;
        })(titleLine);
        rowList.forEach((rowString => {
          const [pid, ppid, memory, command] = itemSwapFunc(((rowString, colStartIndexList, itemList = []) => {
            for (let index = 0, indexMax = colStartIndexList.length; index < indexMax; index++) itemList.push(rowString.slice(colStartIndexList[index], colStartIndexList[index + 1]));
            return itemList;
          })(rowString, colStartIndexList));
          processList.push({
            pid: parseInt(pid),
            ppid: parseInt(ppid),
            memory: parseInt(memory),
            command: command.trim()
          });
        }));
        return processList;
      })(String(await (0, _run.runStdout)(argList)), lineSeparator, itemSwapFunc);
    };
    const getProcessListAsyncLinux = createGetProcessListAsync("ps -wweo pid,ppid,rss=MEMORY__,args", "\n", (([pid, ppid, rss, command]) => [ pid, ppid, 1024 * parseInt(rss), command ]));
    const getProcessListAsyncWin32 = createGetProcessListAsync("WMIC PROCESS get Commandline,Name,ParentProcessId,ProcessId,WorkingSetSize", "\r\r\n", (([command, name, ppid, pid, wss]) => [ pid, ppid, wss, command.trim() || name ]));
    Object.assign(GET_PROCESS_LIST_ASYNC_MAP, {
      INIT: !0,
      linux: getProcessListAsyncLinux,
      win32: getProcessListAsyncWin32,
      darwin: getProcessListAsyncLinux,
      android: getProcessListAsyncLinux
    });
  })();
  const getAsync = GET_PROCESS_LIST_ASYNC_MAP[process.platform];
  if (!getAsync) throw new Error(`unsupported platform: ${process.platform}`);
  return getAsync();
};
exports.getProcessListAsync = getProcessListAsync;
const PROCESS_LIST_SORT_MAP = {
  "pid++": (a, b) => a.pid - b.pid,
  "pid--": (a, b) => b.pid - a.pid,
  "ppid++": (a, b) => a.ppid - b.ppid || a.pid - b.pid,
  "ppid--": (a, b) => b.ppid - a.ppid || a.pid - b.pid,
  "memory++": (a, b) => a.memory - b.memory,
  "memory--": (a, b) => b.memory - a.memory
};
const sortProcessList = (processList, sortOrder = "pid--") => processList.sort(PROCESS_LIST_SORT_MAP[sortOrder]);
exports.sortProcessList = sortProcessList;
exports.toProcessPidMap = processList => processList.reduce(((o, info) => {
  o[info.pid] = info;
  return o;
}), {});
const toProcessTree = processList => {
  const rootInfo = {
    pid: 0,
    ppid: -1,
    memory: 0,
    command: "ROOT"
  };
  const processMap = {
    0: rootInfo
  };
  const subTreeMap = {
    0: {}
  };
  for (const info of processList) {
    const {pid, ppid} = info;
    if (0 !== pid) {
      processMap[pid] = info;
      void 0 === subTreeMap[ppid] && (subTreeMap[ppid] = {});
      subTreeMap[ppid][pid] = info;
    }
  }
  processMap[rootInfo.pid] = rootInfo;
  Object.entries(subTreeMap).forEach((([ppid, subTree]) => {
    let info = processMap[ppid];
    if (!info) {
      info = {
        pid: ppid,
        ppid: rootInfo.pid,
        memory: 0,
        command: "..."
      };
      subTreeMap[info.ppid][info.pid] = info;
    }
    info.subTree = subTree;
  }));
  return rootInfo;
};
exports.toProcessTree = toProcessTree;
exports.flattenProcessTree = (processTree, processList = []) => {
  processTreeDepthFirstSearch(processTree, (info => {
    processList.push(info);
  }));
  return processList;
};
const isInfoMatch = ({pid, ppid, command}, info) => info && info.pid === pid && (void 0 === ppid || info.ppid === ppid) && (void 0 === command || info.command === command);
const findProcessListInfo = (info, processList) => processList.find((v => isInfoMatch(info, v)));
exports.findProcessListInfo = findProcessListInfo;
exports.findProcessPidMapInfo = (info, processPidMap) => isInfoMatch(info, processPidMap[info.pid]) ? processPidMap[info.pid] : void 0;
exports.findProcessTreeInfo = (info, processTree) => processTreeDepthFirstSearch(processTree, (searchInfo => isInfoMatch(info, searchInfo)));
const killProcessInfoAsync = async info => {
  const isExistAsync = async () => isPidExist(info.pid) && findProcessListInfo(info, await getProcessListAsync());
  for (const [wait, signal] of [ [ 50 ], [ 100 ], [ 250 ], [ 500 ], [ 500 ], [ 1e3, "SIGKILL" ] ]) {
    if (!await isExistAsync()) return;
    try {
      process.kill(info.pid, signal);
    } catch (error) {}
    await (0, _time.setTimeoutAsync)(wait);
  }
  if (await isExistAsync()) throw new Error(`failed to stop process, pid: ${info.pid}, ppid: ${info.ppid}, command: ${info.command}`);
};
exports.killProcessInfoAsync = killProcessInfoAsync;
exports.killProcessTreeInfoAsync = async (processTreeInfo, isMatchPpid = !1) => {
  const killAsync = isMatchPpid ? killProcessInfoAsync : ({pid, command}) => killProcessInfoAsync({
    pid,
    command
  });
  await processTreeBottomUpSearchAsync(processTreeInfo, killAsync);
  await killAsync(processTreeInfo);
};
const getSubNodeListFunc = info => info.subTree && Object.values(info.subTree);
const processTreeDepthFirstSearch = (0, _Tree.createTreeDepthFirstSearch)(getSubNodeListFunc);
const processTreeBottomUpSearchAsync = (0, _Tree.createTreeBottomUpSearchAsync)(getSubNodeListFunc);
const getAllProcessStatusAsync = async outputMode => {
  const processList = await getProcessListAsync();
  return outputMode.startsWith("t") ? toProcessTree(processList) : sortProcessList(processList, outputMode);
};
exports.getAllProcessStatusAsync = getAllProcessStatusAsync;
exports.describeAllProcessStatusAsync = async outputMode => {
  const status = await getAllProcessStatusAsync(outputMode);
  if (outputMode.startsWith("t")) {
    const text = prettyStringifyProcessTree(status);
    return "tree-wide" === outputMode | "tw" === outputMode ? text : text.split("\n").map((line => (0, _string.autoEllipsis)(line, 128, 96, 16))).join("\n");
  }
  return (0, _format.padTable)({
    table: [ [ "PID", "PPID", "MEMORY", "COMMAND" ], ...status.map((({pid, ppid, memory, command}) => [ pid, ppid, memory, command ])) ],
    padFuncList: [ "R", "R", "R", "L" ]
  });
};
const prettyStringifyProcessTree = processRootInfo => {
  const resultList = [];
  const pad = (value, count) => `${value}`.padStart(count, " ");
  const addLine = (prefix, {pid, memory, command}) => {
    return resultList.push(`${pad(pid, 8)} ${value = memory, pad(-1 === value ? "MEMORY" : `${(0, _format.binary)(value)}B`, 10)} ${prefix}${command}`);
    var value;
  };
  addLine("", {
    pid: "PID",
    memory: -1,
    command: "COMMAND"
  });
  (0, _Tree.prettyStringifyTreeNode)((([info, level, hasMore]) => info.subTree && Object.values(info.subTree).map(((subInfo, subIndex, {length}) => [ subInfo, level + 1, subIndex !== length - 1 ]))), [ processRootInfo, -1, !1 ], addLine);
  return resultList.join("\n");
};
const isPidExist = pid => {
  try {
    process.kill(pid, 0);
    return !0;
  } catch (error) {
    return "EPERM" === error.code;
  }
};
exports.isPidExist = isPidExist;