import { createCanvasElement, canvasElementToCanvasImageData, canvasImageDataToCanvasElement } from "../ImageData.js";
import { scale as scaleCanvasImageData } from "../ImageDataOperation.js";
const createFontGenerator = (getSymbolMetrics = (() => {
  const bufferCanvasContext2d = createCanvasElement(256, 256).getContext("2d");
  return (symbol, fontConfig) => {
    bufferCanvasContext2d.font = fontConfig.attribute;
    return bufferCanvasContext2d.measureText(symbol);
  };
})()) => {
  const symbolCanvasElementMap = {};
  const scaledSymbolCanvasElementMap = {};
  const renderSymbol = (symbol, fontConfig) => symbolCanvasElementMap[symbol] && symbolCanvasElementMap[symbol][fontConfig.cacheTag] || ((symbol, fontConfig, metricsWidth) => {
    const context = createCanvasElement(metricsWidth, fontConfig.lineHeight).getContext("2d");
    context.font = fontConfig.attribute;
    context.textAlign = "start";
    context.textBaseline = "middle";
    context.fillStyle = fontConfig.fillStyle;
    context.fillText(symbol, 0, .5 * fontConfig.lineHeight);
    void 0 === symbolCanvasElementMap[symbol] && (symbolCanvasElementMap[symbol] = {});
    symbolCanvasElementMap[symbol][fontConfig.cacheTag] = context.canvas;
    return context.canvas;
  })(symbol, fontConfig, getSymbolMetrics(symbol, fontConfig).width);
  return {
    getFontConfig: ({fontSize = 12, lineHeight = 16, fontStyle = "normal", fontFamily = "monospace", fillStyle = "#000"}) => ({
      fontSize,
      lineHeight,
      fontStyle,
      fontFamily,
      fillStyle,
      attribute: `${fontStyle} ${fontSize}px/${lineHeight}px ${fontFamily}`,
      cacheTag: `${fontStyle}|${fontSize}|${lineHeight}|${fontFamily}|${fillStyle}`
    }),
    renderSymbol,
    renderSymbolScaled: (symbol, scaleRatio, fontConfig) => {
      const cacheKey = `${symbol}|${scaleRatio}:${fontConfig.cacheTag}`;
      let scaledSymbolCanvasElement = scaledSymbolCanvasElementMap[cacheKey];
      if (!scaledSymbolCanvasElement) {
        const symbolCanvasElement = renderSymbol(symbol, fontConfig);
        const canvasImageData = canvasElementToCanvasImageData(symbolCanvasElement);
        const scaledCanvasImageData = scaleCanvasImageData(canvasImageData, scaleRatio, scaleRatio);
        scaledSymbolCanvasElement = canvasImageDataToCanvasElement(scaledCanvasImageData);
        scaledSymbolCanvasElementMap[cacheKey] = scaledSymbolCanvasElement;
      }
      return scaledSymbolCanvasElement;
    }
  };
};
export { createFontGenerator };