import { loadText, loadImage } from "../function.js";
import { createCanvasElement, applyImageElementExt, canvasElementToCanvasImageData, canvasImageDataToCanvasElement } from "../ImageData.js";
import { scale as scaleCanvasImageData } from "../ImageDataOperation.js";
const createFontGeneratorBitmap = () => {
  let defaultSymbol = null;
  let symbolCanvasElementMap = {};
  let scaledSymbolCanvasElementMap = {};
  const renderSymbol = symbol => symbolCanvasElementMap[symbol] || symbolCanvasElementMap[defaultSymbol];
  return {
    getDefaultSymbol: () => defaultSymbol,
    loadBitmapFontData: async bitmapFontDataSrc => {
      const {symbolDataList, defaultSymbol: bitmapDefaultSymbol, fontImageSrc} = JSON.parse(await loadText(bitmapFontDataSrc));
      const bitMapImageElementExt = applyImageElementExt(await loadImage(fontImageSrc));
      const bitmapSymbolMetricsMap = {};
      const bitmapSymbolCanvasElementMap = {};
      for (let index = 0, indexMax = symbolDataList.length; index < indexMax; index++) {
        const [symbol, rectX, rectY, rectWidth, rectHeight, xInc, xOffset, yOffset] = symbolDataList[index];
        const symbolCanvasElement = createCanvasElement(rectWidth, rectHeight);
        bitMapImageElementExt.drawClip(symbolCanvasElement.getContext("2d"), 0, 0, rectX, rectY, rectWidth, rectHeight);
        bitmapSymbolMetricsMap[symbol] = {
          xInc,
          xOffset,
          yOffset
        };
        bitmapSymbolCanvasElementMap[symbol] = symbolCanvasElement;
      }
      defaultSymbol = bitmapDefaultSymbol || "?";
      symbolCanvasElementMap = bitmapSymbolCanvasElementMap;
      scaledSymbolCanvasElementMap = {};
    },
    renderSymbol,
    renderSymbolScaled: (symbol, scaleRatio) => {
      const cacheKey = `${symbol}|${scaleRatio}`;
      let scaledSymbolCanvasElement = scaledSymbolCanvasElementMap[cacheKey];
      if (!scaledSymbolCanvasElement) {
        const symbolCanvasElement = renderSymbol(symbol);
        const canvasImageData = canvasElementToCanvasImageData(symbolCanvasElement);
        const scaledCanvasImageData = scaleCanvasImageData(canvasImageData, scaleRatio, scaleRatio);
        scaledSymbolCanvasElement = canvasImageDataToCanvasElement(scaledCanvasImageData);
        scaledSymbolCanvasElementMap[cacheKey] = scaledSymbolCanvasElement;
      }
      return scaledSymbolCanvasElement;
    }
  };
};
export { createFontGeneratorBitmap };