const DEFAULT_ON_MISSING_REQUEST = symbol => null;
const createFontMapper = (fontSize = 16, lineHeight = 20, onMissingRequest = DEFAULT_ON_MISSING_REQUEST) => {
  let configFontSize = fontSize;
  let configLineHeight = lineHeight;
  const symbolMetricsMap = {};
  let symbolDefault;
  let symbolMetricsDefault;
  const setDefaultSymbol = (symbol = "?", deltaX = configFontSize, offsetX = 0, offsetY = 0) => {
    symbolDefault = symbol;
    symbolMetricsDefault = {
      deltaX,
      offsetX,
      offsetY
    };
  };
  setDefaultSymbol("?");
  return {
    setDefaultSymbol,
    setOnMissingRequest: nextOnMissingRequest => {
      onMissingRequest = nextOnMissingRequest;
    },
    setConfig: (fontSize, lineHeight) => {
      configFontSize = fontSize;
      configLineHeight = lineHeight;
    },
    addSymbol: (symbol, deltaX, offsetX = 0, offsetY = 0) => {
      const symbolMetrics = {
        deltaX,
        offsetX,
        offsetY
      };
      symbolMetricsMap[symbol] = symbolMetrics;
      return symbolMetrics;
    },
    autoMapping: (text, scaleRatio, limitWidth, callback) => {
      const mapLimitWidth = limitWidth || 0;
      const mapLineHeight = configLineHeight * (scaleRatio = scaleRatio || 1);
      const cursorPosition = {
        x: 0,
        y: 0
      };
      let widthMax = 0;
      for (let index = 0, length = text.length; index < length; index++) {
        let symbol = text.charAt(index);
        let symbolMetrics = symbolMetricsMap[symbol] || onMissingRequest(symbol);
        if (!symbolMetrics) {
          console.warn(`[FontMapper][autoMapping] missing metrics for symbol: ${symbol}`);
          symbol = symbolDefault;
          symbolMetrics = symbolMetricsDefault;
        }
        switch (symbol) {
         case "\n":
         case "\r":
          cursorPosition.x = 0;
          cursorPosition.y += mapLineHeight;
          break;
         case "\t":
          cursorPosition.x += symbolMetricsDefault.deltaX * scaleRatio * 4;
          break;
         default:
          if (0 !== mapLimitWidth && cursorPosition.x + symbolMetrics.deltaX * scaleRatio > mapLimitWidth) {
            cursorPosition.x = 0;
            cursorPosition.y += mapLineHeight;
          }
          callback && callback(index, symbol, cursorPosition.x + symbolMetrics.offsetX * scaleRatio, cursorPosition.y + symbolMetrics.offsetY * scaleRatio);
          cursorPosition.x += symbolMetrics.deltaX * scaleRatio;
        }
        widthMax = Math.max(widthMax, cursorPosition.x);
      }
      return {
        size: {
          width: widthMax,
          height: cursorPosition.y + mapLineHeight
        },
        position: cursorPosition
      };
    }
  };
};
export { createFontMapper };