import { createCanvasElement } from "../ImageData.js";
import { createFontMapper } from "./fontMapper.js";
import { createFontGenerator } from "./fontGenerator.js";
const createFontRender = (fontMapper = createFontMapper(), fontGenerator = createFontGenerator()) => {
  let fontConfig;
  fontMapper.setOnMissingRequest((symbol => {
    const symbolImageData = fontGenerator.renderSymbol(symbol, fontConfig);
    return fontMapper.addSymbol(symbol, symbolImageData.width, 0, 0);
  }));
  const applyFontConfig = (nextFontConfig = {}) => {
    fontConfig = fontGenerator.getFontConfig(nextFontConfig);
    const {fontSize, lineHeight} = fontConfig;
    const defaultSymbolImageData = fontGenerator.renderSymbol("?", fontConfig);
    fontMapper.setConfig(fontSize, lineHeight);
    fontMapper.setDefaultSymbol("?", defaultSymbolImageData.width, 0, 0);
  };
  applyFontConfig();
  return {
    applyFontConfig,
    renderText: (text, scaleRatio, limitWidth, textCanvasElement = createCanvasElement(0, 0)) => {
      let textAreaMetrics = fontMapper.autoMapping(text, scaleRatio, limitWidth, void 0);
      textCanvasElement.width = textAreaMetrics.size.width;
      textCanvasElement.height = textAreaMetrics.size.height;
      const textCanvasElementContext2d = textCanvasElement.getContext("2d");
      textAreaMetrics = fontMapper.autoMapping(text, scaleRatio, limitWidth, ((index, symbol, x, y) => {
        const scaledSymbolCanvasElement = fontGenerator.renderSymbolScaled(symbol, scaleRatio, fontConfig);
        textCanvasElementContext2d.drawImage(scaledSymbolCanvasElement, x, y);
      }));
      return {
        textCanvasElement,
        textAreaMetrics,
        textEndPosition: textAreaMetrics.position
      };
    }
  };
};
export { createFontRender };