import { createCanvasElement } from "../ImageData.js";
import { createFontMapper } from "./fontMapper.js";
import { createFontGeneratorBitmap } from "./fontGeneratorBitmap.js";
const createFontRenderBitmap = (fontMapper = createFontMapper(), fontGenerator = createFontGeneratorBitmap()) => {
  fontMapper.setOnMissingRequest((symbol => {
    const symbolImageData = fontGenerator.renderSymbol(symbol);
    return fontMapper.addSymbol(symbol, symbolImageData.width, 0, 0);
  }));
  return {
    loadBitmapFontData: async (bitmapFontDataSrc, fontSize, lineHeight) => {
      fontMapper.setConfig(fontSize, lineHeight);
      await fontGenerator.loadBitmapFontData(bitmapFontDataSrc);
      fontMapper.setDefaultSymbol(fontGenerator.getDefaultSymbol(), fontGenerator.renderSymbol().width, 0, 0);
    },
    renderText: (text, scaleRatio, limitWidth, textCanvasElement = createCanvasElement(0, 0)) => {
      let textAreaMetrics = fontMapper.autoMapping(text, scaleRatio, limitWidth, void 0);
      textCanvasElement.width = textAreaMetrics.size.width;
      textCanvasElement.height = textAreaMetrics.size.height;
      const textCanvasElementContext2d = textCanvasElement.getContext("2d");
      textAreaMetrics = fontMapper.autoMapping(text, scaleRatio, limitWidth, ((index, symbol, x, y) => {
        const scaledSymbolCanvasElement = fontGenerator.renderSymbolScaled(symbol, scaleRatio);
        textCanvasElementContext2d.drawImage(scaledSymbolCanvasElement, x, y);
      }));
      return {
        textCanvasElement,
        textAreaMetrics,
        textEndPosition: textAreaMetrics.position
      };
    }
  };
};
export { createFontRenderBitmap };