import { CLOCK_PER_SECOND } from "../../common/time.js";
import { linear } from "../../common/math/easing.js";
import { add, sub, getLength, scale, lerp } from "../../common/geometry/D2/Vector.js";
import { createInterpolationAutoTimer, createVectorAccumulator } from "../module/MotionAutoTimer.js";
import { POINTER_EVENT_TYPE, ENHANCED_POINTER_EVENT_TYPE } from "./PointerEvent.js";
const {START} = POINTER_EVENT_TYPE;
const {DRAG_MOVE, DRAG_END, DRAG_CANCEL} = ENHANCED_POINTER_EVENT_TYPE;
const createSwipeEnhancedEventProcessor = ({getPointStart, updatePoint, getExitInfo, normalizeVector = vector => vector, timeFunction = linear, accumulatedTimeRange}) => {
  let isPause = !1;
  let pointStart;
  let pointExit;
  let prevTime;
  let prevPoint;
  const {stop, start} = createInterpolationAutoTimer({
    func: rate => updatePoint(lerp(pointStart, pointExit, timeFunction(rate)))
  });
  const {reset: resetVectorAccumulator, accumulate, getAverageVector} = createVectorAccumulator(accumulatedTimeRange);
  const reset = () => {
    stop();
    resetVectorAccumulator();
    isPause = !1;
    pointStart = void 0;
    pointExit = void 0;
    prevTime = void 0;
    prevPoint = void 0;
  };
  return {
    reset,
    pause: () => {
      reset();
      isPause = !0;
    },
    onEnhancedEvent: (name, eventState) => {
      if (isPause || name !== DRAG_END && name !== DRAG_CANCEL && name !== DRAG_MOVE) return;
      const {timeStart, pointStart: eventPointStart, time, point} = eventState;
      const pointCurrent = add(pointStart, normalizeVector(sub(point, eventPointStart)));
      updatePoint(pointCurrent, eventState);
      const duration = time - (prevTime || timeStart);
      accumulate(scale(normalizeVector(sub(point, prevPoint || pointStart)), CLOCK_PER_SECOND / duration), duration);
      prevTime = time;
      prevPoint = point;
      if (name === DRAG_MOVE) return;
      const exitVector = normalizeVector(getAverageVector());
      const {pointExit: nextPointExit, exitDuration} = getExitInfo({
        exitVector,
        pointCurrent,
        pointStart
      });
      if (exitDuration) {
        pointStart = pointCurrent;
        pointExit = nextPointExit;
        start(exitDuration);
      }
    },
    onEvent: (name, event, calcState) => {
      if (!isPause && name === START) {
        reset();
        pointStart = getPointStart(calcState(event));
      }
    }
  };
};
export { createSwipeEnhancedEventProcessor };