import { CLOCK_PER_SECOND, clock, createTimer, requestFrameUpdate, cancelFrameUpdate } from "../../common/time.js";
import { clamp } from "../../common/math/base.js";
import { fromOrigin, getLength, add, scale } from "../../common/geometry/D2/Vector.js";
const createInterpolationAutoTimer = ({func, queueTask = requestFrameUpdate, cancelTask = cancelFrameUpdate}) => {
  let prevRate = 1;
  let timeStart = 0;
  let timeDuration = 0;
  const {start: startTimer, stop, isActive} = createTimer({
    func: () => {
      prevRate = clamp((clock() - timeStart) / timeDuration, 0, 1);
      1 === prevRate && (() => {
        stop();
        prevRate = 1;
        timeStart = 0;
        timeDuration = 0;
      })();
      func(prevRate);
    },
    queueTask,
    cancelTask
  });
  return {
    stop,
    start: duration => {
      if (duration <= 0) throw new Error(`[InterpolationAutoTimer] invalid nextDurationTime: ${duration}`);
      prevRate = 0;
      timeStart = clock();
      timeDuration = duration * CLOCK_PER_SECOND;
      startTimer();
    },
    isActive,
    getRate: () => prevRate
  };
};
const createVectorAccumulator = (accumulatedTimeRange = 24) => {
  let accumulatedTimeVector = fromOrigin();
  let accumulatedTime = 0;
  return {
    reset: () => {
      accumulatedTimeVector = fromOrigin();
      accumulatedTime = 0;
    },
    accumulate: (vector, time) => {
      if (time >= accumulatedTimeRange) {
        accumulatedTimeVector = scale(vector, accumulatedTimeRange);
        accumulatedTime = accumulatedTimeRange;
      } else if (time + accumulatedTime <= accumulatedTimeRange) {
        accumulatedTimeVector = add(accumulatedTimeVector, scale(vector, time));
        accumulatedTime += time;
      } else {
        const keepRate = accumulatedTime ? (accumulatedTimeRange - time) / accumulatedTime : 0;
        accumulatedTimeVector = add(scale(accumulatedTimeVector, keepRate), scale(vector, time));
        accumulatedTime = accumulatedTime * keepRate + time;
      }
    },
    getAverageVector: () => accumulatedTime ? scale(accumulatedTimeVector, 1 / accumulatedTime) : fromOrigin()
  };
};
export { createInterpolationAutoTimer, createVectorAccumulator };