/** @type { (value: unknown) => value is string } */
const isString = value => "string" == typeof value
/** @type { (value: unknown) => value is boolean } */;
const isBoolean = value => "boolean" == typeof value
/** @type { (value: unknown) => boolean } */;
const isTruthy = value => Boolean(value)
/** @type { (value: unknown) => value is number } */;
const isNumber = value => "number" == typeof value
/** @type { (value: unknown) => value is number } */;
const isInteger = /** @type { (value: unknown) => value is number } */ /** @type { unknown } */ Number.isInteger;
/** @type { (value: unknown) => value is RegExp } */ const isRegExp = value => value instanceof RegExp
/** @type { (value: unknown) => value is ArrayBuffer } */;
const isArrayBuffer = value => value instanceof ArrayBuffer
/** @type { (value: unknown) => value is Object } */;
const isObjectAlike = value => {
  const type = typeof value;
  return "object" === type && null !== value || "function" === type;
};
/** @typedef { Object & { [Symbol.iterator]?: never } & { [Symbol.hasInstance]?: never } } BasicObject */
/** @type { (value: unknown) => value is BasicObject } */ const isBasicObject = value => "object" == typeof value && null !== value && !Array.isArray(value)
/** @type { (value: unknown, key: string) => value is BasicObject } */;
const isObjectKey = (value, key) => isBasicObject(value) && Object.prototype.hasOwnProperty.call(value, key)
/** @type { (value: unknown, target: Record<string, unknown>) => value is BasicObject } */;
const isObjectContain = (value, target) => isBasicObject(value) && Object.entries(target).every((([key, targetValue]) => value[key] === targetValue))
/** @type { (value: unknown) => value is Array } */;
const isBasicArray = Array.isArray;
/** @type { (value: unknown, length: number) => value is Array } */ const isArrayLength = (value, length) => isBasicArray(value) && value.length === length
/** @type { (value: unknown) => value is Function } */;
const isBasicFunction = value => "function" == typeof value
/** @type { (value: unknown) => value is Promise } */;
const isPromiseAlike = value => Boolean(value && "function" == typeof /** @type { Promise } */ /** @type { unknown } */ value.then)
/** @type { (value: unknown, validList: unknown[]) => boolean } */;
const isOneOf = (value, validList) => validList.includes(value)
/** @type { (func: Function) => boolean } */;
const isFunctionThrow = func => {
  try {
    return func() && !1;
  } catch (error) {
    return !0;
  }
};
/** @type { (func: Function) => Promise<boolean> } */ const isFunctionThrowAsync = async func => {
  try {
    return await func() && !1;
  } catch (error) {
    return !0;
  }
};
/** @type { <T> (actual: unknown, expect: T) => actual is T } */ const isStrictEqual = (actual, expect) => Object.is(actual, expect)
/** @type { <T> (actual: unknown, expect: T) => actual is T } */;
const isStringifyEqual = (actual, expect) => !!Object.is(actual, expect) || !(!isObjectAlike(actual) || !isObjectAlike(expect)) && JSON.stringify(actual) === JSON.stringify(expect);
export { isString, isBoolean, isTruthy, isNumber, isInteger, isRegExp, isArrayBuffer, isObjectAlike, isBasicObject, isObjectKey, isObjectContain, isBasicArray, isArrayLength, isBasicFunction, isPromiseAlike, isOneOf, isFunctionThrow, isFunctionThrowAsync, isStrictEqual, isStringifyEqual };