import { tryRequire } from "../../env/tryRequire.js";
/** @type { (a: ArrayBuffer, b: ArrayBuffer) => boolean } */ const isEqualArrayBuffer = (a, b) => {
  if (a === b) return !0;
  if (a.byteLength !== b.byteLength) return !1;
  const va = new Uint8Array(a);
  const vb = new Uint8Array(b);
  for (let index = 0, indexMax = a.byteLength; index < indexMax; index++) if (va[index] !== vb[index]) return !1;
  return !0;
};
/** @type { (v: ArrayBuffer[]) => ArrayBuffer } */ const concatArrayBuffer = (arrayBufferList = []) => {
  const resultTypedArray = new Uint8Array(arrayBufferList.reduce(((o, arrayBuffer) => o + arrayBuffer.byteLength), 0));
  let byteOffset = 0;
  arrayBufferList.forEach((arrayBuffer => {
    const {byteLength} = arrayBuffer;
    resultTypedArray.set(new Uint8Array(arrayBuffer), byteOffset);
    byteOffset += byteLength;
  }));
  return resultTypedArray.buffer;
};
/** @type { (v: ArrayBuffer, a: number[], b: number) => ArrayBuffer[] } */ const deconcatArrayBuffer = (concatedArrayBuffer, byteLengthList, byteOffset = 0) => byteLengthList.map((byteLength => {
  const arrayBuffer = concatedArrayBuffer.slice(byteOffset, byteOffset + byteLength);
  byteOffset += byteLength;
  return arrayBuffer;
}))
/** @typedef { string } U16String */;
const PREFIX_ODD = Uint8Array.of(0);
const PREFIX_EVEN = Uint8Array.of(255, 255);
const encodeU16Chunk = (dataView, index, indexMax) => {
  const charCodeList = [];
  for (;index < indexMax; index++) charCodeList.push(String.fromCharCode(dataView.getUint16(2 * index, !1)));
  return charCodeList.join("");
};
/** @type { (v: ArrayBuffer) => U16String } */ const toU16String = arrayBuffer => {
  const packArrayBuffer = concatArrayBuffer([ arrayBuffer.byteLength % 2 ? PREFIX_ODD : PREFIX_EVEN, arrayBuffer ]);
  const dataView = new DataView(packArrayBuffer);
  const stringList = [];
  for (let index = 0, indexMax = dataView.byteLength / 2; index < indexMax; index += 12288) stringList.push(encodeU16Chunk(dataView, index, Math.min(indexMax, index + 12288)));
  return stringList.join("");
};
/** @type { (v: U16String) => ArrayBuffer } */ const fromU16String = (string = "") => {
  const dataView = new DataView(new ArrayBuffer(2 * string.length));
  for (let index = 0, indexMax = string.length; index < indexMax; index++) dataView.setUint16(2 * index, string.charCodeAt(index), !1);
  return dataView.buffer.slice(0 === dataView.getUint8(0) ? PREFIX_ODD.byteLength : PREFIX_EVEN.byteLength);
};
/** @type { (v: Buffer) => ArrayBuffer } */ const fromNodejsBuffer = nodejsBuffer => {
  const {buffer: arrayBuffer, byteOffset, byteLength} = nodejsBuffer;
  return arrayBuffer.byteLength === byteLength ? arrayBuffer : arrayBuffer.slice(byteOffset, byteOffset + byteLength);
};
/** @type { (v: ArrayBuffer) => Promise<ArrayBuffer> } */ const calcSHA256ArrayBuffer = (() => {
  try {
    const {crypto, isSecureContext} = globalThis;
    const calcSHA256ArrayBuffer = async arrayBuffer => crypto.subtle.digest("SHA-256", arrayBuffer);
    if (isSecureContext && crypto.subtle.digest) return calcSHA256ArrayBuffer;
  } catch (error) {}
  try {
    const {createHash} = tryRequire("node:crypto");
    const calcSHA256ArrayBuffer = async arrayBuffer => {
      const buffer = createHash("sha256").update(Buffer.from(arrayBuffer)).digest();
      return fromNodejsBuffer(buffer);
    };
    if (createHash && createHash("sha256")) return calcSHA256ArrayBuffer;
  } catch (error) {}
  return async arrayBuffer => {
    throw new Error("not supported");
  };
})();
export { isEqualArrayBuffer, concatArrayBuffer, deconcatArrayBuffer, fromU16String, toU16String, fromNodejsBuffer, calcSHA256ArrayBuffer };