import { concatArrayBuffer, deconcatArrayBuffer, fromU16String, toU16String } from "./ArrayBuffer.js";
import { decodeUTF8, encodeUTF8 } from "../module/TextEnDecoder.js";
/** @typedef { import("./ArrayBuffer.js").U16String } U16String */ const MAX_PACKET_HEADER_SIZE = 4294967295;
const HEADER_BYTE_SIZE = 4;
const EMPTY_ARRAY_BUFFER = new ArrayBuffer(0);
/** @type { (v: ArrayBuffer) => [ headerSize: ArrayBuffer, header: ArrayBuffer ] } */ const packArrayBufferHeader = headerArrayBuffer => {
  const headerSize = headerArrayBuffer.byteLength;
  if (headerSize > 4294967295) throw new Error(`headerArrayBuffer exceeds max size 4294967295, get: ${headerSize}`);
  const headerSizeDataView = new DataView(new ArrayBuffer(4));
  headerSizeDataView.setUint32(0, headerSize, !1);
  return [ headerSizeDataView.buffer, headerArrayBuffer ];
};
/** @type { (v: ArrayBuffer) => [ header: ArrayBuffer, payloadOffset: number ] } */ const parseArrayBufferHeader = arrayBufferPair => {
  const headerSize = new DataView(arrayBufferPair, 0, 4).getUint32(0, !1);
  return [ arrayBufferPair.slice(4, 4 + headerSize), 4 + headerSize ];
};
/** @type { (header: string, payload?: ArrayBuffer) => ArrayBuffer } */ const packArrayBufferPacket2 = (headerString, payloadArrayBuffer = EMPTY_ARRAY_BUFFER) => concatArrayBuffer([ ...packArrayBufferHeader(encodeUTF8(headerString).buffer), payloadArrayBuffer ])
/** @type { (v: ArrayBuffer) => [ header: U16String, payload: ArrayBuffer ] } */;
const parseArrayBufferPacket2 = arrayBufferPacket => {
  const [headerArrayBuffer, payloadOffset] = parseArrayBufferHeader(arrayBufferPacket);
  return [ decodeUTF8(headerArrayBuffer), arrayBufferPacket.slice(payloadOffset) ];
};
/** @type { (header: U16String, payload?: ArrayBuffer) => ArrayBuffer } */ const packArrayBufferPacket = (headerU16String, payloadArrayBuffer = EMPTY_ARRAY_BUFFER) => concatArrayBuffer([ ...packArrayBufferHeader(fromU16String(headerU16String)), payloadArrayBuffer ])
/** @type { (v: ArrayBuffer) => [ header: U16String, payload: ArrayBuffer ] } */;
const parseArrayBufferPacket = arrayBufferPacket => {
  const [headerArrayBuffer, payloadOffset] = parseArrayBufferHeader(arrayBufferPacket);
  return [ toU16String(headerArrayBuffer), arrayBufferPacket.slice(payloadOffset) ];
};
/** @type { (v: ArrayBuffer[]) => ArrayBuffer } */ const packChainArrayBufferPacket = (arrayBufferList = []) => {
  const headerDataView = new DataView(new ArrayBuffer(4 * arrayBufferList.length));
  arrayBufferList.forEach((({byteLength}, index) => headerDataView.setUint32(4 * index, byteLength, !1)));
  return concatArrayBuffer([ ...packArrayBufferHeader(headerDataView.buffer), ...arrayBufferList ]);
};
/** @type { (v: ArrayBuffer) => ArrayBuffer[] } */ const parseChainArrayBufferPacket = chainArrayBufferPacket => {
  const [headerArrayBuffer, payloadOffset] = parseArrayBufferHeader(chainArrayBufferPacket);
  const headerDataView = new DataView(headerArrayBuffer);
  const byteLengthList = [];
  for (let index = 0, indexMax = headerDataView.byteLength / 4; index < indexMax; index++) byteLengthList.push(headerDataView.getUint32(4 * index, !1));
  return deconcatArrayBuffer(chainArrayBufferPacket, byteLengthList, payloadOffset);
};
/** @type { (v: ArrayBuffer[]) => ArrayBuffer } */ const packArrayBufferListPacket = arrayBufferList => {
  const packetList = [];
  for (const arrayBuffer of arrayBufferList) {
    const headerSize = arrayBuffer.byteLength;
    if (headerSize > 4294967295) throw new Error(`arrayBuffer exceeds max size 4294967295, get: ${headerSize}`);
    const headerSizeDataView = new DataView(new ArrayBuffer(4));
    headerSizeDataView.setUint32(0, headerSize, !1);
    packetList.push(headerSizeDataView.buffer, arrayBuffer);
  }
  return concatArrayBuffer(packetList);
};
/** @type { (v: ArrayBuffer) => ArrayBuffer[] } */ const parseArrayBufferListPacket = arrayBufferPacket => {
  const arrayBufferList = [];
  let index = 0;
  for (;index < arrayBufferPacket.byteLength; ) {
    const headerSize = new DataView(arrayBufferPacket.slice(index, index + 4)).getUint32(0, !1);
    arrayBufferList.push(arrayBufferPacket.slice(index + 4, index + 4 + headerSize));
    index += 4 + headerSize;
  }
  return arrayBufferList;
};
export { MAX_PACKET_HEADER_SIZE, HEADER_BYTE_SIZE, packArrayBufferHeader, parseArrayBufferHeader, packArrayBufferPacket, parseArrayBufferPacket, packArrayBufferPacket2, parseArrayBufferPacket2, packChainArrayBufferPacket, parseChainArrayBufferPacket, packArrayBufferListPacket, parseArrayBufferListPacket };