import { isString } from "../check.js";
import { objectFromEntries } from "../immutable/Object.js";
import { encode as encodeBase64, decode as decodeBase64 } from "./Base64.js";
/** @typedef { {
 *    value: string | ArrayBuffer,
 *    mime?: string,
 *    paramMap?: {},
 * } } DataUriSource */
/** @type { (option: DataUriSource) => string } */ const encode = ({value, mime, paramMap}) => {
  const isPayloadBase64 = !isString(value);
  const payloadString = (isPayloadBase64 ? encodeBase64 : encodeURIComponent)(value);
  return `data:${[ mime, paramMap && String(new URLSearchParams(paramMap)).replace(/&/g, ";"), isPayloadBase64 && "base64" ].filter(Boolean).join(";")},${payloadString}`;
};
/** @type { (v: string) => DataUriSource } */ const decode = (string = "") => {
  const payloadIndex = string.indexOf(",");
  const headerStringList = string.slice(5, payloadIndex).split(";").filter(Boolean);
  const isPayloadBase64 = "base64" === headerStringList[headerStringList.length - 1];
  isPayloadBase64 && headerStringList.pop();
  const mime = headerStringList.length && !headerStringList[0].includes("=") && headerStringList.shift() || void 0;
  const paramMap = 0 === headerStringList.length ? void 0 : objectFromEntries(new URLSearchParams(headerStringList.join("&")).entries());
  return {
    value: (isPayloadBase64 ? decodeBase64 : decodeURIComponent)(string.slice(payloadIndex + 1)),
    mime,
    paramMap
  };
};
export { encode, decode };