const createTreeDepthFirstSearch = getSubNodeListFunc => {
  const unshiftStack = (stack, node, extra) => {
    const nodeList = getSubNodeListFunc(node, extra);
    Array.isArray(nodeList) && stack.unshift(...nodeList);
  };
  return (initialNode, func, extra) => {
    const stack = [];
    unshiftStack(stack, initialNode, extra);
    let node;
    for (;node = stack.shift(); ) {
      if (func(node)) return node;
      unshiftStack(stack, node, extra);
    }
  };
};
const createTreeDepthFirstSearchAsync = getSubNodeListFunc => {
  const unshiftStackAsync = async (stack, node, extra) => {
    const nodeList = await getSubNodeListFunc(node, extra);
    Array.isArray(nodeList) && stack.unshift(...nodeList);
  };
  return async (initialNode, func, extra) => {
    const stack = [];
    await unshiftStackAsync(stack, initialNode, extra);
    let node;
    for (;node = stack.shift(); ) {
      if (await func(node)) return node;
      await unshiftStackAsync(stack, node, extra);
    }
  };
};
const createTreeBreadthFirstSearch = getSubNodeListFunc => {
  const pushStack = (stack, node, extra) => {
    const nodeList = getSubNodeListFunc(node, extra);
    Array.isArray(nodeList) && stack.push(...nodeList);
  };
  return (initialNode, func, extra) => {
    const stack = [];
    pushStack(stack, initialNode, extra);
    let node;
    for (;node = stack.shift(); ) {
      if (func(node)) return node;
      pushStack(stack, node, extra);
    }
  };
};
const createTreeBreadthFirstSearchAsync = getSubNodeListFunc => {
  const pushStackAsync = async (stack, node, extra) => {
    const nodeList = await getSubNodeListFunc(node, extra);
    Array.isArray(nodeList) && stack.push(...nodeList);
  };
  return async (initialNode, func, extra) => {
    const stack = [];
    await pushStackAsync(stack, initialNode, extra);
    let node;
    for (;node = stack.shift(); ) {
      if (await func(node)) return node;
      await pushStackAsync(stack, node, extra);
    }
  };
};
const createTreeBottomUpSearch = getSubNodeListFunc => {
  const pushPairStack = (pairStack, node, extra) => {
    const nodeList = getSubNodeListFunc(node, extra);
    return Boolean(Array.isArray(nodeList) && pairStack.push([ node, nodeList ]));
  };
  return (initialNode, func, extra) => {
    const pairStack = [];
    pushPairStack(pairStack, initialNode, extra);
    for (;0 !== pairStack.length; ) {
      const [node, subNodeList] = pairStack[pairStack.length - 1];
      if (0 === subNodeList.length) {
        pairStack.pop();
        if (0 !== pairStack.length && func(node)) return node;
      } else {
        const subNode = subNodeList.shift();
        if (!pushPairStack(pairStack, subNode, extra) && func(subNode)) return subNode;
      }
    }
  };
};
const createTreeBottomUpSearchAsync = getSubNodeListFunc => {
  const pushPairStackAsync = async (pairStack, node, extra) => {
    const nodeList = await getSubNodeListFunc(node, extra);
    return Boolean(Array.isArray(nodeList) && pairStack.push([ node, nodeList ]));
  };
  return async (initialNode, func, extra) => {
    const pairStack = [];
    await pushPairStackAsync(pairStack, initialNode, extra);
    for (;0 !== pairStack.length; ) {
      const [node, subNodeList] = pairStack[pairStack.length - 1];
      if (0 === subNodeList.length) {
        pairStack.pop();
        if (0 !== pairStack.length && await func(node)) return node;
      } else {
        const subNode = subNodeList.shift();
        if (!await pushPairStackAsync(pairStack, subNode, extra) && await func(subNode)) return subNode;
      }
    }
  };
};
const prettyStringifyTreeNode = (getSubNodeLevelHasMoreListFunc, initialNode, func, extra) => {
  const studList = [];
  createTreeDepthFirstSearch(getSubNodeLevelHasMoreListFunc)(initialNode, (([node, level, hasMore]) => {
    studList.length = level;
    0 !== level && (studList[level - 1] = hasMore ? "├─ " : "└─ ");
    func(studList.join(""), node);
    0 !== level && (studList[level - 1] = hasMore ? "│  " : "   ");
  }), extra);
};
export { createTreeDepthFirstSearch, createTreeDepthFirstSearchAsync, createTreeBreadthFirstSearch, createTreeBreadthFirstSearchAsync, createTreeBottomUpSearch, createTreeBottomUpSearchAsync, prettyStringifyTreeNode };