/** @type { <T extends Error>(error: T, nextMessage: string) => T } } */
const remessageError = (error, nextMessage) => {
  error.message && error.stack && (error.stack = error.stack.replace(error.message, nextMessage));
  error.message = nextMessage;
  return error;
};
/** @type { (error: Error) => never } } */ const rethrowError = error => {
  console.warn(error);
  throw error;
};
/** @type { <A extends any[], R, FR> (fallbackResult: FR, func: (...args: A) => R, ...args: A) => FR | R } } */ const withFallbackResult = (fallbackResult, func, ...args) => {
  try {
    return func(...args);
  } catch (error) {
    return fallbackResult;
  }
};
/** @type { <A extends any[], R, FR> (fallbackResult: FR, func: (...args: A) => R, ...args: A) => Promise<FR | R> } } */ const withFallbackResultAsync = async (fallbackResult, func, ...args) => {
  try {
    return await func(...args);
  } catch (error) {
    return fallbackResult;
  }
};
/** @type { <A extends any[], R> (func: (...args: A) => R, ...args: A) => { result: R, error: undefined } | { result: undefined, error: Error } } } */ const catchSync = (func, ...args) => {
  let result, resultError;
  try {
    result = func(...args);
  } catch (error) {
    resultError = error || new Error;
  }
  return {
    result,
    error: resultError
  };
};
/** @type { <A extends any[], R> (func: (...args: A) => R, ...args: A) => Promise<{ result: R, error: undefined } | { result: undefined, error: Error }> } } */ const catchAsync = async (func, ...args) => {
  let result, resultError;
  try {
    result = await func(...args);
  } catch (error) {
    resultError = error || new Error;
  }
  return {
    result,
    error: resultError
  };
};
/** @type { <T> (promise: Promise<T>) => Promise<{ result: T, error: undefined } | { result: undefined, error: Error }> } } */ const catchPromise = promise => promise.then(packResult, packError);
const packResult = result => ({
  result,
  error: void 0
});
const packError = error => ({
  result: void 0,
  error: error || new Error
});
export { remessageError, rethrowError, withFallbackResult, withFallbackResultAsync, catchSync, catchAsync, catchPromise };