const typeNameOf = value => Object.prototype.toString.call(value).slice(8, -1);
const escapeString = string => JSON.stringify(string).slice(1, -1);
const quickStringify = (valueType, value) => "String" === valueType ? JSON.stringify(value) : "Object" === valueType ? `{${escapeString(Object.keys(value))}}` : "Array" === valueType ? value.length <= 8 ? `[${value.map(describe).join(", ")}]` : `[#${value.length}]` : "RegExp" === valueType ? String(value) : valueType.endsWith("Function") ? value.name || "anonymous" : escapeString(String(value));
const describe = value => {
  const valueType = typeNameOf(value);
  return `<${valueType}> ${quickStringify(valueType, value)}`;
};
const percent = value => `${(100 * value).toFixed(2)}%`;
const twoDigit = value => String(Math.floor(value)).padStart(2, "0");
const mediaTime = value => {
  const abs = Math.abs(value);
  return `${value < 0 ? "-" : ""}${twoDigit(abs / 60)}:${twoDigit(abs % 60)}`;
};
const decimal = value => {
  const abs = .75 * Math.abs(value);
  return 0 === abs ? "0" : abs < 1e-9 ? `${(value / 1e-12).toFixed(2)}pico` : abs < 1e-6 ? `${(value / 1e-9).toFixed(2)}nano` : abs < .001 ? `${(value / 1e-6).toFixed(2)}micro` : abs < 1 ? `${(value / .001).toFixed(2)}milli` : abs < 1e3 ? `${(value / 1).toFixed(2)}` : abs < 1e6 ? `${(value / 1e3).toFixed(2)}kilo` : abs < 1e9 ? `${(value / 1e6).toFixed(2)}mega` : abs < 1e12 ? `${(value / 1e9).toFixed(2)}giga` : abs < 1e15 ? `${(value / 1e12).toFixed(2)}tera` : abs < 1e18 ? `${(value / 1e15).toFixed(2)}peta` : `${(value / 1e18).toFixed(2)}exa`;
};
const time = value => {
  const abs = .75 * Math.abs(value);
  return abs < 1e3 ? `${Math.floor(value)}ms` : abs < 6e4 ? `${(value / 1e3).toFixed(2)}s` : abs < 36e5 ? `${(value / 6e4).toFixed(2)}m` : abs < 864e5 ? `${(value / 36e5).toFixed(2)}h` : `${(value / 864e5).toFixed(2)}d`;
};
const binary = value => {
  const abs = .75 * Math.abs(value);
  return abs < 1024 ? `${Math.floor(value)}` : abs < 1048576 ? `${(value / 1024).toFixed(2)}Ki` : abs < 1073741824 ? `${(value / 1048576).toFixed(2)}Mi` : abs < 1099511627776 ? `${(value / 1073741824).toFixed(2)}Gi` : `${(value / 1099511627776).toFixed(2)}Ti`;
};
const padTable = ({table, padFuncList = [], cellPad = " | ", rowPad = "\n", widthMaxList = table.reduce(((o, rowList) => {
  rowList.forEach(((value, index) => {
    o[index] = Math.max(String(value).length, o[index] || 0);
  }));
  return o;
}), [])}) => table.map((rowList => rowList.map(((value, index) => {
  const string = String(value);
  const padFunc = padFuncList[index];
  const maxWidth = widthMaxList[index];
  return padFunc && "L" !== padFunc ? "R" === padFunc ? string.padStart(maxWidth) : padFunc(string, maxWidth) : string.padEnd(maxWidth);
})).join(cellPad))).join(rowPad);
const prettyStringifyJSON = (value, unfoldLevel = 2, pad = "  ") => {
  const stringifySwitch = (resultList, value, level, padString) => {
    if (level >= 1 && value) {
      if (Array.isArray(value)) return stringifyArray(resultList, value, level, padString);
      if ("object" == typeof value) return stringifyObject(resultList, value, level, padString);
    }
    const result = JSON.stringify(value);
    const isSkippedResult = void 0 === result;
    !isSkippedResult && resultList.push(result);
    return isSkippedResult;
  };
  const stringifyObject = (resultList, object, level, padString) => {
    const keyList = Object.keys(object);
    resultList.push("{\n");
    const resultListLength = resultList.length;
    const nextLevel = level - 1;
    const nextPadString = `${padString}${pad}`;
    for (let index = 0, indexMax = keyList.length; index < indexMax; index++) {
      const key = keyList[index];
      const value = object[key];
      const startIndex = resultList.length;
      resultList.push("");
      if (stringifySwitch(resultList, value, nextLevel, nextPadString)) resultList.length--; else {
        resultList[startIndex] = `${nextPadString}${JSON.stringify(key)}: `;
        resultList.push(",\n");
      }
    }
    resultList[resultList.length - 1] = resultList.length === resultListLength ? "{}" : `\n${padString}}`;
  };
  const stringifyArray = (resultList, array, level, padString) => {
    resultList.push("[\n");
    const resultListLength = resultList.length;
    const nextLevel = level - 1;
    const nextPadString = `${padString}${pad}`;
    for (let index = 0, indexMax = array.length; index < indexMax; index++) {
      const value = array[index];
      resultList.push(nextPadString);
      stringifySwitch(resultList, value, nextLevel, nextPadString) && resultList.push("null");
      resultList.push(",\n");
    }
    resultList[resultList.length - 1] = resultList.length === resultListLength ? "[]" : `\n${padString}]`;
  };
  const resultList = [];
  stringifySwitch(resultList, value, Math.max(unfoldLevel, 0) || 0, "");
  return resultList.join("");
};
const prettyStringifyConfigObject = (value, pad = "  ", padStringInitial = "") => {
  if ("string" != typeof pad || pad.length < 2) throw new Error(`invalid pad: ${String(pad)}`);
  const isLastResultTag = resultList => {
    const value = resultList[resultList.length - 1];
    return ": " === value || "- " === value;
  };
  const stringifySwitch = (resultList, value, padString) => {
    const valueType = typeNameOf(value);
    if ("Array" === valueType) return stringifyArray(resultList, value, padString);
    if ("Object" === valueType) return stringifyObject(resultList, value, padString);
    resultList.push(valueType.endsWith("Function") ? valueType : quickStringify(valueType, value), "\n");
  };
  const stringifyObject = (resultList, object, padString) => {
    const keyList = Object.keys(object);
    if (0 === keyList.length) return resultList.push("{}", "\n");
    isLastResultTag(resultList) && resultList.push("\n");
    for (let index = 0, indexMax = keyList.length; index < indexMax; index++) {
      const key = keyList[index];
      const value = object[key];
      resultList.push(padString, escapeString(key), ": ");
      stringifySwitch(resultList, value, `${padString}${pad}`);
    }
  };
  const stringifyArray = (resultList, array, padString) => {
    if (0 === array.length) return resultList.push("[]", "\n");
    isLastResultTag(resultList) && resultList.push("\n");
    for (let index = 0, indexMax = array.length; index < indexMax; index++) {
      const value = array[index];
      resultList.push(padString, "- ");
      stringifySwitch(resultList, value, `${padString}${pad}`);
    }
  };
  const resultList = [];
  stringifySwitch(resultList, value, padStringInitial);
  resultList.length--;
  1 === resultList.length && resultList.unshift(padStringInitial);
  return resultList.join("");
};
export { typeNameOf, describe, percent, mediaTime, decimal, time, binary, padTable, prettyStringifyJSON, prettyStringifyConfigObject };