const fromEmpty = () => ({
  left: 1 / 0,
  right: -1 / 0,
  top: 1 / 0,
  bottom: -1 / 0
});
const fromPoint = (a, b) => ({
  left: Math.min(a.x, b.x),
  right: Math.max(a.x, b.x),
  top: Math.min(a.y, b.y),
  bottom: Math.max(a.y, b.y)
});
const fromWidget = ({center, size, rotate}) => {
  const cos = Math.cos(rotate);
  const sin = Math.sin(rotate);
  const halfBoundingWidth = .5 * (Math.abs(size.x * cos) + Math.abs(size.y * sin));
  const halfBoundingHeight = .5 * (Math.abs(size.x * sin) + Math.abs(size.y * cos));
  return {
    left: center.x - halfBoundingWidth,
    right: center.x + halfBoundingWidth,
    top: center.y - halfBoundingHeight,
    bottom: center.y + halfBoundingHeight
  };
};
const fromWidgetList = widgetList => widgetList.reduce(((boundingRect, {center, size, rotate}) => {
  const cos = Math.cos(rotate);
  const sin = Math.sin(rotate);
  const halfBoundingWidth = .5 * (Math.abs(size.x * cos) + Math.abs(size.y * sin));
  const halfBoundingHeight = .5 * (Math.abs(size.x * sin) + Math.abs(size.y * cos));
  boundingRect.left = Math.min(boundingRect.left, center.x - halfBoundingWidth);
  boundingRect.right = Math.max(boundingRect.right, center.x + halfBoundingWidth);
  boundingRect.top = Math.min(boundingRect.top, center.y - halfBoundingHeight);
  boundingRect.bottom = Math.max(boundingRect.bottom, center.y + halfBoundingHeight);
  return boundingRect;
}), {
  left: 1 / 0,
  right: -1 / 0,
  top: 1 / 0,
  bottom: -1 / 0
});
const getCenter = ({left, right, top, bottom}) => ({
  x: .5 * (left + right),
  y: .5 * (top + bottom)
});
const getUnion = (a, b) => ({
  left: Math.min(a.left, b.left),
  right: Math.max(a.right, b.right),
  top: Math.min(a.top, b.top),
  bottom: Math.max(a.bottom, b.bottom)
});
const isIntersect = (a, b) => !(a.left > b.right || a.right < b.left || a.top > b.bottom || a.bottom < b.top);
const isContainPoint = ({left, right, top, bottom}, {x, y}) => x >= left && x <= right && y >= top && y <= bottom;
export { fromEmpty, fromPoint, fromWidget, fromWidgetList, getCenter, getUnion, isIntersect, isContainPoint };