import { roundFloat } from "../../math/base.js";
import { getDist, getRotate, round as roundVector } from "./Vector.js";
const PI_HALF = .5 * Math.PI;
const fromPoint = (pointA, pointB, data) => ({
  ...data,
  center: {
    x: .5 * (pointA.x + pointB.x),
    y: .5 * (pointA.y + pointB.y)
  },
  size: {
    x: Math.abs(pointA.x - pointB.x),
    y: Math.abs(pointA.y - pointB.y)
  },
  rotate: 0
});
const fromLine = ({begin, end}, width, data) => ({
  ...data,
  center: {
    x: .5 * (begin.x + end.x),
    y: .5 * (begin.y + end.y)
  },
  size: {
    x: width,
    y: getDist(end, begin)
  },
  rotate: getRotate(end, begin) - PI_HALF
});
const fromBoundingRect = ({left, right, top, bottom}, data) => ({
  ...data,
  center: {
    x: .5 * (left + right),
    y: .5 * (top + bottom)
  },
  size: {
    x: right - left,
    y: bottom - top
  },
  rotate: 0
});
const getBoundingSize = ({size: {x, y}, rotate}) => {
  const cos = Math.cos(rotate);
  const sin = Math.sin(rotate);
  return {
    x: Math.abs(x * cos) + Math.abs(y * sin),
    y: Math.abs(x * sin) + Math.abs(y * cos)
  };
};
const getBoundingWidth = ({size: {x, y}, rotate}) => Math.abs(x * Math.cos(rotate)) + Math.abs(y * Math.sin(rotate));
const getBoundingHeight = ({size: {x, y}, rotate}) => Math.abs(x * Math.sin(rotate)) + Math.abs(y * Math.cos(rotate));
const getBoundingLeft = widget => widget.center.x - .5 * getBoundingWidth(widget);
const getBoundingRight = widget => widget.center.x + .5 * getBoundingWidth(widget);
const getBoundingTop = widget => widget.center.y - .5 * getBoundingHeight(widget);
const getBoundingBottom = widget => widget.center.y + .5 * getBoundingHeight(widget);
const round = widget => ({
  ...widget,
  center: roundVector(widget.center),
  size: roundVector(widget.size),
  rotate: roundFloat(widget.rotate)
});
const localPoint = ({center, rotate}, {x, y}) => {
  const offsetX = x - center.x;
  const offsetY = y - center.y;
  const cos = Math.cos(-rotate);
  const sin = Math.sin(-rotate);
  return {
    x: offsetX * cos - offsetY * sin,
    y: offsetX * sin + offsetY * cos
  };
};
const localBoundingRect = ({center, rotate}, targetWidget) => {
  const offsetX = targetWidget.center.x - center.x;
  const offsetY = targetWidget.center.y - center.y;
  const offsetRotate = targetWidget.rotate - rotate;
  const cos = Math.cos(offsetRotate);
  const sin = Math.sin(offsetRotate);
  const x = offsetX * cos - offsetY * sin;
  const y = offsetX * sin + offsetY * cos;
  const halfBoundingWidth = .5 * (targetWidget.size.x * cos - targetWidget.size.y * sin);
  const halfBoundingHeight = .5 * (targetWidget.size.x * sin + targetWidget.size.y * cos);
  return {
    left: x - halfBoundingWidth,
    right: x + halfBoundingWidth,
    top: y - halfBoundingHeight,
    bottom: y + halfBoundingHeight
  };
};
const isContainBoundingRect = ({center, size}, boundingRect) => center.x - .5 * size.x <= boundingRect.left && center.x + .5 * size.x >= boundingRect.right && center.y - .5 * size.y <= boundingRect.top && center.y + .5 * size.y >= boundingRect.bottom;
const isInterceptBoundingRect = ({center, size}, boundingRect) => !(center.x + .5 * size.x < boundingRect.left || center.x - .5 * size.x > boundingRect.right || center.y + .5 * size.y < boundingRect.top || center.y - .5 * size.y > boundingRect.bottom);
export { fromPoint, fromLine, fromBoundingRect, getBoundingSize, getBoundingWidth, getBoundingHeight, getBoundingLeft, getBoundingRight, getBoundingTop, getBoundingBottom, round, localPoint, localBoundingRect, isContainBoundingRect, isInterceptBoundingRect };