import { basicObject as verifyBasicObject, basicFunction as verifyBasicFunction } from "../verify.js";
import { objectMerge } from "./Object.js";
import { arrayMatchPush, arrayMatchDelete } from "./Array.js";
const createStateStore = state => {
  verifyBasicObject(state, "initialState should be basic Object");
  let listenerList = [];
  return {
    subscribe: listener => {
      listenerList = arrayMatchPush(listenerList, listener);
    },
    unsubscribe: listener => {
      listenerList = arrayMatchDelete(listenerList, listener);
    },
    getState: () => state,
    setState: nextState => {
      if ((nextState = objectMerge(state, nextState)) === state) return state;
      const prevState = state;
      state = nextState;
      listenerList.forEach((listener => listener(state, prevState)));
      return state;
    }
  };
};
/** @type { <T extends Record<string, any>> (state: T) => { getState: () => T, setState: (nextState: Partial<T>) => T } } */ const createStateStoreLite = state => ({
  getState: () => state,
  setState: nextState => state = {
    ...state,
    ...nextState
  }
});
const createStateStoreEnhanced = ({initialState, enhancer, reducer}) => {
  verifyBasicFunction(enhancer, "enhancer function required");
  verifyBasicFunction(reducer, "reducer function required");
  const {subscribe, unsubscribe, getState: getStoreState, setState: setStoreState} = createStateStore(initialState);
  let dispatchingState = null;
  let isDispatching = !1;
  let isReducing = !1;
  const getState = () => isDispatching ? dispatchingState : getStoreState();
  const dispatch = action => isDispatching ? (action => {
    enhancer(enhancerStore, action);
    dispatchingState = reducer(dispatchingState, action);
  })(action) : (action => {
    dispatchingState = getStoreState();
    isDispatching = !0;
    enhancer(enhancerStore, action);
    isDispatching = !1;
    isReducing = !0;
    const nextState = reducer(dispatchingState, action);
    isReducing = !1;
    dispatchingState = null;
    setStoreState(nextState);
  })(action);
  const enhancerStore = {
    getState,
    dispatch
  };
  return {
    subscribe,
    unsubscribe,
    getState,
    dispatch
  };
};
const toReduxStore = store => {
  const {subscribe: subscribeStore, unsubscribe} = store;
  verifyBasicFunction(subscribeStore, "store.subscribe required");
  verifyBasicFunction(unsubscribe, "store.unsubscribe required");
  return {
    ...store,
    subscribe: listener => {
      subscribeStore(listener);
      return () => unsubscribe(listener);
    }
  };
};
const reducerFromMap = reducerMap => {
  const keyList = Object.keys(reducerMap);
  const keyListLength = keyList.length;
  return (state, action) => {
    const nextState = {};
    let isChanged = !1;
    for (let index = 0; index < keyListLength; index++) {
      const key = keyList[index];
      const keyState = state[key];
      const nextKeyState = reducerMap[key](keyState, action);
      nextState[key] = nextKeyState;
      isChanged = isChanged || nextKeyState !== keyState;
    }
    return isChanged ? nextState : state;
  };
};
const createEntryEnhancer = entryMap => (enhancerStore, action) => {
  const entryFunction = entryMap[action.type];
  return entryFunction && entryFunction(enhancerStore, action);
};
const createStoreStateSyncReducer = (actionType, {getState, setState}) => (state, {type, payload}) => {
  type === actionType && setState(payload);
  return getState();
};
export { createStateStore, createStateStoreLite, createStateStoreEnhanced, toReduxStore, reducerFromMap, createEntryEnhancer, createStoreStateSyncReducer };