import { getTimestamp } from "../time.js";
import { createAsyncFuncQueue } from "./AsyncFuncQueue.js";
import { ASYNC_TASK_KEY_MAP, runAsyncTask } from "./AsyncTask.js";
const {PLAN_PROMISE, OUTPUT} = ASYNC_TASK_KEY_MAP;
const ASYNC_TASK_QUEUE_KEY_MAP = {
  ...ASYNC_TASK_KEY_MAP,
  STATUS: "status"
};
const createAsyncTaskQueue = () => {
  const {getLength, getTailPromise, reset, push: pushAsyncFunc} = createAsyncFuncQueue();
  return {
    getLength,
    getTailPromise,
    reset,
    push: asyncTask => {
      asyncTask.status = {
        pushAt: getTimestamp(),
        runAt: void 0,
        doneAt: void 0
      };
      return pushAsyncFunc((() => {
        asyncTask.status.runAt = getTimestamp();
        const promise = runAsyncTask(asyncTask);
        const onTaskDone = () => {
          asyncTask.status.doneAt = getTimestamp();
        };
        asyncTask[PLAN_PROMISE].then(onTaskDone, onTaskDone);
        return promise;
      }));
    }
  };
};
const createFilterStaleAsyncTask = (maxStaleTime, maxDoneAt = getTimestamp() - Math.abs(maxStaleTime)) => asyncTask => !asyncTask[OUTPUT] || asyncTask.status.doneAt > maxDoneAt;
export { ASYNC_TASK_QUEUE_KEY_MAP, createAsyncTaskQueue, createFilterStaleAsyncTask };