import { isEqualArrayBuffer, fromU16String, toU16String, calcSHA256ArrayBuffer } from "../data/ArrayBuffer.js";
import { packChainArrayBufferPacket, parseChainArrayBufferPacket } from "../data/ArrayBufferPacket.js";
import { encode } from "../data/Base64.js";
const packArrayBufferChunk = async ({chunkArrayBuffer, key, chunkIndex, chunkTotal}, isSkipVerifyHash = !1) => packChainArrayBufferPacket([ fromU16String(JSON.stringify({
  key,
  chunkIndex,
  chunkTotal,
  chunkByteLength: chunkArrayBuffer.byteLength
})), isSkipVerifyHash ? new ArrayBuffer(0) : await calcSHA256ArrayBuffer(chunkArrayBuffer), chunkArrayBuffer ]);
const parseArrayBufferChunk = async (arrayBufferPacket, allowSkipHashVerify = !0) => {
  const [headerArrayBuffer, chunkHashArrayBuffer, chunkArrayBuffer] = parseChainArrayBufferPacket(arrayBufferPacket);
  const {key, chunkIndex, chunkTotal, chunkByteLength} = JSON.parse(toU16String(headerArrayBuffer));
  if (chunkByteLength !== chunkArrayBuffer.byteLength) throw new Error(`chunk length mismatch, get: ${chunkArrayBuffer.byteLength}, expect ${chunkByteLength}`);
  if (chunkHashArrayBuffer.byteLength) {
    const verifyChunkHashBuffer = await calcSHA256ArrayBuffer(chunkArrayBuffer);
    if (!isEqualArrayBuffer(chunkHashArrayBuffer, verifyChunkHashBuffer)) throw new Error(`chunk ${chunkIndex} of ${key} hash mismatch, get: ${encode(verifyChunkHashBuffer)}, expect ${encode(chunkHashArrayBuffer)}`);
  } else if (!allowSkipHashVerify) throw new Error(`missing chunk ${chunkIndex} of ${key} hash to verify`);
  return {
    chunkArrayBuffer,
    key,
    chunkIndex,
    chunkTotal
  };
};
const uploadArrayBufferByChunk = async ({arrayBuffer, size = arrayBuffer && arrayBuffer.byteLength, getChunk = arrayBuffer && ((index, chunkSize) => arrayBuffer.slice(index, index + chunkSize)), key, chunkSizeMax = 1048576, isSkipVerifyHash = !1, uploadChunk, onProgress}) => {
  let chunkIndex = 0;
  const chunkTotal = Math.ceil(size / chunkSizeMax) || 1;
  for (;chunkIndex < chunkTotal; ) {
    onProgress && await onProgress(chunkIndex * chunkSizeMax, size);
    const chunkSize = chunkIndex < chunkTotal - 1 ? chunkSizeMax : size % chunkSizeMax;
    const chunkInfo = {
      chunkArrayBuffer: await getChunk(chunkIndex * chunkSizeMax, chunkSize),
      key,
      chunkIndex,
      chunkTotal
    };
    await uploadChunk(await packArrayBufferChunk(chunkInfo, isSkipVerifyHash), chunkInfo);
    chunkIndex++;
  }
  onProgress && onProgress(size, size);
};
export { packArrayBufferChunk, parseArrayBufferChunk, uploadArrayBufferByChunk };