import { filterJoin } from "../string.js";
const COMMON_LAYOUT = (extraHeadList = [], extraBodyList = []) => `<!DOCTYPE html>\n<html>\n<head>\n<meta charset="utf-8">\n<meta name="viewport" content="minimum-scale=1, width=device-width">\n${filterJoin(extraHeadList)}\n</head>\n<body>\n${filterJoin(extraBodyList)}\n</body>\n</html>`;
const COMMON_STYLE = ({boxReset = !0, bodyReset = !0} = {}) => `<style>\n:root { --c-dr: #63aeff; --c-warn: #f22;\n  --c-fill-s: hsla(0, 0%, 53%, 0.1); --c-fill-l: hsla(0, 0%, 53%, 0.3); --c-fill-n: hsla(0, 0%, 53%, 0.5); --c-fill-d: hsla(0, 0%, 53%, 0.7);\n  --ct-fg-n: #000; --ct-fg-d: #222; --ct-bg-n: #fff; --ct-bg-d: #ddd;\n}\n@media (prefers-color-scheme: dark) { :root { --ct-fg-n: #fff; --ct-fg-d: #bbb; --ct-bg-n: #000; --ct-bg-d: #222; } }\n\n* { scrollbar-color: var(--c-fill-d) var(--c-fill-l); scrollbar-width: thin; }\n::-webkit-scrollbar { width: 6px; height: 6px; }\n::-webkit-scrollbar-track { background: var(--c-fill-l); }\n::-webkit-scrollbar-thumb { background: var(--c-fill-d); }\n::-webkit-scrollbar-thumb:hover { background: var(--c-fill-n); }\n\n*, *::before, *::after { ${boxReset ? "margin: 0; padding: 0; box-sizing: border-box;" : ""}\n  outline-color: var(--c-warn); border-color: var(--c-fill-n);\n}\nbody { ${bodyReset ? "overflow: auto; display: flex; flex-flow: column; height: 100vh; font-family: monospace; font-size: 16px;" : ""}\n}\n\nbody, input, textarea, select, button { color: var(--ct-fg-n); }\nbody, input, textarea, select, option { background: var(--ct-bg-n); }\np, li, td { color: var(--ct-fg-d); }\na { color: var(--c-dr); }\n\nbutton, .button { text-decoration: none; cursor: pointer; margin: 4px; padding: 4px; min-width: 32px; border: 0; border-radius: 4px; background: var(--c-fill-l); box-shadow: inset 0 0 0 1px var(--c-fill-n); }\nbutton:hover, .button:hover { background: var(--c-fill-n); box-shadow: inset 0 0 0 1px var(--c-fill-d); }\nbutton.select, .button.select { color: var(--c-warn); box-shadow: inset 0 0 0 1px var(--c-warn); }\nbutton:disabled, .button:disabled { cursor: default; background: var(--c-fill-s); box-shadow: unset; }\n@media (pointer: fine) { button, .button, .auto-height { min-height: 20px; font-size: 14px; } }\n@media (pointer: coarse) { button, .button, .auto-height { min-height: 32px; font-size: 18px; } }\n</style>`;
const COMMON_SCRIPT = injectMap => {
  const valueObject = {};
  const functionScriptList = [];
  Object.entries({
    ...COMMON_FUNC_MAP(),
    ...injectMap
  }).forEach((([key, value]) => {
    "function" == typeof value ? functionScriptList.push(`<script>window[${JSON.stringify(key)}] = ${String(value)}<\/script>`) : valueObject[key] = value;
  }));
  return filterJoin([ `<script>Object.assign(window, ${JSON.stringify(valueObject)})<\/script>`, ...functionScriptList ]);
};
const COMMON_FUNC_MAP = () => ({
  qS: querySelector,
  qSA: querySelectorAll,
  cE: createElement,
  aCL: appendChildList,
  mECN: modifyElementClassName,
  mEA: modifyElementAttribute,
  iDR: isDocumentReady,
  tDR: tillDocumentReady
});
const querySelector = (selector, innerHTML = void 0) => {
  const element = window.document.querySelector(selector);
  element && "string" == typeof innerHTML && (element.innerHTML = innerHTML);
  return element;
};
const querySelectorAll = selector => [ ...window.document.querySelectorAll(selector) ];
const createElement = (tagName, attributeMap = {}, childElementList = []) => {
  const element = Object.assign(window.document.createElement(tagName), attributeMap);
  childElementList.forEach((childElement => childElement && element.appendChild(childElement)));
  return element;
};
const appendChildList = (element, childElementList = []) => childElementList.forEach((childElement => childElement && element.appendChild(childElement)));
const modifyElementClassName = (element, isAdd, ...args) => element.classList[isAdd ? "add" : "remove"](...args);
const modifyElementAttribute = (element, isAdd, key, value = "") => element[isAdd ? "setAttribute" : "removeAttribute"](key, value);
const isDocumentReady = () => "complete" === window.document.readyState;
const tillDocumentReady = func => {
  const {iDR, document} = window;
  if (iDR()) return func();
  const onReady = () => {
    if (iDR()) {
      document.removeEventListener("readystatechange", onReady);
      func();
    }
  };
  document.addEventListener("readystatechange", onReady);
};
const styleTagMerge = styleTagStringList => `<style>\n${styleTagStringList.map((styleTagString => {
  if (styleTagString.startsWith("<style>") && styleTagString.endsWith("</style>")) return styleTagString.slice(7, -8);
  throw new Error(`invalid styleTagString: ${styleTagString}`);
})).join("\n")}\n</style>`;
const simpleCompactCSS = CSSString => CSSString.replace(/\s*([,{}();:>])\s*/g, "$1").replace(/;}/g, "}");
export { COMMON_LAYOUT, COMMON_STYLE, COMMON_SCRIPT, COMMON_FUNC_MAP, styleTagMerge, simpleCompactCSS };