import { isBasicObject, isString } from "../check.js";
import { dupJSON } from "../data/function.js";
import { objectSortKey } from "../mutable/Object.js";
const PACKAGE_KEY_SORT_REQUIRED = [ "bundledDependencies", "peerDependencies", "dependencies", "devDependencies", "optionalDependencies" ];
const PACKAGE_KEY_ORDER = [ "private", "name", "version", "description", "author", "contributors", "maintainers", "license", "keywords", "repository", "homepage", "bugs", "type", "main", "bin", "browser", "man", "files", "directories", "scripts", "config", "publishConfig", "os", "cpu", "engines", "engineStrict", ...PACKAGE_KEY_SORT_REQUIRED ];
const getPackageKeyOrder = key => {
  const index = PACKAGE_KEY_ORDER.indexOf(key);
  return -1 === index ? PACKAGE_KEY_ORDER.length : index;
};
const sortPackageJSON = packageJSON => {
  packageJSON = dupJSON(packageJSON);
  PACKAGE_KEY_SORT_REQUIRED.forEach((key => {
    packageJSON[key] && objectSortKey(packageJSON[key]);
  }));
  objectSortKey(packageJSON, ((a, b) => getPackageKeyOrder(a) - getPackageKeyOrder(b)));
  return packageJSON;
};
const packPackageJSON = packageJSON => `${JSON.stringify(packageJSON, null, 2)}\n`;
const toPackageInfo = ({packageJSON, packageJSONPath = "", packageRootPath = ""}) => ({
  sourcePackageJSON: dupJSON(packageJSON),
  packageJSON,
  packageJSONPath,
  packageRootPath
});
const collectDependency = (packageInfo, {dependencyMap = {}, dependencyInfoMap = {}, duplicateInfoList = []} = {}) => {
  const {dependencies = {}, devDependencies = {}, peerDependencies = {}, optionalDependencies = {}} = packageInfo.packageJSON;
  for (const dependencyObject of [ dependencies, devDependencies, peerDependencies, optionalDependencies ]) for (const [name, versionSpec] of Object.entries(dependencyObject)) if (dependencyInfoMap[name]) duplicateInfoList.push({
    name,
    versionSpec,
    packageInfo,
    existPackageInfo: dependencyInfoMap[name]
  }); else {
    dependencyMap[name] = versionSpec;
    dependencyInfoMap[name] = {
      name,
      versionSpec,
      packageInfo
    };
  }
  return {
    dependencyMap,
    dependencyInfoMap,
    duplicateInfoList
  };
};
const getFirstBinPath = packageJSON => isString(packageJSON.bin) ? packageJSON.bin : isBasicObject(packageJSON.bin) ? Object.values(packageJSON.bin)[0] : void 0;
const parsePackageNameAndVersion = nameAndVersion => {
  const nameAndVersionList = nameAndVersion.split("@");
  if (nameAndVersionList.length < 2) return [];
  const version = nameAndVersionList.pop();
  const name = nameAndVersionList.join("@");
  return name && version ? [ name, version ] : [];
};
const toPackageTgzName = (name, version) => `${name}-${version}.tgz`.replace(/^@/, "").replace(/\//, "-");
export { sortPackageJSON, packPackageJSON, toPackageInfo, collectDependency, getFirstBinPath, parsePackageNameAndVersion, toPackageTgzName };