const createPatchKit = ({NAME_KEY = "key", NAME_MODIFY_TIME = "mtime", EPSILON_MODIFY_TIME = 1, isChanged = (data, prevData) => data[NAME_MODIFY_TIME] !== prevData[NAME_MODIFY_TIME], shouldApplyDelete = (data, prevData) => data[NAME_MODIFY_TIME] >= prevData[NAME_MODIFY_TIME], shouldApplyUpdate = (data, prevData) => data[NAME_MODIFY_TIME] > prevData[NAME_MODIFY_TIME], getPatchData = (data, prevData) => data[NAME_MODIFY_TIME] < prevData[NAME_MODIFY_TIME] ? {
  ...data,
  [NAME_MODIFY_TIME]: prevData[NAME_MODIFY_TIME] + EPSILON_MODIFY_TIME
} : data}) => {
  const arrayMapReducer = (o, value) => o.set(value[NAME_KEY], value);
  return {
    NAME_KEY,
    NAME_MODIFY_TIME,
    isChanged,
    shouldApplyDelete,
    shouldApplyUpdate,
    hasPatch: (dataMap, prevDataList) => {
      for (let index = 0, indexMax = prevDataList.length; index < indexMax; index++) {
        const prevData = prevDataList[index];
        const key = prevData[NAME_KEY];
        const data = dataMap.get(key);
        if (void 0 === data) return !0;
        if (isChanged(data, prevData)) return !0;
        dataMap.delete(key);
      }
      return dataMap.size >= 1;
    },
    countPatch: (dataMap, prevDataList) => {
      let count = 0;
      for (let index = 0, indexMax = prevDataList.length; index < indexMax; index++) {
        const prevData = prevDataList[index];
        const key = prevData[NAME_KEY];
        const data = dataMap.get(key);
        (void 0 === data || isChanged(data, prevData)) && count++;
        dataMap.delete(key);
      }
      count += dataMap.size;
      return count;
    },
    generatePatch: (dataMap, prevDataList) => {
      const deleteList = [];
      const updateList = [];
      for (let index = 0, indexMax = prevDataList.length; index < indexMax; index++) {
        const prevData = prevDataList[index];
        const key = prevData[NAME_KEY];
        const data = dataMap.get(key);
        void 0 === data ? deleteList.push({
          [NAME_KEY]: key,
          [NAME_MODIFY_TIME]: prevData[NAME_MODIFY_TIME]
        }) : isChanged(data, prevData) && updateList.push(getPatchData(data, prevData));
        dataMap.delete(key);
      }
      dataMap.forEach((data => {
        updateList.push(data);
      }));
      return {
        deleteList,
        updateList
      };
    },
    applyPatch: (deleteList, updateList, dataList, onData) => {
      let isChanged = !1;
      const deleteMap = deleteList.reduce(arrayMapReducer, new Map);
      const updateMap = updateList.reduce(arrayMapReducer, new Map);
      for (const data of dataList) {
        const key = data[NAME_KEY];
        const deleteData = deleteMap.get(key);
        if (void 0 !== deleteData && shouldApplyDelete(deleteData, data)) {
          onData(void 0, data);
          isChanged = !0;
          continue;
        }
        const updateData = updateMap.get(key);
        if (void 0 !== updateData) {
          updateMap.delete(key);
          if (shouldApplyUpdate(updateData, data)) {
            onData(updateData, data);
            isChanged = !0;
            continue;
          }
        }
        onData(data, data);
      }
      updateMap.forEach((data => {
        onData(data, void 0);
        isChanged = !0;
      }));
      return isChanged;
    },
    arrayMapReducer
  };
};
const toObjectPatchKit = patchKit => {
  const {NAME_KEY, hasPatch, countPatch, generatePatch, applyPatch} = patchKit;
  return {
    hasObjectPatch: (object, prevObject) => hasPatch(new Map(Object.entries(object)), Object.values(prevObject)),
    countObjectPatch: (object, prevObject) => countPatch(new Map(Object.entries(object)), Object.values(prevObject)),
    generateObjectPatch: (object, prevObject) => generatePatch(new Map(Object.entries(object)), Object.values(prevObject)),
    applyObjectPatch: (object, {deleteList, updateList}) => {
      const nextObject = {};
      return applyPatch(deleteList, updateList, Object.values(object), ((data, prevData) => {
        void 0 !== data && (nextObject[data[NAME_KEY]] = data);
      })) ? nextObject : object;
    }
  };
};
const toArrayWithKeyPatchKit = patchKit => {
  const {hasPatch, countPatch, generatePatch, applyPatch, arrayMapReducer} = patchKit;
  return {
    hasArrayWithKeyPatch: (array, prevArray) => hasPatch(array.reduce(arrayMapReducer, new Map), prevArray),
    countArrayWithKeyPatch: (array, prevArray) => countPatch(array.reduce(arrayMapReducer, new Map), prevArray),
    generateArrayWithKeyPatch: (array, prevArray) => generatePatch(array.reduce(arrayMapReducer, new Map), prevArray),
    applyArrayWithKeyPatch: (array, {deleteList, updateList}) => {
      const nextArray = [];
      return applyPatch(deleteList, updateList, array, ((data, prevData) => {
        void 0 !== data && nextArray.push(data);
      })) ? nextArray : array;
    }
  };
};
export { createPatchKit, toObjectPatchKit, toArrayWithKeyPatchKit };