import { createMapMap } from "../data/MapMap.js";
const END = Symbol("runlet:hint:end");
const SKIP = Symbol("runlet:hint:skip");
const REDO = Symbol("runlet:hint:redo");
const LOUD_FUNC = error => {
  console.error(error);
  throw error;
};
const createPack = (value, hint) => [ value, hint, void 0 ];
const clearPack = pack => {
  const [, hint] = pack;
  hint === END && (pack[0] = void 0);
  hint === REDO && (pack[1] = void 0);
  pack[2] = void 0;
  return pack;
};
const describePack = pack => pack ? `${pack[0] && JSON.stringify(String(pack[0]).slice(0, 16))}|${String(pack[1] || "-")}|${pack[2] ? "RUN" : "IDLE"}` : "";
const createRunlet = ({poolMap = new Map, chipMap = new Map, endChipKeySet = new Set, runningChipMap = new Map, onError = LOUD_FUNC}) => {
  const triggerChipFuncMap = new Map;
  const onPendPackMapMap = createMapMap();
  let isPause = !1;
  let isValid = !0;
  const pushWakeKeySetMapMap = createMapMap();
  const pullWakeKeySetMap = new Map;
  const wakeKeyFastList = [];
  const wakeKeySlowList = [];
  let isWakeLoopRunning = !1;
  const wakeLoop = () => {
    if (!0 === isWakeLoopRunning || 0 === wakeKeyFastList.length && 0 === wakeKeySlowList.length) return;
    isWakeLoopRunning = !0;
    let chipKey;
    for (;void 0 !== (chipKey = wakeKeyFastList.shift() || wakeKeySlowList.shift()); ) triggerChipFuncMap.get(chipKey)();
    isWakeLoopRunning = !1;
  };
  const queueChipResolveWake = (chipKey, pushWakeKeySet, pullWakeKeySet) => {
    void 0 !== chipKey && wakeKeyFastList.unshift(chipKey);
    if (void 0 !== pushWakeKeySet) {
      for (const chipKey of pushWakeKeySet) wakeKeyFastList.push(chipKey);
      pushWakeKeySet.clear();
    }
    if (void 0 !== pullWakeKeySet) {
      for (const chipKey of pullWakeKeySet) wakeKeySlowList.push(chipKey);
      pullWakeKeySet.clear();
    }
    wakeLoop();
  };
  const onChipResolveUnbind = (chip, prevPool, nextPool, nextPushWakeKeySet, prevPullWakeKeySet, {pack, state}) => {
    if (!1 === isValid) return;
    const [value, hint] = pack;
    if (hint !== SKIP) {
      hint === END && endChipKeySet.add(chip.key);
      nextPool.pushPend(chip.nextPendKey, clearPack(pack));
      if (hint === END && value >= 2) {
        let extraEND = value - 1;
        for (;(extraEND -= 1) >= 0; ) nextPool.pushPend(chip.nextPendKey, createPack(void 0, END));
      }
      hint === REDO && clearPack(runningChipMap.get(chip.key));
    }
    hint !== REDO && runningChipMap.delete(chip.key);
    chip.state = state;
    isPause || (hint === END ? queueChipResolveWake(void 0, nextPushWakeKeySet, prevPullWakeKeySet) : hint === REDO ? queueChipResolveWake(chip.key, nextPushWakeKeySet, void 0) : queueChipResolveWake(chip.key, hint === SKIP ? void 0 : nextPushWakeKeySet, prevPullWakeKeySet));
  };
  const onChipRejectUnbind = (chipErrored, error) => {
    if (!1 === isValid) return onError(error);
    isValid = !1;
    for (const chip of chipMap.values()) {
      if (endChipKeySet.has(chip.key)) continue;
      endChipKeySet.add(chip.key);
      const pack = chip === chipErrored ? void 0 : runningChipMap.get(chip.key);
      if (chip.sync) try {
        chip.process(pack, chip.state, error);
      } catch (error) {
        onError(error);
      } else chip.process(pack, chip.state, error).catch(onError);
    }
  };
  const obtainChipPackUnbind = (chip, prevPool, nextPool, prevPushWakeKeySet, nextPullWakeKeySet) => {
    let pack = runningChipMap.get(chip.key);
    if (isPause || endChipKeySet.has(chip.key) || void 0 !== pack && void 0 !== pack[2]) ; else {
      if (void 0 === pack) {
        if (void 0 === (pack = prevPool.pullPend(chip.prevPendKey))) {
          prevPushWakeKeySet.add(chip.key);
          return;
        }
        runningChipMap.set(chip.key, pack);
      }
      if (!nextPool.isPendLimited(chip.nextPendKey)) return pack;
      nextPullWakeKeySet.add(chip.key);
    }
  };
  const triggerAsyncChipUnbind = (chip, obtainChipPack, onChipResolve, onChipReject) => {
    const pack = obtainChipPack();
    if (void 0 === pack) return;
    const processPromise = chip.process(pack, chip.state, void 0);
    processPromise.then(onChipResolve, onChipReject);
    pack[2] = processPromise;
  };
  const triggerSyncChipUnbind = (chip, obtainChipPack, onChipResolve, onChipReject) => {
    const pack = obtainChipPack();
    if (void 0 !== pack) try {
      onChipResolve(chip.process(pack, chip.state, void 0));
    } catch (error) {
      onChipReject(error);
    }
  };
  const onPendPullUnbind = pullWakeKeySet => {
    isPause || queueChipResolveWake(void 0, void 0, pullWakeKeySet);
  };
  const onPendPushUnbind = pushWakeKeySet => {
    isPause || queueChipResolveWake(void 0, pushWakeKeySet, void 0);
  };
  const getOnPendPack = (poolKey, pendKey) => {
    let onPendPack = onPendPackMapMap.get(poolKey, pendKey);
    if (void 0 === onPendPack) {
      const onPendPush = onPendPushUnbind.bind(null, pushWakeKeySetMapMap.get(poolKey, pendKey));
      const onPendPull = onPendPullUnbind.bind(null, pullWakeKeySetMap.get(poolKey));
      onPendPackMapMap.set(poolKey, pendKey, onPendPack = {
        onPendPull,
        onPendPush
      });
    }
    return onPendPack;
  };
  return {
    poolMap,
    chipMap,
    getIsValid: () => isValid,
    getIsPause: () => isPause,
    setIsPause: value => {
      isPause = value;
    },
    pause: () => {
      isPause = !0;
    },
    resume: () => {
      isPause = !1;
    },
    attach: () => {
      triggerChipFuncMap.clear();
      onPendPackMapMap.clear();
      pushWakeKeySetMapMap.clear();
      pullWakeKeySetMap.clear();
      for (const pool of poolMap.values()) {
        pullWakeKeySetMap.set(pool.key, new Set);
        if (pool.pendKeyGroupMap) for (const pendKeyGroupSet of pool.pendKeyGroupMap.values()) {
          const sharedWakeKeySet = new Set;
          for (const pendKey of pendKeyGroupSet) pushWakeKeySetMapMap.set(pool.key, pendKey, sharedWakeKeySet);
        }
      }
      for (const chip of chipMap.values()) {
        const {key, prevPoolKey, prevPendKey, nextPoolKey, nextPendKey} = chip;
        const prevPool = poolMap.get(prevPoolKey);
        const nextPool = poolMap.get(nextPoolKey);
        if (!prevPool) throw new Error(`missing prevPool: ${prevPendKey}`);
        if (!nextPool) throw new Error(`missing nextPool: ${nextPoolKey}`);
        const prevPushWakeKeySet = pushWakeKeySetMapMap.getSet(prevPoolKey, prevPendKey, (() => new Set));
        const nextPushWakeKeySet = pushWakeKeySetMapMap.getSet(nextPoolKey, nextPendKey, (() => new Set));
        const prevPullWakeKeySet = pullWakeKeySetMap.get(prevPoolKey);
        const nextPullWakeKeySet = pullWakeKeySetMap.get(nextPoolKey);
        const obtainChipPack = obtainChipPackUnbind.bind(null, chip, prevPool, nextPool, prevPushWakeKeySet, nextPullWakeKeySet);
        const onChipResolve = onChipResolveUnbind.bind(null, chip, prevPool, nextPool, nextPushWakeKeySet, prevPullWakeKeySet);
        const onChipReject = onChipRejectUnbind.bind(null, chip);
        const triggerChip = (chip.sync ? triggerSyncChipUnbind : triggerAsyncChipUnbind).bind(null, chip, obtainChipPack, onChipResolve, onChipReject);
        triggerChipFuncMap.set(key, triggerChip);
      }
    },
    detach: () => {
      isValid = !1;
      return {
        poolMap,
        chipMap,
        endChipKeySet,
        runningChipMap
      };
    },
    trigger: () => (chipKeyList => {
      for (const chipKey of chipKeyList) wakeKeySlowList.push(chipKey);
      wakeLoop();
    })(chipMap.keys()),
    createPendInput: (poolKey, pendKey) => {
      const {onPendPush} = getOnPendPack(poolKey, pendKey);
      const pool = poolMap.get(poolKey);
      return {
        pool,
        push: pack => {
          pool.pushPend(pendKey, pack);
          onPendPush();
          return pack;
        },
        canPush: () => !pool.isPendLimited(pendKey)
      };
    },
    createPendOutput: (poolKey, pendKey) => {
      const {onPendPull} = getOnPendPack(poolKey, pendKey);
      const pool = poolMap.get(poolKey);
      return {
        pool,
        pull: () => {
          const pack = pool.pullPend(pendKey);
          onPendPull();
          return pack;
        },
        canPull: () => Boolean(pool.getPendSize(pendKey))
      };
    },
    describe: (stringList = []) => {
      isValid || stringList.push("[DETACHED]");
      for (const chip of chipMap.values()) stringList.push([ `#${chip.key}`, `[${endChipKeySet.has(chip.key) ? "END" : describePack(runningChipMap.get(chip.key))}]`, `${String(chip.prevPendKey)}@${String(chip.prevPoolKey)} >> ${String(chip.nextPendKey)}@${String(chip.nextPoolKey)}`, chip.describe && chip.describe() ].filter(Boolean).join(" "));
      for (const pool of poolMap.values()) stringList = pool.describe(stringList, (poolKey => `pullWake[${[ ...pullWakeKeySetMap.get(poolKey) ]}]`), ((poolKey, pendKey) => `pushWake[${[ ...pushWakeKeySetMapMap.get(poolKey, pendKey) || [ "EXTERNAL" ] ]}]`));
      return stringList;
    }
  };
};
const createCountPool = ({key = "default", sizeLimit: poolSizeLimit}) => {
  let poolSize = 0;
  const pendMap = new Map;
  return {
    key,
    pendKeyGroupMap: void 0,
    reset: () => {
      poolSize = 0;
      pendMap.clear();
    },
    getPoolSize: () => poolSize,
    configPend: (pendKey, sizePrivate = 0, sizeLimit = 1 / 0) => {
      pendMap.has(pendKey) && (poolSize -= pendMap.get(pendKey).sizePrivate);
      pendMap.set(pendKey, {
        packQueue: [],
        size: 0,
        sizePrivate,
        sizeLimit
      });
      poolSize += sizePrivate;
    },
    isPendLimited: pendKey => {
      const {size, sizePrivate, sizeLimit} = pendMap.get(pendKey);
      return size > sizePrivate && (poolSize >= poolSizeLimit || size >= sizeLimit);
    },
    getPendSize: pendKey => pendMap.get(pendKey).size,
    pushPend: (pendKey, pack) => {
      const pend = pendMap.get(pendKey);
      pend.packQueue.push(pack);
      pend.size++;
      pend.size > pend.sizePrivate && poolSize++;
    },
    pullPend: pendKey => {
      const pend = pendMap.get(pendKey);
      if (0 !== pend.packQueue.length) {
        pend.size > pend.sizePrivate && poolSize--;
        pend.size--;
        return pend.packQueue.shift();
      }
    },
    describe: (stringList = [], getPoolExtraInfo, getPendExtraInfo) => {
      stringList.push(`@${String(key)} ${poolSize}/${poolSizeLimit} ${getPoolExtraInfo(key)}`);
      for (const [pendKey, {packQueue, size, sizePrivate}] of pendMap) stringList.push(` -${String(pendKey)} ${size},${sizePrivate}P [${packQueue.map(describePack).join(", ")}] ${getPendExtraInfo(key, pendKey)}`);
      return stringList;
    }
  };
};
const KEY_POOL_IO = Symbol("runlet:pool:io");
const KEY_PEND_INPUT = Symbol("runlet:pend:input");
const KEY_PEND_OUTPUT = Symbol("runlet:pend:output");
const PoolIO = {
  key: KEY_POOL_IO,
  pendKeyGroupMap: void 0,
  reset: () => {},
  getPoolSize: () => 0,
  configPend: (pendKey, sizePrivate = 0) => {
    if (pendKey !== KEY_PEND_INPUT && pendKey !== KEY_PEND_OUTPUT) throw new Error(`invalid IO config pendKey: ${String(pendKey)}`);
    if (sizePrivate) throw new Error(`invalid IO config sizePrivate: ${sizePrivate} for pendKey: ${String(pendKey)}`);
  },
  isPendLimited: pendKey => pendKey !== KEY_PEND_OUTPUT,
  getPendSize: pendKey => pendKey === KEY_PEND_INPUT ? 1 : 0,
  pushPend: (pendKey, value) => {
    if (pendKey !== KEY_PEND_OUTPUT || value[1] !== END) throw new Error(`invalid IO push pendKey: ${String(pendKey)}, value: ${String(value)}`);
  },
  pullPend: pendKey => {
    if (pendKey === KEY_PEND_INPUT) return createPack(void 0, SKIP);
    throw new Error(`invalid IO shift pendKey: ${String(pendKey)}`);
  },
  describe: (stringList = [], getPoolExtraInfo, getPendExtraInfo) => {
    stringList.push(`@${String(KEY_POOL_IO)}`);
    return stringList;
  }
};
const TYPE_LOGICAL_PENDVIEW = Symbol("runlet:pend:logical:pendview");
const TYPE_LOGICAL_PENDVIEWEE = Symbol("runlet:pend:logical:pendviewee");
const createLogicalCountPool = ({key = "default-logical", sizeLimit: poolSizeLimit, innerPool = createCountPool({
  key,
  sizeLimit: poolSizeLimit
})}) => {
  const pendKeyGroupMap = new Map;
  const pendLogicalMap = new Map;
  const configPendLogic = (pendKey, pendLogic) => {
    pendLogic.isPendLimitedUnbind && (pendLogic.isPendLimited = pendLogic.isPendLimitedUnbind.bind(null, innerPool, pendLogicalMap, pendLogic));
    pendLogic.getPendSizeUnbind && (pendLogic.getPendSize = pendLogic.getPendSizeUnbind.bind(null, innerPool, pendLogicalMap, pendLogic));
    pendLogic.pushPendUnbind && (pendLogic.pushPend = pendLogic.pushPendUnbind.bind(null, innerPool, pendLogicalMap, pendLogic));
    pendLogic.pullPendUnbind && (pendLogic.pullPend = pendLogic.pullPendUnbind.bind(null, innerPool, pendLogicalMap, pendLogic));
    if (pendLogic.type === TYPE_LOGICAL_PENDVIEW) {
      if (!pendLogic.isPendLimited) throw new Error(`expect isPendLimited in pendLogic of: ${String(pendKey)}`);
      if (!pendLogic.getPendSize) throw new Error(`expect getPendSize in pendLogic of: ${String(pendKey)}`);
      if (!pendLogic.pushPend) throw new Error(`expect pushPend in pendLogic of: ${String(pendKey)}`);
      if (!pendLogic.pullPend) throw new Error(`expect pullPend in pendLogic of: ${String(pendKey)}`);
    } else {
      pendLogic.isPendLimited || (pendLogic.isPendLimited = innerPool.isPendLimited);
      pendLogic.getPendSize || (pendLogic.getPendSize = innerPool.getPendSize);
      pendLogic.pushPend || (pendLogic.pushPend = innerPool.pushPend);
      pendLogic.pullPend || (pendLogic.pullPend = innerPool.pullPend);
    }
  };
  return {
    ...innerPool,
    key,
    pendKeyGroupMap,
    reset: () => {
      innerPool.reset();
      pendKeyGroupMap.clear();
      pendLogicalMap.clear();
    },
    configPend: (pendKey, sizePrivate = 0, sizeLimit = 1 / 0, pendLogic) => {
      if (!pendLogic) throw new Error(`expect pendLogic for: ${String(pendKey)}`);
      if (pendLogic.type === TYPE_LOGICAL_PENDVIEW) {
        configPendLogic(pendKey, pendLogic);
        pendLogicalMap.set(pendKey, pendLogic);
        pendKeyGroupMap.set(pendKey, new Set([ pendKey, ...pendLogic.pendVieweeKeyList ]));
      } else {
        if (pendLogic.type !== TYPE_LOGICAL_PENDVIEWEE) throw new Error(`invalid pendLogic for: ${String(pendKey)}`);
        configPendLogic(pendKey, pendLogic);
        pendLogicalMap.set(pendKey, pendLogic);
        innerPool.configPend(pendKey, sizePrivate, sizeLimit);
      }
    },
    isPendLimited: pendKey => pendLogicalMap.get(pendKey).isPendLimited(pendKey),
    getPendSize: pendKey => pendLogicalMap.get(pendKey).getPendSize(pendKey),
    pushPend: (pendKey, pack) => pendLogicalMap.get(pendKey).pushPend(pendKey, pack),
    pullPend: pendKey => pendLogicalMap.get(pendKey).pullPend(pendKey),
    describe: (stringList = [], getPoolExtraInfo, getPendExtraInfo) => {
      const innerStringList = innerPool.describe([], getPoolExtraInfo, getPendExtraInfo);
      innerStringList[0] = `@${String(key)} INNER ${innerStringList[0]}`;
      stringList.push(...innerStringList);
      for (const [pendKey, pendLogic] of pendLogicalMap) pendLogic.type === TYPE_LOGICAL_PENDVIEW && stringList.push(` -${String(pendKey)} PENDVIEW ${getPendExtraInfo(key, pendKey)} ${pendLogic.describe ? pendLogic.describe() : ""}`);
      return stringList;
    }
  };
};
const ChipSyncBasic = {
  key: "chip:sync-basic",
  prevPoolKey: void 0,
  prevPendKey: void 0,
  prevPendSizePrivate: 0,
  prevPendSizeLimit: 1 / 0,
  prevPendLogic: {},
  nextPoolKey: void 0,
  nextPendKey: void 0,
  nextPendSizePrivate: 0,
  nextPendSizeLimit: 1 / 0,
  nextPendLogic: {},
  state: null,
  sync: !0,
  process: (pack, state, error) => error ? void 0 : {
    pack,
    state
  },
  describe: () => "CHIP-SYNC-BASIC"
};
const toPoolMap = (poolList = []) => {
  const poolMap = new Map;
  for (const pool of poolList) {
    if (poolMap.has(pool.key)) throw new Error(`duplicated pool key: ${pool.key}`);
    poolMap.set(pool.key, pool);
  }
  return poolMap;
};
const toChipMap = (chipList = []) => {
  const chipMap = new Map;
  for (const chip of chipList) {
    if (chipMap.has(chip.key)) throw new Error(`duplicated chip key: ${chip.key}`);
    chipMap.set(chip.key, chip);
  }
  return chipMap;
};
const toLinearChipList = (chipList, {poolKey = "default", pendKeyBase = "pend:", chipKeyBase = "chip:", counter = 0} = {}) => {
  for (const chip of chipList) {
    void 0 === chip.prevPoolKey && (chip.prevPoolKey = poolKey);
    void 0 === chip.prevPendKey && (chip.prevPendKey = `${pendKeyBase}${counter}`);
    counter++;
    void 0 === chip.key && (chip.key = `${chipKeyBase}${counter}`);
    void 0 === chip.nextPoolKey && (chip.nextPoolKey = poolKey);
    void 0 === chip.nextPendKey && (chip.nextPendKey = `${pendKeyBase}${counter}`);
  }
  return chipList;
};
const quickConfigPend = (poolMap, chipMap, extraConfig) => {
  for (const chip of chipMap.values()) poolMap.get(chip.prevPoolKey).configPend(chip.prevPendKey, chip.prevPendSizePrivate, chip.prevPendSizeLimit, chip.prevPendLogic);
  for (const chip of chipMap.values()) poolMap.get(chip.nextPoolKey).configPend(chip.nextPendKey, chip.nextPendSizePrivate, chip.nextPendSizeLimit, chip.nextPendLogic);
  return {
    ...extraConfig,
    poolMap,
    chipMap
  };
};
export { END, SKIP, REDO, createPack, clearPack, describePack, createRunlet, createCountPool, KEY_POOL_IO, KEY_PEND_INPUT, KEY_PEND_OUTPUT, PoolIO, TYPE_LOGICAL_PENDVIEW, TYPE_LOGICAL_PENDVIEWEE, createLogicalCountPool, ChipSyncBasic, toPoolMap, toChipMap, toLinearChipList, quickConfigPend };