import { unwrap, createLockStepAsyncIter } from "../data/Iter.js";
import { createInsideOutPromise } from "../function.js";
import { END, SKIP, createPack, KEY_POOL_IO, KEY_PEND_INPUT, KEY_PEND_OUTPUT } from "./Runlet.js";
const createArrayInputChip = ({array = [], state = {
  index: 0,
  indexMax: array.length,
  array
}, key = "chip:input:array", ...extra}) => ({
  ...extra,
  key,
  prevPoolKey: KEY_POOL_IO,
  prevPendKey: KEY_PEND_INPUT,
  state,
  process: async (pack, state, error) => {
    if (!error) {
      if (state.index === state.indexMax) pack[1] = END; else {
        pack[0] = state.array[state.index];
        pack[1] = void 0;
        state.index++;
      }
      return {
        pack,
        state
      };
    }
  },
  describe: () => `[${state.index}/${state.indexMax}]`
});
const createArrayOutputChip = ({array = [], state = {
  array
}, IOP = createInsideOutPromise(), key = "chip:output:array", ...extra}) => ({
  ...extra,
  key,
  nextPoolKey: KEY_POOL_IO,
  nextPendKey: KEY_PEND_OUTPUT,
  state,
  process: async (pack, state, error) => {
    if (error) return IOP.reject(error);
    if (pack[1] === END) IOP.resolve(state.array); else {
      state.array.push(pack[0]);
      pack[0] = void 0;
      pack[1] = SKIP;
    }
    return {
      pack,
      state
    };
  },
  describe: () => `[${array.length}]`,
  promise: IOP.promise
});
const createAsyncIterInputChip = ({iterable, iterator, next = unwrap({
  iterable,
  iterator
}), key = "chip:input:async-iter", ...extra}) => ({
  ...extra,
  key,
  prevPoolKey: KEY_POOL_IO,
  prevPendKey: KEY_PEND_INPUT,
  process: async (pack, state, error) => {
    if (error) return;
    const {value, done} = await next();
    const isEND = !0 === done;
    pack[0] = isEND ? void 0 : value;
    pack[1] = isEND ? END : void 0;
    return {
      pack,
      state
    };
  }
});
const createAsyncIterOutputChip = ({LSAI = createLockStepAsyncIter(), key = "chip:output:async-iter", ...extra}) => ({
  ...LSAI,
  ...extra,
  key,
  nextPoolKey: KEY_POOL_IO,
  nextPendKey: KEY_PEND_OUTPUT,
  process: async (pack, state, error) => {
    if (error) return LSAI.throw(error);
    const [value, hint] = pack;
    const isEND = hint === END;
    await LSAI.send(value, isEND);
    return {
      pack: isEND ? pack : createPack(void 0, SKIP),
      state
    };
  }
});
const createENDRegulatorChip = ({inputChipCount = 1, outputChipCount = 1, key = "chip:end-regulator", ...extra}) => ({
  ...extra,
  key,
  state: {
    inputEND: inputChipCount,
    outputEND: outputChipCount
  },
  process: async (pack, state, error) => {
    if (!error) {
      if (pack[1] === END) {
        state.inputEND--;
        return {
          pack: state.inputEND > 0 ? createPack(void 0, SKIP) : createPack(state.outputEND >= 2 ? state.outputEND : void 0, END),
          state
        };
      }
      return {
        pack,
        state
      };
    }
  }
});
export { createArrayInputChip, createArrayOutputChip, createAsyncIterInputChip, createAsyncIterOutputChip, createENDRegulatorChip };