import { isInteger } from "../check.js";
import { oneOf } from "../verify.js";
const SLOT_MAP = {
  year: 7,
  month: 6,
  week: 5,
  day: 4,
  hour: 3,
  minute: 2,
  second: 1
};
const parseTimerTag = (tag = "") => {
  const string = tag.replace(/\s/g, "");
  const type = string.charAt(0);
  oneOf(type, [ "@", "+" ]);
  const result = REGEXP_TIMER_TAG.exec(string.slice(1));
  if (!result) throw new Error(`invalid format: "${tag}" ${string.slice(1)}`);
  const [, year, month, week, day, hour, minute, second] = result;
  const tagData = {
    type,
    tag,
    year: toNumber(year),
    month: toNumber(month),
    week: toNumber(week),
    day: toNumber(day),
    hour: toNumber(hour),
    minute: toNumber(minute),
    second: toNumber(second),
    maxSlot: "",
    sumSecond: void 0
  };
  tagData.maxSlot = Object.keys(SLOT_MAP).find((slotName => void 0 !== tagData[slotName])) || "";
  if (!tagData.maxSlot) throw new Error(`empty @tag: ${tagData.tag}`);
  "@" === tagData.type ? verifyAtNext(tagData) : verifyFromNow(tagData);
  return tagData;
};
const packTimerTag = tagData => [ tagData.type, void 0 !== tagData.year && `${tagData.year}Y`, void 0 !== tagData.month && `${tagData.month}M`, void 0 !== tagData.week && `${tagData.week}W`, void 0 !== tagData.day && `${tagData.day}D`, void 0 !== tagData.hour && `${tagData.hour}h`, void 0 !== tagData.minute && `${tagData.minute}m`, void 0 !== tagData.second && `${tagData.second}s` ].filter(Boolean).join("");
const calcDate = (tagData, refDate = new Date) => {
  const date = new Date(refDate);
  if ("@" === tagData.type) {
    if (tagData.week) {
      const utcDay = refDate.getUTCDay();
      const offset = (7 + tagData.week - utcDay) % 7;
      date.setUTCDate(refDate.getUTCDate() + offset);
    } else {
      tagData.year && date.setUTCFullYear(tagData.year);
      tagData.month && date.setUTCMonth(tagData.month - 1);
      tagData.day && date.setUTCDate(tagData.day);
    }
    void 0 !== tagData.hour && date.setUTCHours(tagData.hour);
    void 0 !== tagData.minute && date.setUTCMinutes(tagData.minute);
    void 0 !== tagData.second && date.setUTCSeconds(tagData.second);
    if (date <= refDate) if ("week" === tagData.maxSlot) date.setUTCDate(date.getUTCDate() + 7); else if ("month" === tagData.maxSlot) date.setUTCFullYear(date.getUTCFullYear() + 1); else if ("day" === tagData.maxSlot) date.setUTCMonth(date.getUTCMonth() + 1); else if ("hour" === tagData.maxSlot) date.setUTCDate(date.getUTCDate() + 1); else if ("minute" === tagData.maxSlot) date.setUTCHours(date.getUTCHours() + 1); else {
      if ("second" !== tagData.maxSlot) throw new Error(`failed to calc next date for: ${tagData.tag}`);
      date.setUTCMinutes(date.getUTCMinutes() + 1);
    }
  } else date.setUTCSeconds(date.getUTCSeconds() + tagData.sumSecond);
  return date;
};
const REGEXP_TIMER_TAG = /^(?:([\d.]+)Y)?(?:([\d.]+)M)?(?:([\d.]+)W)?(?:([\d.]+)D)?(?:([\d.]+)h)?(?:([\d.]+)m)?(?:([\d.]+)s)?$/;
const toNumber = value => void 0 === value || "" === value ? void 0 : Number(value);
const verifyAtNext = tagData => {
  if (!(checkAtRange(tagData.year, 1, 9999) && checkAtRange(tagData.month, 1, 12) && checkAtRange(tagData.week, 1, 7) && checkAtRange(tagData.day, 1, 31) && checkAtRange(tagData.hour, 0, 23) && checkAtRange(tagData.minute, 0, 59) && checkAtRange(tagData.second, 0, 59))) throw new Error(`invalid "@" value range for: ${tagData.tag}`);
  const isYMD = tagData.year || tagData.month || tagData.day;
  if (tagData.week && isYMD) throw new Error(`"week" conflict with "year/month/day": ${tagData.tag}`);
  if (void 0 === tagData.month && SLOT_MAP[tagData.maxSlot] > SLOT_MAP.month) throw new Error(`"month" expected: ${tagData.tag}`);
  if (void 0 === tagData.day && SLOT_MAP[tagData.maxSlot] >= SLOT_MAP.month) throw new Error(`"day" expected: ${tagData.tag}`);
  void 0 === tagData.hour && SLOT_MAP[tagData.maxSlot] > SLOT_MAP.hour && (tagData.hour = 0);
  void 0 === tagData.minute && SLOT_MAP[tagData.maxSlot] > SLOT_MAP.minute && (tagData.minute = 0);
  void 0 === tagData.second && SLOT_MAP[tagData.maxSlot] > SLOT_MAP.second && (tagData.second = 0);
};
const checkAtRange = (number, from, to) => void 0 === number || isInteger(number) && number >= from && number <= to;
const verifyFromNow = tagData => {
  const sumDay = withScale(tagData.year, 365) + withScale(tagData.month, 30) + withScale(tagData.week, 7) + withScale(tagData.day, 1);
  tagData.sumSecond = 24 * sumDay * 60 * 60 + withScale(tagData.hour, 3600) + withScale(tagData.minute, 60) + withScale(tagData.second, 1);
};
const withScale = (number, scale = 1) => number ? number * scale : 0;
export { parseTimerTag, packTimerTag, calcDate };