import { CLOCK_TO_SECOND, requestFrameUpdate, cancelFrameUpdate, createTimer } from "../time.js";
import { objectSet, objectDelete } from "../immutable/Object.js";
const createUpdater = () => {
  let funcList;
  let funcMap;
  let funcMapValueList;
  let prevTime;
  const clear = () => {
    funcList = [];
    funcMap = {};
    funcMapValueList = [];
    prevTime = Date.now();
  };
  clear();
  return {
    clear,
    pushFunc: func => {
      funcList.push(func);
    },
    setFunc: (key, func) => {
      funcMap = objectSet(funcMap, key, func);
      funcMapValueList = Object.values(funcMap);
    },
    deleteFunc: key => {
      funcMap = objectDelete(funcMap, key);
      funcMapValueList = Object.values(funcMap);
    },
    update: () => {
      const time = Date.now();
      const deltaTime = time - prevTime;
      const doUpdate = func => func(deltaTime * CLOCK_TO_SECOND);
      prevTime = time;
      funcList.length && (funcList = funcList.filter(doUpdate));
      funcMapValueList.forEach(doUpdate);
    }
  };
};
const createUpdateLoop = ({delay, queueTask = requestFrameUpdate, cancelTask = cancelFrameUpdate} = {}) => {
  const {clear, pushFunc, setFunc, deleteFunc, update} = createUpdater();
  const {start, stop} = createTimer({
    func: update,
    queueTask,
    cancelTask,
    delay
  });
  return {
    start,
    stop,
    clear,
    pushFunc,
    setFunc,
    deleteFunc
  };
};
export { createUpdater, createUpdateLoop };