const CLOCK_PER_SECOND = 1e3;
const CLOCK_TO_SECOND = .001;
const clock = (() => {
  try {
    const {performance} = globalThis;
    const clock = () => performance.now();
    if (clock() <= clock()) return clock;
  } catch (error) {}
  try {
    const {process} = globalThis;
    const clock = () => {
      const [seconds, nanoseconds] = process.hrtime();
      return 1e3 * seconds + 1e-6 * nanoseconds;
    };
    if (clock() <= clock()) return clock;
  } catch (error) {}
  return Date.now;
})();
const getTimestamp = () => Math.floor(.001 * Date.now());
const getUTCDateTag = (date = new Date) => [ date.getUTCFullYear(), pad00(date.getUTCMonth() + 1), pad00(date.getUTCDate()) ].join("");
const pad00 = number => String(number).padStart(2, "0");
const [setWeakTimeout, setWeakInterval] = (() => {
  try {
    const toWeakTimer = func => (...args) => {
      const token = func(...args);
      token.unref();
      return token;
    };
    const token = setTimeout((() => {}), 0);
    clearTimeout(token);
    if ("function" == typeof token.unref) return [ setTimeout, setInterval ].map(toWeakTimer);
  } catch (error) {}
  return [ setTimeout, setInterval ];
})();
const setTimeoutAsync = (wait = 0) => new Promise((resolve => setTimeout(resolve, wait)));
const setWeakTimeoutAsync = (wait = 0) => new Promise((resolve => setWeakTimeout(resolve, wait)));
const setAwaitAsync = async (awaitCount = 0) => {
  for (;awaitCount > 0; ) {
    await null;
    awaitCount--;
  }
};
const [requestFrameUpdate, cancelFrameUpdate] = (() => {
  try {
    const {requestAnimationFrame, cancelAnimationFrame} = globalThis;
    cancelAnimationFrame(requestAnimationFrame((() => {})));
    return [ requestAnimationFrame, cancelAnimationFrame ];
  } catch (error) {}
  return [ func => setTimeout(func, 1e3 / 60), clearTimeout ];
})();
const createTimer = ({func, delay, queueTask = setTimeout, cancelTask = clearTimeout}) => {
  let token = null;
  const update = () => {
    if (token) {
      token = queueTask(update, delay);
      func();
    }
  };
  const start = () => {
    token || (token = queueTask(update, delay));
  };
  const stop = () => {
    if (token) {
      cancelTask(token);
      token = null;
    }
  };
  return {
    start,
    stop,
    isActive: () => Boolean(token),
    getDelay: () => delay,
    setDelay: nextDelay => {
      if (nextDelay !== delay) {
        delay = nextDelay;
        stop();
        start();
      }
    }
  };
};
const createStepper = (prevTime = clock()) => () => {
  const nextTime = clock();
  const result = nextTime - prevTime;
  prevTime = nextTime;
  return result;
};
export { CLOCK_PER_SECOND, CLOCK_TO_SECOND, clock, getTimestamp, getUTCDateTag, setWeakTimeout, setWeakInterval, setTimeoutAsync, setWeakTimeoutAsync, setAwaitAsync, requestFrameUpdate, cancelFrameUpdate, createTimer, createStepper };