import { randomFill, createHash } from "node:crypto";
import { promisify } from "node:util";
const randomFillAsync = promisify(randomFill);
const getRandomBufferAsync = size => randomFillAsync(Buffer.allocUnsafe(size));
const calcHash = (bufferOrString, algorithm = "sha1", encoding = "base64") => createHash(algorithm).update(bufferOrString).digest(encoding);
const createBufferRefragPool = () => {
  /** @type Buffer[] */
  const pool = [];
  let poolSumLength = 0;
  /** @type (v: Buffer) => void */  return {
    pushFrag: bufferFrag => {
      pool.push(bufferFrag);
      poolSumLength += bufferFrag.length;
    },
    hasLength: length => poolSumLength >= length,
    tryGetRefragBuffer: length => {
      if (!(poolSumLength < length)) {
        poolSumLength -= length;
        if (length === pool[0].length) return pool.shift();
        if (length < pool[0].length) {
          const buffer = pool[0].subarray(0, length);
          pool[0] = pool[0].subarray(length);
          return buffer;
        }
        {
          const buffer = Buffer.allocUnsafe(length);
          let offset = 0;
          for (;length > 0; ) {
            const frag = pool[0];
            const fragLength = frag.length;
            if (length >= fragLength) {
              frag.copy(buffer, offset);
              pool.shift();
              offset += fragLength;
              length -= fragLength;
            } else {
              frag.copy(buffer, offset, 0, length);
              pool[0] = frag.subarray(length);
              length = 0;
            }
          }
          return buffer;
        }
      }
    }
  };
};
export { getRandomBufferAsync, calcHash, createBufferRefragPool };