import { resolve, relative } from "node:path";
import { createReadStream } from "node:fs";
import { createHash } from "node:crypto";
import { compareString } from "../../common/compare.js";
import { createAsyncLane, extendAutoSelectLane } from "../../common/module/AsyncLane.js";
import { setupStreamPipe, readableStreamToBufferAsync } from "../data/Stream.js";
import { PATH_TYPE, getPathStat, getPathTypeFromStat } from "./Path.js";
import { getDirInfoTree, walkDirInfoTree } from "./Directory.js";
const getChecksumInfoOfFile = async (absolutePath, fromPath) => [ relative(fromPath, absolutePath), (await readableStreamToBufferAsync(setupStreamPipe(createReadStream(absolutePath), createHash("sha1")))).toString("base64") ];
const getChecksumInfoListOfPath = async (path = ".", fromPath = process.cwd(), checksumInfoList = []) => {
  path = resolve(fromPath, path);
  const collector = async filePath => {
    checksumInfoList.push(await getChecksumInfoOfFile(filePath, fromPath));
  };
  const pathType = getPathTypeFromStat(await getPathStat(path));
  switch (pathType) {
   case PATH_TYPE.File:
    await collector(path);
    break;
   case PATH_TYPE.Directory:
    {
      const {getTailPromise, pushAuto} = extendAutoSelectLane(createAsyncLane({
        laneSize: 4
      }));
      await walkDirInfoTree(await getDirInfoTree(path), (async dirInfo => {
        dirInfo.type === PATH_TYPE.File && pushAuto((() => collector(dirInfo.path)));
      }));
      await getTailPromise();
      break;
    }
   default:
    throw new Error(`invalid pathType: ${pathType} for ${path}`);
  }
  return checksumInfoList;
};
const getChecksumInfoListOfPathList = async (pathList = [], fromPath = process.cwd(), checksumInfoList = []) => {
  for (const path of pathList) await getChecksumInfoListOfPath(path, fromPath, checksumInfoList);
  return checksumInfoList;
};
const describeChecksumInfoList = checksumInfoList => checksumInfoList.sort((([a], [b]) => compareString(a, b))).map((([relativePath, hashBase64]) => `${hashBase64} ${relativePath}`)).join("\n");
const describeChecksumOfPathList = async ({pathList = [], fromPath = process.cwd()}) => describeChecksumInfoList(await getChecksumInfoListOfPathList(pathList, fromPath));
export { getChecksumInfoOfFile, getChecksumInfoListOfPath, getChecksumInfoListOfPathList, describeChecksumInfoList, describeChecksumOfPathList };