import { lstatSync, statSync, symlinkSync, readlinkSync, renameSync, unlinkSync, accessSync, mkdirSync, rmdirSync, copyFileSync, promises as fsAsync } from "node:fs";
import { join, resolve, dirname, sep } from "node:path";
import { homedir } from "node:os";
import { withFallbackResult, withFallbackResultAsync } from "../../common/error.js";
/** @typedef { Stats | BigIntStats } FSStat */
/** @type { FSStat } */ const STAT_ERROR = /** @type { Stats } */ /** @type { unknown } */ {
  isSymbolicLink: () => !1,
  isDirectory: () => !1,
  isFile: () => !1
};
/** @typedef { 'File' | 'Directory' | 'Symlink' | 'Other' | 'Error' } PathType */
/** @type { Record<PathType, PathType> } */ const PATH_TYPE = {
  File: "File",
  Directory: "Directory",
  Symlink: "Symlink",
  Other: "Other",
  Error: "Error"
};
/** @type { (stat: FSStat) => PathType } */ const getPathTypeFromStat = stat => stat.isSymbolicLink() ? PATH_TYPE.Symlink : stat.isDirectory() ? PATH_TYPE.Directory : stat.isFile() ? PATH_TYPE.File : stat === STAT_ERROR ? PATH_TYPE.Error : PATH_TYPE.Other
/** @type { (path: string) => Promise<FSStat> } */;
const getPathLstat = async path => withFallbackResultAsync(STAT_ERROR, fsAsync.lstat, path)
/** @type { (path: string) => FSStat } */;
const getPathLstatSync = path => withFallbackResult(STAT_ERROR, lstatSync, path)
/** @type { (path: string) => Promise<FSStat> } */;
const getPathStat = async path => withFallbackResultAsync(STAT_ERROR, fsAsync.stat, path)
/** @type { (path: string) => FSStat } */;
const getPathStatSync = path => withFallbackResult(STAT_ERROR, statSync, path);
const copyPath = async (pathFrom, pathTo, pathStat) => {
  void 0 === pathStat && (pathStat = await getPathLstat(pathFrom));
  return pathStat.isSymbolicLink() ? fsAsync.symlink(await fsAsync.readlink(pathFrom), pathTo) : pathStat.isDirectory() ? (await getPathLstat(pathTo)).isDirectory() ? void 0 : fsAsync.mkdir(pathTo) : fsAsync.copyFile(pathFrom, pathTo);
};
const copyPathSync = (pathFrom, pathTo, pathStat) => {
  void 0 === pathStat && (pathStat = getPathLstatSync(pathFrom));
  return pathStat.isSymbolicLink() ? symlinkSync(readlinkSync(pathFrom), pathTo) : pathStat.isDirectory() ? getPathLstatSync(pathTo).isDirectory() ? void 0 : mkdirSync(pathTo) : copyFileSync(pathFrom, pathTo);
};
const renamePath = async (pathFrom, pathTo) => fsAsync.rename(pathFrom, pathTo);
const renamePathSync = (pathFrom, pathTo) => renameSync(pathFrom, pathTo);
const deletePath = async (path, pathStat) => {
  void 0 === pathStat && (pathStat = await getPathLstat(path));
  pathStat.isDirectory() ? await fsAsync.rmdir(path) : await fsAsync.unlink(path);
};
const deletePathSync = (path, pathStat) => {
  void 0 === pathStat && (pathStat = getPathLstatSync(path));
  pathStat.isDirectory() ? rmdirSync(path) : unlinkSync(path);
};
const deletePathForce = async (path, pathStat) => withFallbackResultAsync(void 0, deletePath, path, pathStat);
const deletePathForceSync = (path, pathStat) => withFallbackResult(void 0, deletePathSync, path, pathStat);
const existPath = async path => ErrorNotExist !== await withFallbackResultAsync(ErrorNotExist, fsAsync.access, path);
const existPathSync = path => ErrorNotExist !== withFallbackResult(ErrorNotExist, accessSync, path);
const ErrorNotExist = new Error;
const nearestExistPath = async path => {
  for (;path && !await existPath(path); ) path = dirname(path);
  return path;
};
const nearestExistPathSync = path => {
  for (;path && !existPathSync(path); ) path = dirname(path);
  return path;
};
const toPosixPath = path => path.replace(REGEXP_PATH_SEP_WIN32, "/");
const REGEXP_PATH_SEP_WIN32 = /\\/g;
const addTrailingSep = path => join(path, sep);
const dropTrailingSep = path => join(path, ".");
const expandHome = path => "~" === path ? homedir() : REGEXP_HOME_PREFIX.test(path) ? `${homedir()}${path.slice(1)}` : path;
const REGEXP_HOME_PREFIX = /^~[/\\]/;
const resolveHome = (...argList) => resolve(...argList.map(expandHome));
const createPathPrefixLock = rootPath => {
  rootPath = resolve(rootPath);
  return relativePath => {
    const absolutePath = resolve(rootPath, relativePath);
    if (!absolutePath.startsWith(rootPath)) throw new Error(`invalid relativePath: ${relativePath}`);
    return absolutePath;
  };
};
export { STAT_ERROR, PATH_TYPE, getPathTypeFromStat, getPathLstat, getPathLstatSync, getPathStat, getPathStatSync, copyPath, copyPathSync, renamePath, renamePathSync, deletePath, deletePathSync, deletePathForce, deletePathForceSync, existPath, existPathSync, nearestExistPath, nearestExistPathSync, toPosixPath, addTrailingSep, dropTrailingSep, expandHome, resolveHome, createPathPrefixLock };