import { dirname } from "node:path";
import { watch, promises as fsAsync } from "node:fs";
import { throttleT } from "../../common/function.js";
import { createHub } from "../../common/module/Event.js";
import { nearestExistPath } from "./Path.js";
const EMPTY_FUNC = () => {};
const createFileWatcherExot = ({path, wait = 250, persistent = !1, id = "exot:file-watcher"}) => {
  const hub = createHub();
  let prevStat;
  let watcherPath;
  let watcherStat;
  let watcher;
  let watcherUpper;
  const emitThrottled = throttleT((async () => {
    const stat = await fsAsync.lstat(path).catch(EMPTY_FUNC);
    const hasChange = Boolean(prevStat) !== Boolean(stat);
    if (void 0 === stat) {
      path === watcherPath && await setupWatch();
      if (!1 === hasChange) return;
    }
    const changeState = {
      path,
      stat,
      hasChange
    };
    prevStat = stat;
    hub.send(changeState);
  }), wait);
  const onErrorEvent = async (...args) => {
    await setupWatch();
    onChangeEvent("rename");
  };
  const onChangeEvent = (changeType, path) => {
    emitThrottled();
  };
  const clearWatch = () => {
    watcher && watcher.close();
    watcherUpper && watcherUpper.close();
    watcherPath = void 0;
    watcherStat = void 0;
    watcher = void 0;
    watcherUpper = void 0;
  };
  const setupWatch = async () => {
    const stat = await fsAsync.lstat(path).catch(EMPTY_FUNC);
    if (stat) {
      clearWatch();
      watcherPath = path;
      watcherStat = stat;
      watcher = watch(path, {
        persistent,
        recursive: !1
      });
      watcher.addListener("error", onErrorEvent);
      watcher.addListener("change", onChangeEvent);
      if (stat.isDirectory()) {
        watcherUpper = watch(dirname(path));
        watcherUpper.addListener("error", EMPTY_FUNC);
        watcherUpper.addListener("change", onChangeEvent);
      }
    } else {
      const pathNearest = await nearestExistPath(path);
      if (pathNearest === watcherPath) return;
      clearWatch();
      watcherPath = pathNearest;
      watcherStat = stat;
      watcher = watch(pathNearest, {
        persistent,
        recursive: !1
      });
      watcher.addListener("error", onErrorEvent);
      watcher.addListener("change", onErrorEvent);
    }
  };
  return {
    id,
    up: async onExotError => {
      await setupWatch();
      prevStat = watcherPath === path ? watcherStat : void 0;
    },
    down: () => {
      clearWatch();
      hub.clear();
      prevStat = void 0;
    },
    isUp: () => void 0 !== watcher,
    subscribe: listener => hub.subscribe(listener),
    unsubscribe: listener => hub.unsubscribe(listener)
  };
};
export { createFileWatcherExot };