import { dirname } from "node:path";
import { createDirectory } from "../../fs/Directory.js";
import { run } from "../../run.js";
import { check as checkAuto, compressAutoAsync, extractAutoAsync } from "../Archive/archive.js";
import { check as checkNpmTar, compressAsync as compressNpmTarAsync, extractAsync as extractNpmTarAsync } from "../Archive/npmTar.js";
import { check as checkTar, extractArgs as compressArgsTar, extractArgs as extractArgsTar } from "../Archive/tar.js";
const PATH_COMPRESS_TAR = "path.compress-tar";
const PATH_EXTRACT_TAR = "path.extract-tar";
const PATH_COMPRESS_AUTO = "path.compress-auto";
const PATH_EXTRACT_AUTO = "path.extract-auto";
const ACTION_TYPE = {
  PATH_COMPRESS_TAR: "path.compress-tar",
  PATH_EXTRACT_TAR: "path.extract-tar",
  PATH_COMPRESS_AUTO: "path.compress-auto",
  PATH_EXTRACT_AUTO: "path.extract-auto"
};
const ACTION_CORE_MAP = checkAuto() && {
  [PATH_COMPRESS_TAR]: async (sourceDirectory, outputFile) => {
    await compressAutoAsync(sourceDirectory, outputFile);
  },
  [PATH_EXTRACT_TAR]: async (sourceFile, outputPath) => {
    await extractAutoAsync(sourceFile, outputPath);
  },
  [PATH_COMPRESS_AUTO]: async (sourceDirectory, outputFile) => {
    await compressAutoAsync(sourceDirectory, outputFile);
  },
  [PATH_EXTRACT_AUTO]: async (sourceFile, outputPath) => {
    await extractAutoAsync(sourceFile, outputPath);
  }
} || checkNpmTar() && {
  [PATH_COMPRESS_TAR]: async (sourceDirectory, outputFile) => {
    await compressNpmTarAsync(sourceDirectory, outputFile);
  },
  [PATH_EXTRACT_TAR]: async (sourceFile, outputPath) => {
    await extractNpmTarAsync(sourceFile, outputPath);
  }
} || checkTar() && {
  [PATH_COMPRESS_TAR]: async (sourceDirectory, outputFile) => {
    await createDirectory(dirname(outputFile));
    await run(compressArgsTar(sourceDirectory, outputFile)).promise;
  },
  [PATH_EXTRACT_TAR]: async (sourceFile, outputPath) => {
    await createDirectory(outputPath);
    await run(extractArgsTar(sourceFile, outputPath)).promise;
  }
} || {};
export { ACTION_TYPE, ACTION_CORE_MAP };