import { createReadStream, createWriteStream, promises as fsAsync } from "node:fs";
import { constants, createGzip, createGunzip, createBrotliCompress, createBrotliDecompress } from "node:zlib";
import { quickRunletFromStream } from "../../data/Stream.js";
const REGEXP_TGZ = /\.t(?:ar\.)?gz$/;
const REGEXP_TBR = /\.t(?:ar\.)?br$/;
const REGEXP_T7Z = /\.t(?:ar\.)?7z$/;
const REGEXP_TXZ = /\.t(?:ar\.)?xz$/;
const isBufferGzip = buffer => buffer && buffer.length > 3 && 31 === buffer[0] && 139 === buffer[1] && 8 === buffer[2];
const isFileGzip = async file => {
  const buffer = Buffer.allocUnsafe(3);
  await fsAsync.read(file, buffer, 0, 3, 0);
  return isBufferGzip(buffer);
};
const createGzipMax = () => createGzip({
  level: 9,
  windowBits: 15,
  memLevel: 9,
  chunkSize: 131072
});
const createBrotliCompressMax = () => createBrotliCompress({
  params: {
    [constants.BROTLI_PARAM_QUALITY]: 11
  },
  chunkSize: 131072
});
const REGEXP_GZ = /\.t?gz$/;
const REGEXP_BR = /\.t?br$/;
const REGEXP_GZBR = /\.t?(?:gz|br)$/;
const compressGzBrFileAsync = async (fileFrom, fileTo, isGzip = REGEXP_GZ.test(fileTo)) => quickRunletFromStream(createReadStream(fileFrom), isGzip ? createGzipMax() : createBrotliCompressMax(), createWriteStream(fileTo));
const extractGzBrFileAsync = async (fileFrom, fileTo, isGzip = REGEXP_GZ.test(fileFrom)) => quickRunletFromStream(createReadStream(fileFrom), isGzip ? createGunzip() : createBrotliDecompress(), createWriteStream(fileTo));
export { REGEXP_TGZ, REGEXP_TBR, REGEXP_T7Z, REGEXP_TXZ, isBufferGzip, isFileGzip, createGzipMax, createBrotliCompressMax, REGEXP_GZ, REGEXP_BR, REGEXP_GZBR, compressGzBrFileAsync, extractGzBrFileAsync };