import { resolve } from "node:path";
import { createBrotliCompress, createBrotliDecompress } from "node:zlib";
import { createReadStream, createWriteStream, promises as fsAsync } from "node:fs";
import { tryRequire } from "../../../env/tryRequire.js";
import { quickRunletFromStream } from "../../data/Stream.js";
import { fromNpmNodeModules } from "../Software/npm.js";
import { REGEXP_TGZ, REGEXP_TBR } from "./function.js";
const REGEXP_NPM_TAR = /\.(?:tar|tgz|tar\.gz|tbr|tar\.br)$/;
let cacheNpmTar;
const getNpmTar = () => {
  void 0 === cacheNpmTar && (cacheNpmTar = tryRequire(fromNpmNodeModules("tar")) || null);
  return cacheNpmTar;
};
const check = () => Boolean(getNpmTar());
const verify = () => {
  const npmTar = getNpmTar();
  if (!npmTar) throw new Error('expect "npm/node_modules/tar"');
  return npmTar;
};
const createCompressStream = (sourceDirectory, option = {
  gzip: !0
}, nameList = [ "./" ]) => {
  sourceDirectory = resolve(sourceDirectory);
  nameList.forEach(((name, index) => name.startsWith("@") && (nameList[index] = `./${name}`)));
  return getNpmTar().create({
    cwd: sourceDirectory,
    ...option
  }, nameList);
};
const createExtractStream = (outputPath, option) => {
  outputPath = resolve(outputPath);
  return getNpmTar().extract({
    cwd: outputPath,
    ...option
  });
};
const compressAsync = async (sourceDirectory, outputFile) => quickRunletFromStream.apply(null, [ createCompressStream(sourceDirectory, {
  gzip: REGEXP_TGZ.test(outputFile)
}, await fsAsync.readdir(sourceDirectory)), REGEXP_TBR.test(outputFile) && createBrotliCompress(), createWriteStream(resolve(outputFile)) ].filter(Boolean));
const extractAsync = async (sourceFile, outputPath) => quickRunletFromStream.apply(null, [ createReadStream(resolve(sourceFile)), REGEXP_TBR.test(sourceFile) && createBrotliDecompress(), createExtractStream(outputPath) ].filter(Boolean));
const REGEXP_PACKAGE_JSON = /^[^/]+\/package\.json$/;
const extractPackageJSON = async sourceFile => {
  const chunkList = [];
  await getNpmTar().list({
    file: sourceFile,
    onentry: entry => {
      REGEXP_PACKAGE_JSON.test(entry.path) && entry.on("data", (chunk => chunkList.push(chunk)));
    }
  });
  if (!chunkList.length) throw new Error(`expect "${REGEXP_PACKAGE_JSON}" in ".tgz"`);
  return JSON.parse(String(Buffer.concat(chunkList)));
};
export { REGEXP_NPM_TAR, getNpmTar, check, verify, createCompressStream, createExtractStream, compressAsync, extractAsync, extractPackageJSON };