import { indentLine, splitKebabCase, joinCamelCase, joinSnakeCase } from "../../../common/string.js";
const FORMAT_DEFAULT = {
  name: "",
  nameListENV: [],
  nameListCONFIG: [],
  shortName: "",
  aliasNameList: [],
  optional: !1,
  argumentCount: "0",
  argumentLengthMin: 0,
  argumentLengthMax: 0,
  argumentListNormalize: argumentList => argumentList,
  argumentListVerify: argumentList => {},
  description: "",
  extendFormatList: []
};
const createOptionParser = ({formatList, prefixENV = "", prefixCONFIG = ""}) => {
  const nameMapCLI = new Map;
  const nameMapCLIShort = new Map;
  const nameMapENV = new Map;
  const nameMapCONFIG = new Map;
  const nonOptionalFormatSet = new Set;
  const optionalFormatCheckSet = new Set;
  const parseFormat = (format, index, upperFormat) => {
    const {name, shortName, aliasNameList, argumentCount} = format;
    format.optional = parseOptional(format.optional, upperFormat);
    const [, argumentLengthMinString, argumentLengthSep, argumentLengthMaxString] = REGEXP_FORMAT_ARGUMENT_COUNT.exec(String(argumentCount));
    format.argumentLengthMin = parseInt(argumentLengthMinString);
    format.argumentLengthMax = argumentLengthSep ? argumentLengthMaxString ? parseInt(argumentLengthMaxString) : 1 / 0 : format.argumentLengthMin;
    nameMapCLI.has(name) && throwFormatError(`duplicate name '${name}'`, format, index, upperFormat);
    nameMapCLI.set(name, format);
    shortName && nameMapCLIShort.has(shortName) && throwFormatError(`duplicate shortName '${shortName}'`, format, index, upperFormat);
    shortName && nameMapCLIShort.set(shortName, format);
    {
      const duplicateAliasName = aliasNameList.find((aliasName => nameMapCLI.has(aliasName)));
      duplicateAliasName && throwFormatError(`duplicate aliasName '${duplicateAliasName}'`, format, index, upperFormat);
      aliasNameList.forEach((aliasName => nameMapCLI.set(aliasName, format)));
    }
    format.nameListENV = [];
    format.nameListCONFIG = [];
    for (const aliasName of [ name, ...aliasNameList ].filter((v => v.length > 1)).reverse()) {
      const nameENV = joinSnakeCase(splitKebabCase(prefixENV ? `${prefixENV}-${aliasName}` : aliasName));
      const nameCONFIG = joinCamelCase(splitKebabCase(prefixCONFIG ? `${prefixCONFIG}-${aliasName}` : aliasName));
      nameMapENV.has(nameENV) && throwFormatError(`duplicate nameENV '${nameENV}'`, format, index, upperFormat);
      nameMapCONFIG.has(nameCONFIG) && throwFormatError(`duplicate nameCONFIG '${nameCONFIG}'`, format, index, upperFormat);
      nameMapENV.set(nameENV, format);
      nameMapCONFIG.set(nameCONFIG, format);
      format.nameListENV.unshift(nameENV);
      format.nameListCONFIG.unshift(nameCONFIG);
    }
    format.optional ? format.optional !== OPTIONAL_TRUE && optionalFormatCheckSet.add({
      format,
      checkOptional: format.optional
    }) : nonOptionalFormatSet.add(format);
    format.extendFormatList.forEach(((extendFormat, index) => parseFormat(extendFormat, index, format)));
  };
  (formatList = formatList.map(((format, index) => normalizeFormat(format, index, null)))).forEach(((format, index) => parseFormat(format, index, null)));
  return {
    parseCLI: getParseCLI(getParseArgvList(nameMapCLI, nameMapCLIShort)),
    parseENV: getParseENV(nameMapENV),
    parseCONFIG: getParseCONFIG(nameMapCONFIG),
    processOptionMap: getProcessOptionMap(nonOptionalFormatSet, optionalFormatCheckSet),
    formatUsage: (message, isSimple = !1) => join(message && `Message:\n${indent(String(message))}`, `CLI Usage:\n${indent(usageCLI(formatList))}`, !isSimple && `ENV Usage:\n${indent(usageENV(formatList))}`, !isSimple && `CONFIG Usage:\n${indent(usageCONFIG(formatList))}`)
  };
};
const REGEXP_FORMAT_ARGUMENT_COUNT = /^(\d+)(-)?(\d+)?$/;
const parseOptional = (optional, upperFormat) => {
  "function" != typeof optional && (optional = !!optional && OPTIONAL_TRUE);
  return upperFormat ? (optionMap, optionFormatSet, format) => optional && optional(optionMap, optionFormatSet, format) || !optionFormatSet.has(upperFormat) : optional;
};
const OPTIONAL_TRUE = () => !0;
const normalizeFormat = (format, index, upperFormat) => {
  format = {
    ...FORMAT_DEFAULT,
    ...format
  };
  !REGEXP_FORMAT_NAME.test(format.name) && throwFormatError(`name '${format.name}'`, format, index, upperFormat);
  format.shortName && !REGEXP_FORMAT_SHORT_NAME.test(format.shortName) && throwFormatError(`shortName '${format.shortName}'`, format, index, upperFormat);
  {
    const errorIndex = format.aliasNameList.findIndex((aliasName => !REGEXP_FORMAT_NAME.test(aliasName)));
    -1 !== errorIndex && throwFormatError(`aliasNameList #${errorIndex} '${format.aliasNameList[errorIndex]}'`, format, index, upperFormat);
  }
  {
    !REGEXP_FORMAT_ARGUMENT_COUNT.test(format.argumentCount) && throwFormatError(`argumentCount '${format.argumentCount}'`, format, index, upperFormat);
    const [, from, , to] = REGEXP_FORMAT_ARGUMENT_COUNT.exec(format.argumentCount);
    to && parseInt(to) < parseInt(from) && throwFormatError(`argumentCount '${format.argumentCount}'`, format, index, upperFormat);
  }
  format.extendFormatList.length && (format.extendFormatList = format.extendFormatList.map(((extendFormat, index) => normalizeFormat(extendFormat, index, format))));
  return format;
};
const REGEXP_FORMAT_NAME = /^[A-Za-z][A-Za-z0-9-]*$/;
const REGEXP_FORMAT_SHORT_NAME = /^[A-Za-z]$/;
const throwFormatError = (message, format, index, upperFormat) => {
  throw new Error(`[Format] ${formatSimple(format)} #${index}${upperFormat ? ` of ${formatSimple(upperFormat)}` : ""} | ${message}`);
};
const getParseArgvList = (nameMapCLI, nameMapCLIShort) => argvList => {
  const optionList = [];
  const getLastOptionArgumentList = () => optionList[optionList.length - 1].argumentList;
  for (let index = 0, indexMax = argvList.length; index < indexMax; index++) {
    const value = argvList[index];
    if (REGEXP_FORMAT_CLI_NAME.test(value)) {
      const [, name, , extraArgument] = REGEXP_FORMAT_CLI_NAME.exec(value);
      const format = nameMapCLI.get(name);
      !format && throwParseArgvError(name, "invalid option");
      optionList.push({
        format,
        argumentList: []
      });
      extraArgument && getLastOptionArgumentList().push(extraArgument);
    } else if (REGEXP_FORMAT_CLI_SHORT_NAME.test(value)) {
      const [, shortNameString, , extraArgument] = REGEXP_FORMAT_CLI_SHORT_NAME.exec(value);
      shortNameString.split("").forEach((shortName => {
        const format = nameMapCLIShort.get(shortName);
        !format && throwParseArgvError(shortName, `from '${shortNameString}'`);
        optionList.push({
          format,
          argumentList: []
        });
      }));
      extraArgument && getLastOptionArgumentList().push(extraArgument);
    } else {
      !optionList.length && throwParseArgvError(value, "no leading option found");
      if ("--" === value) {
        getLastOptionArgumentList().push(...argvList.slice(index + 1));
        break;
      }
      getLastOptionArgumentList().push(value);
    }
  }
  return optionList;
};
const REGEXP_FORMAT_CLI_NAME = /^--([A-Za-z][A-Za-z0-9-]*)(=(.*))?$/;
const REGEXP_FORMAT_CLI_SHORT_NAME = /^-([A-Za-z]+)(=(.*))?$/;
const throwParseArgvError = (arg, detail) => {
  throw new Error(`[ParseArgv] unexpected '${arg}', ${detail}`);
};
const getParseCLI = parseArgvList => (argvList, optionMap = {}) => parseArgvList(argvList).reduce(((o, {format, argumentList}) => {
  o[format.name] ? o[format.name].argumentList.push(...argumentList) : o[format.name] = {
    format,
    argumentList,
    source: "CLI"
  };
  return o;
}), optionMap);
const getParseENV = nameMapENV => (envObject, optionMap = {}) => {
  nameMapENV.forEach(((format, nameENV) => {
    let value = envObject[nameENV];
    if (void 0 !== value) {
      try {
        value = JSON.parse(value);
      } catch (error) {}
      optionMap[format.name] = {
        format,
        argumentList: Array.isArray(value) ? value : [ value ],
        source: "ENV"
      };
    }
  }));
  return optionMap;
};
const getParseCONFIG = nameMapCONFIG => (object, optionMap = {}) => {
  nameMapCONFIG.forEach(((format, nameCONFIG) => {
    const value = object[nameCONFIG];
    void 0 !== value && (optionMap[format.name] = {
      format,
      argumentList: Array.isArray(value) ? value : [ value ],
      source: "CONFIG"
    });
  }));
  return optionMap;
};
const getProcessOptionMap = (nonOptionalFormatSet, optionalFormatCheckSet) => (optionMap, extendOption) => {
  const optionFormatSet = new Set;
  Object.entries(optionMap).forEach((([name, option]) => {
    const {format, argumentList} = option;
    format.argumentLengthMin > argumentList.length && throwProcessError(`expect ${format.argumentLengthMin - argumentList.length} more argument`, format);
    format.argumentLengthMax < argumentList.length && throwProcessError(`expect ${argumentList.length - format.argumentLengthMax} less argument`, format);
    option.argumentList = format.argumentListNormalize(argumentList, extendOption);
    format.argumentListVerify(option.argumentList, extendOption);
    optionFormatSet.add(format);
  }));
  nonOptionalFormatSet.forEach((format => !optionFormatSet.has(format) && throwProcessError("non-optional option", format)));
  optionalFormatCheckSet.forEach((({format, checkOptional}) => !checkOptional(optionMap, optionFormatSet, format) && !optionFormatSet.has(format) && throwProcessError("non-optional option", format)));
  return optionMap;
};
const throwProcessError = (message, format) => {
  throw new Error(`[Process] ${formatSimple(format)} | ${message}`);
};
const formatSimple = ({name, shortName, aliasNameList}) => `${name}${aliasNameList.length ? `|${aliasNameList.join("|")}` : ""}${shortName ? ` [-${shortName}]` : ""}`;
const usageCLI = formatList => mapJoin(formatList, formatUsageCLI);
const formatUsageCLI = format => join(formatUsageBase(format, `--${format.name}${format.aliasNameList.length ? ` --${format.aliasNameList.join(" --")}` : ""}`, format.shortName && `-${format.shortName}`), format.description && indent(format.description, 4), formatExtendList(format.extendFormatList, formatUsageCLI, 2));
const usageENV = formatList => join('"', indent(`#!/usr/bin/env bash\n${mapJoin(formatList, formatUsageENV)}`), '"');
const formatUsageENV = format => join(`export ${format.nameListENV[0]}="${formatUsageBase(format)}${1 === format.nameListENV.length ? "" : ` [ALIAS=${format.nameListENV.slice(1).join(",")}]`}"`, formatExtendList(format.extendFormatList, formatUsageENV, 0));
const usageCONFIG = formatList => join("{", indent(mapJoin(formatList, formatUsageCONFIG)), "}");
const formatUsageCONFIG = format => join(`"${format.nameListCONFIG[0]}": [ "${formatUsageBase(format)}${1 === format.nameListCONFIG.length ? "" : ` [ALIAS=${format.nameListCONFIG.slice(1).join(",")}]`}" ],`, formatExtendList(format.extendFormatList, formatUsageCONFIG, 0));
const formatUsageBase = ({optional, argumentLengthMin, argumentLengthMax}, ...args) => [ ...args, optional === OPTIONAL_TRUE && "[OPTIONAL]", (argumentLengthMin || argumentLengthMax) && `[ARGUMENT=${formatRange(argumentLengthMin, argumentLengthMax)}]` ].filter(Boolean).join(" ");
const formatExtendList = (extendFormatList, formatUsage, indentCount) => extendFormatList.length && indent(mapJoin(extendFormatList, formatUsage), indentCount);
const formatRange = (min, max) => `${min}${max === 1 / 0 ? "+" : max > min ? `-${max}` : ""}`;
const mapJoin = (array, func) => join(...array.map(func));
const join = (...fragList) => fragList.filter(Boolean).join("\n");
const indent = (text, count = 2) => count ? indentLine(text, " ".repeat(count)) : text;
export { createOptionParser };