import { resolve, basename } from "node:path";
import { sortPackageJSON, packPackageJSON, toPackageInfo, collectDependency } from "../../common/module/PackageJSON.js";
import { editText, readJSON, writeText } from "../fs/File.js";
import { getFileList } from "../fs/Directory.js";
const toPackageJSONPath = (path = process.cwd()) => resolve(...[ path, path.endsWith("package.json") ? "" : "package.json" ].filter(Boolean));
const toPackageRootPath = (path = process.cwd()) => resolve(...[ path, path.endsWith("package.json") ? ".." : "" ].filter(Boolean));
const writePackageJSON = async (path, packageJSON, isSortKey = !1) => writeText(path, packPackageJSON(isSortKey ? sortPackageJSON(packageJSON) : packageJSON));
const editPackageJSON = async (editFunc, pathFrom, pathTo) => editText((async string => {
  let packageJSON = JSON.parse(string);
  packageJSON = await editFunc(packageJSON);
  return packPackageJSON(packageJSON);
}), pathFrom, pathTo);
const loadPackageInfo = async path => {
  const packageJSONPath = toPackageJSONPath(path);
  return toPackageInfo({
    packageJSON: await readJSON(packageJSONPath),
    packageJSONPath,
    packageRootPath: toPackageRootPath(packageJSONPath)
  });
};
const loadPackageInfoList = async pathRoot => {
  const pathPackageJSONList = (await getFileList(pathRoot)).filter((path => "package.json" === basename(path)));
  const packageInfoList = [];
  for (const pathPackageJSON of pathPackageJSONList) packageInfoList.push(await loadPackageInfo(pathPackageJSON));
  return packageInfoList;
};
const loadPackageCombo = async pathRoot => {
  const packageInfoList = await loadPackageInfoList(pathRoot);
  const {dependencyMap, dependencyInfoMap, duplicateInfoList} = packageInfoList.reduce(((o, packageInfo) => collectDependency(packageInfo, o)), {});
  return {
    packageInfoList,
    dependencyMap,
    dependencyInfoMap,
    duplicateInfoList
  };
};
const savePackageInfo = async ({packageJSON, packageJSONPath, isSortKey = !1}) => writePackageJSON(packageJSONPath, packageJSON, isSortKey);
export { toPackageJSONPath, toPackageRootPath, writePackageJSON, editPackageJSON, loadPackageInfo, loadPackageInfoList, loadPackageCombo, savePackageInfo };