import { tryRequire } from "../../env/tryRequire.js";
import { isBasicFunction } from "../../common/check.js";
const DENY_ALL = () => !1;
const ALLOW_ALL = () => !0;
const configurePermission = async ({permissionType, permissionFunc, permissionFile, loggerExot}) => {
  let configurePermissionFunc;
  switch (permissionType) {
   case "deny":
    loggerExot.add("use permission: deny-all");
    return {
      checkPermission: DENY_ALL
    };
   case "allow":
    loggerExot.add("use permission: allow-all");
    return {
      checkPermission: ALLOW_ALL
    };
   case "func":
    configurePermissionFunc = permissionFunc;
    if (!isBasicFunction(configurePermissionFunc)) throw new Error("invalid permissionFunc");
    loggerExot.add("use permissionFunc");
    break;
   case "file":
    configurePermissionFunc = (tryRequire(permissionFile) || {
      configurePermission: null
    }).configurePermission;
    if (!isBasicFunction(configurePermissionFunc)) throw new Error(`failed to load permissionFile: ${permissionFile}`);
    loggerExot.add("use permissionFile");
    break;
   default:
    throw new Error(`invalid permissionType: ${permissionType}`);
  }
  const permissionPack = await configurePermissionFunc({
    loggerExot
  });
  if (!isBasicFunction(permissionPack.checkPermission)) throw new Error("expect permissionPack.checkPermission to be function");
  return permissionPack;
};
export { configurePermission };