import { dirname } from "node:path";
import { catchSync } from "../../common/error.js";
import { readTextSync, writeTextSync } from "../fs/File.js";
import { createDirectorySync } from "../fs/Directory.js";
import { deletePathSync } from "../fs/Path.js";
import { addExitListenerSync } from "../system/ExitListener.js";
import { isPidExist } from "../system/Process.js";
const configurePid = ({filePid, shouldIgnoreExistPid = !1} = {}) => {
  if (filePid) {
    !shouldIgnoreExistPid && catchSync((() => {
      const existingPid = Number(readTextSync(filePid).trim());
      if (existingPid && isPidExist(existingPid)) {
        console.warn(`[Pid] get existing pid: ${existingPid}, exit process...`);
        process.exit(-1);
      }
    }));
    createDirectorySync(dirname(filePid));
    writeTextSync(filePid, `${process.pid}`);
    addExitListenerSync((exitState => {
      catchSync(deletePathSync, filePid);
    }));
  }
};
export { configurePid };