import { createInsideOutPromise, withTimeoutPromise } from "../../common/function.js";
import { clock } from "../../common/time.js";
import { requestHttp } from "../net.js";
import { run } from "../run.js";
const batchRequestUrlList = (onResponse, urlList, option, body) => {
  const requestSet = new Set;
  const subProcessSet = new Set;
  void 0 === option.lookup && (option.lookup = (subProcessSet => (hostname, option, callback) => {
    const {subProcess, promise, stdoutPromise} = run([ process.execPath, "-e", `require('node:dns').lookup(${JSON.stringify(hostname)}, ${JSON.stringify(option)}, (error, ...args) => console.log(JSON.stringify([ error ? error.stack : null, ...args ])))` ], {
      quiet: !0
    });
    promise.then((async () => {
      const [errorStack, ...args] = JSON.parse(String(await stdoutPromise));
      callback(errorStack ? new Error(errorStack) : void 0, ...args);
    })).catch((error => {
      callback(error);
    })).then((() => subProcessSet.delete(subProcess)));
    subProcessSet.add(subProcess);
  })(subProcessSet));
  const promise = Promise.all(urlList.map((url => {
    const {request, promise} = requestHttp(url, option, body);
    requestSet.add(request);
    return promise.then((response => {
      requestSet.delete(request);
      response.destroy();
      onResponse(url);
    }), (error => {
      requestSet.delete(request);
    }));
  })));
  return {
    requestSet,
    promise,
    clear: () => {
      for (const request of requestSet) request.destroy();
      for (const subProcess of subProcessSet) subProcess.kill();
      return promise;
    }
  };
};
const pingRaceUrlList = async (urlList = [], {timeout = 5e3, body = null, ...option} = {}) => {
  if (urlList.length < 2) return urlList[0];
  const {promise, resolve} = createInsideOutPromise();
  const batchRequest = batchRequestUrlList(resolve, urlList, {
    timeout,
    ...option
  }, body);
  const resultUrl = await withTimeoutPromise(promise, timeout).catch((error => urlList[0]));
  await batchRequest.clear();
  return resultUrl;
};
const PING_STAT_ERROR = null;
const pingStatUrlList = async (urlList = [], {timeout = 5e3, body = null, ...option} = {}) => {
  const statMap = urlList.reduce(((o, url) => {
    o[url] = null;
    return o;
  }), {});
  const timeStart = clock();
  const batchRequest = batchRequestUrlList((url => {
    statMap[url] = clock() - timeStart;
  }), urlList, {
    timeout,
    ...option
  }, body);
  await withTimeoutPromise(batchRequest.promise, timeout).catch((error => {}));
  await batchRequest.clear();
  return statMap;
};
export { pingRaceUrlList, pingStatUrlList, PING_STAT_ERROR };