import { openSync, write, writeSync, closeSync } from "node:fs";
import { rethrowError } from "../../common/error.js";
/** @typedef { { pathOutputFile: string, flag?: OpenMode, mode?: Mode, fileDescriptor?: number, onError?: (error: Error) => never } } OptCreateSafeWriteStream */
/** @type { (opt: OptCreateSafeWriteStream) => { write: (string: string) => void, end: () => void } } } */ const createSafeWriteStream = ({pathOutputFile, flag = "w", mode = 438, fileDescriptor = openSync(pathOutputFile, flag, mode), onError = rethrowError}) => {
  let writingQueue = [];
  let pendingQueue = [];
  const onWriteDone = error => error ? onError(error) : writingQueue.length = 0;
  const doWrite = string => void 0 !== fileDescriptor && write(fileDescriptor, string, onWriteDone);
  return {
    write: string => {
      if (string) {
        pendingQueue.push(string);
        if (0 === writingQueue.length) {
          [writingQueue, pendingQueue] = [ pendingQueue, [] ];
          process.nextTick(doWrite, writingQueue.join(""));
        }
      }
    },
    end: () => {
      const remainQueue = [ ...writingQueue, ...pendingQueue ];
      0 !== remainQueue.length && writeSync(fileDescriptor, remainQueue.join(""));
      closeSync(fileDescriptor);
      fileDescriptor = void 0;
      pendingQueue.length = writingQueue.length = 0;
    }
  };
};
export { createSafeWriteStream };