import { encode as encodeB62S } from "../../../common/data/B62S.js";
import { getRandomId62S } from "../../../common/math/random.js";
import { run, runStdout, runSync, runStdoutSync } from "../../run.js";
import { probeSync, createArgListPack } from "../function.js";
const {getArgs, setArgs, check, verify} = createArgListPack((() => probeSync([ "bash", "--version" ], "GNU bash") ? [ "bash" ] : void 0), 'expect "bash" in PATH');
const runBash = (argList = [], option) => run([ ...verify(), ...argList ], option);
const runBashSync = (argList = [], option) => runSync([ ...verify(), ...argList ], option);
const runBashStdout = (argList = [], option) => runStdout([ ...verify(), ...argList ], option);
const runBashStdoutSync = (argList = [], option) => runStdoutSync([ ...verify(), ...argList ], option);
const runBashCommand = (commandList = [], option) => runStdout([ ...verify(), "-ec", joinCommand(commonCommandList(commandList)) ], option);
const runBashCommandSync = (commandList = [], option) => runSync([ ...verify(), "-ec", joinCommand(commonCommandList(commandList)) ], option);
const commonBashArgList = commandList => [ "bash", "-ec", joinCommand(commandList) ];
const joinCommand = commandList => commandList.filter(Boolean).join("\n");
const commonCommandList = commandList => [ "shopt -s dotglob nullglob", "set -ex", ...commandList ];
const subShellCommandList = commandList => [ "( # ".padEnd(16, "="), ...commandList, ") # ".padEnd(16, "=") ];
const toHeredocNoMagic = (string = "", extraPipeCommand = "", EOM = getRandomId62S(`EOM-${encodeB62S(string.length)}-`)) => `<< '${EOM}' ${extraPipeCommand}\n${string}\n${EOM}`;
const catStringToVarCommand = string => `$(cat ${toHeredocNoMagic(string)}\n)`;
const catStringToFileCommand = (string, pathFile) => `cat ${toHeredocNoMagic(string, `> "${pathFile}"`)}`;
const gitFetchBranchCommandList = (branchName, branchCommitHash = "HEAD", remoteName = "origin") => [ `git fetch --force --no-tags "${remoteName}" "${branchName}:remotes/${remoteName}/${branchName}"`, `git checkout --force -B "${branchName}"`, `git reset --hard "${branchCommitHash}"` ];
const gitCleanUpCommandList = () => [ "git gc --auto", "( git branch -D $(git branch | grep -v \\* | xargs) 2> /dev/null || true )" ];
const commonSourceProfileCommandList = () => [ "[[ -f /etc/profile ]] && source /etc/profile", "[[ -f ~/.profile ]] && source ~/.profile", "[[ -f ~/.bash_profile ]] && source ~/.bash_profile" ];
export { getArgs, setArgs, check, verify, runBash, runBashSync, runBashStdout, runBashStdoutSync, runBashCommand, runBashCommandSync, commonBashArgList, joinCommand, commonCommandList, subShellCommandList, toHeredocNoMagic, catStringToVarCommand, catStringToFileCommand, gitFetchBranchCommandList, gitCleanUpCommandList, commonSourceProfileCommandList };