import { catchAsync } from "../../../common/error.js";
import { run, runStdout, runSync, runStdoutSync } from "../../run.js";
import { probeSync, createArgListPack } from "../function.js";
const {getArgs, setArgs, check, verify} = createArgListPack((() => probeSync([ "docker", "version" ], "Server:") ? [ "docker" ] : probeSync([ "sudo", "docker", "version" ], "Server:") ? [ "sudo", "docker" ] : void 0), 'expect "docker" in PATH, with dockerd up');
const runDocker = (argList = [], option) => run([ ...verify(), ...argList ], option);
const runDockerSync = (argList = [], option) => runSync([ ...verify(), ...argList ], option);
const runDockerStdout = (argList = [], option) => runStdout([ ...verify(), ...argList ], option);
const runDockerStdoutSync = (argList = [], option) => runStdoutSync([ ...verify(), ...argList ], option);
const parseDockerImage = (image = "") => {
  if (image.toLowerCase().includes("@sha256:")) throw new Error(`expect tag without digest: ${image}`);
  const stubList = image.split(":");
  if (stubList.length <= 1) throw new Error(`expect explicit tag: ${image}`);
  const tag = stubList.pop();
  if (!/^\w[\w-.]+$/.test(tag)) throw new Error(`invalid tag: ${image}`);
  return [ stubList.join(":"), tag ];
};
const checkLocalImage = async (imageRepo, imageTag) => {
  const stdoutString = String(await runDockerStdout([ "image", "ls", `${imageRepo}:${imageTag}` ]));
  return stdoutString.includes(imageRepo) && stdoutString.includes(imageTag);
};
const pullImage = async (imageRepo, imageTag) => runDockerStdout([ "pull", `${imageRepo}:${imageTag}` ]);
const checkPullImage = async (imageRepo, imageTag) => {
  if (await checkLocalImage(imageRepo, imageTag)) return !0;
  await catchAsync(pullImage, imageRepo, imageTag);
  return checkLocalImage(imageRepo, imageTag);
};
const getContainerLsList = async (isListAll = !1) => String(await runDockerStdout([ "container", "ls", "--format", "{{.ID}}|{{.Image}}|{{.Names}}", isListAll && "--all" ].filter(Boolean))).trim().split("\n").filter(Boolean).map((string => {
  const [id, image, names] = string.split("|");
  return {
    id,
    image,
    names
  };
}));
const patchContainerLsListStartedAt = async (containerLsList = []) => {
  const idList = containerLsList.map((({id}) => id));
  String(await runDockerStdout([ "container", "inspect", "--format", "{{.Id}}|{{.State.StartedAt}}", ...idList ])).trim().split("\n").filter(Boolean).forEach((string => {
    const [id, startedAtString] = string.split("|");
    const item = containerLsList[idList.findIndex((v => id.startsWith(v)))];
    item && (item.startedAt = new Date(startedAtString));
  }));
};
const matchContainerLsList = (containerLsList = [], processList = []) => {
  containerLsList.forEach((item => {
    item.pid = (processList.find((({command}) => command.includes(item.id))) || {}).pid;
  }));
};
const {getArgs: getArgsCompose, setArgs: setArgsCompose, check: checkCompose, verify: verifyCompose} = createArgListPack((() => {
  if (!check()) return;
  const argsListP = [ ...getArgs(), "compose" ];
  if (probeSync([ ...argsListP, "version" ], "ompose version")) return argsListP;
  const argsList = [ ...getArgs().slice(0, -1), "docker-compose" ];
  return probeSync([ ...argsList, "version" ], "ompose version") ? argsList : void 0;
}), 'expect both "docker" and "docker compose|docker-compose" in PATH, with dockerd up');
const runCompose = (argList = [], option) => run([ ...verifyCompose(), ...argList ], option);
const runComposeSync = (argList = [], option) => runSync([ ...verifyCompose(), ...argList ], option);
const runComposeStdout = (argList = [], option) => runStdout([ ...verifyCompose(), ...argList ], option);
const runComposeStdoutSync = (argList = [], option) => runStdoutSync([ ...verifyCompose(), ...argList ], option);
export { getArgs, setArgs, check, verify, runDocker, runDockerSync, runDockerStdout, runDockerStdoutSync, parseDockerImage, checkLocalImage, pullImage, checkPullImage, getContainerLsList, patchContainerLsListStartedAt, matchContainerLsList, getArgsCompose, setArgsCompose, checkCompose, verifyCompose, runCompose, runComposeSync, runComposeStdout, runComposeStdoutSync };