import { binary } from "../../../common/format.js";
import { isString, isBasicFunction } from "../../../common/check.js";
import { runStdout } from "../../run.js";
import { describeSystemStatus } from "../../system/Status.js";
const IS_WIN32 = "win32" === process.platform;
const IS_DARWIN = "darwin" === process.platform;
const COMMON_HOST_STATUS_COMMAND_LIST = [ [ "Disk", async rootPath => {
  if (IS_WIN32) {
    const freeByteString = (await runQuick("cmd.exe /C dir", rootPath)).match(/ ([\d,]+) bytes free/)[1].replace(/\D/g, "");
    return `${binary(Number(freeByteString))}B free storage`;
  }
  {
    const diskStatus = await runQuick("df -h .", rootPath);
    return diskStatus.includes("/dev/") ? [ diskStatus, "Usage", await runQuick("du -hd1", rootPath) ].join("\n") : diskStatus;
  }
} ], !IS_WIN32 && [ "Network", "vnstat -s" ], [ "System", IS_DARWIN ? "top -l1 -n0" : async rootPath => (await runQuick("top -bn1", rootPath)).split("\n").slice(0, 5).join("\n"), () => describeSystemStatus() ], [ "Time", () => (new Date).toISOString() ] ].filter(Boolean);
const runQuick = async (command, rootPath) => String(await runStdout(command.split(" "), {
  cwd: rootPath
}));
const runStatusCommand = async (statusCommand, rootPath) => {
  let output = "";
  isString(statusCommand) ? output = await runQuick(statusCommand, rootPath) : isBasicFunction(statusCommand) && (output = await statusCommand(rootPath));
  return output;
};
const getCommonHostStatus = async (rootPath, statusCommandList = COMMON_HOST_STATUS_COMMAND_LIST) => {
  const resultList = [];
  for (const [title, ...tryList] of statusCommandList) {
    let output = "";
    for (const statusCommand of tryList) {
      output = await runStatusCommand(statusCommand, rootPath).catch((() => ""));
      if (output) break;
    }
    resultList.push([ title, output ]);
  }
  return resultList;
};
export { COMMON_HOST_STATUS_COMMAND_LIST, getCommonHostStatus };