import { resolve } from "node:path";
import { statSync, realpathSync, existsSync } from "node:fs";
import { tryRequire } from "../../../env/tryRequire.js";
import { compareSemVer } from "../../../common/module/SemVer.js";
import { resolveCommandName } from "../../system/ResolveCommand.js";
import { fetchLikeRequest, fetchWithJump } from "../../net.js";
import { run, runStdout, runSync, runStdoutSync } from "../../run.js";
import { spawnString } from "../function.js";
const findUpPackageRoot = (path = __dirname) => {
  path = resolve(path);
  let prevPath;
  for (;path !== prevPath; ) {
    if (existsSync(resolve(path, "package.json"))) return path;
    prevPath = path;
    path = resolve(path, "..");
  }
};
const findUpNpmrc = (path = __dirname) => {
  path = resolve(path);
  let prevPath;
  for (;path !== prevPath; ) {
    const npmrcPath = resolve(path, ".npmrc");
    if (existsSync(npmrcPath)) return npmrcPath;
    prevPath = path;
    path = resolve(path, "..");
  }
};
let cachePathNpmExecutable;
const getPathNpmExecutable = () => {
  if (void 0 === cachePathNpmExecutable) {
    const command = resolveCommandName("npm");
    cachePathNpmExecutable = command && realpathSync(command);
  }
  return cachePathNpmExecutable;
};
const getSudoArgs = () => "win32" === process.platform ? [ getPathNpmExecutable() ] : [ "sudo", getPathNpmExecutable() ];
const runNpm = (argList = [], option) => run([ getPathNpmExecutable(), ...argList ], option);
const runNpmSync = (argList = [], option) => runSync([ getPathNpmExecutable(), ...argList ], option);
const runNpmStdout = (argList = [], option) => runStdout([ getPathNpmExecutable(), ...argList ], option);
const runNpmStdoutSync = (argList = [], option) => runStdoutSync([ getPathNpmExecutable(), ...argList ], option);
const runSudoNpm = (argList = [], option) => run([ ...getSudoArgs(), ...argList ], option);
const runSudoNpmSync = (argList = [], option) => runSync([ ...getSudoArgs(), ...argList ], option);
const runSudoNpmStdout = (argList = [], option) => runStdout([ ...getSudoArgs(), ...argList ], option);
const runSudoNpmStdoutSync = (argList = [], option) => runStdoutSync([ ...getSudoArgs(), ...argList ], option);
let cachePathNpmGlobalRoot;
const getPathNpmGlobalRoot = () => {
  void 0 === cachePathNpmGlobalRoot && (cachePathNpmGlobalRoot = getPathNpmExecutable() && spawnString([ getPathNpmExecutable(), "root", "-g" ]).trim());
  return cachePathNpmGlobalRoot;
};
const fromGlobalNodeModules = (...args) => resolve(getPathNpmGlobalRoot(), ...args);
let cachePathNpm;
const getPathNpm = () => {
  if (void 0 === cachePathNpm) {
    {
      const npmHelpText = spawnString([ getPathNpmExecutable() ]).trim();
      const npmHelpLastLine = npmHelpText && npmHelpText.split("\n").pop();
      const npmFullPath = npmHelpLastLine && npmHelpLastLine.split(" ").pop();
      npmFullPath && (cachePathNpm = npmFullPath);
    }
    const tryPath = (pathFrom, relativeNpm) => {
      try {
        statSync(resolve(pathFrom, relativeNpm, "./node_modules/semver/")).isDirectory() && (cachePathNpm = resolve(pathFrom, relativeNpm));
      } catch (error) {}
    };
    !cachePathNpm && getPathNpmExecutable().endsWith("npm-cli.js") && tryPath(getPathNpmExecutable(), "../../");
    !cachePathNpm && tryPath(getPathNpmGlobalRoot(), "./npm/");
    !cachePathNpm && getPathNpmExecutable().endsWith("npm.cmd") && tryPath(getPathNpmExecutable(), "../node_modules/npm/");
    cachePathNpm || (cachePathNpm = "");
  }
  return cachePathNpm;
};
const fromNpmNodeModules = (...args) => getPathNpm() && resolve(getPathNpm(), "./node_modules/", ...args);
const hasRepoVersion = async (packageName, packageVersion, cwd = process.cwd()) => {
  try {
    return JSON.parse(String(await runStdout([ getPathNpmExecutable(), "view", `${packageName}@${packageVersion}`, "versions", "--json" ], {
      cwd
    }))).includes(packageVersion);
  } catch (error) {
    return !1;
  }
};
const fetchLikeRequestWithProxy = (url, option = {}) => {
  const npmVer = tryRequire(fromNpmNodeModules("../package.json")).version;
  tryRequire("node:https").request.__agent_base_https_request_patched__ = !0;
  return fetchLikeRequest(url, {
    ...option,
    agent: (compareSemVer(npmVer, "10.0.0") < 0 ? tryRequire(fromNpmNodeModules(compareSemVer(npmVer, "7.16.0") < 0 ? "make-fetch-happen/agent" : "make-fetch-happen/lib/agent")) : tryRequire(fromNpmNodeModules("@npmcli/agent/lib")).getAgent)(url, {
      dns: {
        ttl: 3e5,
        lookup: tryRequire("node:dns").lookup
      },
      ...option
    }),
    secureEndpoint: "https:" === new URL(url).protocol
  });
};
const fetchWithJumpProxy = (initialUrl, option) => fetchWithJump(initialUrl, {
  fetch: fetchLikeRequestWithProxy,
  ...option
});
export { findUpPackageRoot, findUpNpmrc, getPathNpmExecutable, getSudoArgs, runNpm, runNpmSync, runNpmStdout, runNpmStdoutSync, runSudoNpm, runSudoNpmSync, runSudoNpmStdout, runSudoNpmStdoutSync, getPathNpmGlobalRoot, fromGlobalNodeModules, getPathNpm, fromNpmNodeModules, hasRepoVersion, fetchLikeRequestWithProxy, fetchWithJumpProxy };