import { createSecureContext } from "node:tls";
import { objectMap } from "../../../../common/immutable/Object.js";
import { createExotGroup } from "../../../../common/module/Exot.js";
import { readBufferSync } from "../../../fs/File.js";
import { configureLog } from "../../../module/Log.js";
import { configurePid } from "../../../module/Pid.js";
import { addExitListenerLossyOnce } from "../../../system/ExitListener.js";
import { responderEnd, responderEndWithStatusCode, responderEndWithRedirect, createResponderLog, createResponderLogEnd } from "../../Responder/Common.js";
import { createResponderRouter, createRouteMap, describeRouteMap, createResponderRouteListHTML } from "../../Responder/Router.js";
import { createResponderFavicon } from "../../Responder/Send.js";
import { createServerExot, createRequestListener, describeServerOption } from "../../Server.js";
import { enableWSServer, createUpgradeRequestListener } from "../../WS/Server.js";
/** @deprecated */ const configureServerExot = ({protocol = "http:", hostname = "127.0.0.1", port, TLSSNIConfig, TLSDHParam, ...extraOption}) => createServerExot({
  protocol,
  hostname,
  port,
  ..."https:" === protocol && loadTLS(TLSSNIConfig, TLSDHParam),
  ...extraOption
});
const loadTLS = (TLSSNIConfig, TLSDHParam) => {
  TLSSNIConfig.key && (TLSSNIConfig = {
    default: TLSSNIConfig
  });
  TLSSNIConfig.default || (TLSSNIConfig.default = Object.values(TLSSNIConfig)[0]);
  if (!TLSSNIConfig.default) throw new Error("no default TLS config");
  const dhparam = TLSDHParam && autoLoadBuffer(TLSDHParam);
  const optionMap = objectMap(TLSSNIConfig, (({key, cert, ca}) => ({
    key: autoLoadBuffer(key),
    cert: autoLoadBuffer(cert),
    ca: ca && autoLoadBuffer(ca),
    dhparam
  })));
  const secureContextMap = objectMap(optionMap, (option => createSecureContext(option)));
  return {
    ...optionMap.default,
    SNICallback: Object.keys(optionMap).length >= 2 ? (hostname, callback) => callback(null, secureContextMap[hostname] || secureContextMap.default) : void 0
  };
};
const autoLoadBuffer = bufferOrPath => Buffer.isBuffer(bufferOrPath) ? bufferOrPath : readBufferSync(bufferOrPath);
const setupServer = ({protocol = "http:", hostname = "127.0.0.1", port, TLSSNIConfig, TLSDHParam, ...extraOption}) => {
  configurePid(extraOption);
  const {loggerExot} = configureLog(extraOption);
  return {
    loggerExot,
    serverExot: createServerExot({
      protocol,
      hostname,
      port,
      ..."https:" === protocol && loadTLS(TLSSNIConfig, TLSDHParam),
      ...extraOption
    })
  };
};
/** @deprecated */ const configureFeature = (option, featureList = []) => setupFeature(featureList, option);
const setupFeature = (featureList = [], {serverExot, loggerExot, isRawServer = !1, isFavicon = !0, isDebugRoute = !1, rootRouteResponder, preResponderList = [], preRouteList = [], responderLogEnd = createResponderLogEnd({
  loggerExot
})}) => {
  serverExot.featureMap = new Map;
  serverExot.wsSet = void 0;
  preRouteList = preRouteList.filter(Boolean);
  let featureUrl;
  const featureRouteList = [];
  const featureWSRouteList = [];
  featureList.forEach((feature => {
    if (feature) {
      if (!feature.name) throw new Error(`expect feature.name of ${feature}`);
      if (serverExot.featureMap.has(feature.name)) throw new Error(`duplicate feature.name: ${feature.name}`);
      serverExot.featureMap.set(feature.name, feature);
      featureUrl || (featureUrl = feature.URL_HTML);
      feature.routeList && featureRouteList.push(...feature.routeList.filter(Boolean));
      feature.wsRouteList && featureWSRouteList.push(...feature.wsRouteList.filter(Boolean));
    }
  }));
  if (isRawServer) {
    if (preRouteList.length || featureRouteList.length || featureWSRouteList.length) throw new Error("route setting conflict with isRawServer");
    return;
  }
  const routeMap = createRouteMap([ ...preRouteList, ...featureRouteList, isFavicon && [ [ "/favicon", "/favicon.ico" ], "GET", createResponderFavicon() ], [ [ "/", "" ], "GET", rootRouteResponder || (isDebugRoute ? createResponderRouteListHTML({
    getRouteMap: () => routeMap
  }) : featureUrl ? store => responderEndWithRedirect(store, {
    redirectUrl: featureUrl
  }) : store => responderEndWithStatusCode(store, {
    statusCode: 400
  })) ] ].filter(Boolean));
  serverExot.server.on("request", createRequestListener({
    responderList: [ ...preResponderList, createResponderLog({
      loggerExot
    }), createResponderRouter({
      routeMap,
      serverExot
    }) ].filter(Boolean),
    responderEnd: store => {
      responderEnd(store);
      return responderLogEnd(store);
    }
  }));
  if (0 !== featureWSRouteList.length) {
    const routeMap = createRouteMap(featureWSRouteList);
    serverExot.wsSet = enableWSServer(serverExot.server, {
      onUpgradeRequest: createUpgradeRequestListener({
        responderList: [ createResponderLog({
          loggerExot
        }), createResponderRouter({
          routeMap,
          serverExot
        }) ]
      })
    });
  }
  return {
    RouteList: describeRouteMap(routeMap),
    RouteListWS: 0 === featureWSRouteList.length ? [] : describeRouteMap(createRouteMap(featureWSRouteList))
  };
};
const setupServerExotGroup = (serverExot, loggerExot, serverExotGroup = createExotGroup({
  id: "exot:group-server",
  getOnExotError: serverExotGroup => error => {
    console.log("[exot-group-error]", error);
    return serverExotGroup.down();
  }
})) => {
  loggerExot && serverExotGroup.set(loggerExot);
  if (serverExot.featureMap) for (const {exotList} of serverExot.featureMap.values()) if (exotList && exotList.length) for (const exot of exotList) serverExotGroup.set(exot);
  serverExotGroup.set(serverExot);
  return serverExotGroup;
};
const startServerExotGroup = async ({loggerExot, serverExot, serverExotGroup = loggerExot ? setupServerExotGroup(serverExot, loggerExot) : serverExot, isMuteLog = !loggerExot || !1}) => {
  addExitListenerLossyOnce((eventPack => {
    isMuteLog || loggerExot.add(`[SERVER] down... ${JSON.stringify(eventPack)}${eventPack.error ? ` ${eventPack.error.stack || eventPack.error}` : ""}`);
    return serverExotGroup.down().then((() => isMuteLog || loggerExot.add("[SERVER] down")));
  }));
  isMuteLog || loggerExot.add("[SERVER] up...");
  await serverExotGroup.up();
  isMuteLog || loggerExot.add("[SERVER] up");
};
/** @deprecated */ const runServerExotGroup = async pack => {
  await startServerExotGroup(pack);
  return pack;
};
/** @deprecated */ const runServer = async (configureServer, serverOption, featureOption, serverInfoTitle = (featureOption.packageName ? `${featureOption.packageName}@${featureOption.packageVersion}` : void 0)) => {
  const {loggerExot, serverExot} = setupServer(serverOption);
  const serverInfo = describeServerOption(serverExot.option, serverInfoTitle, featureOption);
  await configureServer({
    serverExot,
    loggerExot,
    serverInfo,
    ...featureOption
  });
  loggerExot.add(serverInfo);
  return runServerExotGroup({
    serverOption,
    featureOption,
    loggerExot,
    serverExot,
    serverExotGroup: setupServerExotGroup(serverExot, loggerExot)
  });
};
export { setupServer, setupFeature, setupServerExotGroup, startServerExotGroup, configureServerExot, configureFeature, runServerExotGroup, runServer };