import { dirname } from "node:path";
import { createWriteStream, promises as fsAsync } from "node:fs";
import { withRetryAsync } from "../../../../common/function.js";
import { percent, binary } from "../../../../common/format.js";
import { lazyEncodeURI } from "../../../../common/string.js";
import { quickRunletFromStream } from "../../../data/Stream.js";
import { createDirectory } from "../../../fs/Directory.js";
import { uploadFileByChunk } from "../../../module/FileChunkUpload.js";
const fileUpload = async ({fileInputPath: filePath, fileBuffer, key, urlFileUpload, timeout = 3e4, maxRetry = 4, wait = 1e3, authFetch, log, onProgress = (uploadedSize, totalSize) => log && log(`[Upload] upload: ${percent(uploadedSize / totalSize)}`)}) => {
  const fileSize = fileBuffer ? fileBuffer.length : (await fsAsync.stat(filePath)).size;
  log && log(`[Upload] key: ${key}, size: ${binary(fileSize)}B`);
  await uploadFileByChunk({
    fileBuffer,
    filePath,
    fileSize,
    key,
    onProgress,
    uploadChunk: async (arrayBufferPacket, {key, chunkIndex, chunkTotal}) => withRetryAsync((async () => authFetch(urlFileUpload, {
      method: "POST",
      body: arrayBufferPacket,
      timeout
    }).catch((error => {
      const message = `[ERROR][Upload] upload chunk ${chunkIndex}/${chunkTotal} of ${key}, packet size: ${arrayBufferPacket.byteLength}`;
      log && log(message, error);
      throw new Error(message);
    }))), maxRetry, wait)
  });
  log && log(`[Upload] done: ${key}`);
};
const fileDownload = async ({fileOutputPath, key, urlFileDownload, timeout = 3e4, authFetch, log}) => {
  log && log(`[Download] key: ${key}`);
  fileOutputPath && await createDirectory(dirname(fileOutputPath));
  const response = await authFetch(`${urlFileDownload}/${lazyEncodeURI(key)}`, {
    method: "GET",
    timeout
  });
  if (!fileOutputPath) {
    const fileBuffer = await response.buffer();
    log && log(`[Download] get: ${binary(fileBuffer.length)}B`);
    return fileBuffer;
  }
  await quickRunletFromStream(response.stream(), createWriteStream(fileOutputPath));
  log && log(`[Download] done: ${fileOutputPath}`);
};
export { fileUpload, fileDownload };