import { catchAsync } from "../../../common/error.js";
import { createCacheMap2 } from "../../../common/data/CacheMap2.js";
import { responderEndWithStatusCode } from "./Common.js";
const DEFAULT_RESPONDER_DENY = (store, limitLeft) => responderEndWithStatusCode(store, {
  statusCode: limitLeft <= 0 ? 429 : 400
});
const DEFAULT_GET_REQUEST_KEY = store => `${store.request.socket.remoteAddress}`;
const createResponderRateLimit = ({responderNext, responderDeny = DEFAULT_RESPONDER_DENY, getRequestKey = DEFAULT_GET_REQUEST_KEY, limitCount = 32, expireTime = 6e5, rateCacheMap = createCacheMap2({
  sizeMax: 32768,
  expireAfter: expireTime
})}) => store => {
  const cacheKey = getRequestKey(store);
  let limitLeft = rateCacheMap.get(cacheKey);
  void 0 === limitLeft && (limitLeft = limitCount);
  if (limitLeft <= 0) return responderDeny(store, limitLeft);
  rateCacheMap.set(cacheKey, limitLeft - 1);
  return responderNext(store, limitLeft);
};
const createResponderCheckRateLimit = ({checkFunc, responderNext, responderDeny = DEFAULT_RESPONDER_DENY, getRequestKey = DEFAULT_GET_REQUEST_KEY, limitCount = 32, expireTime = 6e5, rateCacheMap = createCacheMap2({
  sizeMax: 32768,
  expireAfter: expireTime
})}) => async store => {
  const cacheKey = getRequestKey(store);
  let limitLeft = rateCacheMap.get(cacheKey);
  void 0 === limitLeft && (limitLeft = limitCount);
  if (limitLeft <= 0) return responderDeny(store, limitLeft);
  const {error, result: isCheckPass} = await catchAsync(checkFunc, store, limitLeft);
  if (isCheckPass) return responderNext(store, limitLeft);
  rateCacheMap.set(cacheKey, limitLeft - 1);
  return responderDeny(store, limitLeft, error);
};
export { createResponderRateLimit, createResponderCheckRateLimit };