import { isBasicObject } from "../../../common/check.js";
import { createTreeDepthFirstSearch } from "../../../common/data/Tree.js";
import { parseRouteToMap, findRouteFromMap, getRouteParamAny as getRouteMapParamAny, getRouteParam as getRouteMapParam } from "../../../common/module/RouteMap.js";
import { BASIC_EXTENSION_MAP } from "../../../common/module/MIME.js";
import { COMMON_LAYOUT, COMMON_STYLE } from "../../../common/module/HTML.js";
import { responderSendBufferCompress, prepareBufferData } from "./Send.js";
const METHOD_MAP = {
  GET: "/GET",
  POST: "/POST",
  PUT: "/PUT",
  PATCH: "/PATCH",
  DELETE: "/DELETE",
  HEAD: "/HEAD",
  OPTIONS: "/OPTIONS",
  CONNECT: "/CONNECT",
  TRACE: "/TRACE"
};
const appendRouteMap = (routeMap, routeInput = "/", method = "GET", routeResponder, routePrefix = "") => {
  if (Array.isArray(routeInput)) return routeInput.reduce(((o, v) => appendRouteMap(routeMap, v, method, routeResponder, routePrefix)), routeMap);
  if (Array.isArray(method)) return method.reduce(((o, v) => appendRouteMap(routeMap, routeInput, v, routeResponder, routePrefix)), routeMap);
  const route = routePrefix ? `${routePrefix}${routeInput}` : routeInput;
  const {routeNode, paramNameList} = parseRouteToMap(routeMap, route);
  const methodTag = METHOD_MAP[method];
  if (!methodTag) throw new Error(`invalid method [${method}] for: ${route}`);
  if (routeNode[methodTag]) throw new Error(`duplicate method [${method}] for: ${route}`);
  if ("function" != typeof routeResponder) throw new Error(`invalid responder for: ${route}`);
  routeNode[methodTag] = {
    route,
    paramNameList,
    routeResponder
  };
  return routeMap;
};
const createRouteMap = (configList, routePrefix) => configList.reduce(((o, [route, method, routeResponder]) => appendRouteMap(o, route, method, routeResponder, routePrefix)), {});
const createResponderRouter = ({routeMap, serverExot, baseUrl = serverExot ? serverExot.option.baseUrl : "", getMethodUrl = createGetMethodUrl(new URL(baseUrl))}) => store => {
  const {method, url} = getMethodUrl(store);
  const methodTag = METHOD_MAP[method];
  if (void 0 === methodTag) return;
  const routeData = findRouteFromMap(routeMap, url.pathname);
  if (void 0 === routeData) return;
  const {routeNode, paramValueList} = routeData;
  if (void 0 === routeNode[methodTag]) return;
  const {route, paramNameList, routeResponder} = routeNode[methodTag];
  const paramMap = paramNameList.reduce(((o, paramName, index) => {
    o[paramName] = paramValueList[index];
    return o;
  }), {});
  return routeResponder(store, store.setState({
    method,
    url,
    route,
    paramMap
  }));
};
const createGetMethodUrl = baseUrl => ({request: {method, url}}) => ({
  method,
  url: new URL(url.replace(REGEXP_URL_REPLACE, "/"), baseUrl)
});
const REGEXP_URL_REPLACE = /\/\//g;
const getRouteParamAny = store => getRouteMapParamAny(store.getState());
const getRouteParam = (store, paramName) => getRouteMapParam(store.getState(), paramName);
const describeRouteMap = routeMap => {
  const routeInfoList = [];
  const methodValueSet = new Set(Object.values(METHOD_MAP));
  routeMapDepthFirstSearch([ "ROOT", routeMap ], (([key, {route}]) => {
    methodValueSet.has(key) && routeInfoList.push({
      method: key,
      route
    });
  }));
  return routeInfoList;
};
const routeMapDepthFirstSearch = createTreeDepthFirstSearch((([key, routeNode]) => isBasicObject(routeNode) && Object.entries(routeNode)));
const createResponderRouteListHTML = ({getRouteMap, extraBodyList}) => {
  let bufferData;
  return async store => {
    void 0 === bufferData && (bufferData = await prepareBufferData(Buffer.from(getRouteListHTML(getRouteMap(), extraBodyList)), BASIC_EXTENSION_MAP.html));
    return responderSendBufferCompress(store, bufferData);
  };
};
const getRouteListHTML = (routeMap, extraBodyList = []) => COMMON_LAYOUT([ COMMON_STYLE(), "<style>body { align-items: start; }</style>" ], [ "<h2>Route List</h2>", "<table>", ...describeRouteMap(routeMap).map((({method, route}) => `<tr><td><b>${method}</b></td><td>${"/GET" === method ? `<a href="${route}">${route}</a>` : route}</td></tr>`)), "</table>", ...extraBodyList ]);
export { METHOD_MAP, createRouteMap, createResponderRouter, appendRouteMap, getRouteParamAny, getRouteParam, describeRouteMap, createResponderRouteListHTML };