import { createInsideOutPromise } from "../../../common/function.js";
import { setWeakTimeout } from "../../../common/time.js";
import { wrapAsync, createLockStepAsyncIter } from "../../../common/data/Iter.js";
import { createAsyncFuncQueue } from "../../../common/module/AsyncFuncQueue.js";
import { createRunlet, createCountPool, PoolIO, toPoolMap, toChipMap, toLinearChipList, quickConfigPend } from "../../../common/module/Runlet.js";
import { createAsyncIterInputChip, createAsyncIterOutputChip } from "../../../common/module/RunletChip.js";
import { createReadableStreamInputChip, createWritableStreamOutputChip } from "../../data/Stream.js";
import { createFrameDecodeChip } from "./frameDecode.js";
import { createFrameEncodeChip, encodeTextFramePack, encodeBinaryFramePack, createCloseFramePack, encodePingFramePack, encodePongFramePack } from "./frameEncode.js";
const DEFAULT_BUFFER = Buffer.from("Dr");
const createWSBase = ({socket, info = {}, dataLengthLimit = 8388608, isMask = !1, shouldPing = !1}) => {
  let readyState = 0;
  const closeClear = async error => {
    if (3 !== readyState) {
      readyState = 3;
      Object.assign(info, {
        error
      });
      closeTimeoutToken && clearTimeout(closeTimeoutToken);
      pingCheckTimeoutToken && clearTimeout(pingCheckTimeoutToken);
      if (runletRead.getIsValid()) {
        LSAIRead.abort();
        runletRead.detach();
      }
      if (runletWrite.getIsValid()) {
        await pushLSAI(void 0, !0);
        await promise;
        runletWrite.detach();
      }
      socket.destroy();
    }
  };
  let isCloseReceived = !1;
  let closeTimeoutToken = null;
  const close = async (code = 1e3, reason = "") => {
    if (3 === readyState || 2 === readyState) return closeClear();
    if (1 !== readyState) throw new Error(`invalid readyState on close: ${readyState}`);
    readyState = 2;
    closeTimeoutToken = setWeakTimeout(closeClear, 5e3);
    return pushLSAI(createCloseFramePack(code, reason)).then(isCloseReceived ? closeClear : () => promise, closeClear);
  };
  let pingBuffer = null;
  let pongBuffer = null;
  let pingCheckTimeoutToken = null;
  const ping = async (buffer = DEFAULT_BUFFER) => {
    if (1 === readyState && !pingBuffer) {
      pingCheckTimeoutToken && clearTimeout(pingCheckTimeoutToken);
      pingCheckTimeoutToken = setWeakTimeout(pingCheck, 6e4);
      pingBuffer = buffer;
      pongBuffer = null;
      return pushLSAI(encodePingFramePack(buffer));
    }
  };
  const pingCheck = async () => pongBuffer && 0 === Buffer.compare(pingBuffer, pongBuffer) ? shouldPing && ping() : close(1006, "ping check failed");
  const onError = error => runletWrite.getIsValid() ? close(1006, `error: ${error.message}`) : closeClear(error);
  const LSAIRead = createLockStepAsyncIter();
  const LSAIWrite = createLockStepAsyncIter();
  const {push} = createAsyncFuncQueue();
  const pushLSAI = (framePack, done) => push((() => LSAIWrite.send(framePack, done)));
  const IOP = createInsideOutPromise();
  const promise = IOP.promise.catch(onError);
  const runletRead = createRunlet(quickConfigPend(toPoolMap([ PoolIO, createCountPool({
    sizeLimit: 16
  }) ]), toChipMap(toLinearChipList([ createReadableStreamInputChip({
    stream: socket
  }), createFrameDecodeChip({
    respondClose: async (code, reason) => {
      Object.assign(info, {
        code,
        reason
      });
      isCloseReceived = !0;
      return close(code, reason);
    },
    respondPing: async buffer => {
      if (1 === readyState) return pushLSAI(encodePongFramePack(buffer));
    },
    respondPong: buffer => {
      pongBuffer = buffer;
    },
    dataLengthLimit
  }), createAsyncIterOutputChip({
    LSAI: LSAIRead
  }) ])), {
    onError
  }));
  const runletWrite = createRunlet(quickConfigPend(toPoolMap([ PoolIO, createCountPool({
    sizeLimit: 16
  }) ]), toChipMap(toLinearChipList([ createAsyncIterInputChip({
    next: LSAIWrite.next
  }), createFrameEncodeChip({
    isMask,
    dataLengthLimit
  }), createWritableStreamOutputChip({
    stream: socket,
    IOP
  }) ])), {
    onError
  }));
  readyState = 1;
  runletRead.attach();
  runletWrite.attach();
  runletRead.trigger();
  runletWrite.trigger();
  shouldPing && ping();
  return {
    ...wrapAsync(LSAIRead.next),
    promise,
    CONNECTING: 0,
    OPEN: 1,
    CLOSING: 2,
    CLOSED: 3,
    socket,
    info,
    dataLengthLimit,
    isMask,
    shouldPing,
    getIsOpen: () => 1 === readyState,
    close,
    closeClear,
    sendText: async string => {
      if (1 !== readyState) throw new Error(`not open yet: readyState = ${readyState}`);
      return pushLSAI(encodeTextFramePack(string));
    },
    sendBinary: async buffer => {
      if (1 !== readyState) throw new Error(`not open yet: readyState = ${readyState}`);
      return pushLSAI(encodeBinaryFramePack(buffer));
    },
    ping
  };
};
export { createWSBase };