import { createInsideOutPromise } from "../../../common/function.js";
import { requestHttp } from "../../net.js";
import { createWSBase } from "./Base.js";
import { WEBSOCKET_VERSION, getRequestKey, getRespondKey, packProtocolList } from "./function.js";
const VALID_PROTOCOL_SET = new Set([ "wss:", "ws:", "https:", "http:" ]);
const SECURE_PROTOCOL_SET = new Set([ "wss:", "https:" ]);
const createWSClient = (url, {protocolList, requestKey = getRequestKey(), onUpgradeResponse = (response, socket, headBuffer, info) => {}, dataLengthLimit, isMask = !0, shouldPing = !1, ...option}) => {
  url = url instanceof URL ? url : new URL(url);
  if (!VALID_PROTOCOL_SET.has(url.protocol)) throw new Error(`invalid url protocol: ${url.protocol}`);
  if (!url.host) throw new Error(`invalid url host: ${url.host}`);
  const isSecure = SECURE_PROTOCOL_SET.has(url.protocol);
  url.protocol = isSecure ? "https:" : "http:";
  option.headers = {
    ...option.headers,
    upgrade: "websocket",
    connection: "upgrade",
    "sec-websocket-version": WEBSOCKET_VERSION,
    "sec-websocket-key": requestKey,
    "sec-websocket-protocol": packProtocolList(protocolList)
  };
  const IOP = createInsideOutPromise();
  const onError = error => {
    request.destroy();
    IOP.reject(error);
  };
  const {request, promise} = requestHttp(url, option);
  promise.then((response => {
    onError(new Error("unexpected response"));
  }), onError);
  request.once("upgrade", (async (response, socket, headBuffer) => {
    try {
      const protocol = response.headers["sec-websocket-protocol"];
      if (!protocolList.includes(protocol)) throw new Error(`unexpected protocol ${protocol}`);
      if (getRespondKey(requestKey) !== response.headers["sec-websocket-accept"]) throw new Error("wrong sec-websocket-accept");
      const info = {
        isSecure,
        protocol,
        protocolList,
        headers: response.headers,
        url
      };
      await onUpgradeResponse(response, socket, headBuffer, info);
      IOP.resolve(createWSBase({
        socket,
        info,
        dataLengthLimit,
        isMask,
        shouldPing
      }));
    } catch (error) {
      onError(error);
    }
  }));
  return IOP.promise;
};
export { createWSClient };