import { END, SKIP, REDO, createPack } from "../../../common/module/Runlet.js";
import { createBufferRefragPool } from "../../data/Buffer.js";
import { OPCODE_TYPE, BUFFER_MAX_LENGTH, applyMaskQuadletBufferInPlace } from "./function.js";
const decodeRawFrame = ({dataLengthLimit, refragPool, decodeState}) => {
  let mergedBuffer = null;
  switch (decodeState.stage) {
   case 0:
    if (mergedBuffer = refragPool.tryGetRefragBuffer(2)) {
      const initialQuadlet = mergedBuffer.readUInt16BE(0);
      const quadbitFIN = initialQuadlet >>> 12 & 8;
      const quadbitOpcode = initialQuadlet >>> 8 & 15;
      const initialLength = 127 & initialQuadlet;
      decodeState.isMask = !!(128 & initialQuadlet);
      decodeState.rawFrameData.isFIN = 8 === quadbitFIN;
      decodeState.rawFrameData.opcode = quadbitOpcode;
      if (0 === initialLength) {
        decodeState.dataLength = 0;
        decodeState.stage = decodeState.isMask ? 3 : 5;
      } else if (initialLength <= 125) {
        decodeState.dataLength = initialLength;
        if (decodeState.dataLength > dataLengthLimit) throw new Error(`dataLength ${decodeState.dataLength} exceeds limit: ${dataLengthLimit}`);
        decodeState.stage = decodeState.isMask ? 3 : 4;
      } else decodeState.stage = 126 === initialLength ? 1 : 2;
      return !0;
    }
    break;
   case 1:
    if (mergedBuffer = refragPool.tryGetRefragBuffer(2)) {
      decodeState.dataLength = mergedBuffer.readUInt16BE(0);
      if (decodeState.dataLength > dataLengthLimit) throw new Error(`dataLength ${decodeState.dataLength} exceeds limit: ${dataLengthLimit}`);
      decodeState.stage = decodeState.isMask ? 3 : 4;
      return !0;
    }
    break;
   case 2:
    if (mergedBuffer = refragPool.tryGetRefragBuffer(8)) {
      decodeState.dataLength = 4294967296 * mergedBuffer.readUInt32BE(0) + mergedBuffer.readUInt32BE(4);
      if (decodeState.dataLength > BUFFER_MAX_LENGTH) throw new Error(`dataLength too big: ${decodeState.dataLength}`);
      if (decodeState.dataLength > dataLengthLimit) throw new Error(`dataLength ${decodeState.dataLength} exceeds limit: ${dataLengthLimit}`);
      decodeState.stage = decodeState.isMask ? 3 : 4;
      return !0;
    }
    break;
   case 3:
    if (mergedBuffer = refragPool.tryGetRefragBuffer(4)) {
      decodeState.maskQuadletBuffer = mergedBuffer;
      decodeState.stage = decodeState.dataLength ? 4 : 5;
      return !0;
    }
    break;
   case 4:
    if (mergedBuffer = refragPool.tryGetRefragBuffer(decodeState.dataLength)) {
      decodeState.isMask && applyMaskQuadletBufferInPlace(mergedBuffer, decodeState.maskQuadletBuffer);
      decodeState.rawFrameData.buffer = mergedBuffer;
      decodeState.stage = 5;
      return !0;
    }
  }
  return !1;
};
const tryPickRawFrameData = state => {
  if (5 !== state.decodeState.stage) return;
  const {decodeState: {rawFrameData}} = state;
  state.decodeState = {
    stage: 0,
    isMask: !1,
    maskQuadletBuffer: null,
    dataLength: 0,
    rawFrameData: {
      isFIN: !1,
      opcode: 0,
      buffer: null
    }
  };
  return rawFrameData;
};
const processRawFrameData = (state, rawFrameData) => {
  const {respondClose, respondPing, respondPong} = state;
  const {isFIN, opcode, buffer} = rawFrameData;
  switch (opcode) {
   case OPCODE_TYPE.CONTINUATION:
   case OPCODE_TYPE.TEXT:
   case OPCODE_TYPE.BINARY:
    {
      const {framePack} = state;
      0 === framePack.bufferList.length && (framePack.opcode = opcode);
      framePack.bufferList.push(buffer);
      if (isFIN) {
        framePack.buffer = 1 === framePack.bufferList.length ? framePack.bufferList[0] : Buffer.concat(framePack.bufferList);
        framePack.bufferList = void 0;
        state.framePack = {
          opcode: null,
          buffer: null,
          bufferList: []
        };
        return framePack;
      }
      break;
    }
   case OPCODE_TYPE.CLOSE:
    respondClose(buffer.length >= 2 && buffer.readUInt16BE(0) || 1e3, buffer.length >= 3 && String(buffer.slice(2, buffer.length)) || "");
    break;
   case OPCODE_TYPE.PING:
    respondPing(buffer);
    break;
   case OPCODE_TYPE.PONG:
    respondPong(buffer);
    break;
   default:
    throw new Error(`invalid opcode: ${opcode}`);
  }
};
const createFrameDecodeChip = ({respondFrame, respondClose, respondPing, respondPong, dataLengthLimit = 16777216, key = "chip:frame-decode", ...extra}) => ({
  ...extra,
  key,
  state: {
    respondFrame,
    respondClose,
    respondPing,
    respondPong,
    dataLengthLimit,
    rawFrameDataList: [],
    refragPool: createBufferRefragPool(),
    decodeState: {
      stage: 0,
      isMask: !1,
      maskQuadletBuffer: null,
      dataLength: 0,
      rawFrameData: {
        isFIN: !1,
        opcode: 0,
        buffer: null
      }
    },
    framePackList: [],
    framePack: {
      opcode: null,
      buffer: null,
      bufferList: []
    }
  },
  sync: !0,
  process: (pack, state, error) => {
    if (!error) {
      void 0 !== pack[0] && state.refragPool.pushFrag(pack[0]);
      for (;!0 === decodeRawFrame(state); ) {
        const rawFrameData = tryPickRawFrameData(state);
        void 0 !== rawFrameData && state.rawFrameDataList.push(rawFrameData);
      }
      for (;0 !== state.rawFrameDataList.length; ) {
        const framePack = processRawFrameData(state, state.rawFrameDataList.shift());
        void 0 !== framePack && state.framePackList.push(framePack);
      }
      if (0 === state.framePackList.length) pack[1] !== END && (pack = createPack(void 0, SKIP)); else if (state.framePackList.length >= 2 || pack[1] === END) {
        pack[0] = void 0;
        pack = createPack(state.framePackList.shift(), REDO);
      } else pack[0] = state.framePackList.shift();
      return {
        pack,
        state
      };
    }
  }
});
export { createFrameDecodeChip };