import { createServer as createNetServer } from "node:net";
import { tryRequireResolve } from "../../env/tryRequire.js";
import { catchAsync } from "../../common/error.js";
import { readableStreamToBufferAsync } from "../data/Stream.js";
import { readTextSync } from "../fs/File.js";
const DR_BROWSER_FILE_PATH = () => [ "./Dr.browser.js", "../Dr.browser.js", "@dr-js/core/library/Dr.browser.js" ].reduce(((o, path) => o || tryRequireResolve(path)), null);
let cache = "";
const DR_BROWSER_SCRIPT_TAG = () => {
  "" === cache && (cache = `<script>${readTextSync(DR_BROWSER_FILE_PATH())}<\/script>`);
  return cache;
};
const getUnusedPort = (expectPort = 0, hostname = "0.0.0.0") => new Promise(((resolve, reject) => {
  const server = createNetServer();
  server.on("error", reject);
  server.listen({
    host: hostname,
    port: expectPort,
    exclusive: !0
  }, (error => {
    if (error) return reject(error);
    const {port} = server.address();
    server.close((() => resolve(port)));
  }));
}));
const autoTestServerPort = async (expectPortList, hostname) => {
  for (const expectPort of expectPortList) {
    const {result, error} = await catchAsync(getUnusedPort, expectPort, hostname);
    if (result) return result;
  }
  return getUnusedPort(0, hostname);
};
const parseHostString = (host, defaultHostname) => {
  const hostnameList = host.split(":");
  const port = Number(hostnameList.pop()) || void 0;
  return {
    hostname: hostnameList.join(":") || defaultHostname || void 0,
    port
  };
};
const parseCookieString = cookieString => cookieString.split(";").reduce(((o, v) => {
  const [key, ...valueList] = v.split("=");
  const value = valueList.join("=").trim();
  "" !== value && (o[key.trim()] = value);
  return o;
}), {});
const isRequestAborted = store => Boolean(store.request.destroyed || store.request.aborted);
const getRequestParam = (store, key) => {
  const {headers} = store.request;
  return headers[key] || store.getState().url.searchParams.get(key) || headers.cookie && decodeURIComponent(parseCookieString(headers.cookie)[key] || "") || store.info && getWSProtocolListParam(store.info.protocolList, key);
};
const getRequestBuffer = store => readableStreamToBufferAsync(store.request);
const getRequestJSON = store => getRequestBuffer(store).then((buffer => 0 === buffer.length ? void 0 : JSON.parse(String(buffer))));
const getWSProtocolListParam = (protocolList = [], key) => {
  const header = `${encodeURIComponent(key)}=`;
  const protocol = protocolList.find((protocol => protocol.startsWith(header)));
  return protocol && decodeURIComponent(protocol.slice(header.length));
};
const packWSProtocolListParam = (key, value) => `${encodeURIComponent(key)}=${encodeURIComponent(value)}`;
const isPrivateAddress = address => "127.0.0.1" === address || "0.0.0.0" === address || "[::1]" === address || "[::]" === address || address.startsWith("192.168.") || address.startsWith("127.") || address.startsWith("10.") || address.startsWith("[fc") || address.startsWith("[fd") || address.startsWith("[fe") || "localhost" === address;
export { DR_BROWSER_FILE_PATH, DR_BROWSER_SCRIPT_TAG, getUnusedPort, autoTestServerPort, parseHostString, parseCookieString, isRequestAborted, getRequestParam, getRequestBuffer, getRequestJSON, getWSProtocolListParam, packWSProtocolListParam, isPrivateAddress };