import { catchAsync, catchSync, catchPromise } from "../../common/error.js";
import { once } from "../../common/function.js";
const EXIT_LISTENER_LIST = [ [ "exit", code => exitListenerSync({
  eventType: "exit",
  code
}) ], [ "uncaughtException", error => exitListenerAsync({
  eventType: "uncaughtException",
  error
}) ], [ "unhandledRejection", (error, promise) => exitListenerAsync({
  eventType: "unhandledRejection",
  error,
  promise
}) ], ...[ "SIGINT", "SIGHUP", "SIGQUIT", "SIGTERM" ].map((eventName => [ eventName, () => exitListenerAsync({
  eventType: "signal",
  signalEventType: eventName
}) ])) ];
const listenerSyncSet = new Set;
const listenerAsyncSet = new Set;
const exitListenerSync = eventPack => {
  isBind && unbindExitListener();
  for (const listenerSync of Array.from(listenerSyncSet)) catchSync(listenerSync, eventPack);
  process.exitCode = eventPack.code || eventPack.error && 1 || 0;
};
const exitListenerAsync = async eventPack => {
  isBind && unbindExitListener();
  for (const listenerAsync of Array.from(listenerAsyncSet)) await catchAsync(listenerAsync, eventPack);
  exitListenerSync(eventPack);
};
let isBind = !1;
const bindExitListener = () => {
  isBind = !0;
  EXIT_LISTENER_LIST.forEach((([eventName, listenerFunc]) => process.on(eventName, listenerFunc)));
};
const unbindExitListener = () => {
  isBind = !1;
  EXIT_LISTENER_LIST.forEach((([eventName, listenerFunc]) => process.off(eventName, listenerFunc)));
};
const clearExitListener = () => {
  isBind && unbindExitListener();
  listenerSyncSet.clear();
  listenerAsyncSet.clear();
};
const addExitListenerSync = (...listenerSyncList) => {
  listenerSyncList.forEach((listenerSync => listenerSyncSet.add(listenerSync)));
  !isBind && bindExitListener();
};
const addExitListenerAsync = (...listenerSyncList) => {
  listenerSyncList.forEach((listenerAsync => listenerAsyncSet.add(listenerAsync)));
  !isBind && bindExitListener();
};
const addExitListenerLossyOnce = listener => {
  const onceListener = once(listener);
  addExitListenerAsync(onceListener);
  addExitListenerSync(onceListener);
  return onceListener;
};
const deleteExitListenerSync = (...listenerSyncList) => {
  listenerSyncList.forEach((listenerSync => listenerSyncSet.delete(listenerSync)));
};
const deleteExitListenerAsync = (...listenerSyncList) => {
  listenerSyncList.forEach((listenerAsync => listenerAsyncSet.delete(listenerAsync)));
};
const guardPromiseEarlyExit = async (callback, promise) => {
  const earlyExitCheck = code => 0 === code && !0 === isTestRunning && callback();
  let isTestRunning = !0;
  process.on("exit", earlyExitCheck);
  const {result, error} = await catchPromise(promise);
  isTestRunning = !1;
  process.off("exit", earlyExitCheck);
  if (error) throw error;
  return result;
};
export { clearExitListener, addExitListenerSync, addExitListenerAsync, addExitListenerLossyOnce, deleteExitListenerSync, deleteExitListenerAsync, guardPromiseEarlyExit };