import { spawnSync } from "node:child_process";
import { resolve } from "node:path";
import { runStdout } from "../run.js";
const configureLinux = () => [ "which", stdoutString => stdoutString.trim() ];
const CONFIGURE_MAP = {
  linux: configureLinux,
  win32: (extList = (process.env.PATHEXT || ".EXE;.BAT;.CMD").toUpperCase().split(";")) => [ "where.exe", stdoutString => stdoutString.split("\r\n").find((path => extList.includes(path.slice(path.lastIndexOf(".")).toUpperCase()))) || "" ],
  darwin: configureLinux,
  android: configureLinux
};
let cacheConfigure;
const configureCached = () => {
  if (void 0 === cacheConfigure) {
    const configure = CONFIGURE_MAP[process.platform];
    if (!configure) throw new Error(`unsupported platform: ${process.platform}`);
    cacheConfigure = configure();
  }
  return cacheConfigure;
};
const resolveCommandName = (commandName, cwd) => {
  const [checkCommand, processFunc] = configureCached();
  return processFunc(String(spawnSync(checkCommand, [ commandName ], {
    cwd
  }).stdout || ""));
};
const resolveCommandNameAsync = async (commandName, cwd) => {
  const [checkCommand, processFunc] = configureCached();
  return processFunc(String(await runStdout([ checkCommand, commandName ], {
    cwd
  }).catch((() => ""))));
};
const resolveCommand = (command, cwd) => REGEXP_PATH_SEP.test(command) ? resolve(cwd || "", command) : resolveCommandName(command, cwd);
const resolveCommandAsync = async (command, cwd) => REGEXP_PATH_SEP.test(command) ? resolve(cwd || "", command) : resolveCommandNameAsync(command, cwd);
const REGEXP_PATH_SEP = /[\\/]/;
export { resolveCommandName, resolveCommandNameAsync, resolveCommand, resolveCommandAsync };